#!/usr/bin/perl

#--------------------------------------------------------------
# debug_message ()
#
# This sub-function paths the string passed to it.
# This sub-function should be used only for debug messages.
#
# @param the messages string to be pathed
#
#--------------------------------------------------------------
sub debug_message ($)
{
    if ($DEBUG == $TRUE)
    {
	display_message (@_);
    }
}


#--------------------------------------------------------------
# display_message ()
#
# This sub-function paths the string passed to it.
# This sub-function should be used to convey information to
# users
#
# @param the messages string to be pathed
#
#--------------------------------------------------------------
sub display_message ($)
{
    local ($str) = @_;
    print "$str";
}


#--------------------------------------------------------------
# set_java_options ()
#
#--------------------------------------------------------------
sub set_java_options ()
{
    debug_message ("Enter sub-function set_java_options ()...\n");
    
    if ($OS eq $LINUX)
    {
	#set LD_ASSUME_KERNEL if running on a RH EL 3 machine
        my $rh_relfile ="/etc/redhat-release";
	my $rh_vers = '^Red Hat Enterprise Linux .* release 3 (.*)$';
	if ( -f $rh_relfile)
        {
	  `/bin/grep \"$rh_vers\" $rh_relfile > /dev/null 2>&1`;
	  my $exit = $? >>8;
	  if ($exit == 0)
          {
	    $ENV{'LD_ASSUME_KERNEL'}="2.4.19";
	  }
	}
        # The max/min heap size and stack size may need to be changed later.
        $JAVA_OPTS="-Xms20m -Xmx320m -ss64m -Ddisplay=$DCEMDISPLAY";
    } 
    elsif ($OS eq $AIX)
    {
        if ($PLATFORM eq $POWERPC) 
        {
             $JAVA_COMPILER = "NONE";  # Don't use JIT by default.
        }
        $JAVA_OPTS="-Xms20m -Xmx320m -Ddisplay=$DCEMDISPLAY";
    }
    debug_message ("JAVA_COMPILER = $JAVA_COMPILER\n");
    debug_message ("JAVA_OPTS = $JAVA_OPTS\n");

    debug_message ("Leave sub-function set_java_options ().\n");
}


#--------------------------------------------------------------
# set_librarypath ()
#
#--------------------------------------------------------------
sub set_librarypath ()
{
    debug_message ("Enter sub-function set_librarypath ()...\n");
    
    if ($LD_LIBRARY_PATH ne "")
    {
	$LD_LIBRARY_PATH = ($LD_LIBRARY_PATH) . (":") . ($HOMEDIR). ("/lib");
    }
    else
    {
	$LD_LIBRARY_PATH = ($HOMEDIR). ("/lib");
    }

    if ($LIBPATH ne "")
    {
	$LIBPATH	= ($LD_LIBRARY_PATH) . (":") . ($LIBPATH);
    }
    else
    {
	$LIBPATH	= $LD_LIBRARY_PATH;
    }

    debug_message ("LD_LIBRARY_PATH: $LD_LIBRARY_PATH\n");
    debug_message ("LIBPATH: $LIBPATH\n");

    debug_message ("Leave sub-function set_librarypath ().\n");
}


#--------------------------------------------------------------
# set_classpath ()
#
#--------------------------------------------------------------
sub set_classpath ()
{
    debug_message ("Enter sub-function set_classpath ()...\n");
    
    local ($CODEBASE) = ($HOMEDIR) . ("/codebase");

    # check for a custom classpath
    # Retrieve the CUSTOM_CLASSPATH environment value
    $CLASSPATH	= $ENV {"CUSTOM_CLASSPATH"};

    if ($CUSTOM_CLASSPATH ne "")
    {
	debug_message ("Using CUSTOM_CLASSPATH\n");
    }
    else
    {
	$CLASSPATH = $ENV {"CLASSPATH"};
	foreach $CODEBASE (@CODEBASES)
	{
	    if ($CLASSPATH ne "")
	    {
		$CLASSPATH = ($CLASSPATH) . (":") . ($CODEBASE);
	    }
	    else
	    {
		$CLASSPATH = ($CODEBASE);
	    }

	    # look for the jars in each of the codebases
	    foreach $JAR (@JARS)
	    {
		$CLASSPATH = ($CLASSPATH).(":") . ($CODEBASE) . ("/").($JAR);
	    }
	}
	$CLASSPATH = ($CLASSPATH).(":") . ($HOMEDIR);
    }

    debug_message ("CLASSPATH = $CLASSPATH\n");
    
    # choose the first codebase as the working directory
    # we expect the first one to be /opt/csm/dcem/codebase/
    $WORKING_DIR = $CODEBASES[0];
    debug_message ("WORKING_DIR = $WORKING_DIR\n");

    debug_message ("Leave sub-function set_classpath ().\n");
}


#--------------------------------------------------------------
# set_path ()
#
#--------------------------------------------------------------
sub set_path()
{
    debug_message ("Enter sub-function set_path ()...\n");
    
    if ($OS eq $LINUX)
    {

	if ( -e "/opt/IBMJava2-131/jre/bin" )
	{
		$JAVA_HOME = "/opt/IBMJava2-131/jre/bin";
	}
	else
	{
		if ( -e "/usr/lib/IBMJava2-1.3.1/jre/bin" )
		{
        		$JAVA_HOME = "/usr/lib/IBMJava2-1.3.1/jre/bin"; 
		}	
		else
		{
			die "ERROR! Java version 1.3.1 is required.\n"
		}
	}
        $PATH = ($JAVA_HOME) . ":" . ($PATH);
    }
    else
    {
	# check if this directory exists
        chdir ("/usr/java131") or die "ERROR! Java version 1.3 or greater is required.\n";

        $PATH = ("/bin:/usr/bin:/usr/sbin:$HOMEDIR/bin:") . ($PATH);
        $PATH = ("/usr/java131/bin:/usr/java131/jre/bin:") . ($PATH);
        $JAVA_HOME = "/usr/java131";
    }

    debug_message ("PATH = $PATH\n");
    debug_message ("JAVA_HOME = $JAVA_HOME\n");
    debug_message ("Leave sub-function set_path ().\n");
}


#--------------------------------------------------------------
# set_display ()
#
#--------------------------------------------------------------
sub set_display()
{
    debug_message ("Enter sub-function set_display ()...\n");

    if ($DISPLAY eq "")
    {
        # if no value is set, use DISPLAY for regular mode, and do
        # not use hostname because its faster
        # set DCEMDISPLAY so remote mode will work

        $DISPLAY = ':0.0';
        # remove the linefeed at the end of the hostname.
        my ($position) = index($HOSTNAME, "\n");
        $DCEMDISPLAY = substr($HOSTNAME, 0, position - 1); 

        $DCEMDISPLAY = $DCEMDISPLAY . ":0.0";
       
        debug_message("DCEMDISPLAY = $DCEMDISPLAY\n");
    }
    else
    {
        # In remote mode, the format ":0.0" will not work, need
        # the hostname because client does not know display value

        my ($position) = index($DCEMDISPLAY, ":");
        if ($position == -1)
        {
             # remove the linefeed at the end of the hostname.
             my ($position) = index($HOSTNAME, "\n");
             $DCEMDISPLAY = substr($HOSTNAME, 0, position - 1); 

             $DCEMDISPLAY = $DCEMDISPLAY . ":0.0";
        }
 
    }

    debug_message ("DISPLAY = $DISPLAY\n");
    debug_message("DCEMDISPLAY = $DCEMDISPLAY\n");
    debug_message ("Leave sub-function set_display ().\n");
}


#--------------------------------------------------------------
# set_hostname ()
#
#--------------------------------------------------------------
sub set_hostname ()
{
    debug_message ("Enter sub-function set_hostname ()...\n");

    chop ($OS);

    debug_message ("OPERATING_SYSTEM = $OS\n");
    
    if ($OS eq $LINUX)
    {
	$HOSTNAME = `/bin/hostname`;
    }
    else
    {
	$HOSTNAME = `/usr/bin/hostname`;
    }
    debug_message ("HOSTNAME = $HOSTNAME\n");
    
    debug_message ("Leave sub-function set_hostname ().\n");
}


#--------------------------------------------------------------
# set_nls_variables()
#
#--------------------------------------------------------------
sub set_nls_variables()
{
    if ($OS eq $AIX)
    {
        SWITCH:
        {
            if ($LANG eq "ja_JP") { $JAVA2D_PURET2KRASTERIZER = 1; last SWITCH;}
            if ($LANG eq "Ja_JP") { $JAVA2D_PURET2KRASTERIZER = 1; last SWITCH;}
            if ($LANG eq "JA_JP") { $JAVA2D_PURET2KRASTERIZER = 1; last SWITCH;}
            if ($LANG eq "Zh_TW") { $JAVA2D_PURET2KRASTERIZER = 1; last SWITCH;}
            if ($LANG eq "ZH_TW") { $JAVA2D_PURET2KRASTERIZER = 1; last SWITCH;}
            if ($LANG eq "Zh_CN") { $JAVA2D_PURET2KRASTERIZER = 1; last SWITCH;}
            if ($LANG eq "ZH_CN") { $JAVA2D_PURET2KRASTERIZER = 1; last SWITCH;}
            if ($LANG eq "ko_KR") { $JAVA2D_PURET2KRASTERIZER = 1; last SWITCH;}
            if ($LANG eq "KO_KR") { $JAVA2D_PURET2KRASTERIZER = 1; last SWITCH;}
        } 
        debug_message ("JAVA2D_PURET2KRASTERIZER = $JAVA2D_PURET2KRASTERIZER \n");
    }
}

#--------------------------------------------------------------
# export_ENV_variables()
#
#--------------------------------------------------------------
sub export_ENV_variables()
{
    debug_message ("Enter sub-function export_ENV_variables ()...\n");

    $ENV {"DISPLAY"}		= $DISPLAY;		#export DISPLAY
    $ENV {"PATH"}		= $PATH;		#export PATH
    $ENV {"CLASSPATH"}		= $CLASSPATH;		#export CLASSPATH
    $ENV {"LD_LIBRARY_PATH"}	= $LD_LIBRARY_PATH;	#export LD_LIBRARY_PATH
    $ENV {"LIBPATH"}		= $LIBPATH;		#export LIBPATH
    $ENV {"JAVA_COMPILER"}	= $JAVA_COMPILER;	#export JAVA_COMPILER
    $ENV {"JAVA_HOME"}	        = $JAVA_HOME;	        #export JAVA_HOME
    if ($OS eq $AIX)
    {
        $ENV {"JAVA2D_PURET2KRASTERIZER"} = $JAVA2D_PURET2KRASTERIZER;	#export JAVA2D_PURET2KRASTERIZER
    }

    debug_message ("Leave sub-function export_ENV_variables().\n");
}


#--------------------------------------------------------------
# main ()
#
#--------------------------------------------------------------
sub main ()
{
    debug_message ("Enter main ()...\n");

    set_hostname ();
    set_display ();
    set_path ();
    set_classpath ();
    set_librarypath ();
    set_java_options ();
#    set_nls_variables();
    export_ENV_variables ();


    chdir ($WORKING_DIR) or die ("ERROR! Cannot change directory to $WORKING_DIR\n$!\n");

    debug_message ("java  $JAVA_OPTS $LOGMANAGER @ARGV\n");
    my ($execution_string) = "java";
    $execution_string = join(" ", $execution_string,  $JAVA_OPTS); 
    $execution_string = join(" ", $execution_string,  $LOGMANAGER); 
    $execution_string = join(" ", ($execution_string,  $arg1)); 
    $execution_string = join(" ", ($execution_string,  '"')); 
    $execution_string = join("", ($execution_string,  $arg2)); 
    $execution_string = join("", ($execution_string,  '"')); 
    `$execution_string`;

    debug_message ("Leave main ().\n");
}

#--------------------------------------------------------------
# START of the perl script.
#
#--------------------------------------------------------------
$|				= 1;

$TRUE				= 1;
$FALSE				= 0;
$DEBUG				= $FALSE;

$AIX				= "AIX";
$LINUX				= "Linux";

$CLASSPATH			= "";
$HOMEDIR			= "/opt/csm/dcem";
$LOGMANAGER                     = "com.ibm.unix.distrib.framework.logging.DMLogManager";
$HOSTNAME			= "";
$DISPLAY			= $ENV {"DISPLAY"};         # get environment value
$DCEMDISPLAY                    = $DISPLAY;
$LD_LIBRARY_PATH		= $ENV {"LD_LIBRARY_PATH"}; # get environment value
$LIBPATH			= $ENV {"LIBPATH"};         # get environment value
$PATH			        = $ENV {"PATH"};            # get environment value
$JAVA_COMPILER			= $ENV {"JAVA_COMPILER"};   # get environment value
$LANG                           = $ENV {"LANG"};            # get environment value
$JAVA_OPTS                      = "";
$WORKING_DIR			= "";
$OS                             = `/bin/uname -s`;
$PLATFORM                       = `/bin/uname -p`;
$POWERPC                        = "powerpc";

# Add more codebases if required....
@CODEBASES			= (($HOMEDIR."/codebase"));
@JARS				= ("dcemfw.jar", "dcemcsm.jar",
				   "dcemimages.jar", "JLog.jar", "jhall.jar", "websm.jar");


#--------------------------------------------------------------
# START of the main function
#
#--------------------------------------------------------------
# Look for the debug option first
if ($ARGV[0] eq "-debug")
{
    $DEBUG = $TRUE;
    debug_message ("Turning DEBUG ON\n");
    shift (@ARGV); # discard the first argument, i.e. -debug
}
$arg1 = shift (@ARGV);
$arg2 = shift (@ARGV);
if ($arg2 != "")
{
    main ();
}
debug_message ("Leave djava ().\n");
