#!/usr/bin/perl

#--------------------------------------------------------------
# display_message ()
#
# This sub-function prints the string passed to it.
# This sub-function should be used to convey information to
# users
#
# @param the messages string to be printed
#
#--------------------------------------------------------------
sub display_message ($)
{
    local ($str) = @_;
    print "$str";
}

#-------------------------------------------------------------
# 
# Check for IBM Self Voicing Kit installed. 
#
#--------------------------------------------------------------
sub check_svk()
{

	if ($OS eq $AIX)
	{
		$svk_install = `lslpp -L | fgrep svk`;
	}
	if ($OS eq $LINUX)
	{
		$svk_install = `rpm -aq | grep svk`;
	}
	chomp($svk_install);
	if ($svk_install eq "" )
	{
		help();
	}
}


#--------------------------------------------------------------
# run_dcem_assistive_technology ()
#
# Executes the "dcemAccess" command with the appropriate arguments
#--------------------------------------------------------------
sub run_dcem ()
{
    check_svk();
    local ($count)	= 0;

    display_message ("Distributed Command Execution Manager\nStarting Assistive Technology ...\n");


    $count = @ARGV;
    
    while ($count > 0 )
    {
	$DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
        $DISTRIBUTED_CMD = join("", ($DISTRIBUTED_CMD, " "));
        shift (@ARGV);
	$count--;
    } 
    local ($CMD) = ($DJAVA);
    $CMD = ($CMD) . (" ") . ($JAVA_ARGS) . (" ") . ($DISTRIBUTED_CMD); 
    select(STDOUT);
    $| = 1;
    select(STDERR);
    $| = 1;
    exec ("$CMD");

}

#-------------------------------------------------------------
#
# help ()
# 
#
#-------------------------------------------------------------
sub help ()
{
	display_message("Need to have IBM Self-Voicing Development Kit installed.\n");
        display_message("The IBM SVDK jars needs to be visible either:\n");
	display_message("1. Export into CUSTOM_CLASSPATH.\n");
	display_message("2. Copy IBM SVDK jars to /opt/csm/dcem/codebase/.\n");
	exit (0);

}

#--------------------------------------------------------------
# main ()
#
#--------------------------------------------------------------
sub main ()
{
    if (($ARGV [$0] eq "--help") || ($ARGV [$0] eq "-h"))
    {
        help();
    }
    run_dcem ();
}


#--------------------------------------------------------------
# START of the perl script.
#
#--------------------------------------------------------------
$|				= 1;
$OS                             = `/bin/uname -s`;
$AIX                            = "AIX";
$LINUX                          = "Linux";

$DJAVA				= "/opt/csm/dcem/bin/dcem";
$JAVA_ARGS			= "-DsvkEnabled=true";
$DISTRIBUTED_CMD		= "";


# Drop the trailing newline character
chop ($OS);

main ();
