# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2000,2003 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# -*- perl -*-
# @(#)54   1.3   src/csm/core/pm/DBIutils.pm.perl, csmcore, csm_rsirl, rsirl0403a 3/11/02 14:39:18

# This perl module provides useful functions to help in accessing the DBI layer.

package DBIutils;
use strict;
require NodeUtils;
require DBI;

#$etc_csm_db;     # this is global so we don't need to declare it
my $dbiConnStr;
my $msgs;

BEGIN
  {
	#todo: use CSM_ROOT instead
	$NodeUtils::etc_csm_db = '/etc/opt/csm/csm_db';
    my $test_etc_csm_db = $ENV{'ETC_CSM_DB'};      # only for development
	if ($test_etc_csm_db) { $NodeUtils::etc_csm_db = $test_etc_csm_db; }
	#print "etc_csm_db=$NodeUtils::etc_csm_db\n";

	$msgs = {
			 CANT_CONNECT => 'Can not connect to database:  %s.  Error message from the DBD layer:  %s',
			 TABLE_EXISTS => 'Skipping creation of table %s - it already exists.',
			 PREPARE_FAILED => 'Can not prepare SQL statement:  %s.  Error message from the DBD layer:  %s',
			 EXECUTE_FAILED => 'Can not execute SQL statement:  %s.  Error message from the DBD layer:  %s',
			 ANY_MSG => '%s'
			};
  }

sub getDbiConnStr
  {
	my ($class, $dbd) = @_;
	if (!$dbd)          # return the default in the csm_db file
	  {
		if (!$dbiConnStr)
		  {
			#print "getting dbi conn string...\n";
			#todo:  replace this with perl i/o
			chop($dbiConnStr=`cat $NodeUtils::etc_csm_db`);
		  }
		#print "dbiConnStr=$dbiConnStr\n";
		return $dbiConnStr;
	  }
	elsif ($dbd eq 'rmc') { return 'DBI:RMC:dbn=IBM'; }
	elsif ($dbd eq 'file') { return "dbi:CSV:f_dir=$NodeUtils::file_db;csv_sep_char=|"; }
	elsif ($dbd eq 'mysql') { return 'dbi:mysql:dbname=csm'; }
	elsif ($dbd eq 'postgresql') { return 'dbi:Pg:dbname=csm'; }
	else { return undef; }
  }

sub isRMC
  {
	my ($class, $dbd) = @_;
	if (NodeUtils->getDbiConnStr($dbd) =~ /^dbi:rmc/i) { return 1; } else { return 0; }
  }
sub isMySQL
  {
	my ($class, $dbd) = @_;
	if (NodeUtils->getDbiConnStr($dbd) =~ /^dbi:mysql/i) { return 1; } else { return 0; }
  }
sub isPostgreSQL
  {
	my ($class, $dbd) = @_;
	if (NodeUtils->getDbiConnStr($dbd) =~ /^dbi:pg/i) { return 1; } else { return 0; }
  }
sub isFile
  {
	my ($class, $dbd) = @_;
	if (NodeUtils->getDbiConnStr($dbd) =~ /^dbi:csv/i) { return 1; } else { return 0; }
  }

sub dbiConnect
  {
	# dbd is optional.  If it's not passed in, we will use the one in the etc_csm_db file
	my ($class, $dbd, $skip) = @_;
	my $dbn = NodeUtils->getDbiConnStr($dbd);
	#print "dbn=$dbn\n";
	if (!$skip)
	  {
		NodeUtils->checkMgmtSvr();    # set remote rmc env var, if applicable
		DBIutils->initDb($dbd);      # create database, if file
	  }
	#Note: as the 4th arg use { PrintError => 0 } to suppress the DBD from printing
	#      the error msg, then use DBI::err and DBI::errstr to handle errors
	DBI->connect($dbn,"","", {PrintError => ($::VERBOSE?1:0)})
	  or NodeUtils->msg($msgs, "E$DBI::err", 'CANT_CONNECT', $dbn, $DBI::errstr);
  }


# Create the ManagedNode and the NodeGroup tables in the database
sub initDb
  {
	my ($class, $dbd) = @_;

	# This is an optimization because the only dbd this function does anything for
	# currently is file
	if (!(NodeUtils->isFile($dbd))) { return; }

	# If this is the file dbd, make sure the file_db dir is created
	if (NodeUtils->isFile($dbd) && !(-d $NodeUtils::file_db))
	  {
		my $rc = system("mkdir -pm 755 $NodeUtils::file_db") >> 8;
		#todo:  handle failure
	  }
	
	my $dbh = DBIutils->dbiConnect($dbd, 1);
	my $s;

	# Test to see if the ManagedNode table exists.  If not, create it
	if (!DBIutils->isTableThere($dbd, 'ManagedNode'))
	  {
		$s = 'CREATE TABLE ManagedNode (Hostname VARCHAR(100), UniversalId VARCHAR(160))';
		$dbh->do($s)  or NodeUtils->msg($msgs,"E$DBI::err",'EXECUTE_FAILED',$s,$DBI::errstr);
	  }
	else { NodeUtils->msg($msgs,'V','TABLE_EXISTS','ManagedNode'); }

	# Test to see if the PreManagedNode table exists.  If not, create it
	if (!DBIutils->isTableThere($dbd, 'PreManagedNode'))
	  {
		$s = 'CREATE TABLE PreManagedNode (Hostname VARCHAR(100))';
		$dbh->do($s)  or NodeUtils->msg($msgs,"E$DBI::err",'EXECUTE_FAILED',$s,$DBI::errstr);
	  }
	else { NodeUtils->msg($msgs,'V','TABLE_EXISTS','PreManagedNode'); }

	# Test to see if the NodeGroup table exists.  If not, create it
	if (!DBIutils->isTableThere($dbd, 'NodeGroup'))
	  {
		$s = 'CREATE TABLE NodeGroup (Name VARCHAR(100), MemberList VARCHAR(1000), SelectStr VARCHAR(200))';
		$dbh->do($s)  or NodeUtils->msg($msgs,"E$DBI::err",'EXECUTE_FAILED',$s,$DBI::errstr);
	  }
	else { NodeUtils->msg($msgs,'V','TABLE_EXISTS','NodeGroup'); }
	
	$dbh->disconnect();
  }


# Test to see if a table exists via SQL.
sub isTableThere
  {
	my ($class, $dbd, $table) = @_;

	#todo: I should do it using SQL so that is works for any DBD, now that I know
	#      how to suppress the error msg when the table doesnt exist (PrintError=0
	#      on the connect, which is done unless the -v option is used).
	#my $ary_ref = $dbh->selectcol_arrayref("SELECT $attr FROM $table");
	#print "err=$DBI::err, errstr=$DBI::errstr\n";
	#if (defined($ary_ref)) { return 1; }
	#else { return 0; }

	if (NodeUtils->isRMC($dbd)) { return 1; }
	elsif (NodeUtils->isFile($dbd)) { return -f "$NodeUtils::file_db/$table"; }
	else { return 1; }
  }


# Copy the contents of 1 db to another.  We assume the other one has already been
# initialized via initDb()
sub copyDb
  {
	my ($class, $dbdfrom, $dbdto) = @_;
	my $dbhfrom = DBIutils->dbiConnect($dbdfrom);
	my $dbhto = DBIutils->dbiConnect($dbdto);
	DBIutils->copyTable($dbhfrom, $dbhto, 'ManagedNode');
	DBIutils->copyTable($dbhfrom, $dbhto, 'PreManagedNode');
	DBIutils->copyTable($dbhfrom, $dbhto, 'NodeGroup');
	$dbhfrom->disconnect();
	$dbhto->disconnect();
  }

sub copyTable
  {
	my ($class, $dbhfrom, $dbhto, $table) = @_;
	my $s = "SELECT * FROM $table";
	my $sth = $dbhfrom->prepare($s)  or NodeUtils->msg($msgs,"E$DBI::err",'PREPARE_FAILED',$s,$DBI::errstr);
	$sth->execute  or NodeUtils->msg($msgs,"E$DBI::err",'EXECUTE_FAILED',$s,$DBI::errstr);
	my $attrnames = '';
	while ( my $hashref = $sth->fetchrow_hashref )
	  {
		if (!$attrnames) { $attrnames = join(', ', keys(%$hashref)); }

		# Get the values.  For the strings, we have to put '' back around them.
		my @vals = values(%$hashref);
		foreach my $v (@vals)
		  {
			#todo: for now assume all the attributes are strings
			#if (!($v =~ /^-?[0-9]+$/))
			  { $v = $dbhto->quote($v); }
		  }
		my $vals = join(', ', @vals);

		# Now add this row to the other table
		my $s = "INSERT INTO $table ($attrnames) VALUES ($vals)";
		#if ($::VERBOSE) { print "$s\n"; }
		NodeUtils->msg($msgs,'V','ANY_MSG',$s);
		$dbhto->do($s)   or NodeUtils->msg($msgs,"E$DBI::err",'EXECUTE_FAILED',$s,$DBI::errstr);
	  }
	$sth->finish;
  }


1;
