#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2003,2004 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

#---------------------------------------------------------------------------
# Module:SetupBootUtils
#
# Description:
#   This module consists of subroutines that would be helpful in setting
# up net boots for various purposes like gettting mac address or for kickstar
# installation of nodes etc. Most of the net boots involve setting up similar
# steps like creating dhcp, setting up pxe files, creating ram disk etc.By
# consolidating the subroutines here, future additions can make use of already 
# existing infrastructure.
# 
#----------------------------------------------------------------------------
# @(#)06   1.50.1.6   src/csm/install/SetupBootUtils.pm.perl, setup, csm_rsirl, rsirl0403a 12/15/03 11:19:57

package SetupBootUtils;

use strict;
BEGIN { $::csmpm = $ENV{'CSM_PM'} ? $ENV{'CSM_PM'} : '/opt/csm/pm'; }
use lib $::csmpm;
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Std;
use File::Path;	# Provides mkpath() and rmtree();
use File::Copy; # Provides copy();
use NodeUtils;
use CSMDefs;
use InstallUtils;

#--Begin add for autoyast
use XML::DOM;
#--End

# Script to set up Kickstart, dhcp, pxe

#---------------------
# Global Variables
#---------------------
%::BOOT_ATTRIBUTES;
$::BOOT_EXIT=0;
# get the definitions we need for this command
#    in this case get_OSDefs uses the attrs of the management
#    server to identify the correct definitions
%::msOSDefs= InstallUtils->get_OSDefs();


# Directories
$::INSTALLDIR		= $::CSMINSTDIR;
$::TFTPBOOT             ="/tftpboot";
$::PXELINUXCFG          =$::TFTPBOOT   .  "/pxelinux.cfg";
$::OLDGETMACSPXE        =$::PXELINUXCFG  . "/getmacs.pxe";
$::GETMACZ              =$::TFTPBOOT   .  "/csm/getmac.gz";
# Files
$::PXE_INSTALL_TMPL	= "/opt/csm/install/install.pxe";
$::PXE_YASTINSTALL_TMPL	= "/opt/csm/install/yastinstall.pxe";
$::PXE_GETMACS_TMPL	= "/opt/csm/install/getmacs.pxe";
$::PXE_DEFAULT_TMPL	= "/opt/csm/install/default.pxe";
$::FIRSTBOOT_TMPL	= "/opt/csm/install/firstboot.tmpl";

# Defaults
$::DEFAULT_PKGPATH	= "/mnt/cdrom";
$::DEFAULT_KSCFG_TMPL	= "/opt/csm/install/kscfg.tmpl";
my @VALID_ATTRIBUTES	= ( "Netmask", 
			    "Gateway", 
			    "Nameserver", 
			    "Nameservers"); 

#SSS
my @VALID_BOOT_METHODS	= ( "getmacs", 
			    "kickstart", 
			    "autoyast", 
		            "sis",
			    "disk" );
my @VALID_HEX_FILE_OPTIONS = ( "yes", 
			      "no", 
		              "onlyhex" );


#Local Variables
my $bootMethod;
my $hexFileOption="yes";
my @inputNodes;
my @targetNodes;
my @okNodes;
#my %attributes;
my @targetNodes;
my @nodeInfo;
my %nodeHash;
# Subroutines
#---------------------


sub commonNetBootSetup
{
  createBootDirectory();
  createDefaultPxe();
}


sub doFullSetup()
{
  &commonNetBootSetup();
  &setupDHCP();
  &setupPxe();
  &copyKernel();
  &makeRAMDisk();
  &setupPxeHex();
}

sub doFullSetupGenerateNoHex
{
 &commonNetBootSetup();
 &setupDHCP();
 &setupPxe();
 &copyKernel();
 &makeRAMDisk();
  
}
#--Begin add for ppc64
sub doPPC64Setup
{
 &setupDHCP();
 &copyPPC64Kernel();
 &createAutoyastFiles();
}
#--End
sub doSetupGenerateOnlyHex()
{
  &verifyDHCP();
  &verifyPxe();
  &verifyKernel();
  &verifyRAMDisk();
  &verifyTFTP();
  &verifyOthers();
  &setupPxeHex();
}

sub setupDiskBoot
{
  &setupPxe();
  &setupDHCP();
  &setBootLocation();
}

sub createBootDirectory
{
    my ($rc) = 0;
    # Create the /tftpboot/pxelinux.cfg directory.
    mkpath ("$::PXELINUX_CFG_DIR", $::VERBOSE, 0110);
    InstallUtils->secureFilePermissions($::PXELINUX_CFG_DIR, '0110','0440');
    return $rc;
}

sub createDefaultPxe
{
  my $pxetemplate = $::PXE_DEFAULT_TMPL;
  my $pxefile = "$::PXELINUX_CFG_DIR/default";
  &copyfile($pxetemplate, $pxefile);
}

# Run the createdhcp tool to generate the /etc/dhcpd.conf file for use
# during MAC address collection.
sub setupDHCP
{
 
  if($bootMethod eq 'getmacs' )
  {
     my ($cmd, $output);
     NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgGENERATING_DHCP' ); 
     my @nodelist = @targetNodes;
     my $nodelist = join(',', @nodelist);
     my $verbose;
     $verbose = ($::VERBOSE ? "-v" : "");
     if (!($verbose)) { $verbose = ($::VERBOSE ? "-V" : ""); }

     my $attrs = "";
     foreach my $attr (keys %::BOOT_ATTRIBUTES)
     {
	$attrs .=  "$attr=" . $::BOOT_ATTRIBUTES{$attr} . " ";
        #print ("$attr\n");
        #exit;
     }
     my $nodelist_file = InstallUtils->make_node_list_file(\@nodelist);

     if ($nodelist) { $cmd = "$::CREATEDHCP $verbose  -f $nodelist_file $attrs";}
     else           { $cmd = "$::CREATEDHCP $verbose $attrs";}
     $output = NodeUtils->runcmd($cmd,0);
     my $fstatus = $::RUNCMD_RC;
     NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgShow_Output', $output) if($fstatus); 

     InstallUtils->close_delete_file($::NODE_LIST_FILE, $nodelist_file);

     return $fstatus;
 }elsif($bootMethod eq 'kickstart' || $bootMethod eq 'autoyast' ||$bootMethod eq 'sis' )
{
   
  #unless(-e "/etc/dhcpd.conf")
  #{
     my ($cmd, $output);
     #NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
     #          'csminstall','I', 'IMsgGENERATING_DHCP' ); 
     my $verbose;
     $verbose = ($::VERBOSE ? "-v" : "");
     if (!($verbose)) { $verbose = ($::VERBOSE ? "-V" : ""); }

     my $attrs = "";
     foreach my $attr (keys %::BOOT_ATTRIBUTES)
     {
	$attrs .=  "$attr=" . $::BOOT_ATTRIBUTES{$attr} . " ";
        #print ("$attr\n");
        #exit;
     }

     $cmd = "$::CREATEDHCP $verbose $attrs";
     $output = NodeUtils->runcmd($cmd,0);
     my $fstatus = $::RUNCMD_RC;
     NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgShow_Output', $output) if($fstatus); 

     #return $fstatus;
  #}
  
  
      # Run updatedhcp to add nodes to the /etc/dhcpd.conf file for use during
      # Kickstart install
      my $nodelist = join(',', @targetNodes);
      my $verbose;
      $verbose = ($::VERBOSE ? "-v" : "");
      if (!($verbose)) { $verbose = ($::VERBOSE ? "-V" : ""); }
      if($bootMethod eq 'kickstart') {
          NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgUPDATING_DHCP', $nodelist); 
      }
      elsif ($bootMethod eq 'autoyast') {
      	  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgUPDATING_DHCP_AUTOYAST', $nodelist); 
      }
      else {  #$bootMethod eq 'sis' 
          NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgUPDATING_DHCP_SIS', $nodelist); 
      }
      my $nodelist_file = InstallUtils->make_node_list_file(\@targetNodes);
      
      my $cmd = "$::UPDATEDHCP $verbose -n $nodelist";
      #my $cmd = "/bin/echo $nodelist | $::UPDATEDHCP $verbose -f -";
      my $output = NodeUtils->runcmd($cmd,0);
      my $fstatus = $::RUNCMD_RC;
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgShow_Output', $output) if($fstatus); 
      #remove the nodes for which the update did not work 
      InstallUtils->close_delete_file($::NODE_LIST_FILE, $nodelist_file);
      return $fstatus; # 0 is good  

}
 elsif($bootMethod eq "disk" )
    {
      my $nodelist = join(',', @targetNodes);
      my $verbose;
      $verbose = ($::VERBOSE ? "-v" : "");
      if (!($verbose)) { $verbose = ($::VERBOSE ? "-V" : ""); }
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgUPDATING_DHCP', $nodelist); 
      my $cmd = "$::UPDATEDHCP $verbose -n $nodelist";
      #for disk
      #If a MAC address exists in the managed node class, create a static DHCP
      # entry . If no MAC address , remove the DHCP range entry for the node.
    }
}

sub setupGetmacsPxeWithoutConsoleSerialDevice
{
  my $pxetemplate = $::PXE_GETMACS_TMPL;
  my ($pxefile) = @_;
  
  open (PXETEMPLATE, "<$pxetemplate")
             || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH,'csminstall','E2', 'EMsgCANT_READ_FILE', $pxetemplate);

  open (PXEFILE, ">$pxefile")
            || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_WRITE_FILE', $pxefile);

  while (<PXETEMPLATE>)
  {
    s/console=#CONSOLE_DEVICE#,9600//g; # Console Device e.g. ttyS0
    s/SERIAL #CONSOLE_DEVICE_NUMBER# 9600//g; # and number
    print PXEFILE $_;
  }
  close PXETEMPLATE;
  close PXEFILE;
}

sub setupGetmacsPxeWithConsoleSerialDevice
{
  my $pxetemplate = $::PXE_GETMACS_TMPL;
  my ($pxefile, $console_device) = ($_[0], $_[1]);

  $::CONSOLE_DEVICES{$console_device} = 1;# save list of unique devices
  my $console_device_number = $console_device;
  $console_device_number =~ s/\D//g;      # remove all non-digits
  open (PXETEMPLATE, "<$pxetemplate")
            || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_READ_FILE', $pxetemplate);

  open (PXEFILE, ">$pxefile")
            || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_WRITE_FILE', $pxefile);

  while (<PXETEMPLATE>)
  {    
    # take out the following two entries related to, console serial device
    s/#CONSOLE_DEVICE#/$console_device/g; # Console Device e.g. ttyS0
    s/#CONSOLE_DEVICE_NUMBER#/$console_device_number/g; # and number

    print PXEFILE $_;
  }

  close PXETEMPLATE;
  close PXEFILE;
}

sub isVersion73orAbove
{
  # If the management server is rh7.2 or below, then we need
  # to take the rw option out of the pxegetmacs hex file.

  my ($version73orAbove,$distro,$version,$mod);
  # by default assume it is rh 7.3 or above.
  $version73orAbove = "true";
  # get the distro (i.e.: 7.2, 7.3, 8.0, etc.)
  $distro = InstallUtils->get_DistributionVersion();
  ($version, $mod) = split /\./ , $distro;
  # check if it's less than 7.3:
  if($version <= 7 && $mod <= 2){
    #print "not 7.3 or above...\n";
    $version73orAbove = 0;
  }
  return $version73orAbove;

}


sub setupKickstartPxe
{
    my ($pxefile, $ms_ipaddr, $ksfile, $ksfile_dir, $console_device) = @_;
    my $pxetemplate = $::PXE_INSTALL_TMPL;
    my $ks_level = "ks" . $::DISTRO_VERSION;    # e.g. ks7.3
    open (PXETEMPLATE, "<$pxetemplate")
	|| NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $pxetemplate);
    open (PXEFILE, ">$pxefile")
	|| NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_WRITE_FILE', $pxefile);
    if($console_device){
	my $console_device_number = $console_device;
	$console_device_number =~ s/\D//g;      # remove all non-digits
	while (<PXETEMPLATE>)
	{
	    s/#NFS_IP#/$ms_ipaddr/g;         # Management server (NFS server)
	    s/#KS_LEVEL#/$ks_level/g;        # Kickstart Level e.g. ks71
	    s/#KS_DIR#/$ksfile_dir/g;        # Kickstart File Directory
	    s/#KS_FILE#/$ksfile/g;           # Kickstart File <nodeip>-kickstart
	    s/#CONSOLE_DEVICE#/$console_device/g; # Console Device e.g. ttyS0
	    s/#CONSOLE_DEVICE_NUMBER#/$console_device_number/g; # and number
	    print PXEFILE $_;
	}
    }
    else{
	while (<PXETEMPLATE>)
	{
	    s/#NFS_IP#/$ms_ipaddr/g;         # Management server (NFS server)
	    s/#KS_LEVEL#/$ks_level/g;        # Kickstart Level e.g. ks71
	    s/#KS_DIR#/$ksfile_dir/g;        # Kickstart File Directory
	    s/#KS_FILE#/$ksfile/g;           # Kickstart File <nodeip>-kickstart
	    # take out the following two entries related to, 
	    # console serial device
	    s/console=#CONSOLE_DEVICE#,9600//g; # Console Device e.g. ttyS0
	    s/SERIAL #CONSOLE_DEVICE_NUMBER# 9600//g; # and number
	    print PXEFILE $_;
	}
    }
    close PXETEMPLATE;
    close PXEFILE;
}

#SSSBEGIN
sub setupAutoyastPxe
{
    my ($pxefile, $ms_ipaddr, $yastcfgfile, $yastcfgfile_dir, $nfs_dir, $console_device) = @_;
    my $pxetemplate = $::PXE_YASTINSTALL_TMPL;
    my ($yast_level) = "yast-" . $::DISTRO_NAME . $::DISTRO_VERSION . "-" . $::ARCH;	# e.g. yast-SLES8.1-i386

    open (PXETEMPLATE, "<$pxetemplate")
	|| NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $pxetemplate);
    open (PXEFILE, ">$pxefile")
	|| NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_WRITE_FILE', $pxefile);
    if($console_device){
	my $console_device_number = $console_device;
	$console_device_number =~ s/\D//g;      # remove all non-digits
	while (<PXETEMPLATE>)
	{
	    s/#NFS_IP#/$ms_ipaddr/g;            # Management server (NFS server)
	    s/#YAST_LEVEL#/$yast_level/g;       # YaST Level (yast-SLES8.1-i386)
	    s/#YASTCFG_DIR#/$yastcfgfile_dir/g; # YaST Config File Directory
	    s/#YASTCFG_FILE#/$yastcfgfile/g;    # YaST Config File <nodeip>-autoyast.xml
	    s/#NFS_DIR#/$nfs_dir/g;             # NFS directory (SUSE_TOP)
	    s/#CONSOLE_DEVICE#/$console_device/g; # Console Device e.g. ttyS0
	    s/#CONSOLE_DEVICE_NUMBER#/$console_device_number/g; # and number
	    print PXEFILE $_;
	}
    }
    else{
	while (<PXETEMPLATE>)
	{
	    s/#NFS_IP#/$ms_ipaddr/g;            # Management server (NFS server)
	    s/#YAST_LEVEL#/$yast_level/g;       # YaST Level (yast-SLES8.1-i386)
	    s/#YASTCFG_DIR#/$yastcfgfile_dir/g; # YaST Config File Directory
	    s/#YASTCFG_FILE#/$yastcfgfile/g;    # YaST Config File <nodeip>-autoyast.xml
	    s/#NFS_DIR#/$nfs_dir/g;             # NFS directory (SUSE_TOP)

	    # take out the following two entries related to, 
	    # console serial device
	    s/console=#CONSOLE_DEVICE#,9600//g; # Console Device e.g. ttyS0
	    s/SERIAL #CONSOLE_DEVICE_NUMBER# 9600//g; # and number
	    print PXEFILE $_;
	}
    }
    close PXETEMPLATE;
    close PXEFILE;
}
#SSSEND


sub setupSisInstallPxeWithoutConsoleSerialDevice
{
  my ($pxefile) = @_;
  my $pxetemplate = $::PXE_INSTALL_TMPL;
  my $sis_level = "sis" . $::DISTRO_VERSION;	# e.g. sis7.2

  open (PXETEMPLATE, "<$pxetemplate")
   || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $pxetemplate);
 
  open (PXEFILE, ">$pxefile")
   || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_WRITE_FILE', $pxefile);

  while (<PXETEMPLATE>)
  {
    s/#SIS_LEVEL#/$sis_level/g;	     # SIS Level
    # take out the following two entries related to, 
    # console serial device
    s/console=#CONSOLE_DEVICE#,9600//g; # Console Device e.g. ttyS0
    s/SERIAL #CONSOLE_DEVICE_NUMBER# 9600//g; # and number

    print PXEFILE $_;
  }

}

sub setupSisInstallPxeWithConsoleSerialDevice
{

  my ($pxefile, $console_device) = @_;
  my $pxetemplate = $::PXE_INSTALL_TMPL;
  my $sis_level = "sis" . $::DISTRO_VERSION;	# e.g. sis7.2
  my $console_device_number = $console_device;
  $console_device_number =~ s/\D//g;      # remove all non-digits

  open (PXETEMPLATE, "<$pxetemplate")
    || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $pxetemplate);
  
  open (PXEFILE, ">$pxefile")
    || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_WRITE_FILE', $pxefile);
  
  while (<PXETEMPLATE>)
  {
    s/#SIS_LEVEL#/$sis_level/g;	     # SIS Level
    s/#CONSOLE_DEVICE#/$console_device/g; # Console Device e.g. ttyS0
    s/#CONSOLE_DEVICE_NUMBER#/$console_device_number/g; # and number

      print PXEFILE $_;
  }

  close PXETEMPLATE;
  close PXEFILE;

}

sub setupPxe
{
  if($bootMethod eq 'getmacs')
  {
    my $pxetemplate = $::PXE_GETMACS_TMPL;
    my $version73orAbove = &isVersion73orAbove();

    # This must be expanded to process the REAL list of nodes
    foreach my $node (@targetNodes)
    {
       my ($nodename, $nodeip) = NodeUtils->getHost($node);
       my $pxefile = "$::PXELINUX_CFG_DIR/$nodename.getmacs";
       my $console_device = $nodeHash{$nodename}{"ConsoleSerialDevice"};

       # If the ConsoleSerialDevice attribute is set to 'NONE', this is
       # A System that has no console serial device.
       if ($console_device eq 'NONE') 
       {
         &setupGetmacsPxeWithoutConsoleSerialDevice($pxefile);                            
       }
       else
       {
         # If the ConsoleSerialDevice attribute is unset, assume this is
         # a system that was migrated from early CSM 1.2 which hardcoded ttyS1
         if ($console_device eq '') 
         {
           $console_device = "ttyS1";
         }
         &setupGetmacsPxeWithConsoleSerialDevice($pxefile,$console_device);             
       }
	 
       # one last edit, get rid of the read/write option in the kernel
       if (!($version73orAbove || $::DISTRO_NAME =~ /SLES/ || (($::DISTRO_NAME =~ /RedHatEL-AS/)&&($::DISTRO_VERSION =~ /3/)) || (($::DISTRO_NAME =~ /RedHatEL-ES/)&&($::DISTRO_VERSION =~ /3/)) || (($::DISTRO_NAME =~ /RedHatEL-WS/)&&($::DISTRO_VERSION =~ /3/))))
       {
         NodeUtils->runcmd("perl -pi -e \"s/APPEND rw/APPEND/g\" $pxefile");
       }

     } # end of for loop    
  } # end of if
  elsif($bootMethod eq  'kickstart')
  {
      
      # for kickstart
      # Create an install pxe configuration file for each node.
      # The name of the file is /tftpboot/pxelinux.cfg/<nodename>.install.
      
      my $pxetemplate = $::PXE_INSTALL_TMPL;
      my $ks_level = "ks" . $::DISTRO_VERSION;    # e.g. ks7.3
      
      # This must be expanded to process the REAL list of nodes
      foreach my $node (@targetNodes)
      {
        my ($nodename, $nodeip) = NodeUtils->getHost($node);
        my $ksfile_dir;

        # Get the hostname of the management server as known by this node
        #my ($ms_hostname, $ms_ipaddr) = &get_node_management_server($nodename);
        my ($isvr_hostname, $isvr_ip, $dir) = &get_node_isvr($nodename);
	# Kickstart config file directory.
	# Example:  /csminstall/csm/1.2.0/kickstart.RedHat7.3
	if($dir){ #install server is used
	    $ksfile_dir = "$dir/csm/" . $::CSM_VERSION . "/kickstart." . $::DISTRO_NAME . $::DISTRO_VERSION;
	}
	else{
	    $ksfile_dir = "/csminstall/csm/" . $::CSM_VERSION . "/kickstart." . $::DISTRO_NAME . $::DISTRO_VERSION;
	}       
        my $pxefile = "$::PXELINUX_CFG_DIR/$nodename.install";
        my $ksfile = "${nodeip}-kickstart";

        my $console_device = $nodeHash{$nodename}{"ConsoleSerialDevice"};
        # If the ConsoleSerialDevice attribute is set to 'NONE', this is
        # a system that has no console serial device.
        if ($console_device eq 'NONE') 
        { 
	    &setupKickstartPxe($pxefile, $isvr_ip, $ksfile, $ksfile_dir);
	}
        else
	{
          # If the ConsoleSerialDevice attribute is unset, assume this is
	  # a system that was migrated from early CSM 1.2 which hardcoded ttyS1
	  if ($console_device eq '') 
	  {
            $console_device = "ttyS1";
	  }
	  my $console_device_number = $console_device;
	  $console_device_number =~ s/\D//g;      # remove all non-digits
	  &setupKickstartPxe($pxefile, $isvr_ip, $ksfile, $ksfile_dir, $console_device);
	}
        
      }#end of for loop
      &createKickstartFiles();
    }
#SSSBEGIN
  elsif($bootMethod eq  'autoyast')
  {
      
      # for autoyast
      # Create an install pxe configuration file for each node.
      # The name of the file is /tftpboot/pxelinux.cfg/<nodename>.install.
      
      my $pxetemplate = $::PXE_YASTINSTALL_TMPL;
      #my $yast_level = "yast" . $::DISTRO_VERSION;    # e.g. yast8.1
      
      # This must be expanded to process the REAL list of nodes
      foreach my $node (@targetNodes)
      {
        my ($nodename, $nodeip) = NodeUtils->getHost($node);
        my $ksfile_dir;

        # Get the hostname of the management server as known by this node
        #my ($ms_hostname, $ms_ipaddr) = &get_node_management_server($nodename);
        my ($isvr_hostname, $isvr_ip, $dir) = &get_node_isvr($nodename);
        my $yastcfgfile_dir;
        my $nfs_dir;
	# AutoYaST config file directory.
	# Example:  /csminstall/csm/1.3.2/autoyast.SLES8.1
	if($dir){ #install server is used
	    $yastcfgfile_dir = "$dir/csm/" . $::CSM_VERSION . "/autoyast." . $::DISTRO_NAME . $::DISTRO_VERSION;
	    my $temp = $::SUSE_TOP;
	    $temp =~ s/^\/csminstall/$dir/;
	    $nfs_dir = $temp;
	}
	else{
	    $yastcfgfile_dir = "/csminstall/csm/" . $::CSM_VERSION . "/autoyast." . $::DISTRO_NAME . $::DISTRO_VERSION;
	    $nfs_dir = $::SUSE_TOP;
	}       
        my $pxefile = "$::PXELINUX_CFG_DIR/$nodename.install";
        my $yastcfgfile = "${nodeip}-autoyast.xml";


        my $console_device = $nodeHash{$nodename}{"ConsoleSerialDevice"};
        # If the ConsoleSerialDevice attribute is set to 'NONE', this is
        # a system that has no console serial device.
        if ($console_device eq 'NONE') 
        { 
	    &setupAutoyastPxe($pxefile, $isvr_ip, $yastcfgfile, $yastcfgfile_dir, $nfs_dir);
	}
        else
	{
          # If the ConsoleSerialDevice attribute is unset, assume this is
	  # a system that was migrated from early CSM 1.2 which hardcoded ttyS1
	  if ($console_device eq '') 
	  {
            $console_device = "ttyS1";
	  }
	  my $console_device_number = $console_device;
	  $console_device_number =~ s/\D//g;      # remove all non-digits
	  &setupAutoyastPxe($pxefile, $isvr_ip, $yastcfgfile, $yastcfgfile_dir, $nfs_dir, $console_device);
	}
        
      }#end of for loop
      &createAutoyastFiles();
    }
#SSSEND
    elsif($bootMethod eq 'sis')
      {
	# for SIS
	# Create an install pxe configuration file for each node.
        # The name of the file is /tftpboot/pxelinux.cfg/<nodename>.sis.
	my $pxetemplate = $::PXE_INSTALL_TMPL;
	my $sis_level = "sis" . $::DISTRO_VERSION;	# e.g. sis7.2

	# SIS config file directory.  
	# Example:  /csminstall/csm/1.2.0/sis.SLES7.2
	my $sisfile_dir = 
	"/csminstall/csm/" . $::CSM_VERSION . "/sis." . 
	$::DISTRO_NAME . $::DISTRO_VERSION;

	# This must be expanded to process the REAL list of nodes
	foreach my $node (@targetNodes)
	{
	  my ($nodename, $nodeip) = NodeUtils->getHost($node);
	  my $pxefile = "$::PXELINUX_CFG_DIR/$nodename.sis";

	  # Change for console number support
	  my $console_device = $nodeHash{$nodename}{"ConsoleSerialDevice"};
	  # If the ConsoleSerialDevice attribute is set to 'NONE', this is
	  # a system that has no console serial device.
	  if ($console_device eq 'NONE') 
	  {  
            &setupSisInstallPxeWithoutConsoleSerialDevice($pxefile);          	   
	  }
          else
	  {
	    # If the ConsoleSerialDevice attribute is unset, assume this is
	    # a system that was migrated from early CSM 1.2 which hardcoded ttyS1
	    if ($console_device eq '') 
            {
	      $console_device = "ttyS1";
	    }
	    my $console_device_number = $console_device;
	    $console_device_number =~ s/\D//g;      # remove all non-digits
            &setupSisInstallPxeWithConsoleSerialDevice($pxefile,$console_device);   	
	  }
	}#end of for
      } # end of if (sis)
  if($bootMethod eq 'disk')
  {
     my $InDir = "/tftpboot/pxelinux.cfg";
     my @nodeList = @targetNodes;
     my @ips;
     foreach my $node (@nodeList)
     {
       my $ip = NodeUtils->getHost($node);
       push(@ips, $ip);
     }
     my %hexNames = InstallUtils->genHex(@ips);
     foreach(keys %hexNames)
     {
       my $HEX = $hexNames{$_};
       if( -f "$::PXELINUXCFG\/$HEX")
       {
         NodeUtils->runcmd("$::RM -rf $::PXELINUXCFG\/$HEX", 1);
       }
     }
   }# end of if($bootMEthod='disk') 
}

# Create a kickstart configuration file for each node.
sub createKickstartFiles
{
    my $kscfg_tmpl="";
    my @nodeList = @targetNodes;
   
    # Set some variables that will be substituted into the kickstart
    # configuration template file in place of #VARIABLE#.
    my $ks_level = "ks" . $::DISTRO_VERSION;		# e.g. ks7.3

    # Kickstart config file directory.  
    # Example:  /csminstall/csm/1.2.0/kickstart.RedHat7.3
       my $ksfile_dir = 
	"/csminstall/csm/" . $::CSM_VERSION . "/kickstart." . 
	$::DISTRO_NAME . $::DISTRO_VERSION;

    my $nfs_dir = $::REDHAT_TOP;

    # Get the management server's timezone information
    my $clockcfg = "/etc/sysconfig/clock";
    my $zone;
    my $ms_distro = InstallUtils->get_DistributionName(); #need the OS of MS to determine format of clock file.
    if (( $ms_distro =~ /SLES/ ) || ($ms_distro =~ /SuSE/)){
	$zone= `$::GREP "^TIMEZONE" $clockcfg`;
    }
    else{ #RH machine
	$zone = `$::GREP "ZONE" $clockcfg`;
    }
    my ($junk, $timezone) = split('=', $zone);
    $timezone =~ s/"//g;
    $timezone =~ s/'//g;

    # Get the management server's timezone --utc flag.
    my $utc = `$::GREP "UTC" $clockcfg`;
    my ($junk, $timezone_utc) = split('=', $utc);
    $timezone_utc =~ s/"//g;
    $timezone_utc =~ s/'//g;
    if (($timezone_utc =~ /yes/i) || ($timezone_utc =~ /true/i))
    {
	$timezone_utc = "--utc";
    }
    else { $timezone_utc = ""; }


    my $nameservers = $::ATTRS{"Nameservers"};

    # Generate a hex representation of each node's ip address.
    my %hex = InstallUtils->genHex(@nodeList);

    # Create a file for each node
    foreach my $node (@nodeList)
    {
         # This is the template that will be filled in.
         # Example:  /opt/csm/install/kscfg.tmpl.RedHat7.3 or could be some file name
         #that was set by csmsetupks using -k flag
        $kscfg_tmpl = $nodeHash{$node}{'InstallTemplate'} ;
	my ($node_hostname, $node_ipaddr) = NodeUtils->getHost($node);
	my $node_hex = $hex{$node_hostname};

	# Get the hostname of the management server as known by this node
	my ($ms_hostname, $ms_ipaddr) = 
		    &get_node_management_server($node_hostname);

	my $ms_shorthost = (split('\.', $ms_hostname))[0];
	(my $dnsdomain = $node) =~ s/^.*?\.//;
	if ($dnsdomain eq $node) 
	{
	    # If the node's hostname has no domain name (short hostname), 
	    # use the domain name of the management server.
	    ($dnsdomain = $ms_hostname) =~ s/^.*?\.//;
	    if ($dnsdomain eq $ms_hostname)
	    {
		$dnsdomain = "";
	    }
	}

	# The NFS Hostname and IP addr are taken from the InstallServer attribute
	# or default to the management server.
	my ($nfs_hostname, $nfs_ipaddr, $dir) = &get_node_isvr($node);
	
	my $csm_mount = "/csminstall/csm";
	# The NFS Directory is taken from the NFSDir attribute (no longer used), or defaults
	# to the location on the management server.
  	my $nfs_dir		= $::REDHAT_TOP;
	if($dir){
	    $nfs_dir =~ s/\/csminstall/$dir/;
	    $csm_mount =~ s/\/csminstall/$dir/;
	}

	my $ksfile = $ksfile_dir . "/" . $node_ipaddr . "-kickstart";

	NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','V', 'IMsgCREATING_KS_FILE', $ksfile);
	mkpath ("$ksfile_dir", $::VERBOSE, 0755);

	open (KSTEMPLATE, "<$kscfg_tmpl")
	    || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_READ_FILE', $kscfg_tmpl);

	open (KSFILE, ">$ksfile")
	    || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_WRITE_FILE', $ksfile);

	# Substitute values in the kickstart configuration template file
	# with variables defined above.
	while (<KSTEMPLATE>)
	{
	    s/#MGMTSVR_HOSTNAME#/$ms_hostname/g; # Management Server hostname
	    s/#MGMTSVR_SHORTHOST#/$ms_shorthost/g; # Management Server shorthost
	    s/#MGMTSVR_IP#/$ms_ipaddr/g;	# Management Server IP Address
	    s/#NODE_HOSTNAME#/$node_hostname/g;	# Node hostname
	    s/#NODE_IP#/$node_ipaddr/g;		# Node ip address 
	    s/#NODE_HEX#/$node_hex/g;		# Node ip address in hex.
	    s/#NFS_HOSTNAME#/$nfs_hostname/g;	# NFS server hostname
	    s/#NFS_IP#/$nfs_ipaddr/g;		# NFS server ip address
	    s/#NFS_DIR#/$nfs_dir/g;		# REDHAT_TOP
	    s/#CSM_MOUNT#/$csm_mount/g;         # CSM mount point
	    s/#TFTP_IP#/$ms_ipaddr/g;		# TFTP Server(Management server)
	    s/#NAMESERVERS#/$nameservers/g;	# Nameservers (comma-separated)
	    s/#DNSDOMAIN#/$dnsdomain/g;		# Node's DNS Domain name
	    s/#CSM_VERSION#/$::CSM_VERSION/g;	# Version of CSM
	    s/#TIMEZONE#/$timezone/g;		# Timezone of MS
	    s/#TIMEZONE_UTC#/$timezone_utc/g;	# Timezone --utc flag of MS

	    print KSFILE $_;
	}
	close(KSTEMPLATE);
	close(KSFILE);
    }
}


#--Begin add for ppc64
sub createAutoyastFiles
{
    my $yastcfg_tmpl="";
    my @nodeList = @targetNodes;
   
    
    # Autoyast config file directory.  
    # Example:  /csminstall/csm/1.3.1/autoyast.SLES8.1/9.114.179.14-autoyast.xml
       my $yastfile_dir = 
	"/csminstall/csm/" . $::CSM_VERSION . "/autoyast." . 
	$::DISTRO_NAME . $::DISTRO_VERSION;
	
	my $yastdir_real = $::CSM_VERSION . "/autoyast." . $::DISTRO_NAME . $::DISTRO_VERSION;	
	my $yastdir_link =  "/csminstall/csm/" . $::DISTRO_NAME . $::DISTRO_VERSION;
	
    my $nfs_dir = $::SUSE_TOP;

    # Get the management server's timezone information
    my $clockcfg = "/etc/sysconfig/clock";
    my $zone = `$::GREP -w "TIMEZONE" $clockcfg`;
    my ($junk, $timezone) = split('=', $zone);
    chomp($timezone);
    $timezone =~ s/"//g;
    $timezone =~ s/'//g;

    # Get the management server's timezone --utc flag.
    my $utc = `$::GREP "UTC" $clockcfg`;
    my ($junk, $timezone_utc) = split('=', $utc);
    $timezone_utc =~ s/"//g;
    $timezone_utc =~ s/'//g;
    if (($timezone_utc =~ /yes/i) || ($timezone_utc =~ /true/i))
    {
	$timezone_utc = "--utc";
    }
    else { $timezone_utc = ""; }


    my $nameservers = $::ATTRS{"Nameservers"};

#SSSBEGIN
    # Generate a hex representation of each node's ip address.
    my %hex = InstallUtils->genHex(@nodeList);
#SSSEND

    # Create a file for each node
    foreach my $node (@nodeList)
    {
        $yastcfg_tmpl = $nodeHash{$node}{'InstallTemplate'} ;
	my ($node_hostname, $node_ipaddr) = NodeUtils->getHost($node);
	my $node_hex = $hex{$node_hostname};	#SSSADD
	# change $bootdisk logical
	my $bootdisk= $nodeHash{$node}{'InstallDisk'} ;
	if (! $bootdisk) {
		$bootdisk = $nodeHash{$node}{'InstallDiskType'};
		if ($bootdisk eq "scsi") {
			$bootdisk = "/dev/sda";
		} elsif ($bootdisk eq "ide") {
			$bootdisk = "/dev/hda";
		} else {
			$bootdisk = "/dev/sda";
		}
	}	
	
	
	# Get the hostname of the management server as known by this node
	my ($ms_hostname, $ms_ipaddr) = 
		    &get_node_management_server($node_hostname);

	my $ms_shorthost = (split('\.', $ms_hostname))[0];
	my $node_shorthost = (split('\.', $node_hostname))[0];
	(my $dnsdomain = $node) =~ s/^.*?\.//;
	if ($dnsdomain eq $node) 
	{
	    # If the node's hostname has no domain name (short hostname), 
	    # use the domain name of the management server.
	    ($dnsdomain = $ms_hostname) =~ s/^.*?\.//;
	    if ($dnsdomain eq $ms_hostname)
	    {
		$dnsdomain = "";
	    }
	}

	# The NFS Hostname and IP addr are taken from the InstallServer attribute
	# or default to the management server.
	my ($nfs_hostname, $nfs_ipaddr, $dir) = &get_node_isvr($node);
	
	my $csm_mount = "/csminstall/csm";
	# The NFS Directory is taken from the NFSDir attribute (no longer used), or defaults
	# to the location on the management server.
  	my $nfs_dir		= $::SUSE_TOP;
	if($dir){
	    $nfs_dir =~ s/\/csminstall/$dir/;
	    $csm_mount =~ s/\/csminstall/$dir/;
	}

	my $yastfile = $yastfile_dir . "/" . $node_ipaddr . "-autoyast.xml";
	my $yastfile_real = $node_ipaddr . "-autoyast.xml";
	my $yastfile_link =  $yastfile_dir . "/" .$node_ipaddr . ".xml";

	NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','V', 'IMsgCREATING_YAST_FILE', $yastfile_dir);
	mkpath ("$yastfile_dir", $::VERBOSE, 0755);
	
	unlink $yastdir_link;
	symlink $yastdir_real,$yastdir_link;

	#print "tmpl: $yastcfg_tmpl\n file:$yastfile\n";
	open (YASTTEMPLATE, "<$yastcfg_tmpl")
	    || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_READ_FILE', $yastcfg_tmpl);

	open (YASTFILE, ">$yastfile")
	    || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_WRITE_FILE', $yastfile);
	close(YASTTEMPLATE);
	close(YASTFILE);

	my $xmlparser = new XML::DOM::Parser(KeepCDATA => 1);
	my $xmldoc = $xmlparser->parsefile ($yastcfg_tmpl);
	my $xmlroot=$xmldoc->getDocumentElement();
	my $xmlnodelist;
	
       my ($offset,$xmlnode);
       my ($configurenode,$installnode);
	my ($scripts,$post,$chroot,$scriptsnode,$chrootnode,$postnode);
       $post=0; $chroot=0; $scripts=0;
       my ($modenode,$initnode,$partitioningnode,$usersnode,$networkingnode,$clocknode,$generalnode,$reportnode,$bootloadernode);
       my ($ex_mode,$ex_init,$ex_partitioning,$ex_users,$ex_networking,$ex_clock,$ex_general,$ex_report,$ex_bootloader);
	$ex_mode=0; $ex_init=0; $ex_partitioning=0;$ex_users=0;$ex_networking=0;$ex_report=0;$ex_bootloader=0;
	
	$xmlnodelist=$xmlroot->getElementsByTagName("configure");

	if ($xmlnodelist->getLength() != 1) {
	 NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgPARSING_YAST_FILE', $yastcfg_tmpl);
	 exit(1);
       }
       $configurenode=$xmlnodelist->item(0);

       $xmlnode=$configurenode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
       }


       foreach $xmlnode ($configurenode->getChildNodes) {
       	if (($xmlnode->getNodeType == 1)) {
       		 if ($xmlnode->getNodeName eq "scripts") {
	       		$scriptsnode=$xmlnode;
       			$scripts=1;
	       	}
	       	if ($xmlnode->getNodeName eq "users") {
	       		$usersnode=$xmlnode;
	       		$ex_users=1;
	       	}
	       	if ($xmlnode->getNodeName eq "networking") {
	       		$networkingnode=$xmlnode;
       			$ex_networking=1;
	       	}
       	}
       }
        if ($scripts==0) {
       	$configurenode->insertBefore($xmldoc->createTextNode("$offset "),$configurenode->getLastChild);
       	$scriptsnode=$xmldoc->createElement("scripts");
       	$scriptsnode->appendChild($xmldoc->createTextNode("$offset "));
       	$configurenode->insertBefore($scriptsnode,$configurenode->getLastChild);
       }
       if ($ex_users==0) {
       	$configurenode->insertBefore($xmldoc->createTextNode("$offset "),$configurenode->getLastChild);
       	$usersnode=$xmldoc->createElement("users");
       	$usersnode->appendChild($xmldoc->createTextNode("$offset "));
       	$usersnode->setAttribute("config:type","list");
       	$configurenode->insertBefore($usersnode,$configurenode->getLastChild);
       }
       if ($ex_networking==0) {
       	$configurenode->insertBefore($xmldoc->createTextNode("$offset "),$configurenode->getLastChild);
       	$networkingnode=$xmldoc->createElement("networking");
       	$networkingnode->appendChild($xmldoc->createTextNode("$offset "));
       	$configurenode->insertBefore($networkingnode,$configurenode->getLastChild);
       }

       $xmlnodelist=$xmlroot->getElementsByTagName("install");

	if ($xmlnodelist->getLength() != 1) {
	 NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgPARSING_YAST_FILE', $yastcfg_tmpl);
	 exit(1);
       }
       $installnode=$xmlnodelist->item(0);

       $xmlnode=$installnode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
       }

      

       foreach $xmlnode ($installnode->getChildNodes) {
       	if (($xmlnode->getNodeType == 1)) {
       		 if ($xmlnode->getNodeName eq "init") {
	       		$initnode=$xmlnode;
       			$ex_init=1;
	       	}
	       	if ($xmlnode->getNodeName eq "general") {
	       		$generalnode=$xmlnode;
	       		$ex_general=1;
	       	}
	       	if ($xmlnode->getNodeName eq "partitioning") {
	       		$partitioningnode=$xmlnode;
	       		$ex_partitioning=1;
	       	}
	       	if ($xmlnode->getNodeName eq "report") {
	       		$reportnode=$xmlnode;
	       		$ex_report=1;
	       	}
	       	if ($xmlnode->getNodeName eq "bootloader") {
	       		$bootloadernode=$xmlnode;
	       		$ex_bootloader=1;
	       	}
       	}
       }
       
        if ($ex_init==0) {
       	$installnode->insertBefore($xmldoc->createTextNode("$offset "),$installnode->getLastChild);
       	$initnode=$xmldoc->createElement("init");
       	$initnode->appendChild($xmldoc->createTextNode("$offset "));
       	$installnode->insertBefore($initnode,$installnode->getLastChild);
       }
       if ($ex_report==0) {
       	$installnode->insertBefore($xmldoc->createTextNode("$offset "),$installnode->getLastChild);
       	$reportnode=$xmldoc->createElement("report");
       	$reportnode->appendChild($xmldoc->createTextNode("$offset "));
       	$installnode->insertBefore($reportnode,$installnode->getLastChild);
       }
       if ($ex_general==0) {
       	$installnode->insertBefore($xmldoc->createTextNode("$offset "),$installnode->getLastChild);
       	$generalnode=$xmldoc->createElement("general");
       	$generalnode->appendChild($xmldoc->createTextNode("$offset "));
       	$installnode->insertBefore($generalnode,$installnode->getLastChild);
       }
       if ($ex_partitioning==0) {
       	$installnode->insertBefore($xmldoc->createTextNode("$offset "),$installnode->getLastChild);
       	$partitioningnode=$xmldoc->createElement("partitioning");
       	$partitioningnode->setAttribute("config:type","list");
       	$partitioningnode->appendChild($xmldoc->createTextNode("$offset "));
       	$installnode->insertBefore($partitioningnode,$installnode->getLastChild);
       }
       if ($ex_bootloader==0) {
       	$installnode->insertBefore($xmldoc->createTextNode("$offset "),$installnode->getLastChild);
       	$bootloadernode=$xmldoc->createElement("bootloader");
       	$installnode->insertBefore($initnode,$installnode->getLastChild);
       }

       $xmlnode=$generalnode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
       }
       foreach $xmlnode ($generalnode->getChildNodes) {
       	if (($xmlnode->getNodeType == 1)) {
       		 if ($xmlnode->getNodeName eq "mode") {
	       		$modenode=$xmlnode;
       			$ex_mode=1;
	       	}
	       	if ($xmlnode->getNodeName eq "clock") {
	       		$clocknode=$xmlnode;
	       		$ex_clock=1;
	       	}
       	}
       }
       
        if ($ex_mode==0) {
       	$generalnode->insertBefore($xmldoc->createTextNode("$offset "),$generalnode->getLastChild);
       	$modenode=$xmldoc->createElement("mode");
       	$modenode->appendChild($xmldoc->createTextNode("$offset "));
       	$generalnode->insertBefore($modenode,$generalnode->getLastChild);
       }
       if ($ex_clock==0) {
       	$generalnode->insertBefore($xmldoc->createTextNode("$offset "),$generalnode->getLastChild);
       	$clocknode=$xmldoc->createElement("clock");
       	$clocknode->appendChild($xmldoc->createTextNode("$offset "));
       	$generalnode->insertBefore($clocknode,$generalnode->getLastChild);
       }
       
       #users
       my $ex_root=0;
	foreach $xmlnode ($usersnode->getElementsByTagName("username")) {
		foreach my $kid ($xmlnode->getChildNodes) {
			if (($kid->getNodeType==3) && ($kid->getNodeValue eq "root")) {
				$ex_root=1;
				last;
			}
		}
		if ($ex_root) { last;}
	}

	if ($ex_root == 0) {
		my ($usernode,$encryptednode,$user_passwordnode,$usernamenode);
		$xmlnode=$usersnode->getPreviousSibling;
       	if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       		$offset=$xmlnode->getNodeValue;
       	} else {
       		$offset="\n";
      		 }
		$usersnode->insertBefore($xmldoc->createTextNode("$offset "),$usersnode->getLastChild);
       	$usernode=$xmldoc->createElement("user");
       	$usernode->appendChild($xmldoc->createTextNode("$offset "));       	
       	$usersnode->insertBefore($usernode,$usersnode->getLastChild);

       	$usernode->insertBefore($xmldoc->createTextNode("$offset  "),$usernode->getLastChild);
       	$encryptednode=$xmldoc->createElement("encrypted");
       	$encryptednode->setAttribute("config:type","boolean");   
       	$encryptednode->appendChild($xmldoc->createTextNode("true "));     	
       	$usernode->insertBefore($encryptednode,$usernode->getLastChild);

       	$usernode->insertBefore($xmldoc->createTextNode("$offset  "),$usernode->getLastChild);
       	$user_passwordnode=$xmldoc->createElement("user_password");
       	$user_passwordnode->appendChild($xmldoc->createTextNode("95G9p0q.CBtl."));    	
       	$usernode->insertBefore($user_passwordnode,$usernode->getLastChild);

       	$usernode->insertBefore($xmldoc->createTextNode("$offset  "),$usernode->getLastChild);
       	$usernamenode=$xmldoc->createElement("username"); 
       	$usernamenode->appendChild($xmldoc->createTextNode("root"));    	
       	$usernode->insertBefore($usernamenode,$usernode->getLastChild);

	}

	#partitioning stanza
	my $ex_drive=0;
       $xmlnode=$partitioningnode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
       }
       foreach $xmlnode ($partitioningnode->getChildNodes) {
       	if (($xmlnode->getNodeType == 1)) {
       		 if ($xmlnode->getNodeName eq "drive") {
       			$ex_drive=1;
	       	}
       	}
       }
       if ($ex_drive == 0) {
		my ($drivenode,$usenode,$devicenode);
		$partitioningnode->insertBefore($xmldoc->createTextNode("$offset "),$partitioningnode->getLastChild);
       	$drivenode=$xmldoc->createElement("drive");
       	$drivenode->appendChild($xmldoc->createTextNode("$offset "));
       	$partitioningnode->insertBefore($drivenode,$partitioningnode->getLastChild);

       	$drivenode->insertBefore($xmldoc->createTextNode("$offset  "),$drivenode->getLastChild);
       	$devicenode=$xmldoc->createElement("device");
       	$devicenode->appendChild($xmldoc->createTextNode("$bootdisk"));
       	$drivenode->insertBefore($devicenode,$drivenode->getLastChild);
       	
       	$drivenode->insertBefore($xmldoc->createTextNode("$offset  "),$drivenode->getLastChild);
       	$usenode=$xmldoc->createElement("use");
       	$usenode->appendChild($xmldoc->createTextNode("all"));
       	$drivenode->insertBefore($usenode,$drivenode->getLastChild);
		
	}
	#bootloader stanza
    if ($::ARCH eq "ppc64") {
	foreach $xmlnode ($bootloadernode->getChildNodes) {
       	$bootloadernode->removeChild($xmlnode);
       	}

       $xmlnode=$bootloadernode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
       }
       
       $bootloadernode->appendChild($xmldoc->createTextNode("$offset"));
       
	my ($activatenode,$locationnode,$lba_supportnode,$linearnode);
	
       $activatenode=$xmldoc->createElement("activate");     
       $bootloadernode->insertBefore($xmldoc->createTextNode("$offset "),$bootloadernode->getLastChild);
       $bootloadernode->insertBefore($activatenode,$bootloadernode->getLastChild);
       $activatenode->setAttribute("config:type","boolean");
	$activatenode->appendChild($xmldoc->createTextNode("true"));

	$bootloadernode->insertBefore($xmldoc->createTextNode("$offset "),$bootloadernode->getLastChild);
	$locationnode=$xmldoc->createElement("location"); 
	$locationnode->appendChild($xmldoc->createTextNode("$bootdisk"."1"));    	#????
	$bootloadernode->insertBefore($locationnode,$bootloadernode->getLastChild);

       $lba_supportnode=$xmldoc->createElement("lba_support");     
       $bootloadernode->insertBefore($xmldoc->createTextNode("$offset "),$bootloadernode->getLastChild);
       $bootloadernode->insertBefore($lba_supportnode,$bootloadernode->getLastChild);
       $lba_supportnode->setAttribute("config:type","boolean");
	$lba_supportnode->appendChild($xmldoc->createTextNode("false"));

       $linearnode=$xmldoc->createElement("linear");     
       $bootloadernode->insertBefore($xmldoc->createTextNode("$offset "),$bootloadernode->getLastChild);
       $bootloadernode->insertBefore($linearnode,$bootloadernode->getLastChild);
       $linearnode->setAttribute("config:type","boolean");
	$linearnode->appendChild($xmldoc->createTextNode("false"));
    }
	#  report section 
		
       foreach $xmlnode ($reportnode->getChildNodes) {
       	$reportnode->removeChild($xmlnode);
       	}

       $xmlnode=$reportnode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
       }
       
       $reportnode->appendChild($xmldoc->createTextNode("$offset"));
       
	my ($messagesnode,$errorsnode,$warningsnode,$shownode,$timeoutnode,$lognode);
	
       $messagesnode=$xmldoc->createElement("messages");     
       $reportnode->insertBefore($xmldoc->createTextNode("$offset "),$reportnode->getLastChild);
       $reportnode->insertBefore($messagesnode,$reportnode->getLastChild);
	$messagesnode->appendChild($xmldoc->createTextNode("$offset "));

	$messagesnode->insertBefore($xmldoc->createTextNode("$offset  "),$messagesnode->getLastChild);
	$shownode=$xmldoc->createElement("show"); 
	$shownode->appendChild($xmldoc->createTextNode("true"));    	
	$messagesnode->insertBefore($shownode,$messagesnode->getLastChild);

	$messagesnode->insertBefore($xmldoc->createTextNode("$offset  "),$messagesnode->getLastChild);
	$timeoutnode=$xmldoc->createElement("timeout"); 
	$timeoutnode->appendChild($xmldoc->createTextNode("3"));    	
	$messagesnode->insertBefore($timeoutnode,$messagesnode->getLastChild);

	
	$messagesnode->insertBefore($xmldoc->createTextNode("$offset  "),$messagesnode->getLastChild);
	$lognode=$xmldoc->createElement("log"); 
	$lognode->appendChild($xmldoc->createTextNode("true"));    	
	$messagesnode->insertBefore($lognode,$messagesnode->getLastChild);

	
       $errorsnode=$xmldoc->createElement("errors");    
       $reportnode->insertBefore($xmldoc->createTextNode("$offset "),$reportnode->getLastChild);
       $reportnode->insertBefore($errorsnode,$reportnode->getLastChild);
	$errorsnode->appendChild($xmldoc->createTextNode("$offset "));

	$errorsnode->insertBefore($xmldoc->createTextNode("$offset  "),$errorsnode->getLastChild);
	$shownode=$xmldoc->createElement("show"); 
	$shownode->appendChild($xmldoc->createTextNode("true"));    	
	$errorsnode->insertBefore($shownode,$errorsnode->getLastChild);

	$errorsnode->insertBefore($xmldoc->createTextNode("$offset  "),$errorsnode->getLastChild);
	$timeoutnode=$xmldoc->createElement("timeout"); 
	$timeoutnode->appendChild($xmldoc->createTextNode("3"));    	
	$errorsnode->insertBefore($timeoutnode,$errorsnode->getLastChild);

	
	$errorsnode->insertBefore($xmldoc->createTextNode("$offset  "),$errorsnode->getLastChild);
	$lognode=$xmldoc->createElement("log"); 
	$lognode->appendChild($xmldoc->createTextNode("true"));    	
	$errorsnode->insertBefore($lognode,$errorsnode->getLastChild);


       $warningsnode=$xmldoc->createElement("warnings");    
       $reportnode->insertBefore($xmldoc->createTextNode("$offset "),$reportnode->getLastChild);
       $reportnode->insertBefore($warningsnode,$reportnode->getLastChild);
	$warningsnode->appendChild($xmldoc->createTextNode("$offset "));

	$warningsnode->insertBefore($xmldoc->createTextNode("$offset  "),$warningsnode->getLastChild);
	$shownode=$xmldoc->createElement("show"); 
	$shownode->appendChild($xmldoc->createTextNode("true"));    	
	$warningsnode->insertBefore($shownode,$warningsnode->getLastChild);

	$warningsnode->insertBefore($xmldoc->createTextNode("$offset  "),$warningsnode->getLastChild);
	$timeoutnode=$xmldoc->createElement("timeout"); 
	$timeoutnode->appendChild($xmldoc->createTextNode("3"));    	
	$warningsnode->insertBefore($timeoutnode,$warningsnode->getLastChild);

	
	$warningsnode->insertBefore($xmldoc->createTextNode("$offset  "),$warningsnode->getLastChild);
	$lognode=$xmldoc->createElement("log"); 
	$lognode->appendChild($xmldoc->createTextNode("true"));    	
	$warningsnode->insertBefore($lognode,$warningsnode->getLastChild);


	
	#  Mode section 
		
       foreach $xmlnode ($modenode->getChildNodes) {
       	$modenode->removeChild($xmlnode);
       	}

       $xmlnode=$modenode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
       }
	
       my $confirmnode=$xmldoc->createElement("confirm"); 
       $confirmnode->setAttribute("config:type","boolean");
	$confirmnode->appendChild($xmldoc->createTextNode("false"));
	
       my $rebootnode=$xmldoc->createElement("reboot"); 
       $rebootnode->setAttribute("config:type","boolean");
	$rebootnode->appendChild($xmldoc->createTextNode("false"));
	
       my $forcebootnode=$xmldoc->createElement("forceboot"); 
       $forcebootnode->setAttribute("config:type","boolean");
	$forcebootnode->appendChild($xmldoc->createTextNode("false"));
	
       my $interactive_bootnode=$xmldoc->createElement("interactive_boot"); 
       $interactive_bootnode->setAttribute("config:type","boolean");
	$interactive_bootnode->appendChild($xmldoc->createTextNode("false"));

       foreach $xmlnode ($modenode->getChildNodes) {
       	$modenode->removeChild($xmlnode);
       	}

       $modenode->appendChild($xmldoc->createTextNode("$offset "));
       $modenode->appendChild($confirmnode);
       $modenode->appendChild($xmldoc->createTextNode("$offset "));
       $modenode->appendChild($rebootnode);
       $modenode->appendChild($xmldoc->createTextNode("$offset "));
       $modenode->appendChild($forcebootnode);
       $modenode->appendChild($xmldoc->createTextNode("$offset "));
       $modenode->appendChild($interactive_bootnode);
       $modenode->appendChild($xmldoc->createTextNode("$offset"));

	#networking
	$xmlnode=$networkingnode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
      }
      
       my ($dnsnode,$hostnamenode,$domainnode);
       my $ex_dns = 0;
       my $ex_hostname = 0;
       my $ex_domain=0;
       
       foreach $xmlnode ($networkingnode->getChildNodes) {
       	if (($xmlnode->getNodeType == 1) && ($xmlnode->getNodeName eq "dns")) {
       		$dnsnode=$xmlnode;
       		$ex_dns=1;
       		last;
       	}
       }
       if ($ex_dns==0) {
       	$networkingnode->insertBefore($xmldoc->createTextNode("$offset "),$networkingnode->getLastChild);
       	$dnsnode=$xmldoc->createElement("dns");
       	$dnsnode->appendChild($xmldoc->createTextNode("$offset "));
       	$networkingnode->insertBefore($dnsnode,$networkingnode->getLastChild);
       }
       
       foreach $xmlnode ($dnsnode->getChildNodes) {
       	if (($xmlnode->getNodeType == 1)) {
       		 if ($xmlnode->getNodeName eq "hostname") {
	       		$hostnamenode=$xmlnode;
	       		$ex_hostname=1;
	       	}
	       	if ($xmlnode->getNodeName eq "domain") {
	       		$domainnode=$xmlnode;
	       		$ex_domain=1;
	       	}
       	}
       }

       if ($ex_hostname==0) {
       	$dnsnode->insertBefore($xmldoc->createTextNode("$offset  "),$dnsnode->getLastChild);
       	$hostnamenode=$xmldoc->createElement("hostname");
       	$dnsnode->insertBefore($hostnamenode,$dnsnode->getLastChild);
       }
       if ($ex_domain==0) {
       	$dnsnode->insertBefore($xmldoc->createTextNode("$offset  "),$dnsnode->getLastChild);
       	$domainnode=$xmldoc->createElement("domain");
       	$dnsnode->insertBefore($domainnode,$dnsnode->getLastChild);
       }
       
       foreach $xmlnode ($hostnamenode->getChildNodes) {
       	$hostnamenode->removeChild($xmlnode);
       	}
       	
       $hostnamenode->appendChild($xmldoc->createTextNode($node_shorthost));

       foreach $xmlnode ($domainnode->getChildNodes) {
       	$domainnode->removeChild($xmlnode);
       	}
       $domainnode->appendChild($xmldoc->createTextNode($dnsdomain));

       #clock
       $xmlnode=$clocknode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
       }
       my $timezonenode;
       my $ex_timezone=0;
       foreach $xmlnode ($clocknode->getChildNodes) {
       	if (($xmlnode->getNodeType == 1) && ($xmlnode->getNodeName eq "timezone")) {
       		$timezonenode=$xmlnode;
       		$ex_timezone=1;
       		last;
       	}
       }
       if ($ex_timezone==0) {
       	$clocknode->insertBefore($xmldoc->createTextNode("$offset "),$clocknode->getLastChild);
       	$timezonenode=$xmldoc->createElement("timezone");
       	$clocknode->insertBefore($timezonenode,$clocknode->getLastChild);
       }
       if ($timezonenode->hasChildNodes() != 1 )  {
	       $timezonenode->appendChild($xmldoc->createTextNode($timezone));
	}
	#for init section
       $xmlnode=$initnode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
       }

	foreach $xmlnode ($initnode->getChildNodes) {
       	$initnode->removeChild($xmlnode);
       	}

	my $info_file = <<INFOEND

#
# Don't remove the following line:
# start_linuxrc_conf
#
install: nfs://$nfs_ipaddr$nfs_dir/CD1
Usedhcp: 1
Textmode: 1

# end_linuxrc_conf
# Do not remove the above comment

INFOEND
;
       my $info_filenode=$xmldoc->createElement("info_file"); 
       $info_filenode->appendChild($xmldoc->createTextNode("\n"));
	$info_filenode->appendChild($xmldoc->createCDATASection ($info_file));
	$info_filenode->appendChild($xmldoc->createTextNode("$offset "));
	
       my $insmodenode=$xmldoc->createElement("insmode"); 
	$insmodenode->appendChild($xmldoc->createTextNode("nfs"));

	my $profile_locationnode=$xmldoc->createElement("profile_location"); 
	$profile_locationnode->appendChild($xmldoc->createTextNode($yastfile));
	
       my $profile_protocolnode=$xmldoc->createElement("profile_protocol"); 
	$profile_protocolnode->appendChild($xmldoc->createTextNode("nfs"));
	
	my $servernode=$xmldoc->createElement("server"); 
	$servernode->appendChild($xmldoc->createTextNode($nfs_ipaddr));
	
	my $serverdirnode=$xmldoc->createElement("serverdir"); 
	$serverdirnode->appendChild($xmldoc->createTextNode($nfs_dir));
	
       my $textmodenode=$xmldoc->createElement("textmode"); 
       $textmodenode->setAttribute("config:type","boolean");
	$textmodenode->appendChild($xmldoc->createTextNode("true"));
	
       my $usedhcpnode=$xmldoc->createElement("usedhcp"); 
       $usedhcpnode->setAttribute("config:type","boolean");
	$usedhcpnode->appendChild($xmldoc->createTextNode("true"));

	$initnode->appendChild($xmldoc->createTextNode("$offset "));
       $initnode->appendChild($info_filenode);
	$initnode->appendChild($xmldoc->createTextNode("$offset "));
       $initnode->appendChild($insmodenode);
	$initnode->appendChild($xmldoc->createTextNode("$offset "));
       $initnode->appendChild($profile_locationnode);
	$initnode->appendChild($xmldoc->createTextNode("$offset "));
       $initnode->appendChild($profile_protocolnode);
	$initnode->appendChild($xmldoc->createTextNode("$offset "));
       $initnode->appendChild($servernode);
	$initnode->appendChild($xmldoc->createTextNode("$offset "));
       $initnode->appendChild($serverdirnode);
	$initnode->appendChild($xmldoc->createTextNode("$offset "));
       $initnode->appendChild($textmodenode);
	$initnode->appendChild($xmldoc->createTextNode("$offset "));
       $initnode->appendChild($usedhcpnode);
	$initnode->appendChild($xmldoc->createTextNode("$offset"));

	#post-scripts
	
      

       $xmlnode=$scriptsnode->getPreviousSibling;
       if ((defined $xmlnode) &&  ($xmlnode->getNodeType == 3)) {
       	$offset=$xmlnode->getNodeValue;
       } else {
       	$offset="\n";
       }
       
       foreach $xmlnode ($scriptsnode->getChildNodes) {
       	if (($xmlnode->getNodeType == 1)) {
       		 if ($xmlnode->getNodeName eq "post-scripts") {
	       		$postnode=$xmlnode;
	       		$post=1;
	       	}
	       	if ($xmlnode->getNodeName eq "chroot-scripts") {
	       		$chrootnode=$xmlnode;
	       		$chroot=1;
	       	}
       	}
       }
       
       if ($post==0) {
       	$scriptsnode->insertBefore($xmldoc->createTextNode("$offset "),$scriptsnode->getLastChild);
       	$postnode=$xmldoc->createElement("post-scripts");
       	$postnode->appendChild($xmldoc->createTextNode("$offset "));
       	$postnode->setAttribute("config:type","list");
       	$scriptsnode->insertBefore($postnode,$scriptsnode->getLastChild);
       }
       if ($chroot==0) {
       	$scriptsnode->insertBefore($xmldoc->createTextNode("$offset "),$scriptsnode->getLastChild);
	#SSSBEGIN
       	$chrootnode=$xmldoc->createElement("chroot-scripts");
       	$chrootnode->appendChild($xmldoc->createTextNode("$offset "));
       	$chrootnode->setAttribute("config:type","list");
       	$scriptsnode->insertBefore($chrootnode,$scriptsnode->getLastChild);
	#SSSEND
       }
       $postnode->setAttribute("config:type","list");
       $chrootnode->setAttribute("config:type","list");

       
	#SSSBEGIN
	my ($scriptnode, $filenamenode,$interpreternode,$sourcenode);

	if ($::ARCH ne "ppc64") {
	    #
	    # Edit the chrootscript file and add it as a CDATA node
	    #
	    my $chrootsourcefile = "/opt/csm/install/autoyast.chrootscript.tmpl.".$::ARCH;
	    my $chrootsource;

	    if (-r $chrootsourcefile) {
		$chrootsource = `cat $chrootsourcefile`;
		$chrootsource = "$chrootsource\n";
		$chrootsource=~s/#MGMTSVR_HOSTNAME#/$ms_hostname/g; # Management Server hostname
		$chrootsource=~s/#MGMTSVR_SHORTHOST#/$ms_shorthost/g; # Management Server shorthost
		$chrootsource=~s/#MGMTSVR_IP#/$ms_ipaddr/g;	# Management Server IP Address
		$chrootsource=~s/#NODE_HOSTNAME#/$node_hostname/g;	# Node hostname
		$chrootsource=~s/#NODE_HEX#/$node_hex/g;		# Node ip address in hex. #SSSADD
		$chrootsource=~s/#NFS_HOSTNAME#/$nfs_hostname/g;	# NFS server hostname
		$chrootsource=~s/#NFS_IP#/$nfs_ipaddr/g;		# NFS server ip address
		$chrootsource=~s/#NFS_DIR#/$nfs_dir/g;		# REDHAT_TOP
		$chrootsource=~s/#CSM_MOUNT#/$csm_mount/g;         # CSM mount point
		$chrootsource=~s/#TFTP_IP#/$ms_ipaddr/g;		# TFTP Server(Management server)
		$chrootsource=~s/#NAMESERVERS#/$nameservers/g;	# Nameservers (comma-separated)
		$chrootsource=~s/#DNSDOMAIN#/$dnsdomain/g;		# Node's DNS Domain name
		$chrootsource=~s/#CSM_VERSION#/$::CSM_VERSION/g;	# Version of CSM
		$chrootsource=~s/#BOOTDISK#/$bootdisk/g;	# boot disk
		$chrootsource=~s/#DISTRO_NAME#/$::DISTRO_NAME/g;# Name of Distribution
		$chrootsource=~s/#DISTRO_VERSION#/$::DISTRO_VERSION/g;# Version of Distribution
	    } else {	
		    
	       NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_READ_FILE', $chrootsourcefile);
	    }

	    $scriptnode=$xmldoc->createElement("script"); 
	    $offset="$offset  ";
	    
	    $filenamenode=$xmldoc->createElement("filename"); 
	    $filenamenode->appendChild($xmldoc-> createTextNode("csmchrootscript"));
	    
	    $interpreternode=$xmldoc->createElement("interpreter"); 
	    $interpreternode->appendChild($xmldoc->createTextNode("shell"));

	    $sourcenode=$xmldoc->createElement("source"); 
	    $sourcenode->appendChild($xmldoc->createTextNode("\n"));
	    $sourcenode->appendChild($xmldoc->createCDATASection($chrootsource));
	    $sourcenode->appendChild($xmldoc->createTextNode("$offset  "));

	    $scriptnode->appendChild($xmldoc->createTextNode("$offset "));
	    $scriptnode->appendChild($filenamenode);
	    $scriptnode->appendChild($xmldoc->createTextNode("$offset "));
	    $scriptnode->appendChild($interpreternode);
	    $scriptnode->appendChild($xmldoc->createTextNode("$offset "));
	    $scriptnode->appendChild($sourcenode);
	    $scriptnode->appendChild($xmldoc->createTextNode("$offset"));

	    # Now add the node to the tree.
	    $chrootnode->insertBefore($xmldoc->createTextNode("$offset"),$chrootnode->getLastChild);
	    $chrootnode->insertBefore($scriptnode,$chrootnode->getLastChild);
	}
	#SSSEND

	#
	# Edit the postscript file and add it as a CDATA node
	#
	my $postsourcefile = "/opt/csm/install/autoyast.postscript.tmpl.".$::ARCH;
	my $postsource;

	if (-r $postsourcefile) {
	    $postsource = `cat $postsourcefile`;
	    $postsource = "$postsource\n";
	    $postsource=~s/#MGMTSVR_HOSTNAME#/$ms_hostname/g; # Management Server hostname
	    $postsource=~s/#MGMTSVR_SHORTHOST#/$ms_shorthost/g; # Management Server shorthost
	    $postsource=~s/#MGMTSVR_IP#/$ms_ipaddr/g;	# Management Server IP Address
	    $postsource=~s/#NODE_HOSTNAME#/$node_hostname/g;	# Node hostname
	    $postsource=~s/#NODE_SHORTHOST#/$node_shorthost/g;	# Node short hostname
	    $postsource=~s/#NODE_IP#/$node_ipaddr/g;		# Node ip address
	    $postsource=~s/#NODE_HEX#/$node_hex/g;		# Node ip address in hex. #SSSADD
	    $postsource=~s/#NFS_HOSTNAME#/$nfs_hostname/g;	# NFS server hostname
	    $postsource=~s/#NFS_IP#/$nfs_ipaddr/g;		# NFS server ip address
	    $postsource=~s/#NFS_DIR#/$nfs_dir/g;		# REDHAT_TOP
	    $postsource=~s/#CSM_MOUNT#/$csm_mount/g;         # CSM mount point
	    $postsource=~s/#TFTP_IP#/$ms_ipaddr/g;		# TFTP Server(Management server)
	    $postsource=~s/#NAMESERVERS#/$nameservers/g;	# Nameservers (comma-separated)
	    $postsource=~s/#DNSDOMAIN#/$dnsdomain/g;		# Node's DNS Domain name
	    $postsource=~s/#CSM_VERSION#/$::CSM_VERSION/g;	# Version of CSM
	    $postsource=~s/#BOOTDISK#/$bootdisk/g;	# Version of CSM
	} else {	
		
	   NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_READ_FILE', $postsourcefile);
	
	}
	
	$scriptnode=$xmldoc->createElement("script"); 
	$offset="$offset  ";
	
	$filenamenode=$xmldoc->createElement("filename"); 
	$filenamenode->appendChild($xmldoc-> createTextNode("csmpostscript"));
	
        $interpreternode=$xmldoc->createElement("interpreter"); 
	$interpreternode->appendChild($xmldoc->createTextNode("shell"));

	$sourcenode=$xmldoc->createElement("source"); 
	$sourcenode->appendChild($xmldoc->createTextNode("\n"));
	$sourcenode->appendChild($xmldoc->createCDATASection($postsource));
	$sourcenode->appendChild($xmldoc->createTextNode("$offset  "));

	$scriptnode->appendChild($xmldoc->createTextNode("$offset "));
        $scriptnode->appendChild($filenamenode);
	$scriptnode->appendChild($xmldoc->createTextNode("$offset "));
        $scriptnode->appendChild($interpreternode);
	$scriptnode->appendChild($xmldoc->createTextNode("$offset "));
        $scriptnode->appendChild($sourcenode);
	$scriptnode->appendChild($xmldoc->createTextNode("$offset"));

	# Now add the node to the tree
	$postnode->insertBefore($xmldoc->createTextNode("$offset"),$postnode->getLastChild);
	$postnode->insertBefore($scriptnode,$postnode->getLastChild);
	       
	$xmldoc->printToFile($yastfile);
	
    unlink $yastfile_link;
    symlink $yastfile_real,$yastfile_link;
    }
}


#--End

#get the ip address, and directory, of the install server for this node
sub get_node_isvr{
    my ($nodename) = @_;
    my $isvr_ip;
    my $bad =0;
    my ($hostname, $dir) = split ':', $nodeHash{$nodename}{'InstallServer'};
    if($hostname && $hostname ne $nodeHash{$nodename}{'ManagementServer'}){
	($hostname, $isvr_ip) = NodeUtils->getHost($hostname); 
	my ($node_hostname, $node_ip) = NodeUtils->getHost($nodename);
	if($hostname eq $node_hostname){ #this machine is an install server of itself
		if(!$SetupBootUtils::ISVR_Warning{$nodename}){
			$SetupBootUtils::ISVR_Warning{$nodename}=1;
			#print "Warning: $nodename is an install server of itself. The management server will be used to serve its files during the operating system install.\n";
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH, 'csminstall','W', 'EMsgISVR_OF_ITSELF', $nodename); 
		}
		$bad =1;	
	}
	if(!$dir){
	    $dir = $::SERVER_DIRECTORY;
	}
    }
    else{
	($hostname, $isvr_ip)= &get_node_management_server($nodename);
    }
    if($bad){
	($hostname, $isvr_ip)= &get_node_management_server($nodename);
	undef $dir;
    }
    
    return($hostname, $isvr_ip, $dir);
}


sub copyKernel
{
  #for getmac
  # Copy the kernel, ramdisk and pxelinux files needed for MAC address collection
  if($bootMethod eq 'getmacs')
  {
    my ($cmd, $rc);

    # Copy the current linux kernel
    mkpath ("$::TFTPBOOT/csm", $::VERBOSE, 0110);
    # mkpath can fail to set the mode - so do it explicitly.

    my $linuzFile = "/boot/vmlinuz";
     # SuSE SLES doesn't have the vmlinuz-<version> file.
    if (( $::DISTRO_NAME =~ /SLES/ ) || ($::DISTRO_NAME =~ /SuSE/)) 
    {
	}
	else {
       chomp(my $lversion = NodeUtils->runcmd("/bin/uname -r")); 
       $linuzFile .= "-$lversion";
    }
    #chomp(my $lversion = NodeUtils->runcmd("/bin/uname -r")); 
    #$linuzFile .= "-$lversion";
    &copyfile ($linuzFile, "$::TFTPBOOT/csm/getmacz");
    InstallUtils->secureFilePermissions("$::TFTPBOOT/csm","0110","0440");
  }
  elsif($bootMethod eq 'kickstart')
    {
      # FOR KICKSTART
      my ($cmd);
      my ($rc) = 0;
      my ($returncode) = 0;
      my ($output);

      my ($source_kernel) = "$::REDHAT_TOP/images/pxeboot/vmlinuz";
      my ($ks_kernel) = $::TFTPBOOT . "/csm/ks" . $::DISTRO_VERSION . "z";

      mkpath ("$::TFTPBOOT/csm", $::VERBOSE, 0110);
      # if the path has already been created then we need to secure it:
      &copyfile($source_kernel, $ks_kernel);
      InstallUtils->secureFilePermissions("$::TFTPBOOT/csm","0110","0440");

      return $returncode;
    }
#SSSBEGIN
  elsif($bootMethod eq 'autoyast')
    {
      # FOR AUTOYAST
      my ($cmd);
      my ($rc) = 0;
      my ($returncode) = 0;
      my ($output);

      my ($source_kernel) = "$::SUSE_TOP/boot/loader/linux";
      my ($yast_kernel) = $::TFTPBOOT . "/csm/yast-" . $::DISTRO_NAME . $::DISTRO_VERSION . "-" . $::ARCH . "z";

      mkpath ("$::TFTPBOOT/csm", $::VERBOSE, 0110);
      &copyfile($source_kernel, $yast_kernel);
      InstallUtils->secureFilePermissions("$::TFTPBOOT/csm","0110","0440");

      return $returncode;
    }
#SSSEND
    elsif($bootMethod eq 'sis')
      {
	  &copySISKernel();
      }
}
sub copySISKernel
{
    my ($returncode) = 0;

    my ($source_kernel) = "/usr/share/systemimager/boot/i386/standard/kernel";
    my ($sis_kernel) = $::TFTPBOOT . "/csm/sis" . $::DISTRO_VERSION . "z";

    mkpath ("$::TFTPBOOT/csm", $::VERBOSE, 0110);
    # mkpath can fail to set the mode - so do it explicitly.
    &copyfile($source_kernel, $sis_kernel);
    InstallUtils->secureFilePermissions("$::TFTPBOOT/csm","0110","0440");
    return $returncode;
}

#--Begin add for ppc64
sub copyPPC64Kernel
{
    my $kerneldir=$::SUSE_TOP."/updates/kernel";
    my ($returncode) = 0;

    my $source_rpm=$::SUSE_TOP."/CD1/suse/ppc/kernel-ppc64-2.4.19-108.ppc.rpm";
    my $kernelversion="2.4.19-108";
    my $cmd="ls $kerneldir/kernel-ppc64-2.4.19-*.ppc.rpm 2>/dev/null | sort -dr | head -1";
    my $kernelrpm=`$cmd`;
    chomp($kernelrpm);
    if ($kernelrpm=~/kernel-ppc64-(2\.4\.19-\d+)\.ppc\.rpm/) {
    	$source_rpm=$kernelrpm;
    	$kernelversion=$1;
    }
    
    my $rpmtmpdir = "/tmp/rpmtmp";
      
    my $cmd="rm -rf $rpmtmpdir 1>/dev/null 2>&1 ; mkdir -p $rpmtmpdir/var/lib/rpm 1>/dev/null 2>&1 ; rpm --initdb --root $rpmtmpdir  1>/dev/null 2>&1 ;"
    		."rpm -i --force --nodeps --ignorearch --root $rpmtmpdir $source_rpm 1>/dev/null 2>&1 ";
	
    system($cmd);
    
    my ($source_kernel) = "$rpmtmpdir/boot/zImage.initrd.ppc64-2.4.19";

    my ($ppc_kernel) = $::TFTPBOOT . "/csm/yast" . $::DISTRO_VERSION . "-ppc64-zImage.initrd";

    my $real_kernel="$ppc_kernel-$kernelversion";

    mkpath ("$::TFTPBOOT/csm", $::VERBOSE, 0110);
    # mkpath can fail to set the mode - so do it explicitly.
    InstallUtils->secureFilePermissions("$::TFTPBOOT/csm","0110","0440");

    &copyfile($source_kernel, $real_kernel);

    unlink $ppc_kernel;
    symlink $real_kernel,$ppc_kernel;
    return $returncode;
}
#--End
sub setAttributes
{
     # If netmask is not provided, use the netmask of the management server
    if (!$::BOOT_ATTRIBUTES{"Netmask"})
    {
	$::BOOT_ATTRIBUTES{"Netmask"} = 
		(split('Mask:', `$::IFCONFIG eth0 | $::GREP \'inet addr\'`))[1];
	chomp $::BOOT_ATTRIBUTES{"Netmask"};
    }

    # If Gateway is not provided, use the gateway of the management server
    if (!$::BOOT_ATTRIBUTES{"Gateway"})
    {
	$::BOOT_ATTRIBUTES{"Gateway"} = 
	    (split(' ', `$::NETSTAT -rn | $::GREP 0.0.0.0 | $::TAIL -n 1`))[1];
	chomp $::BOOT_ATTRIBUTES{"Gateway"};
    }

    # If Nameservers is not provided, use the nameservers of the 
    # management server
    if (!$::BOOT_ATTRIBUTES{"Nameservers"})
    {
	my $nameservers = "";
	my $filename = "/etc/resolv.conf";
	open (RESOLV, "<$filename")
	    || NodeUtils->messageFromcat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $filename);

	while (<RESOLV>)
	{
	    my $line = $_;
	    chomp $line;

	    if ($line =~ /^nameserver/i)
	    {

		$nameservers .= "," if ($nameservers);
		$nameservers .= (split(' ', $line))[1];
	    }
	}
	close (RESOLV);
	$::BOOT_ATTRIBUTES{"Nameservers"} = $nameservers;
	chomp $::BOOT_ATTRIBUTES{"Gateway"};
    }

    #print "Netmask     = " . $::BOOT_ATTRIBUTES{"Netmask"} . "\n";
    #print "Gateway     = " . $::BOOT_ATTRIBUTES{"Gateway"} . "\n";
    #print "Nameservers = " . $::BOOT_ATTRIBUTES{"Nameservers"} . "\n";
}

sub copyfile
{
    my ($src, $dest) = @_;

    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'V', 'IMsgCOPYING_SRC_DST', $src, $dest);
    copy($src, $dest);
    my $rc = $!;
    ($rc) && NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E1', 'EMsgNO_FILECOPY_RC', $src, $dest, $rc);

    return $rc;
}



sub makeKickstartRAMDisk
  {
      &prepare_mount();
      # for kickstart
      my ($cmd);
      my ($rc) = 0;
      my ($returncode) = 0;
      my ($output);

      my ($initrd) = "$::REDHAT_TOP/images/pxeboot/initrd-everything.img";
      if (! -f $initrd)
      {
	$initrd = "$::REDHAT_TOP/images/pxeboot/initrd.img";
      }
      my ($ramdisk_basename) = "ks" . $::DISTRO_VERSION;		# e.g. ks7.3
      my ($ks_ramdisk) = $::TFTPBOOT . "/csm/" . $ramdisk_basename . ".gz";

      my $tmpdir = "/tmp/ks" . $::DISTRO_VERSION;  # e.g. /tmp/ks7.3
      if (-d $tmpdir)
      {
        rmtree ($tmpdir, $::VERBOSE, 1);
      }
      mkpath ($tmpdir, $::VERBOSE, 0755);
    
    #-------------------------------------------------
    # Unzip the ramdisk
    #-------------------------------------------------
    my $ramdisk = "$tmpdir/$ramdisk_basename";		# /tmp/ks7.3/ks7.3
    #$cmd = "$::GUNZIP -c $tmpdir/initrd.img > $ramdisk";
    $cmd = "$::GUNZIP -c $initrd > $ramdisk";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    #-------------------------------------------------
    # Mount the ramdisk to kstmp so it can be modified
    #-------------------------------------------------
    my $kstmp = "$tmpdir/kstmp";			# e.g. /tmp/ks7.3/kstmp
    mkpath ($kstmp, $::VERBOSE, 0755);
    $cmd = "$::MOUNT -o loop $ramdisk $kstmp";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    #-------------------------------------------------
    # Copy a terminfo file onto the ramdisk
    #-------------------------------------------------
    mkpath ("$kstmp/etc/terminfo/x", $::VERBOSE, 0755);
    mkpath ("$kstmp/etc/terminfo/v", $::VERBOSE, 0755);
    $cmd = "$::COPY -f /usr/share/terminfo/x/xterm $kstmp/etc/terminfo/v/vt100";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    #-------------------------------------------------
    # Insert the drivers into the ramdisk
    #-------------------------------------------------

    my $kernel_level = "";
    $::FIND = "/usr/bin/find";

    # Copy the modules.cgz file to a temporary location
    $cmd = "$::COPY -f $kstmp/modules/modules.cgz $tmpdir/modules.cgz";
    $output = NodeUtils->runcmd($cmd,0);

    # Unpack the modules.cgz file
    $cmd = "cd $tmpdir; $::GUNZIP -c modules.cgz | $::CPIO -id";
    $output = NodeUtils->runcmd($cmd,0);

    # Determine the kernel level used in the modules.cgz.
    $cmd = "cd $tmpdir; $::LS -d [0-9]*";
    $output = NodeUtils->runcmd($cmd,0);
    chomp $output;
    if (-d "$tmpdir/$output")
    {
        $kernel_level = $output;
    }

    my $csm_moduledir;
    # Define all the modules to insert.  The list depends on the kernel level.
    #my $csm_moduledir	= "/opt/csm/install/drivers/$kernel_level";
    if ( -d "/csminstall/csm/drivers/$kernel_level/$::ARCH" )
    {
		$csm_moduledir= "/csminstall/csm/drivers/$kernel_level/$::ARCH";
    }else{
		$csm_moduledir= "/csminstall/csm/drivers/$kernel_level";
    }
    my @drivers = ("e1000", "bcm5700", "mptscsih", "mptbase", "tg3");
    my @driver_infofiles = ("e1000", "bcm5700", "mpt", "tg3");

    my (@driver_list, @pcitable_list, @modinfo_list, @dep_list);

    # Only add the drivers that actually exist to the @driver_list.
    # Some drivers are only needed for certain kernel levels.
    foreach my $driver (@drivers)
    {
        my $driver = "$csm_moduledir/${driver}.o";
        push(@driver_list, $driver)     if (-f $driver);
    }

    # Only add the drivers' info files that actually exist to the lists.
    # Some drivers are only needed for certain kernel levels.
    foreach my $infofile (@driver_infofiles)
    {
        my $pcitable = "$csm_moduledir/${infofile}.pcitable";
        my $modinfo = "$csm_moduledir/${infofile}.module-info";
        my $dep = "$csm_moduledir/${infofile}.dep";

        push(@pcitable_list, $pcitable) if (-f $pcitable);
        push(@modinfo_list, $modinfo)   if (-f $modinfo);
        push(@dep_list, $dep)           if (-f $dep);
    }


    # Insert the modules into the modules.cgz 
    if (($#driver_list + 1) > 0)
    {
	if (-d "$tmpdir/$kernel_level/$::ARCH")   
	{
        $cmd = "$::COPY -f ". join(' ', @driver_list) ." $tmpdir/$kernel_level/$::ARCH";
	}else{
        $cmd = "$::COPY -f ". join(' ', @driver_list) ." $tmpdir/$kernel_level";
	}
        $output = NodeUtils->runcmd($cmd,0);
    }

    # Remove FAStT modules from the modules.cgz
      if (-d "$tmpdir/$kernel_level/$::ARCH")
      {
        $cmd = "$::RM -f $tmpdir/$kernel_level/$::ARCH/qla*.o";
      }else{
        $cmd = "$::RM -f $tmpdir/$kernel_level/qla*.o";
      }
      $output = NodeUtils->runcmd($cmd,0);

    # Repack the modules.cgz file
    $cmd = "cd $tmpdir; $::FIND $kernel_level -type f -print | " .
           "$::CPIO -o -H crc | " .
           "$::GZIP -c -9 > $tmpdir/modules.cgz";
    $output = NodeUtils->runcmd($cmd,0);

    # Copy modules.cgz back into the ramdisk
    $cmd = "$::COPY -f $tmpdir/modules.cgz $kstmp/modules/modules.cgz";
    $output = NodeUtils->runcmd($cmd,0);

    # Add modules' entries to module-info and pcitable
    if (($#pcitable_list + 1) > 0)
    {
        $cmd = "$::CAT ".join(' ',@pcitable_list)." >> $kstmp/modules/pcitable";
        $output = NodeUtils->runcmd($cmd,0);
    }
    if (($#modinfo_list + 1) > 0)
    {
        $cmd="$::CAT ".join(' ',@modinfo_list)." >> $kstmp/modules/module-info";
        $output = NodeUtils->runcmd($cmd,0);
    }

    if (($#dep_list + 1) > 0)
    {
        $cmd = "$::CAT ". join(' ', @dep_list)." >> $kstmp/modules/modules.dep";
        $output = NodeUtils->runcmd($cmd,0);
    }

    # Copy the drivers into /tmp on the ramdisk to fix a possible bug.
    if (($#driver_list + 1) > 0)
    {
        $cmd = "$::COPY " . join(' ', @driver_list) . " $kstmp/tmp";
        $output = NodeUtils->runcmd($cmd,0);
    }

    # RedHat 7.3 uses the tg3.o driver.  It should be using bcm5700.o
    if ($::DISTRO_VERSION eq "7.3")
    {
	# Modify pcitable to replace all occurences of tg3 with bcm5700
	my $pcitable = "$kstmp/modules/pcitable";
	my $tmp_pcitable = "$tmpdir/pcitable.$$";
	open (PCITABLE, "<$pcitable")
            || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $pcitable);
	open (TMP_PCITABLE, ">$tmp_pcitable")
            || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_WRITE_FILE', $pcitable);
	while (<PCITABLE>)
	{
	    s/tg3/bcm5700/g;
	    print TMP_PCITABLE $_;
	}
	close PCITABLE;
	close TMP_PCITABLE;
	NodeUtils->runcmd ("$::COPY $tmp_pcitable $pcitable");
    }


    #-------------------------------------------------
    # Unmount the ramdisk
    #-------------------------------------------------
    $cmd = "$::UNMOUNT $kstmp";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    #-------------------------------------------------
    # Zip up the ramdisk into <distro_version>.gz (e.g. ks7.3.gz)
    #-------------------------------------------------
    $cmd = "$::GZIP -f $ramdisk";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    # Copy the new ramdisk to /tftpboot
    &copyfile("$ramdisk.gz", $ks_ramdisk);
    InstallUtils->secureFilePermissions("$::TFTPBOOT/csm","0110","0440");

    # Clean up the temporary directory
    rmtree ($tmpdir, $::VERBOSE, 1);

    return $returncode; 
  }


sub makeRAMDisk
{  
  if($bootMethod eq 'getmacs')
  {
    # Make the getmac ramdisk by running mkgetmacrd
    my $cmd1;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','I', 'IMsgCREATING_GETMAC_RAMDISK');
    my $verbose;
    $verbose = ($::VERBOSE ? "-v" : "");
    if (!($verbose)) { $verbose = ($::VERBOSE ? "-V" : ""); }
    $cmd1 = "$::MKGETMACRD $verbose 2>/dev/null";
    print "cmd1\n" if $::VERBOSE;
    system("$cmd1");
    if($? >> 8)
    {
    	NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','I', 'IMsgCreateRAMDISKFAIL');
    }
  }
  elsif($bootMethod eq 'kickstart')
    {
      &makeKickstartRAMDisk();
      &copyDrivers();
    }
#SSSBEGIN
  elsif($bootMethod eq 'autoyast')
    {
      &makeAutoyastRAMDisk();
      #&copyDrivers();	# Maybe we'll need to do this
    }
  elsif($bootMethod eq 'sis')
    {
      &makeSISRAMDisk();
    }
#SSSEND
}

#SSSBEGIN
sub makeAutoyastRAMDisk
{
    &prepare_mount();
    my ($cmd);
    my ($rc) = 0;
    my ($output);

    my ($returncode) = 0;

    my ($initrd) = "$::SUSE_TOP/boot/loader/initrd";
    my ($ramdisk_basename) = "yast-" . $::DISTRO_NAME . $::DISTRO_VERSION . "-" . $::ARCH;	
    my ($yast_ramdisk) = $::TFTPBOOT . "/csm/" . $ramdisk_basename . ".gz";
    # e.g. /tftpboot/csm/yast-SLES8.1-i386.gz

    &copyfile("$initrd", $yast_ramdisk);

    # Just quit for now until we determine if there is anything that needs to
    # go into the ramdisk (drivers, etc.).
    # Maybe there is something that can be added to the ramdisk that will
    # allow rconsole to work during install (termcap?).
    # Some of this code can be copied from makeKickstartRAMDisk.

#ywbegin
    # Need to add drivers ( bcm5700 ) for x335.
    # Copy some code from makeKickstartRAMDisk
      my $tmpdir = "/tmp/yast" . $::DISTRO_VERSION; # e.g. /tmp/yast8.1
      if (-d $tmpdir)
      {
        rmtree ($tmpdir, $::VERBOSE, 1);
      }
      mkpath ($tmpdir, $::VERBOSE, 0755);

    #-------------------------------------------------
    # Unzip the ramdisk
    #-------------------------------------------------
    my $ramdisk = "$tmpdir/$ramdisk_basename";
                                    # e.g. /tmp/yast8.1/yast-SuSE8.1-i386
    $cmd = "$::GUNZIP --quiet -c $initrd > $ramdisk";
    #$cmd = "$::GUNZIP -c $yast_ramdisk > $ramdisk 2> /dev/null";
    # There is a trailing grabage issue ( rc=2 ) when gunzip the initrd file.
    # Don't post any error message for the time being.
    $output = NodeUtils->runcmd($cmd,-1);
    $rc = $::RUNCMD_RC;
    if ($rc != 2 && $rc > 0) {
      NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','I','IMsgShow_Output',$output);
      $returncode = $rc;
    }

    #-------------------------------------------------
    # Mount the ramdisk to yasttmp so it can be modified
    #-------------------------------------------------
    my $yasttmp = $tmpdir."/yasttmp";   # e.g. /tmp/yast8.1/yasttmp
    mkpath ($yasttmp, $::VERBOSE, 0755);
    $cmd = "$::MOUNT -o loop $ramdisk $yasttmp";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    #-------------------------------------------------
    # Create new "info" file in the ramdisk
    #-------------------------------------------------
    my $info = "$yasttmp/info";
    open (INFOFILE, ">$info")
         || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_WRITE_FILE', $info);
    print INFOFILE "Insmod: bcm5700";
    close INFOFILE;

    #-------------------------------------------------
    # Remove FAStT modules from the modules directory
    #-------------------------------------------------
      $cmd = "$::RM -f $yasttmp/modules/qla*.o";
      $output = NodeUtils->runcmd($cmd,0);

    #-------------------------------------------------
    # Unmount the ramdisk
    #-------------------------------------------------
    $cmd = "$::UNMOUNT $yasttmp";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    #-------------------------------------------------
    # Zip up the ramdisk into <distro_version>.gz (e.g. yast-SuSE8.1-i386.gz)
    #-------------------------------------------------
    $cmd = "$::GZIP -f $ramdisk";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    # Copy the new ramdisk to /tftpboot
    &copyfile("$ramdisk.gz", $yast_ramdisk);

    # Clean up the temporary directory
    rmtree ($tmpdir, $::VERBOSE, 1);
#ywend
    return $returncode;
}
#SSSEND

sub makeSISRAMDisk
{
    my ($returncode) = 0;

    my ($initrd) = "/usr/share/systemimager/boot/i386/standard/initrd.img";
    my ($ramdisk_basename) = "sis" . $::DISTRO_VERSION;	
    my ($sis_ramdisk) = $::TFTPBOOT . "/csm/" . $ramdisk_basename . ".gz";

    &copyfile("$initrd", $sis_ramdisk);

    return $returncode;
}

# Copy the drivers (kernel modules) that will be used to update the system
# after it is installed.  The modules are copied to /lib/modules/<kernel>*
# during the kickstart post-install.
sub copyDrivers
{
    my $srcdir = "/opt/csm/install/drivers";
    my $tgtdir = "/csminstall/csm/drivers";
    mkpath ("$tgtdir", $::VERBOSE, 0755);
    my $cmd = "$::COPY -rp $srcdir/* $tgtdir";
    NodeUtils->runcmd("$cmd",0);
    my $rc = $::RUNCMD_RC;

    return ($rc);
}
sub setupPxeHex
{
 
  if($bootMethod eq 'getmacs' )
  {
    my @ips;
    my %extraHex;
    my $file = "/etc/dhcpd.conf";
    my $lnDir = "/tftpboot/pxelinux.cfg";
    my($hostName, $ip) = NodeUtils->getHost(@targetNodes[0]);
    my $firstFile = $hostName . '.getmacs' ;
    my $lnFile = $lnDir . "/" .  $firstFile ; 
    # get all "range" statements in the dhcpd.conf file:
    chomp(my @out = `$::GREP 'range' $file`); 

    # for each "range" entry, get the ip address and put it
    # into a @ip array:
    for (@out)
    {
      s/^\s+//g;
      my $ip = (split(' ', $_))[1];
      $ip =~ s/;//g;
      NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'V','IMsgShow_Output', $ip);
      push @ips, $ip;
    }

    # get the hex representation of these IP addresses and
    # put them in the extraHex hash table:
    %extraHex = InstallUtils->genHex(@ips);
    
    # make a link with each hex to the getmacs file:    
    for(keys %extraHex)
    {
      my $hex = $extraHex{$_};
      NodeUtils->runcmd("$::COPY -f $lnFile $lnDir/$hex");
    }
    InstallUtils->secureFilePermissions($lnDir, '0110','0440');
  }
#SSSBEGIN
  elsif($bootMethod eq 'kickstart' or $bootMethod eq 'autoyast')
#SSSEND
    {
      # Change the pxelinux HEX file to link to <node_hostname>.install
      my @nodes = @targetNodes;

      # Generate a hex representation of each node's ip address.
      my %hex = InstallUtils->genHex(@nodes);

      my $mode = "440";
      my $errors = 0;

      foreach my $node (@nodes)
      {
	# Copy /tftpboot/pxelinux.pxe/node_hostname.install to the file named
	# with the hex representation of the node's ip address.
	my $nodehex = $hex{$node};
        # Use different pxe config file name for different install methods
        my $src = "$::PXELINUX_CFG_DIR/${node}.install";
	my $tgt = "$::PXELINUX_CFG_DIR/$nodehex";
	NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','V', 'IMsgSETTING_PXELINUX_TO_INSTALL', $node);

	# If we do a symlink, the src file gets overwritten when the node 
	# updates the tgt file via tftp.  Therefore, a copy is used instead.
	#NodeUtils->runcmd("$::LN -sf $src $tgt");
	NodeUtils->runcmd("$::COPY $src $tgt");
	if ($::RUNCMD_RC) 
        {
	  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','W', 'EMsgCANT_COPY', $src, $tgt);
         $errors++;
	}
	# secure file:
	InstallUtils->secureFilePermissions($tgt, '0110','0440');
	#NodeUtils->runcmd("$::CHMOD $mode $tgt");
	#if ($::RUNCMD_RC) 
        #{
	#  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
        #       'csminstall', 'W', 'EMsgCANT_CHMOD', $tgt, $mode);
        # $errors++;
	#}
      } # end of for loop
      return $errors;
   }
   elsif($bootMethod eq 'sis' )
   {
      # Change the pxelinux HEX file to link to <node_hostname>.install
      my @nodes = @targetNodes;

      # Generate a hex representation of each node's ip address.
      my %hex = InstallUtils->genHex(@nodes);

      my $mode = "440";
      my $errors = 0;

      foreach my $node (@nodes)
      {
	# Copy /tftpboot/pxelinux.pxe/node_hostname.install to the file named
	# with the hex representation of the node's ip address.
	my $nodehex = $hex{$node};
        # Use different pxe config file name for different install methods
        my $src = "$::PXELINUX_CFG_DIR/${node}.sis";
	my $tgt = "$::PXELINUX_CFG_DIR/$nodehex";
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','V', 'IMsgSETTING_PXELINUX_TO_INSTALL', $node);

	# If we do a symlink, the src file gets overwritten when the node 
	# updates the tgt file via tftp.  Therefore, a copy is used instead.
	#NodeUtils->runcmd("$::LN -sf $src $tgt");
	NodeUtils->runcmd("$::COPY $src $tgt");
	if ($::RUNCMD_RC) 
        {
	  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','W', 'EMsgCANT_COPY', $src, $tgt);
         $errors++;
	}

	InstallUtils->secureFilePermissions($tgt,'0110','0440');
      } # end of for loop
      return $errors;

   }  
}


sub verifyDHCP
{
  if($bootMethod eq 'getmacs' )
  { 
    my @verifiedNodes = @targetNodes;
    my $num = 0;
    my $file = "/etc/dhcpd.conf";
    my @nodeList;
    # make sure the /etc/dhcpd.conf file exists:
    unless(-f $file)
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E1', 'EMsgDOESNT_EXIST',$file);
    }

    # Make sure that each node that we are going to process
    # has an entry in the /etc/dhcpd.conf file in the dynamic
    # entry section. 

    foreach my $node (@verifiedNodes)
    {
      # get the ip address of the node:
      my(undef,$ipaddr) = NodeUtils->getHost($node);

      # check if it's in the dhcpd.conf file:
      my $out = NodeUtils->runcmd("$::GREP 'range $ipaddr' $file", -1);

      # if it is in the dhcpd.conf file, then put it into the @::nodes.
      # @::nodes is the list that will be processed:
      if($out)
      { 
	$num = 1;;
	push @nodeList, $node;

	# if it's not on the dhcpd.conf file, then we signal an error:
	# this node will not be processed.
      }
      else
      { 
        NodeUtils->messageFromCat('W','EMsgNO_DHCP_ENTRY',$node);
      }
   } # end of for  
   @targetNodes = @nodeList ;
 }
  elsif($bootMethod eq 'kickstart') # || $bootMethod eq 'sis' )
  {
   
    my $rc = InstallUtils->verify_dhcp();
    ($rc) && NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall',"E$rc", 'EMsgCANT_START_DHCP');
    return $rc;
  }
}


sub verifyPxe
{
  if($bootMethod eq 'getmacs' )
  {
    my @nodes = @targetNodes ;

    my @verifiedNodes;
    my $pxefile;
    my $return_value = 0; # assume failure
    my @hasSerialDeviceNodes;
    my $out1=0;
    my $out2=0;

    # check that the pxelinux.cfg dir is defined:
    unless( -d $::PXELINUXCFG ){
        NodeUtils->runcmd("$::MKDIR/$::PXELINUXCFG");
    }
    InstallUtils->secureFilePermissions($::PXELINUXCFG,'0110','0440');
    # don't add the nodes that has no console serial device defined,
    # like console=ttyS0,9600 or console=ttyS1,9600, so getmacs won't process
    # such nodes.
    foreach my $node (@nodes){

        # Find the pxe file for this node
        # If it does not exist, assume we are running with early CSM 1.2 files
        # and try using getmacs.pxe
        my ($nodename, $nodeip) = NodeUtils->getHost($node);
        $pxefile = "$::PXELINUXCFG/$nodename.getmacs";
        unless( -f $pxefile ){
            unless( -f $::OLDGETMACSPXE ){
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E','EMsgFILE_NOT_FOUND', $pxefile );
	        return (0); # failure
	    }
	    $pxefile = $::OLDGETMACSPXE;
        }
	# open the PXEFILE
        open (PXEFILE, "<$pxefile")
            || NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E2', 'EMsgCANT_READ_FILE', $pxefile);
	# go through and see if it has a serial port. 
        while (<PXEFILE>){
            if( /SERIAL 0 9600/  || /SERIAL 1 9600/ || /SERIAL 2 9600/  || /SERIAL 3 9600/  ){
                 $out1 = 1;
            }        
	}

        if ( $out1){
            # this node has serial device defined
            push @hasSerialDeviceNodes, $node;
        }else{
            #print "No Console serial device defined. $node will not be processed. \n";
	}
        close PXEFILE;
	InstallUtils->secureFilePermissions($pxefile,'0110','0440');
        $out1=0;        
    }
    @nodes = @hasSerialDeviceNodes; 

       
      

    # Use the pxe file for the first node
    # If it does not exist, assume we are running with early CSM 1.2 files
    # and try using getmacs.pxe
    my ($nodename, $nodeip) = NodeUtils->getHost($nodes[0]);
    $::GETMACSPXE = "$::PXELINUXCFG/$nodename.getmacs";
    unless( -f $::GETMACSPXE ){
        unless( -f $::OLDGETMACSPXE ){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E','EMsgFILE_NOT_FOUND', $::GETMACSPXE );
	    return (0);  # failure
        }
        $::GETMACSPXE = $::OLDGETMACSPXE;
    }


    foreach my $node (@nodes){

        # Find the pxe file for this node
        # If it does not exist, assume we are running with early CSM 1.2 files
        # and try using getmacs.pxe
        ($nodename, $nodeip) = NodeUtils->getHost($node);
        $pxefile = "$::PXELINUXCFG/$nodename.getmacs";
        unless( -f $pxefile ){
            unless( -f $::OLDGETMACSPXE ){
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E','EMsgFILE_NOT_FOUND', $pxefile );
	        return (0); # failure
	    }
	    $pxefile = $::OLDGETMACSPXE;
        }

        # Check if this node's pxe file matches the first node's file
        # If it does, add it to the active node list
	my $out = NodeUtils->runcmd("$::DIFF -Bwq $::GETMACSPXE $pxefile", -1);
 	if($out){ 
        #  need a new error msg!!!!
        #   NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
        #        'csminstall','W','EMsgXXXXX',$pxefile,$::GETMACSPXE,$node)
		print "$pxefile is different from $::GETMACSPXE.  $node will not be processed. \n";
        }else{
	   $return_value = 1;
	   push @verifiedNodes, $node;
        }

    }
    @targetNodes = @verifiedNodes;
    return $return_value;
  }
  elsif($bootMethod eq 'kickstart' || $bootMethod eq 'sis' )
  {
  }

}

sub verifyKernel
{
  if($bootMethod eq 'getmacs' )
  {
    my $getmacs_kernel = "$::TFTPBOOT/csm/getmacz" ;
    unless( -f $getmacs_kernel )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $getmacs_kernel );
    }
  }
  elsif($bootMethod eq 'kickstart' )
  {
    my $ks_kernel = $::TFTPBOOT . "/csm/ks" . $::DISTRO_VERSION . "z";
    unless( -f $ks_kernel )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $ks_kernel );
    }
#SSSBEGIN
  } elsif($bootMethod eq 'autoyast' )
  {
    my ($kernel_basename) = "yast-" . $::DISTRO_NAME . $::DISTRO_VERSION . "-" . $::ARCH;	# e.g. yast-SLES8.1-i386
    my $yast_kernel = $::TFTPBOOT . "/csm/" . $kernel_basename . "z";
    unless( -f $yast_kernel )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $yast_kernel );
    }
#SSSEND
  }elsif($bootMethod eq 'sis' )
  {
    my $sis_kernel = $::TFTPBOOT . "/csm/sis" . $::DISTRO_VERSION . "z";
    unless( -f $sis_kernel )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $sis_kernel );
    }
  }

}

sub verifyRAMDisk
{
  if($bootMethod eq 'getmacs' )
  {
    unless( -f $::GETMACZ )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $::GETMACZ);
    }
  }
  elsif($bootMethod eq 'kickstart' )
  {
    my ($ramdisk_basename) = "ks" . $::DISTRO_VERSION;
    my ($ks_ramdisk) = $::TFTPBOOT . "/csm/" . $ramdisk_basename . ".gz";
    unless( -f $ks_ramdisk )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $ks_ramdisk);
    }
  }
  elsif($bootMethod eq 'sis' )
  {
    my ($ramdisk_basename) = "sis" . $::DISTRO_VERSION;	
    my ($sis_ramdisk) = $::TFTPBOOT . "/csm/" . $ramdisk_basename . ".gz" ;
    unless( -f $sis_ramdisk )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $sis_ramdisk);
    }
  }
#SSSBEGIN
  elsif($bootMethod eq 'autoyast' )
  {
    my ($ramdisk_basename) = "yast-" . $::DISTRO_NAME . $::DISTRO_VERSION . "-" . $::ARCH;	# e.g. yast-SLES8.1-i386
    my ($autoyast_ramdisk) = $::TFTPBOOT . "/csm/" . $ramdisk_basename . ".gz" ;
    unless( -f $autoyast_ramdisk )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $autoyast_ramdisk);
    }
  }
#SSSEND
  
}
sub verifyTFTP
{
  my $rc =0;
  if( $bootMethod eq 'getmacs' || $bootMethod eq 'kickstart' || $bootMethod eq 'sis' )
  {
    # 1 means its running, 0 means its not running.
    $rc = InstallUtils->verifyTftp();
    unless($rc){ NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall',"E1", 'EMsgCANT_START_TFTP');
    }
  }
}

sub verifyOthers
{
  if($bootMethod eq 'getmacs')
  {
  }
  elsif($bootMethod eq 'kickstart' || $bootMethod eq 'sis' )
  {
     #&verify_systemimager();
  }

}


# Verify the systemimager daemon to ensure the rsyncd is running
sub verify_systemimager
{
    my ($InstallMethod);
    &loadNodeAttributes();
	# Find whether there are some nodes use "sis" InstallMethod
    foreach my $node (@targetNodes)
    {
      $InstallMethod = $nodeHash{$node}{'InstallMethod'};
       if ( $InstallMethod =~ "sis" )
        {
       	  my $cmd = "/etc/init.d/systemimager start";
	  my $rc = NodeUtils->runcmd($cmd, 0);
	  return $::RUNCMD_RC;
        }
    }
    
	return 0;
}

sub setBootLocation
{
  #TBD
}
# Get the hostname of the management server for a node.
#
# Since the nodes might need to contact the management server through a
# different address than its primary hostname, we cannot just use the output
# of the hostname command here.
#
# If we cannot contact the node then return the output of the hostname command
# as a backup.
#
sub get_node_management_server
{
    my ($nodename) = @_;

    my ($ms_hostname, $ms_ipaddr);
    $ms_hostname = $nodeHash{$nodename}{"ManagementServer"};
    if ($ms_hostname)
    {
	($ms_hostname, $ms_ipaddr) = NodeUtils->getHost($ms_hostname);
    }
    else
    {
	($ms_hostname, $ms_ipaddr) = 
			    InstallUtils->get_management_server($nodename);
    }

    # If the management server cannot be resolved, just use the output
    # of the hostname command on the management server.  This is done because
    # we require some management server hostname, even if the node cannot
    # be resolved.
    if (! $ms_hostname)
    {
	($ms_hostname, $ms_ipaddr) = &get_management_server_by_hostname();
    }

    return ($ms_hostname, $ms_ipaddr);
}

sub get_management_server_by_hostname
{
    my $h = `hostname`;
    chomp $h;
    my ($hostname, $ipaddr) = NodeUtils->getHost($h);
    chomp $hostname;
    chomp $ipaddr;
    return ($hostname, $ipaddr);
}

# Get the hostname of the management server.
#
# Since the nodes might need to contact the management server through a
# different address than its primary hostname, we cannot just use the output
# of the hostname command here.
#
# On Linux, the ping command outputs a line like the following which is
# parsed to find the originating hostname (m.m.m.m = management server IP addr).
#   PING node_hostname (n.n.n.n) from m.m.m.m : 56(84) bytes of data.
#
# On AIX, the traceroute command outputs a line like the following which is
# parsed to find the originating hostname (m.m.m.m = management server IP addr).
#   source should be m.m.m.m
# (This is not implemented for AIX yet!!!)
#
# If we cannot contact the node via ping (for Linux) or traceroute (for AIX),
# just return nothing.  This indicates that there is no way to get to the node.
#
sub get_management_server
{
    my ($node_hostname) = @_;
    my $ipaddr   = "";
    my $ping_searchstring1 = "\^PING ";
    my $ping_searchstring2 = " from ";
    my $cmd = "$::PING -c1 $node_hostname 2>/dev/null";
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','V', 'IMsgCMD', $cmd);
    open (CMD, "$cmd |")
	|| NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','W', 'EMsgCANT_RUN_CMD', $cmd);
    while (<CMD>)
    {
	# Find the correct line in the ping output
	if (/$ping_searchstring1/)
	{
	    my $line = $_;
	    # Split the line around the "from".
	    my ($before, $after) = split ($ping_searchstring2, $line);
	    # Find the first word after the "from".
	    ($ipaddr) = (split(' ', $after))[0];
	}
    }

    if (!$ipaddr)
    {
	my $ms_hostname;
	($ms_hostname, $ipaddr) = &get_management_server_by_hostname;
    }

    return NodeUtils->getHost($ipaddr);
}

sub getCommonAttributes
{
# Get the attributes that are common across all nodes.
($::DISTRO_NAME,
 $::DISTRO_VERSION, 
 $::ARCH, 
 $::CSM_VERSION) = InstallUtils->get_common_attrs();
}


#prepare_mount sub-routine will check if the director/tmp/... is mounted.
#If mounted, then it will unmount.
sub prepare_mount
{
   &getCommonAttributes();
   my ($cmd, @my_result, $cnt);
#SSSBEGIN
   my ($tmpdir, $check_val);
   if ($bootMethod eq 'autoyast') {
     $tmpdir="/tmp/yast" . $::DISTRO_VERSION;  # e.g. /tmp/yast8.1
     $check_val=$tmpdir."/yasttmp";
   }
   elsif ($bootMethod eq 'kickstart') {
     $tmpdir="/tmp/ks" . $::DISTRO_VERSION;  # e.g. /tmp/ks7.3
     $check_val=$tmpdir."/kstmp";
   }
#SSSEND
   $cmd="$::MOUNT | grep $check_val";
   @my_result=NodeUtils->runcmd($cmd,-1);
   chomp (@my_result);
   $cnt=@my_result;
   if ($cnt > 0)
   {
     my $i;
     for($i=1; $i<=$cnt;$i++)
     {
        $cmd = "$::UNMOUNT $check_val";
        NodeUtils->runcmd($cmd);
     }
   }
}

sub loadNodeAttributes
{
   my @nodeList = @inputNodes;
    # Process the various ways of inputting the list of nodes.  
    my ($ref_DestNode, $ref_lsnode_info, $ref_DestNodeHash) 
		= InstallUtils->get_target_nodes(\@nodeList);
    @targetNodes = @$ref_DestNode;
    @nodeInfo = @$ref_lsnode_info;
    %nodeHash = %$ref_DestNodeHash;
    #@::NODELIST = @$ref_DestNode;
    #@::LSNODE_INFO = @$ref_lsnode_info;
    %::NODEHASH = %$ref_DestNodeHash;
    if(!@targetNodes)
    {
	#NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
       #        'csminstall','E1', 'EMsgINSTALLNODE_NO_TARGET');
    }

} 

#Takes a list of nodes and checks for the InstallAdapterMacaddr attribute
#value. Returns a list with references to both a list of nodes with MAC address  and
#to the list without MAC address.
sub checkMACAddress
{
     shift;
    my @nodeList = @_;
    # First, find the nodes whose InstallAdapterMacaddr attribute
    # is  already set.
    my ($ref_DestNode, $ref_lsnode_info, $ref_DestNodeHash) 
		= InstallUtils->get_target_nodes(\@nodeList);
    my %latestNodeHash = %$ref_DestNodeHash;
    my @hasMACList=();
    my @noMACList=();
    my @result=( \@hasMACList , \@noMACList );
    my $InstallAdapterMacaddr;
    foreach my $hostname (keys %latestNodeHash)
    {
        $InstallAdapterMacaddr =
                $latestNodeHash{$hostname}{'InstallAdapterMacaddr'};
        if ( ($InstallAdapterMacaddr =~ /(..:..:..:..:..:..)/ ))
        {
            push (@hasMACList, $hostname);
        }
        else
        {
            push (@noMACList, $hostname);
        }
   }
   #return the node list for which for which we have MAC address.
   return @result ;
  
}
sub setupBoot
{
  shift(@_) ;
  my($boot, $hexOption, @nodeList) = @_ ;
  #SSS my @commonBootMethods = ("getmacs", "kickstart", "sis");
  my @commonBootMethods = ("getmacs", "kickstart", "sis", "autoyast");
  my $rc=0;
  $bootMethod = $boot ;
  # print("BOOT_METHOD:,$bootMethod\n");

  $hexFileOption = $hexOption;
  #print("HEX OPTION:,$hexFileOption\n");

  @inputNodes = @nodeList ; #("c5bn03.ppd.pok.ibm.com");
  #print("NODE LIST:@nodeList\n");
  if(! (@inputNodes))
  {
    #print("No Nodes to process\n");
    return (());
  }

  &setAttributes();
  &loadNodeAttributes();
  &getCommonAttributes();

#Begin add for ppc64
  if ($::ARCH eq "ppc64") {
  	$::BOOT_EXIT=&doPPC64Setup();
  	return(@targetNodes);
  }
#End

  #Disk boot setup
  if($bootMethod eq "disk" )
  {
    &setupDiskBoot();
  }

  foreach my $bootMethod (@commonBootMethods)
  {
    if($bootMethod =~ /\b$bootMethod\b/i )
    {
       if($hexFileOption eq "yes"  )
       {
         $::BOOT_EXIT = &doFullSetup();
         return (@targetNodes);
       }
       elsif($hexFileOption eq "no"  )
       {
         #print("Before doFullSetupGenerateNoHex...\n");
         $::BOOT_EXIT = &doFullSetupGenerateNoHex();
         return (@targetNodes);
       }
       elsif($hexFileOption eq "onlyhex"  )
       {
	  #print(" before doSetupgenerateOnlyhex..\n");
           $::BOOT_EXIT = &doSetupGenerateOnlyHex();
           return (@targetNodes);
       }
     }
  }
  return (@targetNodes);
}
1;














