# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2003 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
package DshUtils;

#use strict;
use locale;

use dshNodeUtils;

###############################################################################################
#
# The DshUtils package supports several additional dsh features. These features can be invoked
# by using the following command line options and environment variables. 
# 
# Environment variables:
#
# DSH_REPORTS_DIRECTORY 
#     When set to the reports directory path, reporting and output file writing is enabled. 
#     Overridden by the --reports option and the --noReporting option.
# 
# DSH_LOG
#     When set to the absolute path and filename of the log file, logging is enabled.
#     Overridden by the --log option and the --noLogging option.
#
# Options:
# 
# -R | --reports <directory>  
#     Sets the reports directory for reporting. All output from each host is saved to
#     individual output and error files and a report is generated if this option is
#     used. A unique sub directory is created to hold the output files and the report 
#     specific to a run. The sub directory name is created using the default 'unspecified'
#     or the string provided using the --reportName option. An integer extension is 
#     added to the name to create a unique directory name. 
#     Ex. unspecified.0001  lscommand.0001
#     This extension ranges from .0001 to .9999 for each unique report name. When 
#     .9999 is reached, directories must be deleted before more reports can be generated.
#     The option overrides the DSH_REPORTS_DIRECTORY environment variable.
# 
# --log <absolute path + filename>
#     Sets the log file. Log entries will be added for each dsh run when this option
#     is used. This options overrides the DSH_LOG environment variable.
# 
# -W | --noFileWriting
#     This option disables output file writing for a run when used with the --reports
#     option or the DSH_REPORTS_DIRECTORY is set. Reporting remains enabled.
#     Required when using reporting with dshbak.
# 
# Unsupported/undocumented options:  
#
# --commandName <name>
#     Sets the command name associated with command to be run.
#     
# --commandDescription <description>
#     Sets a text description of the command as part of the report.
# 
# --reportName <name>
#     Sets the name used to create a unique report directory name and the html report file name.
#
# --version
#     Prints monitoring level and remote shell path.
#
# --lastRun <# of commands> 
#     Displays a list of the last commands run using dsh. Only commands that have
#     been logged will appear. Most recent command names appear first, each followed
#     by its execution start time and end time. The --log option or the DSH_LOG
#     environment variable are required with this option.
# 
# --noReporting
#     This option disables reporting and output file writing. Overrides the 
#     DSH_REPORTS_DIRECTORY environment variable.
# 
# --noLogging
#     This option disables logging. Overrides the DSH_LOG environment variable.
#
#
###############################################################################################
#
#            Initialization subroutines - public
#
###############################################################################################

# Args: ($logPath)
sub initializeSummaryLogging;

# TODO verify fanout here
# return updated fanout
# use getrlimit
# returns fanout
# Args: ($reportsDirectory, $fanout, $reportName)
sub initializeFileWriting; 

# Args: ($reportsDirectory, $reportName)
sub initializeReporting; 

# Args: none
sub initializeMonitoring;

###############################################################################################
#
#            Command Info update subroutines - public
#
###############################################################################################

# Args: $commandName
sub setCommandName;

# Args: $commandDescription
sub setCommandDescription;

# Args: $runAsUser - must be valid
sub setLogin;

# Args: $streamingRequested
sub setStreaming;

# Args: $verifyRequested
sub setVerify;

# Args: $timeout
sub setTimeout;

# Args: $fanout
sub setFanout;

# Args: $remoteShellPath
sub setRemoteShellPath;

# Args: $remoteShellOptions
sub setRemoteShellOptions;

# Args: comma separated string of host names
sub setHosts;

# Args: comma separated string of group names
sub setGroups;
       
# Passes the remote command which may include an export PATH statement, this statement is parsed
# to include as a separate field in the report and log. 
# Args: ($command)
sub setCommand;

###############################################################################################
#
#            General message handling subroutines - public
#
###############################################################################################

# Info messages are sent to the console/monitoring, log status if logging, and report status
# if reporting
sub handleInfoMessage;

# Non-fatal error messages are sent to the console/monitoring, log status if logging, and report status
# if reporting
sub handleErrorMessage;

###############################################################################################
#
#            Execution state update subroutines - public
#
###############################################################################################

sub setStartTime;

# DshUtils does not try to report or log until this state is reached
sub setInitializationCompleted; # ();  

sub setVerifyingHosts; # ();

sub setRemoteExecutionStarted; # (); 

# A host status list is set to pending for all hosts in @nl. This list already contains any hosts that were 
# removed from @nl as a result of the get_nl method call. The handleConnectionFailure and handleExecutionError
# calls update the status list to EXECUTION_FAILED. Calls to these methods before setPendingHosts, add the 
# appropriate entries to $executionStatus. The setPendingHosts() call is made after the get_nl call because
# the hostnames in @nl need to match the user specified  host names before setPendingHosts() is called.
# Pre: @nl contains a list of the user specified host names. Any hosts that were removed from this list
#      must have invoked handleConnectionFailure or handleExecutionError in order to have the hosts status
#      entered into $executionStatus. 
#      $executionStatus{hosts that had an error before setPendingHosts invoked} = EXECUTION_FAILED 
# Post: $executionStatus contains all hosts specified by the user. 
#       $executionStatus{hosts in @nl} = EXECUTION_PENDING
#       $executionStatus{hosts that had an error before setPendingHosts invoked} = EXECUTION_FAILED 
sub setPendingHosts; # ($hosts_ref);  

sub setRemoteCommandStarted; # ($hostname);

sub setRemoteReturnCode; # ($hostname, $rc);

sub setRemoteCommandCompleted; # ($hostname);

###############################################################################################
#
#            Per Host Output file writing subroutines - public
#
###############################################################################################

sub appendOutputLines; # ($hostname, $last_ind, $lines_ref);
    
sub appendOutputLine; # ($hostname, $line_ref);
 
sub appendAllOutputLines; # ($hostname, $lines_ref); 

sub appendErrorLines; # ($hostname, $last_ind, $lines_ref);

sub appendErrorLine; # ($hostname, $line_ref); 

sub appendAllErrorLines; # ($hostname, $lines_ref); 

sub appendErrorMessage; # ($message_ref, $hostname); 

# Arg: scalar hostname
sub closeOutputFile; # ($myHostName); 

# Arg: scalar hostname
sub closeErrorFile; # ($myHostName);

###############################################################################################
#
#            Dsh Per Host Error handling subroutines - public
#
###############################################################################################


# host specific error msg, non-fatal, we don't exit because of this message
sub handleConnectionFailure; # ($messageName, $hostName);

# host specific error msg, non-fatal, we don't exit because of this message
sub handleExecutionError; # ($messageName, $hostName); 

# host specific error msg, non-fatal, we don't exit because of this message
sub handleExecutionCancelled; # ($messageName, $hostName);

# host specific error msg, non-fatal, we don't exit because of this message
sub handleTimeoutError;

###############################################################################################
#
#            Exit subroutines - public
#
###############################################################################################
   
sub isExitPending; # ();

sub setExitPending; # ();

# Method handles non-host-specific error messages that require dsh parent process to exit.
# If $remoteExecutionStarted is false, then it is safe to log and report since no child 
# processes have started.
# If $remoteExecutionStarted is true, then we are not in a safe place to exit because
# child processes may still be running. We have to add appropriate messages to 
# output files, report and log that describe the indeterminate state we are in when
# the fatal error occurred. Some child processes may have time to finish while this routine is running,
# the parent process will never read anymore output so the execution status variables will not 
# change once we are in this method.
# Pre: handleFatalError() should not be called by dsh until after $initCompleted 
#      Call only from parent dsh process.
#      Call only after initialization completed because we don't need to log or report until after.
# $messageName is the bundle name for the message
# @msgArgs is a list of strings to be substituted into the message
# Post:  logging and reporting will be completed.
sub handleFatalError; # ($messageName, @msgArgs);

sub handleStop; # ($messageName, @msgArgs); 

sub handleExit; # ();

# Lists the last command or commands run, includes command name, start and end time 
# Arg: number of commands to list
sub handleLastCommandsRun;

###############################################################################################
#
#
#                         Private Implementation
#
#
###############################################################################################


###############################################################################################
#
#            Global constants - private
#
###############################################################################################

#use constant {
#    TRUE  => 1,
#    FALSE => 0
#};

# target execution states
#use constant {
#    EXECUTION_PENDING        => 0,
#    EXECUTION_STARTED        => 1,
#    EXECUTION_SUCCESSFUL     => 2,
#    EXECUTION_FAILED         => 3,
#    EXECUTION_CANCELLED      => 4
#};
my ($EXECUTION_PENDING) = 0;
my ($EXECUTION_STARTED) = 1;
my ($EXECUTION_SUCCESSFUL) = 2;
my ($EXECUTION_FAILED) = 3;
my ($EXECUTION_CANCELLED) = 4;

# initialized in initializeOutputFileWriting subroutine
my ($OUTPUT_FILE_EXTENSION) = 0;
my ($ERROR_FILE_EXTENSION)  = 0;

my ($fileWritingRequested) = 0;
my ($monitoringRequested) = 0;
my ($loggingRequested) = 0;
my ($reportingRequested) = 0;

# directory, path and fileName variables
my ($logPath) = 0;            # the absolute path and file name of the summary logfile
my ($reportsDirectory) = 0;   # the base reports directory, absolute path, contains index file
my ($reportName) = 0;         # the report sub directory prefix and report html file prefix 
my ($outputDirectory) = 0;    # the subdirectory under reports directory that contains output and report file

#######################
# Command Info data
#######################

my ($commandName);               
my ($commandDescription);

my (@commands);              # array of commands forms command definition
my ($commandDefinition);     # string of commands including \n and spaces
my ($commandPath);                             

my ($runAsUser);

my ($startTime);
my ($endTime);

# target info
my (@hostNames);
my (@groupNames);

# dsh options
my ($verifyRequested);             
my ($streamingRequested);  
my ($timeout);
my ($fanout);

# remote shell options      
my ($remoteShellPath);       # absolute path and filename of remote shell
my ($remoteShellOptions);


# Report state variables
my (@hostStatus);	     # 2 dimensional array to contain status of all the hosts
my ($hostStatusUpdated) = 0;

# Report documents
my ($prolog);
my ($sectionLinks);
my ($cmdExecutionResults);
my ($cmdInfo);
my ($miscInfo);
my ($endHTML);


###############################################################################################
#
#            Program Flow State Variables - private
#
###############################################################################################

my ($initCompleted) = 0; # FALSE            # set to true when dsh arguments have been initialized.
my ($targetUpdatesCompleted) = 0; # FALSE   # set to true when hosts, groups and pending hosts have been set
my ($remoteExecutionStarted) = 0; # FALSE   # set to true before processes begin to fork.
my ($exitPending)  = 0; # FALSE
#my ($exitStarted) = 0;  


###############################################################################################
#
#            Execution Status State Variables - private
#
###############################################################################################

my ($writingFiles) = 1; # TRUE

##
## An entry is added to the array 'outputFiles' when an output file is created for a specific host.
## The key is the host name. The value is 0 if the file has been created but is currently closed.
## The value is the file handle for the file if the file is currently open.
##
my (%outputFiles); 

##
## An entry is added to the array 'errorFiles' when an error file is created for a specific host.
## The key is the host name. The value is 0 if the file has been created but is currently closed.
## The value is the file handle for the file if the file is currently open.
##         
my (%errorFiles);    

   
my (%executionStatus);  # EXECUTION_PENDING, EXECUTION_STARTED, EXECUTION_SUCCESSFUL, FAILED, CANCELLED


##
## Temporarily stores the rc for a host. The entry is undefined after it is evaluated
## by the setRemoteCommandCompleted method
##
my (%remoteRC); 

my (@reportStatusMessages);
my (@logStatusMessages);


###############################################################################################
#
#            Misc. subroutines - private
#
###############################################################################################

#
# Name: printLine
# Description: Writes a line of text to a file.  If it fails it will call
#              the error routine.
# Args:  fp            - file pointer to print out the message
#        msg           - msg to be printed
#        error_routine - function to call if it fails to write to log
# Returns:  TRUE if successful writing to file
#           FALSE if it fails
#
sub printLine
{
    my ($fp) = $_[0];      # file pointer to print out the message
    my ($func) = $_[2];    # function to call if it fails to write to log

    $! = 0;
    # try to print it the file handler
    printf ( $fp "%s", "$_[1]");
#printf (STDOUT "%s", "$_[1]");
    # if it fails call the error routine
    if ($!) {
        &$func();
        return 0; # FALSE # if write error, the output is not sent to console instead
    }

    return 1; # TRUE
} # end printLine


sub executionStatusCompleted
{   
    my ($hostName) = @_;
    return ($executionStatus{$hostName} == $EXECUTION_SUCCESSFUL) ||
        ($executionStatus{$hostName} == $EXECUTION_FAILED) ||
        ($executionStatus{$hostName} == $EXECUTION_CANCELLED);
}

###############################################################################################
#
#            Console message subroutines - private
#
###############################################################################################

# This method should be used for sending stdout messages to the shell console or dcem console.
sub sendInfoMessage {
    my ($messageName, @msgArgs) = @_;

    if ($monitoringRequested) {
        print STDOUT "dsh>  Dsh_begin_info_message\n";
        NodeUtils->message('I', $messageName, @msgArgs);
        print STDOUT "dsh>  Dsh_end_info_message\n";
    } else {        
        NodeUtils->message('I', $messageName, @msgArgs);
    }
}

# This method should be used for sending stderr messages to the shell console or dcem console.
sub sendErrorMessage {
    my ($messageName, @msgArgs) = @_;

    if ($monitoringRequested) {
        print STDERR "dsh:  Dsh_begin_error_message\n";
        NodeUtils->message('E', $messageName, @msgArgs);
        print STDERR "dsh:  Dsh_end_error_message\n";
    } else {   
        NodeUtils->message('E', $messageName, @msgArgs);
    }
}

###############################################################################################
#
#            Application monitoring subroutines - private
#
###############################################################################################

# When the client receives this message, it can expect reporting and logging to occur when
# it tries to stop dsh. Before this message is recieved, no reporting or logging occurs.
sub sendInitializationCompletedMessage {
    print STDOUT "dsh>  Dsh_initialization_completed\n";
}

sub sendVerifyingHostsMessage {
    print STDOUT "dsh>  Dsh_verifying_hosts\n";
}

sub sendRemoteExecutionStartedMessage {
    print STDOUT "dsh>  Dsh_remote_execution_started\n";
}

sub sendRemoteCommandStartedMessage {
    my ($hostname) = @_;
    print STDOUT "dsh>  Remote_command_started $hostname\n";
}

#
# Name: sendOUtputFileCreatedMessage
# Description: A monitoring message is sent on the STDOUT stream
# when a new output file for a specific host is created. When the 
# monitoring application receives this message it is informed that 
# there is viewable output waiting.
# Args:  hostname - host name of the for which the output file belongs
#        filename - absolute path + filename of the the output file
# Returns:  nothing
#
sub sendOutputFileCreatedMessage {
    my ($hostname, $filename) = @_;
    print STDOUT "dsh>  Output_file_created $hostname $filename\n";
}

#
# Name: sendErrorFileCreatedMessage
# Description: A monitoring message is sent on the STDOUT stream
# when a new error file for a specific host is created. When the 
# monitoring application receives this message it is informed that 
# there is viewable output waiting.
# Args:  hostname - host name of the for which the error file belongs
#        filename - absolute path + filename of the the error file
# Returns:  nothing
#
sub sendErrorFileCreatedMessage {
    my ($hostname, $filename) = @_;
    print STDOUT "dsh>  Error_file_created $hostname $filename\n";
}

sub sendSuccessfulMessage {
    my ($hostname) = @_;
    print STDOUT "dsh>  Remote_command_successful $hostname\n";
}

sub sendFailedMessage {
    my ($hostname) = @_;
    print STDOUT "dsh>  Remote_command_failed $hostname\n";
}

sub sendCancelledMessage {
    my ($hostname) = @_;
    print STDOUT "dsh>  Remote_command_cancelled $hostname\n";
}

sub sendReportGeneratedMessage {
    my ($reportPath) = @_;
    print STDOUT "dsh>  Report_generated $reportsDirectory/index.html $reportPath\n";
}

sub sendLogUpdatedMessage {
    print STDOUT "dsh>  Log_updated $logPath\n";
}

sub sendRemoteExecutionCompletedMessage {
    print STDOUT "dsh>  Dsh_remote_execution_completed\n";
}

###############################################################################################
#
#            Reporting subroutines - private
#
###############################################################################################

sub reportInfoMessage  {
    my ($messageName, @msgArgs) = @_;

    push (@reportStatusMessages, NodeUtils->getMessage($messageName, @msgArgs));
}

sub reportErrorMessage  {
    my ($messageName, @msgArgs) = @_;

    push (@reportStatusMessages, NodeUtils->getMessage($messageName, "dsh", @msgArgs));
}

sub updateHostStatus {
    #
    # Fill out the hostStatus.  This will be a 2 dimensional array keyed off the
    # execution status of the machines.  Ex:
    #  
    #    hostStatus = [ 
    #			[[$EXECUTION_SUCCESSFUL], [host1, host5]],
    #			[[$EXECUTION_FAILED],      [host2]],
    #			[[$EXECUTION_CANCELLED],   [host3, host54]
    #                 ]
    #
    while(($key,$value)=each %executionStatus){
#print "key, value = ($key, $value)\n";
        push @{$hostStatus[$value]}, $key;
    }

  
    $hostStatusUpdated = 1;
}

sub initializeReportDocuments {

    $LANGUAGE = $ENV{'LANG'};
    $LISTTITLE = NodeUtils->getMessage("IMsgDSHcliReportListTitle");
    $FANOUT = NodeUtils->getMessage("IMsgDSHcliReportFanout");
    $CMDEXECSTATUS = NodeUtils->getMessage("IMsgDSHcliReportCmdExecStatus");
    $CMDHOSTRESULTS = NodeUtils->getMessage("IMsgDSHcliReportCmdHostResults");
    $CMDINFO = NodeUtils->getMessage("IMsgDSHcliReportCmdInfo");
    $CMDOPTIONS = NodeUtils->getMessage("IMsgDSHcliReportCmdOptions");
    $COMMAND = NodeUtils->getMessage("IMsgDSHcliReportCommand");
    $FAILEDHOSTS = NodeUtils->getMessage("IMsgDSHcliReportFailedHosts");
    $SUCCESSFULHOSTS = NodeUtils->getMessage("IMsgDSHcliReportSuccessfulHosts");
    $CANCELLEDHOSTS = NodeUtils->getMessage("IMsgDSHcliReportCancelledHosts");
    $STATUS = NodeUtils->getMessage("IMsgDSHcliReportStatus");
    $NAME = NodeUtils->getMessage("IMsgDSHcliReportName");
    $CMDDEFINITION = NodeUtils->getMessage("IMsgDSHcliReportCommandDefinition");
    $RUNASUSER = NodeUtils->getMessage("IMsgDSHcliReportRunAsUser");
    $DESCRIPTION = NodeUtils->getMessage("IMsgDSHcliReportDescription");
    $RUNHERE = NodeUtils->getMessage("IMsgDSHcliReportRunHere");
    $HOSTNAMES = NodeUtils->getMessage("IMsgDSHcliReportHostNames");
    $GROUPSOFHOSTS = NodeUtils->getMessage("IMsgDSHcliReportGroupsOfHosts");
    $HOSTWILLRESPOND = NodeUtils->getMessage("IMsgDSHcliReportHostWillRespond");
    $STREAMOUTPUT = NodeUtils->getMessage("IMsgDSHcliReportStreamOutput");
    $TIMEOUT = NodeUtils->getMessage("IMsgDSHcliReportTimeout");
    $GENERATEFILES = NodeUtils->getMessage("IMsgDSHcliReportGenerateFiles");
    $REMOTESHELL = NodeUtils->getMessage("IMsgDSHcliReportRemoteShell");
    $REMOTESHELLOPTIONS = NodeUtils->getMessage("IMsgDSHcliReportRemoteShellOptions");
    $STARTED = NodeUtils->getMessage("IMsgDSHcliReportStarted");
    $ENDED = NodeUtils->getMessage("IMsgDSHcliReportEnded");
    $CMDPATH = NodeUtils->getMessage("IMsgDSHcliReportCmdPath");
    $TRUE_STR = NodeUtils->getMessage("IMsgDSHcliReportTrue");
    $FALSE_STR = NodeUtils->getMessage("IMsgDSHcliReportFalse");
    $OUTPUT_FILE_LABEL = NodeUtils->getMessage("IMsgDSHcliReportOutputFileLabel");
    $ERROR_FILE_LABEL  = NodeUtils->getMessage("IMsgDSHcliReportErrorFileLabel");
    $BLANK = '&nbsp';

    my ($formattedCmdDef) = $commandDefinition;
    if ($formattedCmdDef) {
        $formattedCmdDef =~ s/\n/<BR>\n/g;
    } else {
        $formattedCmdDef = $BLANK;
    }

$prolog=<<END_PROLOG
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<HTML lang="$LANGUAGE">
<HEAD>
<!--
Licensed Materials - Property of IBM
(C) COPYRIGHT International Business Machines Corp. 1994,2003
All Rights Reserved
US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
Author : Generated by dsh
 -->
END_PROLOG
;

    my ($reportDirectory) = $outputDirectory;
    $reportDirectory =~ s/$reportsDirectory\///;  
    #$reportDirectory =~ s/\.(.*)$/ ($1)/;

$reportTitle=<<END_REPORT_TITLE
<TITLE>$reportDirectory</TITLE>
</HEAD>
<BODY>
<CENTER><H1>$reportDirectory</H1></CENTER>
<HR>
END_REPORT_TITLE
;

$listTitle=<<END_LIST_TITLE
<TITLE>$LISTTITLE</TITLE>
</HEAD>
<BODY>
<CENTER><H1>$LISTTITLE</H1></CENTER>
<HR>
END_LIST_TITLE
;

$indexTitle=<<'END_INDEX_TITLE'
</HEAD>
END_INDEX_TITLE
;

$sectionLinks=<<END_SECTION_LINKS
<UL>
<LI><A HREF="#Execution_Status">$CMDEXECSTATUS</A>
<LI><A HREF="#Host_Results">$CMDHOSTRESULTS</A>
<LI><A HREF="#Command_Info">$CMDINFO</A>
<LI><A HREF="#Command_Options">$CMDOPTIONS</A>
</UL>
<HR>
END_SECTION_LINKS
;

$cmdExecutionResults=<<END_EXECUTION_RESULTS

<P><A NAME="Command_Execution_Status"></A><B>$CMDEXECSTATUS</B></P>

<TABLE BORDER COLS=4 WIDTH="100%" CELLPADDING="3">

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="10%" BGCOLOR="silver">$COMMAND</TH>
    <TD ALIGN=LEFT VALIGN=TOP COLSPAN="3">$formattedCmdDef</TD>
</TR>

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="10%" BGCOLOR="silver">$STARTED</TH>
    <TD ALIGN=LEFT VALIGN=TOP COLSPAN="3">$startTime</TD>
</TR>

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="10%" BGCOLOR="silver">$ENDED</TH>
    <TD ALIGN=LEFT VALIGN=TOP COLSPAN="3">$endTime</TD>
</TR>

END_EXECUTION_RESULTS
;

    # commandName, definition, and user are guaranteed to have a value
    my ($path) = ($commandPath) ? $commandPath : $BLANK;
    my ($description) = ($commandDescription) ? $commandDescription : $BLANK;
    my ($hosts) = (@hostNames) ? "@hostNames" : $BLANK;
    my ($groups) = (@groupNames) ? "@groupNames" : $BLANK;

$cmdInfo=<<END_CMDINFO
<P></P><P><A NAME="Command_Info"></A><B>$CMDINFO</B></P>

<TABLE BORDER COLS=3 WIDTH="100%" CELLPADDING="3">

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="5%" BGCOLOR="silver">$NAME</TH>
    <TD ALIGN=LEFT VALIGH=TOP COLSPAN="2">$commandName</TD>
</TR>

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="5%" ROWSPAN="2" BGCOLOR="silver">$CMDDEFINITION</TH>

    <TD ALIGN=LEFT VALIGN=TOP WIDTH="5%" BGCOLOR="whitesmoke">$CMDPATH</TD>
    <TD ALIGN=LEFT VALIGN=TOP>$path</TD>
</TR>

<TR>
    <TD ALIGN=LEFT VALIGN=TOP WIDTH="5%" BGCOLOR="whitesmoke">$COMMAND</TD>
    <TD ALIGN=LEFT VALIGN=TOP>$formattedCmdDef</TD>
</TR>

<TR>
    <TH ALIGN=LEFT VALIGN=TOP BGCOLOR="silver">$RUNASUSER</TH>
    <TD ALIGN=LEFT VALIGN=TOP COLSPAN="2">$runAsUser</TD>
</TR>

<TR>
    <TH ALIGN=LEFT VALIGN=TOP BGCOLOR="silver">$DESCRIPTION</TH>
    <TD ALIGN=LEFT VALIGN=TOP COLSPAN="2">$description</TD>
</TR>

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="5%" ROWSPAN="2" BGCOLOR="silver">$RUNHERE</TH>

    <TD ALIGN=LEFT VALIGN=TOP WIDTH="5%" BGCOLOR="whitesmoke">$HOSTNAMES</TD>
    <TD ALIGN=LEFT VALIGN=TOP>$hosts</TD>
</TR>

<TR>
    <TD ALIGN=LEFT VALIGN=TOP WIDTH="5%" BGCOLOR="whitesmoke">$GROUPSOFHOSTS</TD>
    <TD ALIGN=LEFT VALIGN=TOP>$groups</TD>
</TR>
</TABLE>

END_CMDINFO
;

    my ($verifyStr) = ($verifyRequested) ? $TRUE_STR : $FALSE_STR;
    my ($streamingStr) = ($streamingRequested) ? $TRUE_STR : $FALSE_STR;
    my ($fileWritingStr) = ($fileWritingRequested) ? $TRUE_STR : $FALSE_STR;
    my ($remoteShellOptionsStr) = ($remoteShellOptions) ? $remoteShellOptions : $BLANK;

    my ($timeoutInfo) = '';
    if (defined $timeout) {

$timeoutOption=<<END_TIMEOUT_OPTION

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="90%" BGCOLOR="silver">$TIMEOUT</TH>
    <TD ALIGN=LEFT VALIGN=TOP>$timeout</TD>
</TR>

END_TIMEOUT_OPTION
;

    } 

$miscInfo=<<END_MISCINFO;
<P></P><P><A NAME="Command_Options"></A><B>$CMDOPTIONS</B></P>
	
<TABLE BORDER COLS=2 WIDTH="60%" CELLPADDING="3">
 
<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="90%" BGCOLOR="silver">$FANOUT</TH>
    <TD ALIGN=LEFT VALIGN=TOP>$fanout</TD>
</TR>

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="90%" BGCOLOR="silver">$HOSTWILLRESPOND</TH>
    <TD ALIGN=LEFT VALIGN=TOP>$verifyStr</TD>
</TR>

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="90%" BGCOLOR="silver">$STREAMOUTPUT</TH>
    <TD ALIGN=LEFT VALIGN=TOP>$streamingStr</TD>
</TR>
$timeoutOption
<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="90%" BGCOLOR="silver">$GENERATEFILES</TH>
    <TD ALIGN=LEFT VALIGN=TOP>$fileWritingStr</TD>
</TR>

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="90%" BGCOLOR="silver">$REMOTESHELL</TH>
    <TD ALIGN=LEFT VALIGN=TOP>$remoteShellPath</TD>
</TR>

<TR>
    <TH ALIGN=LEFT VALIGN=TOP WIDTH="90%" BGCOLOR="silver">$REMOTESHELLOPTIONS</TH>
    <TD ALIGN=LEFT VALIGN=TOP>$remoteShellOptionsStr</TD>
</TR>
</TABLE>
END_MISCINFO
;

$endHTML=<<'END_HTML'
</BODY>
</HTML>
END_HTML
;



}

#
# For debuggin purposes
#
sub printHostandStatus
{
    for $i ($EXECUTION_PENDING..$EXECUTION_CANCELLED) {
        if ($i==$EXECUTION_PENDING) {print "$EXECUTION_PENDING: ";}
        elsif ($i==$EXECUTION_STARTED) {print "$EXECUTION_STARTED: ";}
        elsif ($i==$EXECUTION_SUCCESSFUL) {print "$EXECUTION_SUCCESSFUL: ";}
        elsif ($i==$EXECUTION_FAILED) {print "$EXECUTION_FAILED: ";}
        elsif ($i==$EXECUTION_CANCELLED) {print "$EXECUTION_CANCELLED: ";}
        print "\t@{$hostStatus[$i]}\n";
    }
}

#
# For debuggin purposes
#
sub getHostWithStatus
{
    local ($status = $_[0]);   # get requested status from paramater
    foreach $i (@{$hostStatus[$status]}) {
	print "$i ";
    }
    print "\n";
}


#
# Name: printMachinesWithStatus
# Description: Creates an html table of machines based on the status passed in.
# Args:  label  - What the table label should be, ex "Successful Hosts"
#        status - which status to to print out, ex $EXECUTION_SUCCESSFUL
#        fp     - file pointer to print out the message
# Called from: generateReport
# Returns:  0 if error, else 1
#
# Example:
#   printMachinesWithStatus ($FAILEDHOSTS,     $EXECUTION_FAILED, REPORT_HANDLE);
# will create an html table that would look similar to this:
#  |  Failed Hosts  |   host1           output          error
#  |                |   host3                           error
#
sub printMachinesWithStatus
{
    my ($label) = $_[0];
    my ($status) = $_[1];
    my ($fp) = $_[2];		# file pointer to print out messages
    my ($resultsColor) = $_[3];

#print "unsortedList = @{$hostStatus[$status]}\n";
#foreach my $i (@{$hostStatus[$status]}) {
#   print "host = $i\n";
#}
    my (@sortedList) = sort @{$hostStatus[$status]};


    # if no file handle was specified, default to STDOUT
    if (!exists($_[2])) {
        $fp=STDOUT;
    }

#print "sortedList = @sortedList\n";
#foreach my $i (@sortedList) {
#   print "host = $i\n";
#}
    &printLine ($fp, "\n<TR>\n", "handleReportWriteError") || return 0;
    if (scalar(@sortedList)) {
        my ($rowSpan) = scalar(@sortedList);
        &printLine ($fp, "    <TH ALIGN=LEFT VALIGN=TOP WIDTH=\"10%\" ROWSPAN=\"$rowSpan\" BGCOLOR=\"silver\">$label</TH>\n\n",
	    "handleReportWriteError") || return 0;
        my ($hostname) = $sortedList[0];
        shift @sortedList;
        &printLine ($fp, "    <TD ALIGN=LEFT VALIGN=TOP BGCOLOR=$resultsColor>$hostname</TD>\n", 
			"handleReportWriteError") || return 0;

        if (exists $outputFiles{$hostname}) {
            &printLine ($fp, 
                "    <TD ALIGN=CENTER VALIGN=TOP WIDTH=\"20%\" BGCOLOR=$resultsColor> <A HREF=$hostname.$OUTPUT_FILE_EXTENSION>$OUTPUT_FILE_LABEL</A></TD>\n",
                "handleReportWriteError") || return 0;
        } else {
            # print a blank space to keep columns spaced correctly
            &printLine ($fp, "    <TD ALIGN=CENTER VALIGN=TOP WIDTH=\"20%\" BGCOLOR=$resultsColor>$BLANK</TD>\n", 
                "handleReportWriteError") || return 0;
        }

        if (exists $errorFiles{$hostname}) {
	    &printLine ($fp, 
                "    <TD ALIGN=CENTER VALIGN=TOP WIDTH=\"20%\" BGCOLOR=$resultsColor> <A HREF=$hostname.$ERROR_FILE_EXTENSION>$ERROR_FILE_LABEL</A></TD>\n",
                "handleReportWriteError") || return 0;
        } else {
            # print a blank space to keep columns spaced correctly
	    &printLine ($fp, "    <TD ALIGN=CENTER VALIGN=TOP WIDTH=\"20%\" BGCOLOR=$resultsColor>$BLANK</TD>\n",
                "handleReportWriteError") || return 0;
        }

        &printLine ($fp, "</TR>\n\n", "handleReportWriteError");
    } else {
        &printLine ($fp, "    <TH ALIGN=LEFT VALIGN=TOP WIDTH=\"10%\" BGCOLOR=\"silver\">$label</TH>\n\n",
	    "handleReportWriteError") || return 0;
        &printLine ($fp, "    <TD ALIGN=LEFT VALIGN=TOP BGCOLOR=$resultsColor>$BLANK</TD>\n", 
            "handleReportWriteError") || return 0;
        &printLine ($fp, "    <TD ALIGN=LEFT VALIGN=TOP WIDTH=\"20%\" BGCOLOR=$resultsColor>$BLANK</TD>\n", 
            "handleReportWriteError") || return 0;      
	&printLine ($fp, "    <TD ALIGN=LEFT VALIGN=TOP WIDTH=\"20%\" BGCOLOR=$resultsColor>$BLANK</TD>\n",
            "handleReportWriteError") || return 0;
        &printLine ($fp, "</TR>\n\n", "handleReportWriteError") || return 0;
    }

    foreach $i (@sortedList) {
        &printLine ($fp, "<TR>\n", "handleReportWriteError") || return 0;
        &printLine ($fp, "    <TD ALIGN=LEFT VALIGN=TOP BGCOLOR=$resultsColor>$i</TD>\n", 
			"handleReportWriteError") || return 0;

        if (exists $outputFiles{$i}) {
            &printLine ($fp, 
                "    <TD ALIGN=CENTER VALIGN=TOP BGCOLOR=$resultsColor> <A HREF=$i.$OUTPUT_FILE_EXTENSION>$OUTPUT_FILE_LABEL</A></TD>\n",
                "handleReportWriteError") || return 0;
        } else {
            # print a blank space to keep columns spaced correctly
            &printLine ($fp, "    <TD ALIGN=LEFT VALIGN=TOP BGCOLOR=$resultsColor>$BLANK</TD>\n", 
                "handleReportWriteError") || return 0;
        }

        if (exists $errorFiles{$i}) {
	    &printLine ($fp, 
                "    <TD ALIGN=CENTER VALIGN=TOP BGCOLOR=$resultsColor> <A HREF=$i.$ERROR_FILE_EXTENSION>$ERROR_FILE_LABEL</A></TD>\n",
                "handleReportWriteError") || return 0;
        } else {
            # print a blank space to keep columns spaced correctly
	    &printLine ($fp, "    <TD ALIGN=LEFT VALIGN=TOP BGCOLOR=$resultsColor>$BLANK</TD>\n",
                "handleReportWriteError") || return 0;
        }

        &printLine ($fp, "</TR>\n", "handleReportWriteError") || return 0;
    } # endforeach

    return 1;
} # printMachinesWithStatus

#
# Name: printReportStatusMessages
# Description: Creates an html table containing the status of the commands run
# Args: fp     - file pointer to print out the status
# Returns:  nothing
# Called From: generateReport
#
#
sub printReportStatusMessages
{
    local ($fp=$_[0]);   # file pointer to print out messages

    # if no file handle was specified, default to STDOUT
    if (!exists($_[0])) {
       $fp=STDOUT;
    }

    &printLine ($fp, "\n<TR>\n", "handleReportWriteError") || return 0;
    &printLine ($fp, "    <TH ALIGN=LEFT VALIGN=TOP BGCOLOR=\"silver\">$STATUS</TH>\n", "handleReportWriteError") || return 0;
    &printLine ($fp, "    <TD ALIGN=LEFT VALIGN=TOP COLSPAN=\"3\">","handleReportWriteError") || return 0;
    if (scalar(@reportStatusMessages)) {
        foreach $i (@reportStatusMessages) {
            &printLine ($fp, "$i<br>", "handleReportWriteError") || return 0;
        }
    } else {
        &printLine ($fp, "&nbsp;", "handleReportWriteError") || return 0;
    }
    &printLine ($fp, "</TD>\n</TR>\n", "handleReportWriteError") || return 0;

    return 1;
}

#
# Name: printReportSummary
# Description: Creates a text area that contains the total number of hosts that were successful,
#     failed, not run and were attempted.
# Args: fp     - file pointer to print out the status
# Returns:  0 if error, 1 otherwise
# Called From: generateReport
#
#
sub printReportSummary
{
    local ($fp=$_[0]);   # file pointer to print out messages

    # if no file handle was specified, default to STDOUT
    if (!exists($_[0])) {
       $fp=STDOUT;
    }

    my ($totalSuccessful) = scalar(@{$hostStatus[$EXECUTION_SUCCESSFUL]});
    if (!$totalSuccessful) {
        $totalSuccessful = 0;
    }
    my ($totalFailed) = scalar(@{$hostStatus[$EXECUTION_FAILED]});
    if (!$totalFailed) {
        $totalFailed = 0;
    }
    my ($totalCancelled) = scalar(@{$hostStatus[$EXECUTION_CANCELLED]});

    my ($REPORT_SUMMARY_MSG) = NodeUtils->getMessage("IMsgDSHcliReportSummary");
    my ($REPORT_SUMMARY_SUCCESSFUL_HOSTS_MSG) = NodeUtils->getMessage("IMsgDSHcliReportSummarySuccessfulHosts", $totalSuccessful);
    my ($REPORT_SUMMARY_FAILED_HOSTS_MSG) = NodeUtils->getMessage("IMsgDSHcliReportSummaryFailedHosts", $totalFailed);
    my ($REPORT_SUMMARY_CANCELLED_HOSTS_MSG) = NodeUtils->getMessage("IMsgDSHcliReportSummaryCancelledHosts", $totalCancelled); 
    my ($REPORT_SUMMARY_TOTAL_ATTEMPTED_MSG) = NodeUtils->getMessage("IMsgDSHcliReportSummaryTotalAttempted", $totalSuccessful + $totalFailed + $totalCancelled);

    &printLine ($fp, "\n<TR>\n", "handleReportWriteError") || return 0;
    &printLine ($fp, "    <TH ALIGN=LEFT VALIGN=TOP BGCOLOR=\"silver\">$REPORT_SUMMARY_MSG</TH>\n", "handleReportWriteError") || return 0;
    &printLine ($fp, "    <TD ALIGN=LEFT VALIGN=TOP COLSPAN=\"3\">","handleReportWriteError") || return 0;

    &printLine ($fp, "$REPORT_SUMMARY_SUCCESSFUL_HOSTS_MSG<br>", "handleReportWriteError") || return 0;
    &printLine ($fp, "$REPORT_SUMMARY_FAILED_HOSTS_MSG<br>", "handleReportWriteError") || return 0;
    if ($totalCancelled) {
        &printLine ($fp, "$REPORT_SUMMARY_CANCELLED_HOSTS_MSG<br>", "handleReportWriteError") || return 0;
    }
    &printLine ($fp, "$REPORT_SUMMARY_TOTAL_ATTEMPTED_MSG", "handleReportWriteError") || return 0;

    &printLine ($fp, "</TD>\n</TR>\n</TABLE>\n\n", "handleReportWriteError") || return 0;

    return 1;
}

#
# Name: generateListOfReports
# Description: Creates an .html file that has links to all the command reports
# Args:  none
#
sub generateListOfReports {
    unless (open (LIST_HANDLE, "> $reportsDirectory/listOfReports.html")) {
        &handleReportOpenError("$reportsDirectory/listOfReports.html");
        return 0;
    }

    flock (LIST_HANDLE, LOCK_EX);  # lock the file for exclusive use
                                   # if already locked, it will block till rdy


    &printLine (LIST_HANDLE, $prolog, "handleReportWriteError") || goto FAILED; 
    &printLine (LIST_HANDLE, $listTitle, "handleReportWriteError") || goto FAILED;

    # get the list of all the files with a four letter suffix, ex, cmd.0001
    # We use the read operator (the angle brackets) to obtain this list
    @reportFiles=<$reportsDirectory/*.[0-9][0-9][0-9][0-9]>;
    @sortedList = sort @reportFiles;

    foreach $direc (@sortedList) {
        if (-d $direc ) {
            # the variable $direc is full path, need to split it so we can
            # just get the actual directory name
	    @direcArray=split('/', $direc);
            $file=pop @direcArray;  # grab last entry which is directory name

            # now get rid of the .0001 at the end
            @fileArray=split('\.', $file);
            $reportName=@fileArray[0];

	    &printLine (LIST_HANDLE, "<a href=\"$file/$reportName.html\" target=\"detailFrame\">$file </a><br>\n", "handleReportWriteError") || goto FAILED;
        }
    }

    &printLine (LIST_HANDLE, $endHTML, "handleReportWriteError") || goto FAILED;
    
    flock (LIST_HANDLE, LOCK_UN);    # unlock file
    close (LIST_HANDLE);
    return 1;

    FAILED:
        flock (LIST_HANDLE, LOCK_UN);    # unlock file
        close (LIST_HANDLE);
        return 0;

} # end generateListOfReports

sub generateIndex {

    my ($relativeReportPath) = @_;

    unless (open (INDEX_HANDLE, "> $reportsDirectory/index.html")) {
        &handleReportOpenError("$reportsDirectory/index.html");
        return 0;
    }

    flock (INDEX_HANDLE, LOCK_EX);  # lock the file for exclusive use
			            # if already locked, it will block till rdy

    # create relative path to report
    $relativeReportPath =~ s/$reportsDirectory\///;

$indexBody=<<END_INDEX_BODY
<FRAMESET cols="50,100">
<FRAME src="listOfReports.html" name="Report">
<FRAME src="$relativeReportPath" name="detailFrame">
</FRAMESET>
END_INDEX_BODY
;
    &printLine (INDEX_HANDLE, $prolog, "handleReportWriteError") || goto FAILED;
    &printLine (INDEX_HANDLE, $indexTitle, "handleReportWriteError") || goto FAILED;
    &printLine (INDEX_HANDLE, $indexBody, "handleReportWriteError") || goto FAILED;
    &printLine (INDEX_HANDLE, "</HTML>", "handleReportWriteError") || goto FAILED;
    
    flock (INDEX_HANDLE, LOCK_UN);   # unlock file
    close (INDEX_HANDLE);	     # close file
    return 1;

    FAILED:
        flock (INDEX_HANDLE, LOCK_UN);   # unlock file
        close (INDEX_HANDLE);            # close file
        return 0; 
} # end generateIndex

#
# Name: generateReport
# Description: Creates an .html file summarizing the command execution
# Args:  none
#
sub generateReport {
    if (!$writingFiles) {
        return 0;
    }

#while(($key,$value)=each %executionStatus) {
#print "being reportkey, value = ($key, $value)\n";
#}
    &updateHostStatus() if (!$hostStatusUpdated);
    &initializeReportDocuments();

    my ($reportPath) = "$outputDirectory/$reportName.html"; 

    unless (open (REPORT_HANDLE, "> $reportPath")) {
        &handleReportOpenError("$reportPath");
        return 0;
    }

    &printLine (REPORT_HANDLE, $prolog, "handleReportWriteError") || goto FAILED;
    &printLine (REPORT_HANDLE, $reportTitle, "handleReportWriteError") || goto FAILED;
    &printLine (REPORT_HANDLE, $sectionLinks, "handleReportWriteError") || goto FAILED;
    &printLine (REPORT_HANDLE, $cmdExecutionResults, "handleReportWriteError") || goto FAILED;  
    &printReportStatusMessages (REPORT_HANDLE) || goto FAILED;
    &printReportSummary (REPORT_HANDLE) || goto FAILED;


    &printLine (REPORT_HANDLE, "<P><A NAME=\"Host_Results\"></A><B>$CMDHOSTRESULTS</B></P>",  
        "handleReportWriteError") || goto FAILED;  

    &printLine (REPORT_HANDLE, "<TABLE BORDER COLS=3 WIDTH=\"100%\" CELLPADDING=\"3\">\n\n",  
        "handleReportWriteError") || goto FAILED;  

    &printMachinesWithStatus ($SUCCESSFULHOSTS,$EXECUTION_SUCCESSFUL, REPORT_HANDLE, "whitesmoke") || goto FAILED;
    &printMachinesWithStatus ($FAILEDHOSTS,    $EXECUTION_FAILED, REPORT_HANDLE, "white") || goto FAILED;
    if (scalar(@{$hostStatus[$EXECUTION_CANCELLED]})) {
        &printMachinesWithStatus ($CANCELLEDHOSTS,  $EXECUTION_CANCELLED, REPORT_HANDLE, "whitesmoke") || goto FAILED;
    }
    &printLine (REPORT_HANDLE, "</TABLE>\n\n", "handleReportWriteError")  || goto FAILED;
 
    &printLine (REPORT_HANDLE, $cmdInfo, "handleReportWriteError")  || goto FAILED;
    &printLine (REPORT_HANDLE, $miscInfo, "handleReportWriteError") || goto FAILED;
    &printLine (REPORT_HANDLE, $endHTML, "handleReportWriteError")  || goto FAILED;
   
    close (REPORT_HANDLE);

    # generate list of reports
    &generateListOfReports()  || return $reportPath;

    # generate index/frame html
    &generateIndex($reportPath); 

    return $reportPath;  

    FAILED:
        close (REPORT_HANDLE);
        return 0;

} # end generateReport



###############################################################################################
#
#            Summary Logging subroutines - private
#
###############################################################################################

# Args: ($messageName, @msgArgs)
sub logInfoMessage  {
    my ($messageName, @msgArgs) = @_;

    push (@logStatusMessages, NodeUtils->getMessage($messageName, @msgArgs));
}

# Args: ($messageName, @msgArgs)
sub logErrorMessage  {
    my ($messageName, @msgArgs) = @_;

    push (@logStatusMessages, NodeUtils->getMessage($messageName, "dsh", @msgArgs));
}

# creates an array of 
sub printLogHostStatus
{
    my ($statusRef, $statusMsg, $indent) = @_;
    my (@tmpHostList);
    my (@hostLines);
    my ($charCount) = length($indent);
    my ($maxLineLength) = 72;
    my ($hostSeparator) = NodeUtils->getMessage(IMsgDSHcliLogHostSeparator);
    my ($hostSeparatorLength) = length($hostSeparator);
#print "$hostSeparator / $hostSeparatorLength\n";
    foreach my $host (sort @$statusRef) {
#print "host = $host\n";
        $charCount += length($host) + $hostSeparatorLength;
#print "charCount = $charCount\n";
        if ($charCount <= $maxLineLength) {
            # store the host in the tmp array 
#print "pushing host\n";
            push @tmpHostList, $host;
        } else {
#print "tmpHostList = @tmpHostList\n";
            # create the new line of hosts
            push @hostLines, join($hostSeparator, @tmpHostList);
            undef @tmpHostList;
            $charCount = length($indent);
            redo;
        }
    }

    # push last line if necessary
    if (defined @tmpHostList) {
        push @hostLines, join($hostSeparator, @tmpHostList);
    }

    &printLine (LOG_HANDLE, NodeUtils->getMessage($statusMsg, $indent . join("$hostSeparator\n$indent", @hostLines)), 
        "handleLogWriteError") || return 0;  

    return 1; 
}   
              
#
# Name: updateLog
# Description: Updates the log file with the standard format
# Args:  none
# Returns: TRUE if successful
#          FALSE if failed
#
sub updateLog() {
    if (!$writingFiles) {
        return 0;
    }

    # open the logfile for appending
    unless (open (LOG_HANDLE, ">> $logPath")) {
        &handleLogOpenError("$logPath");
        return 0;
    }

    # check to make sure some of the variables we used are initialized
    &updateHostStatus() if (!$hostStatusUpdated);

    # Start the logging update

    my ($separator) = NodeUtils->getMessage("IMsgDSHcliLogSeparator");
    my ($indent) = NodeUtils->getMessage("IMsgDSHcliLogIndent");

    &printLine (LOG_HANDLE, "$separator\n\n", "handleLogWriteError") || goto FAILED;

    &printLine (LOG_HANDLE, NodeUtils->getMessage("IMsgDSHcliLogCommandName") . $commandName . "\n", 
        "handleLogWriteError") || goto FAILED;
    
    &printLine (LOG_HANDLE, NodeUtils->getMessage("IMsgDSHcliLogRunAsUser", $runAsUser), 
        "handleLogWriteError") || goto FAILED;

    # format command
    my ($formattedCommandDefinition) = $commandDefinition;
    $formattedCommandDefinition =~ s/\n/\n$indent/g;
#print "commdef = $commandDefinition\n";
#print "forcomdeh = $formattedCommandDefinition\n";

    &printLine (LOG_HANDLE, NodeUtils->getMessage("IMsgDSHcliLogCommandDefinition", 
        $indent . $formattedCommandDefinition), "handleLogWriteError") || goto FAILED;

    &printLine (LOG_HANDLE, NodeUtils->getMessage("IMsgDSHcliLogStarted") . $startTime . "\n", 
        "handleLogWriteError") || goto Failed;
    &printLine (LOG_HANDLE, NodeUtils->getMessage("IMsgDSHcliLogEnded") . $endTime . "\n\n", 
        "handleLogWriteError") || goto Failed;

    # if successful hosts
    if (scalar(@{$hostStatus[$EXECUTION_SUCCESSFUL]})) {
        &printLogHostStatus(\@{$hostStatus[$EXECUTION_SUCCESSFUL]}, "IMsgDSHcliLogSuccessfulHosts", $indent) || goto FAILED;
    } else {
        &printLine (LOG_HANDLE, NodeUtils->getMessage("IMsgDSHcliLogNoSuccessfulHosts"), 
            "handleLogWriteError") || goto FAILED;
    }

    # if failed hosts
    if (scalar(@{$hostStatus[$EXECUTION_FAILED]})) {
        &printLogHostStatus(\@{$hostStatus[$EXECUTION_FAILED]}, "IMsgDSHcliLogFailedHosts", $indent) || goto FAILED;
    } else {
        &printLine (LOG_HANDLE, NodeUtils->getMessage(IMsgDSHcliLogNoFailedHosts), 
            "handleLogWriteError") || goto FAILED;
    } 
    
    # if cancelled hosts
    if (scalar(@{$hostStatus[$EXECUTION_CANCELLED]})) {
        &printLogHostStatus(\@{$hostStatus[$EXECUTION_CANCELLED]}, "IMsgDSHcliLogCancelledHosts", $indent) || goto FAILED;
    } 

    # format status and print
    my ($logStatus) = join('', @logStatusMessages);
    $logStatus =~ s/\n/\n$indent/g;
#print "logStatusMessages= @logStatusMessages\n";
#print "logStatus = $logStatus\n";

    &printLine (LOG_HANDLE, NodeUtils->getMessage("IMsgDSHcliLogStatus", $indent . $logStatus), 
        "handleLogWriteError") || goto FAILED;

    close (LOG_HANDLE);
    return 1; # TRUE

    FAILED: 

        close (LOG_HANDLE);
        return 0; # FALSE

} # end updateLog

###############################################################################################
#
#            Per Host Output file writing subroutines - private
#
###############################################################################################

# takes a scalar hostname as parameter
# returns the handle if file writing has been requested and writing has not been disabled, 0 otherwise.
# The file will be opened for writing.
# Post:
# 1. If the file already existed, then the file is open for append, else open for write.
# 2. $outputFiles{$hostname} = $fileHandle

sub getOutputFileHandle {
    my ($hostname) = @_;
    my ($fileHandle) = 0;

    if ($fileWritingRequested && $writingFiles) {
        my $filename = "$outputDirectory/$hostname.$OUTPUT_FILE_EXTENSION";

        # if file has been opened before by this run of dsh. Files that exist in the output dir before running dsh will be overwritten.
        if (exists $outputFiles{$hostname}) {

            # then if open return fileHandle
            if ($fileHandle = $outputFiles{$hostname}) {
                return $fileHandle;
            } else {  # open it for appending

                $fileHandle = $hostname . $OUTPUT_FILE_EXTENSION;
                if (open($fileHandle, ">>$filename")) {
                    $outputFiles{$hostname} = $fileHandle; 
                } else {
                    # print STDERR "dsh:  2617-037 The following error occurred while trying to open the output file "$filename:\n\t$!'
                    &handleOutputOpenError("$filename");
                }
            }
        } else { # we need to open the file for writing
            $fileHandle = $hostname . $OUTPUT_FILE_EXTENSION;
            
            if (open($fileHandle, ">$filename")) {
                $monitoringRequested && &sendOutputFileCreatedMessage($hostname, $filename);
                $outputFiles{$hostname} = $fileHandle; 
            } else {
                # print STDERR "dsh:  2617-037 The following error occurred while trying to open the output file "$filename:\n\t$!'
                &handleOutputOpenError("$filename");
            }

        }
    }
    select ($fileHandle);
    $| = 1;
    return $fileHandle;
}

# takes a scalar hostname as parameter
# returns the handle if file writing has been requested and writing has not been disabled, 0 otherwise.
# The file will be opened for writing.
sub getErrorFileHandle {
    my ($hostname) = @_;
    my ($fileHandle) = 0;

    if ($fileWritingRequested && $writingFiles) {
        my $filename = "$outputDirectory/$hostname.$ERROR_FILE_EXTENSION";

        # if file has been opened before by this run of dsh. Files that exist in the output dir before running dsh will be overwritten.
        if (exists $errorFiles{$hostname}) {

            # then if open return fileHandle
            if ($fileHandle = $errorFiles{$hostname}) {
                return $fileHandle;
            } else {  # open it for appending

                $fileHandle = $hostname . $ERROR_FILE_EXTENSION;
                if (open($fileHandle, ">>$filename")) {
                    $errorFiles{$hostname} = $fileHandle; 
                } else {  
                    # print STDERR "dsh:  2617-037 The following error occurred while trying to open the output file "$filename:\n\t$!'
                    &handleOutputOpenError("$filename");
                }
            }
        } else { # we need to open the file for writing
            $fileHandle = $hostname . $ERROR_FILE_EXTENSION;
           
            if (open($fileHandle, ">$filename")) {
                $monitoringRequested && &sendErrorFileCreatedMessage($hostname, $filename);
                $errorFiles{$hostname} = $fileHandle; 
            } else { 
                # print STDERR "dsh:  2617-037 The following error occurred while trying to open the output file "$filename:\n\t$!'
                &handleOutputOpenError("$filename");
            }

        }
    } 
    select ($fileHandle);
    $| = 1;
    return $fileHandle;
}

sub appendErrorMessage {
    my ($message_ref, $hostname) = @_;
    if (my $outputfh = &getOutputFileHandle($hostname)) {
        &printLine($outputfh, $$message_ref, "handleOutputWriteError") || return;
        DshUtils->closeOutputFile($hostname);
    }

    if (my $errorfh = &getErrorFileHandle($hostname)) {
        &printLine($errorfh, $$message_ref, "handleOutputWriteError") || return;
        DshUtils->closeErrorFile($hostname);
    }
}

sub handleOutputOpenError {
    my ($fileName) = @_; 

    &sendErrorMessage("EMsgDSHcliFileOpenError", $fileName, $!);
    if ($reportingRequested && $loggingRequested) {
        &sendErrorMessage("EMsgDSHcli_ORL_Disabled");
    } elsif ($reportingRequested && !$loggingRequested) {
        &sendErrorMessage("EMsgDSHcli_OR_Disabled");
    } elsif (!$reportingRequested && $loggingRequested) {
        &sendErrorMessage("EMsgDSHcli_OL_Disabled");
    } else {
        &sendErrorMessage("EMsgDSHcli_O_Disabled");
    }
    $writingFiles = 0; # FALSE
}

sub handleOutputWriteError {
    
    &sendErrorMessage("EMsgDSHcliFileWritingError", $!);
    if ($reportingRequested && $loggingRequested) {
        &sendErrorMessage("EMsgDSHcli_ORL_Disabled");
    } elsif ($reportingRequested && !$loggingRequested) {
        &sendErrorMessage("EMsgDSHcli_OR_Disabled");
    } elsif (!$reportingRequested && $loggingRequested) {
        &sendErrorMessage("EMsgDSHcli_OL_Disabled");
    } else {
        &sendErrorMessage("EMsgDSHcli_O_Disabled");
    }
    $writingFiles = 0; # FALSE
}

# assumes that output writing have completed
sub handleReportOpenError {
    my ($fileName) = @_; 
    &sendErrorMessage("EMsgDSHcliReportOpenError", $fileName, $!);
    if ($loggingRequested) {
        &sendErrorMessage("EMsgDSHcli_RL_Disabled");
    } else {
        &sendErrorMessage("EMsgDSHcli_R_Disabled");
    } 
    $writingFiles = 0; # FALSE
}

# assumes that output writing have completed
sub handleReportWriteError {
    &sendErrorMessage("EMsgDSHcliReportWritingError", $!);
    if ($loggingRequested) {
        &sendErrorMessage("EMsgDSHcli_RL_Disabled");
    } else {
        &sendErrorMessage("EMsgDSHcli_R_Disabled");
    } 
    $writingFiles = 0; # FALSE
}

# assumes that output writing and report writing have completed
sub handleLogOpenError {
    my ($fileName) = @_; 
    &sendErrorMessage("EMsgDSHcliLogOpenError", $fileName, $!);
    &sendErrorMessage("EMsgDSHcli_L_Disabled");
    $writingFiles = 0; # FALSE
}

# assumes that output writing and report writing have completed
sub handleLogWriteError {
    &sendErrorMessage("EMsgDSHcliLogWritingError", $!);
    &sendErrorMessage("EMsgDSHcli_L_Disabled");
    $writingFiles = 0; # FALSE
}

###############################################################################################
#
#            Initialization subroutines - private
#
###############################################################################################

# Args: $index - used to create suffix number of directory
#       $baseDirectoryPath - the directory that will contain the new directory
#       $directoryPrefix        
sub getDirectory {
    my ($index, $baseDirectoryPath, $directoryPrefix) = @_;
    my ($directoryPath) = "$baseDirectoryPath/$directoryPrefix.";
    if ($index < 10) {
        $directoryPath .= "000" . $index;
    } elsif ($index < 100) {
        $directoryPath .= "00" . $index;
    } elsif ($index < 1000) {
        $directoryPath .= "0" . $index;
    } else {
        $directoryPath .= $index;
    }

    return $directoryPath;
}

# Pre:   $reportName must not be empty
# Args:  $reportName, $reportsDirectory 
sub getUniqueDirectory {
    my ($reportName, $reportsDirectoryPath) = @_;

    # get the list of all the files in the $reportsDirectoryPath
    opendir (DIR, "$reportsDirectoryPath");
    @reportFiles = readdir(DIR);

    my ($suffix, $index);
    my ($maxIndex) = 0;
    my (%usedIndices);
    foreach $tmpfile (@reportFiles) {
        # prepend the directory path so it can be found correctly
        $file="$reportsDirectoryPath/$tmpfile";

        # if the file is a directory and
        # if it has a four letter suffix, ex, cmd.0001
        if (-d $file && $file =~ /$reportName.[0-9][0-9][0-9][0-9]$/) {
            # strip off prefix including leading 0's
            $file =~ s/$reportsDirectory\/$reportName.0*//;

            # add to usedSuffixes array
            $usedIndices{$file} = 1;

            # compare file suffix numerical value to max, if suffix not digit, resolves to 0
            $maxIndex = $file if ($file > $maxIndex);
        }
    }
    $maxIndex++;

    if ($maxIndex > 9999) {  # then we need to find a free index starting at 0001
        for (my $i = 1; $i <= 9999; ++$i) {
            if (exists $usedIndices{$i}) {
                next;
            } else {
                undef %usedIndices;
                return &getDirectory($i, $reportsDirectory, $reportName);
            }
        }
        # No directory index available for the command "$reportName". The report directory could not be created.
        # Please delete unnecessary report directories or run the command without reporting or output file writing.
        &sendErrorMessage("EmsgDSHcliNoFreeDirectoryIndex", $reportName);
        exit (-1);
    } else {
        undef %usedIndices;
        return &getDirectory($maxIndex, $reportsDirectory, $reportName);
    }
}

###############################################################################################
#
#            Initialization subroutines - public
#
###############################################################################################

# private method
# recursively makes directories
sub makeDirectory {
    my ($absolutePath, $errorMsg) = @_;
    my (@directories) = split(/\//, $absolutePath);  # / is system dependant

    my ($tmpDirectory);  
    foreach my $subDirectory (@directories) {
        $tmpDirectory .= "$subDirectory/";

        unless (-e $tmpDirectory and -d $tmpDirectory) {
            #if (exists($ENV{'DSH_REPORTS_USE_DEFAULT_PERMISSIONS'})) {
                unless (mkdir($tmpDirectory)) {
                    &sendErrorMessage($errorMsg, $tmpDirectory, $!);
                    exit(-1);
                }
            #} else {
            #    unless (mkdir($tmpDirectory, 0700)) {
            #        &sendErrorMessage($errorMsg, $tmpDirectory, $!);
            #        exit(-1);
            #    }
            #}
        }
    }
}

# Args: $logPath - absolute path with file name of log file
sub initializeSummaryLogging {
    shift;  # get rid of class name
    ($logPath) = @_;

    my ($logDirectory) = 0;

    # first check that log directory is accessible, create it if necessary
    $logDirectory = substr($logPath, 0, rindex($logPath, "/"));

    if (!(-e $logDirectory)) { 
        &makeDirectory($logDirectory, "EMsgDSHcliCannotMakeLogDirectory");       
    } 

    # if log file exists make sure it is writable, otherwise create it to append later.
    if (!(-e $logPath)) {
        unless (open (LOGFILE, ">$logPath")) {
            #print STDERR "dsh: 2617-xxx Could not create log file. $!\n";
            &sendErrorMessage("EMsgDSHcliLogOpenError", $logPath, $!);
            exit(-1);
        }
        close (LOGFILE);  
    } elsif (!(-w $logPath)) {
         #print STDERR "dsh: 2617-xxx Could not write to log file. $!\n";
         &sendErrorMessage("EMsgDSHcliLogWritingError", $logPath, $!);
         exit(-1);
    }
     
    $loggingRequested = 1; # TRUE
}

# TODO verify fanout here
# return updated fanout
# use getrlimit
# returns fanout
# Pre: $reportsDirectory and $fanout must be defined
# Args: ($reportsDirectory, $fanout, $reportName)
sub initializeFileWriting {
    shift;  # get rid of class name
    ($reportsDirectory, $fanout, $reportName) = @_;

    $OUTPUT_FILE_EXTENSION = NodeUtils->getMessage("IMsgDSHcliReportOutputFileExtension");
    $ERROR_FILE_EXTENSION  = NodeUtils->getMessage("IMsgDSHcliReportErrorFileExtension");

    $reportName = NodeUtils->getMessage("IMsgDSHcliUnspecified") unless defined $reportName;  

    &makeDirectory($reportsDirectory, "EMsgDSHcliCannotMakeReportDirectory");

    # if outputDirectory not set yet, get a unique one
    (!$outputDirectory) && ($outputDirectory = &getUniqueDirectory($reportName, $reportsDirectory));

    # Add code here
    # TODO verify fanout here
    # return updated fanout
    # use getrlimit
    # handleErrorMessage for dcemMon, logging and reporting that informs of temporary fanout
    my ($restrictedFanout) = $fanout;  # = getrlimit() / 2;
    if ($restrictedFanout < $fanout) {
        # don't check request var's because they may not be set yet, add report and log messages
        # just in case
        &reportInfoMessage("IMsgDSHcliFanoutRestricted", $fanout, $restrictedFanout);
        &logInfoMessage("IMsgDSHcliFanoutRestricted", $fanout, $restrictedFanout);

        &sendInfoMessage("IMsgDSHcliFanoutRestricted", $fanout, $restrictedFanout);
    } 
     
    &makeDirectory($outputDirectory, "EMsgDSHcliCannotMakeReportDirectory");

    $fileWritingRequested = 1; # TRUE

    return $restrictedFanout;
}

# Args: ($reportsDirectory, $reportName)
sub initializeReporting {

    shift;  # get rid of class name
    ($reportsDirectory, $reportName) = @_;

    $reportName = NodeUtils->getMessage("IMsgDSHcliUnspecified") unless defined $reportName; 

    &makeDirectory($reportsDirectory, "EMsgDSHcliCannotMakeReportDirectory");

    # if outputDirectory not set yet, get a unique one
    (!$outputDirectory) && ($outputDirectory = &getUniqueDirectory($reportName, $reportsDirectory));

    &makeDirectory($outputDirectory, "EMsgDSHcliCannotMakeReportDirectory");
  
    $reportingRequested = 1; # TRUE
}

# Args: none
sub initializeMonitoring {
    $monitoringRequested = 1; # TRUE
}

###############################################################################################
#
#            Command Info update subroutines - public
#
###############################################################################################

# Args: $commandName
sub setCommandName {
    shift; # get rid of class name
    ($commandName) = @_;
    $commandName = NodeUtils->getMessage("IMsgDSHcliUnspecified") unless defined $commandName; 
}

# Args: $commandDescription
sub setCommandDescription {
    shift; # get rid of class name
    ($commandDescription) = @_;
}

# Args: $runAsUser - must be valid
sub setLogin  {
    shift; # get rid of class name
    ($runAsUser) = @_;
}

# Args: $streamingRequested
sub setStreaming {
    shift; # get rid of class name
    ($streamingRequested) = @_;
}

# Args: $verifyRequested
sub setVerify {
    shift; # get rid of class name
    ($verifyRequested) = @_;
}

# Args: $timeout
sub setTimeout {
    shift; # get rid of class name
    ($timeout) = @_;
}

# Args: $fanout
sub setFanout {
    shift; # get rid of class name
    ($fanout) = @_;
}

# Args: $remoteShellPath
sub setRemoteShellPath {
    shift; # get rid of class name
    ($remoteShellPath) = @_;
}

# Args: $remoteShellOptions
sub setRemoteShellOptions {
    shift; # get rid of class name
    ($remoteShellOptions) = @_;
}

# Args: comma separated string of hostnames
sub setHosts {
    shift; # get rid of class name
    my ($hostNames) = @_;

    # don't want to worry about this arrays changing later, so copying them
    @hostNames = split(/,/, $hostNames);

#print "setting hosts = @hostNames\n";  
}

# Args: comma separated string of group names
sub setGroups {
    shift; # get rid of class name
    my ($groupNames) = @_;

    # don't want to worry about this arrays changing later, so copying them
    @groupNames = split(/,/, $groupNames);

#print "setting groups = @groupNames\n";
}
       
# Passes the remote command which may include an export PATH statement, this statement is parsed
# to include as a separate field in the report and log. 
# Args: ($command)
sub setCommand {
    shift; # get rid of class name
    my ($command) = @_;
    my ($exportStatement);
    
    if ($command =~ /^\s*export\s+PATH\s*=/) {      
        ($exportStatement, $commandDefinition) = split(';', $command, 2);
        $exportStatement =~ s/^\s*export\s+PATH\s*=\s*//;  # remove export PATH =
        #$exportStatement =~ s/;\s*$//;                    # remove ;
        $commandPath = $exportStatement;
    } else {
        $commandDefinition = $command;
    }
#print "In setCommand->\n"; &dumpCommandInfo();

}

###############################################################################################
#
#            Execution state update subroutines - public
#
###############################################################################################

sub setStartTime {
    $startTime = localtime();
}

# DshUtils does not try to report or log until this state is reached
sub setInitializationCompleted {
    $initCompleted = 1;
    $monitoringRequested && &sendInitializationCompletedMessage();
}

sub setVerifyingHosts {
    $monitoringRequested && &sendVerifyingHostsMessage();
}

sub setRemoteExecutionStarted {
    $remoteExecutionStarted = 1;
    $monitoringRequested && &sendRemoteExecutionStartedMessage();
#print "setting remote Ex started\n";
}

# A host status list is set to pending for all hosts in @nl. This list already contains any hosts that were 
# removed from @nl as a result of the get_nl method call. The handleConnectionFailure and handleExecutionError
# calls update the status list to $EXECUTION_FAILED. Calls to these methods before setPendingHosts, add the 
# appropriate entries to $executionStatus. The setPendingHosts() call is made after the get_nl call because
# the hostnames in @nl need to match the user specified  host names before setPendingHosts() is called.
# Pre: @nl contains a list of the user specified host names. Any hosts that were removed from this list
#      must have invoked handleConnectionFailure or handleExecutionError in order to have the hosts status
#      entered into $executionStatus. 
#      $executionStatus{hosts that had an error before setPendingHosts invoked} = $EXECUTION_FAILED 
# Post: $executionStatus contains all hosts specified by the user. 
#       $executionStatus{hosts in @nl} = $EXECUTION_PENDING
#       $executionStatus{hosts that had an error before setPendingHosts invoked} = $EXECUTION_FAILED 
sub setPendingHosts {
    shift;  # get rid of class name
    my ($hosts_ref) = @_;
    my ($host) = 0;

    #if ((@$hosts_ref > 0)

    foreach $host (@$hosts_ref) {
#print "pending host = $host\n";
        $executionStatus{$host} = $EXECUTION_PENDING;
    }
    $targetUpdatesCompleted = 1; # TRUE
}

sub setRemoteCommandStarted {
    shift;  # get rid of class name   
    my ($hostname) = @_;
#print "starting host = $hostname\n";
    $executionStatus{$hostname} = $EXECUTION_STARTED;

    $monitoringRequested && &sendRemoteCommandStartedMessage($hostname);
}

sub setRemoteReturnCode {
    shift;  # get rid of class name
    my($hostname, $rc) = @_;
    $remoteRC{$hostname} = $rc
}

sub setRemoteCommandCompleted {
    shift;  # get rid of class name
    my ($hostname) = @_;



    if (exists $remoteRC{$hostname} && !$remoteRC{$hostname}) {
        # then command was successful
#print "setRemoteCommandCompleted success $hostname\n";

        $monitoringRequested && (!&executionStatusCompleted($hostName)) && &sendSuccessfulMessage($hostname);
        $executionStatus{$hostname} = $EXECUTION_SUCCESSFUL;

    } elsif (exists $remoteRC{$hostname}) {  # then the rc must not be zero
        DshUtils->handleExecutionError("EMsgDSHcliNonZeroRemoteReturnCode", $hostname, $remoteRC{$hostname});
        if (&isExitPending()) {
            DshUtils->handleExecutionError("EMsgDSHcliRunningInterrupt", $hostname);
        } 
    } else {  # we never got the rc, 
        DshUtils->handleExecutionError("EMsgDSHcliRemoteReturnCodeNotFound", $hostname);
        if (&isExitPending()) {
            DshUtils->handleExecutionError("EMsgDSHcliRunningInterrupt", $hostname);
        } 
    }
    undef $remoteRC{$hostname};

    #$monitoringRequested && &sendRemoteCommandCompletedMessage($hostname); 
}

sub setEndTime {
    $endTime = localtime();
}

###############################################################################################
#
#            General message handling subroutines - public
#
###############################################################################################

# Info messages are sent to the console/monitoring, log status if logging, and report status
# if reporting
sub handleInfoMessage {
    my ($messageName, @msgArgs) = @_;

    &sendInfoMessage($messageName, @msgArgs);

    if ($reportingRequested) {   
        &reportInfoMessage($messageName, @msgArgs);
    }

    if ($loggingRequested) {
        &logInfoMessage($messageName, @msgArgs);
    }  
}

# Non-fatal error messages are sent to the console/monitoring, log status if logging, and report status
# if reporting
sub handleErrorMessage { 
    my ($messageName, @msgArgs) = @_;

    &sendErrorMessage($messageName, @msgArgs);

    if ($reportingRequested) {   
        &reportErrorMessage($messageName, @msgArgs);
    }

    if ($loggingRequested) {
        &logErrorMessage($messageName, @msgArgs);
    }  
}

###############################################################################################
#
#            Per Host Output file writing subroutines - public
#
###############################################################################################

sub appendOutputLines {
    shift;  # get rid of class name
    my ($hostname, $last_ind, $lines_ref) = @_;
    my ($outputfh) = 0;

   
    if (($last_ind >= 0) && ($outputfh = &getOutputFileHandle($hostname))) {

        for (my $lind = 0; $lind <= $last_ind; $lind++) {
            &printLine($outputfh, $$lines_ref[$lind], "handleOutputWriteError") || last;
        } 
    }
}

sub appendOutputLine {
    shift;  # get rid of class name
    my ($hostname, $line_ref) = @_;
    my ($outputfh) = 0;

    if ($outputfh = &getOutputFileHandle($hostname)) {
        &printLine($outputfh, $$line_ref, "handleOutputWriteError");
    }
}
 
sub appendAllOutputLines {
    shift;  # get rid of class name
    my ($hostname, $lines_ref) = @_;
    my ($outputfh) = 0;

    if ((@$lines_ref > 0) && ($outputfh = &getOutputFileHandle($hostname))) {

        foreach my $line (@$lines_ref) {          
            &printLine($outputfh, $line, "handleOutputWriteError") || last;
        } 
    }         
} 

sub appendErrorLines {
    shift;  # get rid of class name
    my ($hostname, $last_ind, $lines_ref) = @_;
    my ($outputfh) = 0;
    my ($errorfh) = 0;

    if (($last_ind >= 0) && ($outputfh = &getOutputFileHandle($hostname))
        && ($errorfh = &getErrorFileHandle($hostname))) {

        for (my $lind = 0; $lind <= $last_ind; $lind++) {         
            &printLine($outputfh, $$lines_ref[$lind], "handleOutputWriteError") || last;                
            &printLine($errorfh, $$lines_ref[$lind], "handleOutputWriteError") || last;
        }
    }         
}

sub appendErrorLine {
    shift;  # get rid of class name
    my ($hostname, $line_ref) = @_;
    my ($outputfh) = 0;
    my ($errorfh) = 0;

    if ($outputfh = &getOutputFileHandle($hostname)) {        
        &printLine($outputfh, $$line_ref, "handleOutputWriteError") || return;   
    }

    if ($errorfh = &getErrorFileHandle($hostname)) {        
        &printLine($errorfh, $$line_ref, "handleOutputWriteError") || return;   
    }
}
  
sub appendAllErrorLines {
    shift;  # get rid of class name
    my ($hostname, $lines_ref) = @_;
    my ($outputfh) = 0;
    my ($errorfh) = 0;

    if ((@$lines_ref > 0) && ($outputfh = &getOutputFileHandle($hostname))
        && ($errorfh = &getErrorFileHandle($hostname))) {

        foreach my $line (@$lines_ref) {       
            &printLine($outputfh, $line, "handleOutputWriteError") || last;              
            &printLine($errorfh, $line, "handleOutputWriteError") || last; 
        }
    }         
} 

# Arg: scalar hostname
sub closeOutputFile {
    shift;  # get rid of class name
    my ($hostName) = @_;
    my ($fileHandle) = 0;

    # if fileHandle exists
    if ($fileHandle = $outputFiles{$hostName}) {
        close($fileHandle);
        $outputFiles{$hostName} = 0;
    }
}

# Arg: scalar hostname
sub closeErrorFile {
    shift;  # get rid of class name
    my ($hostName) = @_;
    my ($fileHandle) = 0;

#print STDOUT "closing error file fh = $errorFiles{$hostName}, host = $hostName\n";
    # if fileHandle exists
    if ($fileHandle = $errorFiles{$hostName}) {
        close($fileHandle);
        $errorFiles{$hostName} = 0;
    }
}

###############################################################################################
#
#            Dsh Per Host Error handling subroutines - public
#
###############################################################################################


# host specific error msg, non-fatal, we don't exit because of this message
sub handleConnectionFailure {
    shift;  # get rid of class name
    my ($messageName, $hostName, @msgArgs) = @_;
    my ($message, $lMessage);

    $message = NodeUtils->getMessage($messageName, "dsh", $hostName, @msgArgs);
    #$message =~ s/  TODO remove error code and host name

    # "Connection failed:\n\t{0}"
    $lMessage = NodeUtils->getMessage("IMsgDSHcliConnectionFailure", $message);

    if ($fileWritingRequested) {
        &appendErrorMessage(\$lMessage, $hostName);
    } else { 
        &sendErrorMessage($messageName, $hostName, @msgArgs);
    }

    if ($monitoringRequested && !&executionStatusCompleted($hostName)) {
        &sendCancelledMessage($hostName);
    }

    $executionStatus{$hostName} = $EXECUTION_CANCELLED;
}

# host specific error msg, non-fatal, we don't exit because of this message
sub handleExecutionError {
    shift;  # get rid of class name
    my ($messageName, $hostName, @msgArgs) = @_;
    my ($message, $lMessage);

    $message = NodeUtils->getMessage($messageName, "dsh", $hostName, @msgArgs);

    #$message =~ s/  TODO remove error code and host name

    # "Execution Error:\n\t{0}"
    $lMessage = NodeUtils->getMessage("IMsgDSHcliExecutionError", $message);

    if ($fileWritingRequested) {
        &appendErrorMessage(\$lMessage, $hostName);
    } else {
        &sendErrorMessage($messageName, $hostName, @msgArgs);
    }

    if ($monitoringRequested && !&executionStatusCompleted($hostName)) {
        &sendFailedMessage($hostName);
    }
 
    $executionStatus{$hostName} = $EXECUTION_FAILED;
}

# host specific error msg, non-fatal, we don't exit because of this message
# invoked for the following reasons:
# Hostname Unknown
# Hostname Used
# Host Not Responding
# In each case, the command for this target to not executed.
sub handleExecutionCancelled {
    shift;  # get rid of class name
    my ($messageName, $hostName, @msgArgs) = @_;
    my ($message, $lMessage);

    $message = NodeUtils->getMessage($messageName, "dsh", $hostName, @msgArgs);
    #$message =~ s/  TODO remove error code and host name

    # "Execution Cancelled:\n\t{0}"
    $lMessage = NodeUtils->getMessage("IMsgDSHcliExecutionCancelled", $message);

    if ($fileWritingRequested) {
        &appendErrorMessage(\$lMessage, $hostName);
    } else {
        &sendErrorMessage($messageName, $hostName, @msgArgs);
    }

    if ($monitoringRequested && !&executionStatusCompleted($hostName)) {
        &sendCancelledMessage($hostName);
    }

    $executionStatus{$hostName} = $EXECUTION_CANCELLED;
}

# This method creates individual timeout messages when writing to output files.
# If output files are not being written then a message containing all hosts is
# sent to the console, and report and log if requested.
sub handleTimeoutError {
    shift;  # get rid of class name
    my ($hostsRef) = @_; 

    if ($fileWritingRequested) { 
        # then convert to individual host messages, don't need to send to report or log or monitoring
        foreach my $host (@$hostsRef) {
            DshUtils->handleExecutionError("EMsgDSHcliProcessTimeout", $host);
        }           
    } else { # otherwise send out a general message 
        my ($msgArg) = (join ",", @$hostsRef);
        &sendErrorMessage("EMsgDSHcliTimeout", $msgArg);  # sent to monitoring app or console
        if ($loggingRequested) {
            &logErrorMessage("EMsgDSHcliTimeout", $msgArg);
        }
        if ($reportingRequested) {
            &reportErrorMessage("EMsgDSHcliTimeout", $msgArg);
        }       
    }
    #if ($monitoringRequested) {  # This should not be necessary, the msg should get sent after pipes finish.
    #    foreach $host (@hs) {
    #        (!&executionStatusCompleted($hostName)) && &sendFailedMessage($host);
    #    }
    #}
}

###############################################################################################
#
#            Exit subroutines - public
#
###############################################################################################
   
sub isExitPending {
    return $exitPending;
}

sub setExitPending {
    $exitPending = 1; # TRUE
}

# Method handles non-host-specific error messages that require dsh parent process to exit.
# If $remoteExecutionStarted is false, then it is safe to log and report since no child 
# processes have started.
# If $remoteExecutionStarted is true, then we are not in a safe place to exit because
# child processes may still be running. We have to add appropriate messages to 
# output files, report and log that describe the indeterminate state we are in when
# the fatal error occurred. Some child processes may have time to finish while this routine is running,
# the parent process will never read anymore output so the execution status variables will not 
# change once we are in this method.
# Pre: handleFatalError() should not be called by dsh until after $initCompleted 
#      Call only from parent dsh process.
#      Call only after initialization completed because we don't need to log or report until after.
# $messageName is the bundle name for the message
# @msgArgs is a list of strings to be substituted into the message
# Post:  logging and reporting will be completed.
sub handleFatalError {
    &setExitPending();
    shift;  # get rid of class name
             
    my ($messageName, @msgArgs) = @_;
   
    # always send error message, with or without $monitoringRequested
    &sendErrorMessage($messageName, @msgArgs);

    if (!$initCompleted) {       # just in case it is called before initCompleted
        # remove outputDirectory, nothing will have been written here yet
        if ($outputDirectory) {
            rmdir($outputDirectory);
        }
        exit(-1);
    }       
       
    # handle host specific error messages       
    while (my ($hostname, $status)=each %executionStatus) {

        # handle an execution error msg for running hosts
        # Execution Error: "Running of the command on this host has been interrupted due to
        # an unrecoverable error. As a result of this action, the command may not have completed
        # properly on this host. Also, the output file for this host may be incomplete even if 
        # the command completed properly. See log or report for more information on the specific 
        # error."

        ($status == $EXECUTION_STARTED) && DshUtils->handleExecutionError("EMsgDSHcliRunningError", $hostname);

        # handle cancel messages for pending hosts
        # Execution Cancelled: "Running of the command on this host has been cancelled due to
        # an unrecoverable error. The command was never sent to this host.
        
        ($status == $EXECUTION_PENDING) && DshUtils->handleExecutionCancelled("EMsgDSHcliPendingError", $hostname);
          
    }

    # send monitoring message that command execution aborted
    # "Command execution ended prematurely due to a previous unrecoverable error."
    
    &sendInfoMessage("IMsgDSHcliFatalErrorOccurred");
    (!$remoteExecutionStarted) && &sendInfoMessage("IMsgDSHcliInterruptBeforeExecution");

    &setEndTime();

    if ($reportingRequested) {
        # add error message to report message list
        &reportErrorMessage($messageName, @msgArgs);
        &reportInfoMessage("IMsgDSHcliFatalErrorOccurred");
        (!$targetUpdatesCompleted) && &reportInfoMessage("IMsgDSHcliInterruptBeforeNodeEval");
        (!$remoteExecutionStarted) && &reportInfoMessage("IMsgDSHcliInterruptBeforeExecution");

        my ($reportPath);

        # generate report
        if (($reportPath) = &generateReport()) {

            # if report generated and monitoring, send 
            $monitoringRequested && &sendReportGeneratedMessage($reportPath);
            $loggingRequested && &logInfoMessage("IMsgDSHcliLogReportGenerated", $reportPath);
        }
    }   

    if ($loggingRequested) {
        # add error message to log message list
        &logErrorMessage($messageName, @msgArgs);
        &logInfoMessage("IMsgDSHcliFatalErrorOccurred");
        (!$targetUpdatesCompleted) && &logInfoMessage("IMsgDSHcliInterruptBeforeNodeEval");
        (!$remoteExecutionStarted) && &logInfoMessage("IMsgDSHcliInterruptBeforeExecution");

        (!$reportingRequested) && &logInfoMessage("IMsgDSHcliLogReportNotRequested");

        # update log
        if (&updateLog()) {
                   
            # if log updated and monitoring, send
            $monitoringRequested && &sendLogUpdatedMessage();
        }
    }  
    
    $monitoringRequested && &sendRemoteExecutionCompletedMessage();
        
    exit(-1);  
}

######################################

sub handleStop {
    &setExitPending();
    #shift;  # get rid of class name
    #my ($messageName, @msgArgs) = @_;
   
    # always send error message, with or without $monitoringRequested
    #&sendErrorMessage($messageName, @msgArgs);

    if (!$initCompleted) {       # just in case it is called before initCompleted
        # remove outputDirectory, nothing will have been written here yet
        if ($outputDirectory) {
            rmdir($outputDirectory);
        }
        return;
    }       
       
    # handle host specific error messages       
    while (my ($hostname, $status)=each %executionStatus) {
        # handle an execution error msg for running hosts
        # Execution Error: "Running of the command on this host has been interrupted due to
        # an unexpected error or a stop request from the user. As a result of this action, the 
        # command may not have completed properly on this host. Also, the output file for this host
        # may be incomplete even if the command completed properly. See log or report for more 
        # information on the specific error."

        ($status == $EXECUTION_STARTED) && DshUtils->handleExecutionError("EMsgDSHcliRunningInterrupt", $hostname);

        # handle cancel messages for pending hosts
        # Execution Cancelled: "Running of the command on this host has been cancelled due to
        # an unexpected error or a stop request from the user. The command was never sent to
        # this host.
        
        ($status == $EXECUTION_PENDING) && DshUtils->handleExecutionCancelled("EMsgDSHcliPendingInterrupt", $hostname);
          
    }

    # send monitoring message that command execution aborted
    # "Command execution ended prematurely due to a previous unrecoverable error or a stop request from the user."
    &sendInfoMessage("IMsgDSHcliFatalErrorOrStop");
    (!$remoteExecutionStarted) && &sendInfoMessage("IMsgDSHcliInterruptBeforeExecution");

    &setEndTime();
     
    if ($reportingRequested) {
        # add error message to report message list
        #&reportErrorMessage($messageName, @msgArgs);
        &reportInfoMessage("IMsgDSHcliFatalErrorOrStop");
        (!$targetUpdatesCompleted) && &reportInfoMessage("IMsgDSHcliInterruptBeforeNodeEval");
        (!$remoteExecutionStarted) && &reportInfoMessage("IMsgDSHcliInterruptBeforeExecution");

        # generate report
        if (($reportPath) = &generateReport()) {
 
            # if report generated and monitoring, send 
            $monitoringRequested && &sendReportGeneratedMessage($reportPath);
            $loggingRequested && &logInfoMessage("IMsgDSHcliLogReportGenerated", $reportPath);
        }
    }   

    if ($loggingRequested) {
        # add error message to log message list
        #&logErrorMessage($messageName, @msgArgs);
        &logInfoMessage("IMsgDSHcliFatalErrorOrStop");
        (!$targetUpdatesCompleted) && &logInfoMessage("IMsgDSHcliInterruptBeforeNodeEval");
        (!$remoteExecutionStarted) && &logInfoMessage("IMsgDSHcliInterruptBeforeExecution");

        (!$reportingRequested) && &logInfoMessage("IMsgDSHcliLogReportNotRequested");

        # update log
        if (updateLog()) {
                   
            # if log updated and monitoring, send
            $monitoringRequested && &sendLogUpdatedMessage();
        }
    }  
    
    $monitoringRequested && &sendRemoteExecutionCompletedMessage();
}

#######################################

sub handleExit { 
    &setExitPending();   
    #shift;  # get rid of class name  
    #my ($messageName, @msgArgs) = @_;

    if (!$initCompleted) {       # just in case it is called before initCompleted
        # remove outputDirectory, nothing will have been written here yet
        if ($outputDirectory) {
            rmdir($outputDirectory);
        }
        return;
    }       
     
    &setEndTime();

    if ($reportingRequested) {
        my ($reportPath);

        # report completion status
        if (scalar(keys %errorFiles)) {  # errors occurred
            &reportInfoMessage("IMsgDSHcliExecutionCompletedWithErrors");
        } else {
            &reportInfoMessage("IMsgDSHcliCommandExecutionCompleted");
        }

        # generate report
        if (($reportPath) = &generateReport()) {
 
            # if report generated and monitoring, send 
            $monitoringRequested && &sendReportGeneratedMessage($reportPath);
            $loggingRequested && &logInfoMessage("IMsgDSHcliLogReportGenerated", $reportPath);
        }
    }   

    if ($loggingRequested) {
  
        # log completion status
        if (scalar(keys %errorFiles)) {  # errors occurred
            &logInfoMessage("IMsgDSHcliExecutionCompletedWithErrors");
        } else {
            &logInfoMessage("IMsgDSHcliCommandExecutionCompleted");
        }

        (!$reportingRequested) && &logInfoMessage("IMsgDSHcliLogReportNotRequested");

        # update log
        if (&updateLog()) {
                   
            # if log updated and monitoring, send
            $monitoringRequested && &sendLogUpdatedMessage();
        }
    }  
    
    if ($monitoringRequested) {

        if (scalar(keys %errorFiles)) {  # errors occurred
            &sendInfoMessage("IMsgDSHcliExecutionCompletedWithErrors");
        #} else {   # this is redundant, already sending completed protocol message
        #    &sendInfoMessage("IMsgDSHcliCommandExecutionCompleted");
        }

        &sendRemoteExecutionCompletedMessage();
    }
    
}

#######################################

# Lists the last command or commands run, includes command name, start and end time 
# Arg: number of commands to list
sub handleLastCommandsRun {
    shift;
    my ($lastNumRun) = @_;
    if ($lastNumRun < 1) {
        return;
    }
    my ($commandNameLabel) = NodeUtils->getMessage("IMsgDSHcliLogCommandName");
    my (@commandList) = `egrep '$commandNameLabel' $logPath | tail -n $lastNumRun`;
 
    my ($startTimeLabel) = NodeUtils->getMessage("IMsgDSHcliLogStarted");
    my (@startTimeList) = `egrep '$startTimeLabel' $logPath | tail -n $lastNumRun`;

    my ($endTimeLabel) = NodeUtils->getMessage("IMsgDSHcliLogEnded");
    my (@endTimeList) = `egrep '$endTimeLabel' $logPath | tail -n $lastNumRun`;
  
    for (my $i = scalar(@commandList) - 1; $i >= 0; --$i) { 
       
        $commandList[$i] =~ s/^$commandNameLabel//;
        print STDOUT "$commandList[$i]";
        $startTimeList[$i] =~ s/^$startTimeLabel//;
        print STDOUT "$startTimeList[$i]";
        $endTimeList[$i] =~ s/^$endTimeLabel//;
        print STDOUT "$endTimeList[$i]\n";
    }
}

sub sendDebugMessage {
   shift;
   my ($message) = @_;
    unless (open (D_HANDLE, ">> /dcem/reports/tmpout")) {
       
        return 0;
    }

    print D_HANDLE "$message";
   close (D_HANDLE);
}

###############################################################################################
#
#            Debug subroutines - private
#
###############################################################################################

sub dumpCommandInfo {

    print "CommandName: $commandName\n";
    print "CommandDescription: $commandDescription\n";
    print "CommandPath:$commandPath\n";
    print "CommandDefinition: $commandDefinition\n";
    print "RunAsUser: $runAsUser\n";
    print "StartTime: $startTime\n";
    print "EndTime: $endTime\n";
    print "Hosts: @hostNames\n";
    print "Groups: @groupNames\n";
    print "Verify: $verifyRequested\n";
    print "Streaming: $streamingRequested\n";
    print "Timeout: $timeout\n";
    print "Fanout: $fanout\n";
    print "RemoteShellPath: $remoteShellPath\n";
    print "RemoteShellOptions: $remoteShellOptions\n";
}


1;
