#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2003 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

# sccsid = "@(#)27   1.8   src/csm/probemgr/probes/csm_probes/ibm.csm.ms.perl, probemgr, csm_rsirl, rsirl0403a 9/3/03 15:09:05"

$::STD_PROBE_UTILDIR ="/opt/diagnostics/utilities";
if($ENV{"STD_PROBE_UTILDIR"}){
        $::STD_PROBE_UTILDIR=$ENV{"STD_PROBE_UTILDIR"};
}

BEGIN { $::csmpm = $ENV{'CSM_PM'} ? $ENV{'CSM_PM'} : '/opt/csm/pm'; }
use lib $::csmpm;
use Data::Dumper;
use CSMDefs;
use InstallUtils;

require "$::STD_PROBE_UTILDIR/ProbeSupport.pm";

my $INSTALLMS_LOG_FILE = "/var/log/csm/installms.log";

$::MSGCAT       = 'csmProbes.cat';
$::MSGMAPPATH   = $ENV{'CSM_MSGMAPS'} ? $ENV{'CSM_MSGMAPS'}: '/opt/diagnostics/msgmaps';
$::MSGSET       = "ms";

sub printdep() {

	 if ( ProbeSupport->node_type() ne "ManagementServer" ) {
		  print "";
	 }
	 else {
		  print "rmc:dmsrm:errm:ibm.csm.predefined_conditions:ibm.csm.predefined_nodegroups"; 
	 }
}

ProbeSupport->probe_init();
ProbeSupport->parseargs();



if ( ProbeSupport->node_type() ne "ManagementServer" ) {
	 ProbeSupport->printattn('IMsgNotOnMS');
	 ProbeSupport->probe_exit();	 
}

################################################################################
# Fetch the message that indicates installms success in the log file.
# Grep for that in the log file. If its not there, show the messages.
################################################################################
# This check can be enabled later if its needed. For now, its disbaled.
#  if ( ProbeSupport->isLinux() ) {
#  	 # only Linux based MS use the installms command.
#    	 $SuccessMessage = NodeUtils->getMessageFromCat('csmInstall', $::MSGMAPPATH,
#    																	'csmInstall', 'IMsgINSTALLMS_SUCCESSFUL');
#    	 if ( $SuccessMessage && -f $INSTALLMS_LOG_FILE ) {
#    		  ProbeSupport->printtrace('IMsgCheckInstallMSTraceFile',$INSTALLMS_LOG_FILE, $SuccessMessage);
#    	 }
#  }

###############################################################################
# Check installed packages
################################################################################
if ( ProbeSupport->isLinux() ) {
	 $pack_query = "rpm -qa";
}
elsif ( ProbeSupport->isAix() ) {
	 $pack_query = " lslpp -lJ | grep -v '#'";
}
else { die; }

################################################################################
# Check that the main csm directories are present
################################################################################
# TODO : verify these directories - add AIX specifc ones

if ( ProbeSupport->isLinux() ) {

	 @Dirs = ( 
				  $::CSMTOP,
				  $::CSMBIN,
				  $::CSMCSMBIN,
				  $::OPTCSMINSTALL,
				  $::RSCTBIN,
#			 $::CSMSTATUSDIR,
				  $::CSMINSTDIR,
#			 $::CSMCFGDIR,
#			 $::CSMSTATUSLINK,
				  $::CSMLOG,
				  $::OSDEFS,
				  $::PKGDEFS 
				  );
}
elsif ( ProbeSupport->isAix() ) {
	 @Dirs = ( 
				  $::CSMTOP,
				  $::CSMBIN,
				  $::CSMCSMBIN,
#				  $::OPTCSMINSTALL,
				  $::RSCTBIN,
#			 $::CSMSTATUSDIR,
#				  $::CSMINSTDIR,
#			 $::CSMCFGDIR,
#			 $::CSMSTATUSLINK,
				  $::CSMLOG,
#				  $::OSDEFS,
#				  $::PKGDEFS 
				  );
}
	
foreach my $dir ( sort @Dirs ) {
	 ProbeSupport->printtrace('IMsgCheckDirExists', $dir);
	 if ( ! -d $dir ) {
		  ProbeSupport->printerror('EMsgMissingDirectory',$dir);
		  ProbeSupport->probe_exit();
	 }
}


################################################################################
sub check_package_list {
	 my %input = @_;
	 my @list;

	 foreach $item ( @{$input{list}} ) {
		  if ( $item eq "rsct.basic" ) { 
				next;
		  }
		  push ( @list, $item );
	 }

	 if ( scalar ( @list ) <= 0 ) {
		  return;
	 }

	 ProbeSupport->printtrace('IMsgCheckingPackages', join (", ", sort @list ));

	 @all_packages = `$pack_query`;

	 foreach $pack ( @list ) { 

		  # an example of what we are grepping for is this: compat-libstdc++-*
		  # We need to change this to work with grep to   : compat-libstdc\+\+-.*
		  # We make a separate copy so that when print any missing names, they are not munged up.
		  my $temp_pack = $pack;
		  $temp_pack =~ s/\+/\\\+/g;
		  $temp_pack =~ s/\*/.\*/g;

		  @found = grep { /$temp_pack/ } @all_packages;

		  if ( scalar @found < 1 ) {
				# Not installed
				push @Missing, $pack;
		  }
	 }
	 if ( scalar @Missing > 0 ) {
		  $message = join (",", sort @Missing );
		  ProbeSupport->printerror('EMsgMissingPackages',$message);
		  ProbeSupport->probe_exit();
	 }
}

%::Svrpkgdefs = InstallUtils->get_pkgdefs();

check_package_list( list=>\@{$::Svrpkgdefs{rsct_packages}} );
check_package_list( list=>\@{$::Svrpkgdefs{csm_packages}} );
#check_package_list( list=>\@{$::Svrpkgdefs{director_packages}} );
#check_package_list( list=>\@{$::Svrpkgdefs{opensrc_prereqs}} );
check_package_list( list=>\@{$::Svrpkgdefs{os_prereqs}} );


################################################################################
# Check that the cfmupdate cronjob is installed
################################################################################
ProbeSupport->printtrace('IMsgCheckCFMCrontabNotEnabled', $dir);
my @Crons = `crontab -l | grep cfmupdatenode | grep -v '^#'`;
if ( scalar @Crons == 0 ) {
	 ProbeSupport->printattn('IMsgCFMCrontabNotEnabled');
}

# Done
ProbeSupport->probe_exit();











