#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2001,2003 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

package FileUtils;

=item FileUtils.pm

        The best way to read these comments is to go to the
        directory where this file is and type:
        % pod2text FileUtils.pm


        FileUtils.pm has the following functions that may
        be used by the probe developer:

                fileperms
                freespace
                whoowns
                whatgroup

=cut

=item fileperms

	Checks whether the permissions on a specific file match 
	the specified permission. The permission argument is an 
	octal number representing the file permission.

    Usuage = FileUtils->fileperms("<filename>", "<file permissions in octal>"); 

        This function returns 1 if the perms are not equal else 0;
	Its returns 127 if the file is not found, if the 
	parameters passed are incorrect, the stats call to the file
	not defined 

=cut

sub fileperms {
        shift;
	my $file = shift;
	my $on_mode_string = shift;
	if(!defined($file) || !defined($on_mode_string)){
	  	return 127;
	}

#need to check the file existence.

	unless(-e $file) {
		return 127;	
	}

	my $on_mode = oct($on_mode_string);
	my (undef, undef, $mode)=stat($file);
	unless (defined($mode)){
		return 127;
	}
	$mode &= 07777;
	my $blah = $on_mode & $mode;
	if(($mode != $on_mode)){
		return 1;
	}
	else{
		return 0;
	}
}

=item freespace

	Determines the file system on which directory is located
	and then returns the free space on that file system. 
	An absolute path name must be provided for the directory, 
	not the relative one expect the "." ie., current directory.
	
   Usuage = FileUtils->freespace("<Directory name>");
	
	This function returns freespace if the call is success
        else 1. The functions return 127 if the arguments are 
	not defined, not able to run the command mount, if the 
	file system not found in  mount command output, not 
	able to run df command  

=cut


sub freespace {
    shift;
    my $dir = shift;
    if (!defined($dir))
    {
      return 127;
    }
    if ( $dir eq "." )
    {
       $dir =`pwd`;
    }

    my $PLTFRM = `uname`;
    my $Aix_flg = 0;
    chomp($PLTFRM);
    if ($PLTFRM eq "AIX")
    {
       $Aix_flg=1;
    }

    if ( $Aix_flg == 0) 
    {
      # If the os is other than AIX
      # Find what filesystem the specified directory is on
      open MOUNTS, "mount 2>/dev/null |" || return 127;
      my $matchlen=0;
      my $fs="";
      while (<MOUNTS>) 
      {
         ($mntpt) = /\S+ on (\S+) type/;
         if($dir =~ /^$mntpt/) 
         {
            if(length($mntpt) > $matchlen) 
            {
               $matchlen = length($mntpt);
               $fs=$mntpt;
            }
         }
      }
      close MOUNTS;
      return 127 if $fs eq "";   # Worst case scenario is that we $fs="/".
                         # If we match nothing, then / isn't mounted or
                         # something weird has happened in the utility.
                         # I am assuming the later and returning 127.

      open DF, "df -Pk 2>/dev/null |" || return 127;

      while (<DF>) 
      {
         (undef, undef, undef, $avail, undef, $mntpt, undef) = split(/\s+/, $_, 8);
         if ($mntpt eq $fs) 
         {
            #print "$avail\n";
            close DF;
            return $avail;
            #return 0;
         }
      }

     close DF;
     return 1;
   }
   else
   {
     # if the os is AIX
     # Find what filesystem the specified directory is on
     open MOUNTS, "mount 2>/dev/null |" || return 127;
     my $matchlen=0;
     my $fs="";
     while (<MOUNTS>) 
     {
       #($mntpt) = /\S+ on (\S+) type/;
       (undef, undef, $mntpt, undef) = split(/\s+/, $_, 8);
       if($dir =~ /^$mntpt/) 
       {
           if(length($mntpt) > $matchlen) 
           {
               $matchlen = length($mntpt);
               $fs=$mntpt;
           }
       }
     }
     close MOUNTS;

     return 127 if $fs eq "";   # Worst case scenario is that we $fs="/".
                         # If we match nothing, then / isn't mounted or
                         # something weird has happened in the utility.
                         # I am assuming the later and returning 127.

     open DF, "df -Pk 2>/dev/null |" || return 127;

     while (<DF>) 
     {
       (undef, undef, undef, $avail, undef, $mntpt, undef) = split(/\s+/, $_, 8);
       if ($mntpt eq $fs) 
       {
          #print "$avail\n";
          close DF;
          return  $avail;
          #return  0;
       }
     }

     close DF;
     return 1;

   }

}

=item whoowns

	Prints the user who owns the filename.

   Usuage = FileUtils->whoowns("<filename>");

	If successfull , this function returns the user id of the file.
	It returns 127 if the the function is not able to open the 
	/etc/passwd file, if it could not get the stats of the file


=cut


sub whoowns {
   shift;
   my $file_name = shift;

   if (!defined($file_name))
   {
     return 127;
   }

   my $PLTFRM = `uname`;
   my $Aix_flg = 0;
   chomp($PLTFRM);
   if ($PLTFRM eq "AIX")
   {
      $Aix_flg=1;
   }
   
   if ($Aix_flg == 0) 
   {
      #if the os is other than (NOT) AIX
      open PASSWD, "</etc/passwd" ||  return 127;
      while(<PASSWD>) 
      {
          my ($login,undef,$uid,undef) = split(/:/, $_, 4);
          $users{$uid}=$login;
      }
      close PASSWD;

      my (undef, undef, undef, undef, $uid)=stat($file_name);
      return 127 if not defined($uid);

      if (exists $users{$uid}) 
      {
          #print $users{$uid}, "\n";
          return $users{$uid};
      } 
      else 
      {
         #print $uid, "\n";
         return $uid;
      }

    }
    else
    {
      #if the os is AIX

      open PASSWD, "</etc/passwd" ||  return 127;
      while(<PASSWD>) 
      {
         my ($login,undef,$uid,undef) = split(/:/, $_, 4);
         $users{$uid}=$login;
      }
      close PASSWD;

      my (undef, undef, undef, undef, $uid)=stat($file_name);
      return 127 if not defined($uid);

      if (exists $users{$uid}) 
      {
         #print $users{$uid}, "\n";
         return $users{$uid};
      } 
      else 
      {
         #print $uid, "\n";
         return $uid;
      }
    }
}

sub whatgroup {
   shift;
   my $file_name = shift;

   if (!defined($file_name))
   {
     return 127;
   }

   $PLTFRM = `uname`;
   $Aix_flg = 0;
   chomp($PLTFRM);
   if ($PLTFRM eq "AIX")
   {
      $Aix_flg=1;
   }


   if ($Aix_flg == 0) 
   {
      open GROUP, "</etc/group" ||  return 127;
      while(<GROUP>) 
      {
          ($gname,undef,$gid,undef) = split(/:/, $_, 4);
          $groups{$gid}=$gname;
      }
      close GROUP;

      (undef, undef, undef, undef, undef, $gid)=stat($file_name);
      return 127 if not defined($gid);

      if (exists $groups{$gid}) 
      {
          #print $groups{$gid}, "\n";
          return $groups{$gid};
      } 
      else 
      {
          #print $gid, "\n";
          return $gid;
      }

    }
    else
    {
       #if the os is AIX

       open GROUP, "</etc/group" ||  return 127;
       while(<GROUP>) 
       {
          my ($gname,undef,$gid,undef) = split(/:/, $_, 4);
          $groups{$gid}=$gname;
       }
       close GROUP;

       my (undef, undef, undef, undef, undef, $gid)=stat($file_name);
       return 127 if not defined($gid);

       if (exists $groups{$gid}) 
       {
          #print $groups{$gid}, "\n";
          return $groups{$gid};
       } 
       else 
       {
          #print $gid, "\n";
          return $gid;
       }

     }
}

1;
