#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2000,2004 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
package NetworkUtils;


# Global Variables

$::NETWORK_FILE ="/etc/sysconfig/network";
$::NETWORK_SCRIPTS_DIR="/etc/sysconfig/network-scripts";

$::RCCONFIG_FILE = "/etc/rc.config";

if(ProbeSupport->isAix()){
    # These variables mean nothing to AIX right now.
    $::NETWORK_FILE = "";
    $::NETWORK_SCRIPTS_DIR = "";
}
%::NETVALS;

# add for SuSE SLES 7.2 support
%::RCCONFIG;

# add for SuSE 8.0 support
%::NETCFG;

my $distribution=ProbeSupport->getLinuxDistributionName();
# redhat7.x & redhat8.0 both support
if( $distribution=~/Default/ ||
    $distribution =~ /RedHat8.0/||
   $distribution =~ /RedHat9/ || $distribution =~ /RedHatEL/)
{
        unless(open(FILE,"<$::NETWORK_FILE")){ return 127}
        while(my $line = <FILE>){
        chomp $line;
         my ($var,$val) = split('=', $line);
         $::NETVALS{$var} = $val;
        }

         # close the opened file
         close FILE;

}elsif($distribution =~ /SLES7.2/){
        unless(open(FILE,"<$::RCCONFIG_FILE")) {return 127}
        while(my $line = <FILE>){
                chomp($line);
                next if $line=~/^#/;
                my ($name,$value) = split(/=/,$line);
                $::RCCONFIG{$name}=$value;
        }

        close FILE;
}elsif($distribution =~ /SuSE8.0/ ||
	$distribution =~ /SuSE8.2/ ||
    $distribution=~/SuSE8.1/ || $distribution=~/United1.0/ || $distribution=~/SLES8/){
        my $curCwd=$ENV{'PWD'};
        chdir("/etc/sysconfig/network/");
        my $cmdStr="ls ifcfg-*";
        my @adapters=`$cmdStr`;
        if($?) { return 127 }
        chdir($curCwd);
        foreach my $adapter(@adapters){
                chomp($adapter);
                unless(open(FILE,"</etc/sysconfig/network/$adapter")) { return 127; }
                while(my $line = <FILE>){
                        chomp($line);
                        next if $line=~/^#/;
                        my($var,$val)=split('=',$line);
                        $val=~s/[\"\']//g;
                        $::NETCFG{"$adapter"}{"$var"} = $val;
                }
                close FILE;
        }
}


=item functions

	portavail [tcp | udp] port [ip address]
	load_services
	check-ifaces
	check_dev
	check_up
	portpid

=cut


=item portavail

	portavail [tcp | udp] port [ip address] 
	will return 1 if port is in use or a 0 is the port is free.
	If there is a user error it will return 127.

=cut

sub portavail{
    shift;          # Get rid of NetworkUtils
    my ($mode, @myArgs);
    if((grep /tcp/, @_)&&!(grep /udp/, @_)){
	$mode = 'tcp';
    }
    elsif((grep /udp/,@_)&&!(grep /tcp/, @_)){
	$mode = 'udp';
    }
    elsif(!(grep /udp/,@_)&&!(grep /tcp/,@_)){
	$mode = 'tcp';    # default
    }
    return 127 if not defined($mode);
    foreach(@_){
	if($_ ne $mode){ push @myArgs, $_ }
    }
    $netstat_options="-an";
if(ProbeSupport->isLinux()){
    $netstat_options=$netstat_options." --tcp" if $mode eq 'tcp';
    $netstat_options=$netstat_options." --udp" if $mode eq 'udp';
   }

    $port = shift @myArgs;
    $addr = shift @myArgs;
    # Convert hostnames to IP addresses
    if(defined($addr) && 
	not $addr =~ /[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1-3}\.[0-9]{1-3}/) {
      	(undef, undef, undef, undef, @addrs) = gethostbyname($addr);
      	return 127 if not defined(@addrs);
      	$addr=join('.', unpack("C4", $addrs[0]));
    }
    $addr = '*' if not defined($addr);
    # Convert port names to port numbers
    if(not $port =~ /[0-9]+/) {
	my %name2port = NetworkUtils->load_services;        
        $port = $name2port{$port};
      	return 127 if not defined($port);
    }

    if (ProbeSupport->isLinux()) {
	open NETSTAT, "netstat $netstat_options 2>/dev/null | " || return 127;
  	while(<NETSTAT>) {
      	    (undef, undef, undef, $combaddr, undef) = split(/[ ]+/,$_,5);
      	    ($curaddr, $curport) = split(/:/,$combaddr);
      	    next if $curport != $port;
      	    return(1) if (($addr eq '*') || ($curaddr eq $addr) || ($curaddr eq "0.0.0.0"));
  	}
    }
    elsif(ProbeSupport->isAix){
        open NETSTAT, "netstat $netstat_options | grep $mode 2>/dev/null | " || return 127;
        while(<NETSTAT>) {
    	    (undef, undef, undef, $combaddr, undef) = split(/[ ]+/,$_,5);
            ($curaddr=$combaddr)=~ s/(^.*)\..*$/\1/;
            ($curport=$combaddr)=~ s/^.*\.//;
            next if $curport != $port;
            return(1) if (($addr eq '*') || ($curaddr eq $addr) || ($curaddr eq "0.0.0.0"));
	}    
    }
    close(NETSTAT);
    return(0);
}


=item load_services

	load_services
	returns a hash table that maps services to ports.  For example
	ftp has port 21 on a certain system.  Therefore you could say:
	my %name2port = NetworkUtils->load_services();
	my $ftpPortNum = $name2port{'ftp'}; 
        print $ftpPortNum;      # will print 21.

=cut

sub load_services {
    open SERVICES, "</etc/services" or return 127;
    my ($svc, $port, $proto, $aliases,%hash);
    while(<SERVICES>) {
        chomp;
        ($svc, $port, $proto, $aliases) = /^(\S*)\s*(\d*)\/(\S*)\s*([^#]*)/;  
        if(defined($svc)) {
	    $hash{$svc} = $port;
            foreach (split /\s/,$aliases) {
		$hash{$_} = $port;
            }
        }
    }
    close SERVICES;
    return %hash;
}


=item portpid

	portpid

=cut


sub portpid{


my (%hash,$curaddr, $curport, $combaddr, $pid,$prog);
my ($addr,@addrs,$port,$combpid);
my %args;
my $mode;
my $netstat_options;
chomp($PLTFRM);
# if linux
if(ProbeSupport->isLinux()) {
    NetworkUtils->load_services();
  %hash = NetworkUtils->load_services();
    # Get the mode: tcp udp
  exit 127 if not defined($mode);

  $netstat_options="-anp";
  $netstat_options=$netstat_options." --tcp" if $mode eq 'tcp';
  $netstat_options=$netstat_options." --udp" if $mode eq 'udp';
  $port = shift;
  $addr = shift;

  # Convert hostnames to IP addresses
  if(defined($addr) &&
     not $addr =~ /[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1-3}\.[0-9]{1-3}/) {

      (undef, undef, undef, undef, @addrs) = gethostbyname($addr);
      exit 127 if not defined(@addrs);
      $addr=join('.', unpack("C4", $addrs[0]));
  }
  $addr = '*' if not defined($addr);

  # Convert port names to port numbers
  if(not $port =~ /[0-9]+/) {
      $port=$$mode{$port};
      exit 127 if not defined($port);
  }

  ProbeSupport->printtrace('IMsgChkAvail', $addr,$port);   
  open(NETSTAT, "netstat $netstat_options 2>/dev/null | ") or return 127;
  while(<NETSTAT>) {
      (undef, undef, undef, $combaddr, undef, undef, $combpid, undef) =
          split(/[ ]+/,$_,8);
      ($curaddr, $curport) = split(/:/,$combaddr, 2);
      ($pid,$prog) = split(/\//, $combpid, 2);
      next if $curport != $port;
      if (($addr eq '*') || ($curaddr eq $addr) || ($curaddr eq 
"0.0.0.0")) {
          ProbeSupport->printtrace('TMsgPID',$pid);
          exit(0);
      }

  }
  close(NETSTAT);

  exit(1);

}
else
{
#if the os is AIX

  exit 127 if $#ARGV < 1;

  exit 127 if not defined do 'load-services.pl';
  load_services();

  getopts("tu", \%args);
  if ($args{'t'}) {
      $mode='tcp';
  }
  if ($args{'u'}) {
      if(not defined($mode)) {
          $mode='udp';
      } else {
          exit 127;
      }
  }
  exit 127 if not defined($mode);

  $netstat_options="-anp";
  $netstat_options=$netstat_options." --tcp" if $mode eq 'tcp';
  $netstat_options=$netstat_options." --udp" if $mode eq 'udp';

  $port = shift;
  $addr = shift;

  # Convert hostnames to IP addresses
  if(defined($addr) &&
     not $addr =~ /[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1-3}\.[0-9]{1-3}/) {

      (undef, undef, undef, undef, @addrs) = gethostbyname($addr);
      exit 127 if not defined(@addrs);
      $addr=join('.', unpack("C4", $addrs[0]));
  }
  $addr = '*' if not defined($addr);

  # Convert port names to port numbers
  if(not $port =~ /[0-9]+/) {
      $port=$$mode{$port};
      exit 127 if not defined($port);
  }
 ProbeSupport->printtrace('TMsgChkAvail', $addr,$port);

  open NETSTAT, "netstat $netstat_options 2>/dev/null | " || exit 127;
  while(<NETSTAT>) {
      (undef, undef, undef, $combaddr, undef, undef, $combpid, undef) =
          split(/[ ]+/,$_,8);
      ($curaddr, $curport) = split(/:/,$combaddr, 2);
      ($pid,$prog) = split(/\//, $combpid, 2);
      next if $curport != $port;
      if (($addr eq '*') || ($curaddr eq $addr) || ($curaddr eq 
"0.0.0.0")) {
         ProbeSupport->printtrace('TMsgPID', $pid);
          exit(0);
      }


  }
  close(NETSTAT);

  exit(1);

}


}


=item check_dev

	check_dev checks that the device is defined in ifconfig

=cut




sub check_dev{
    if(ProbeSupport->isAix()){ return 0 } # we do nothing for AIX.
    shift;
    my $dev = shift;
    my $strCmd =  "/sbin/ifconfig $dev 2>&1 | grep -s \"not found\"";
    system("$strCmd > /dev/null");
    unless(($? >> 8)){
        return 20;
    }
    return 0;
}

=item check_up

	check_up checks that a device is UP in ifconfig

=cut

sub check_up{
    if(ProbeSupport->isAix()){ return 0 } # we do nothing for AIX.
    shift;
    my $dev = shift;
    my $strCmd =  "/sbin/ifconfig $dev 2>&1 | grep -s \"UP\"";
    system("$strCmd > /dev/null");
    if(($? >> 8)){
        return 20;
    }
    return 0;
}

=item checkIface

	checkIface checks that the interface is consistant in all of the places
	where it is mentioned.  It also calls check_up and check_dev

=cut

sub Pack{
        shift;

        my $saddr=shift;
        my $iaddr=0;
        my $imask=24;

        my @array=split(/\./,$saddr);
        foreach my $n(@array){
                $n &= 0xFF;
                $iaddr |= ($n<<$imask);
                $imask -= 8;
        }

        return $iaddr;
}

sub Unpack{
        shift;

        my $iaddr=shift;
        my $saddr="";

        for(my $i=3;$i>=0;$i--){
                my $n = ($iaddr>>($i*8)) & 0xFF;
                $saddr.=$n;
                $saddr.="\.";
        }
        chop $saddr;
        return $saddr;
}

sub ipcalc_netmask{
        shift;

        my $ipaddr=shift;
        my $iaddr=0;

        $iaddr=NetworkUtils->Pack($ipaddr);
        if( (($iaddr&0xFF000000) >> 24) <= 127 ){ # class A IP
                 return NetworkUtils->Unpack(0xFF000000);
        }elsif( (($iaddr&0xFF000000) >> 24) <=191 ){ # class B IP
                 return NetworkUtils->Unpack(0xFFFF0000);
        }else{
                 return NetworkUtils->Unpack(0xFFFFFF00);
        }

        return undef;
}

sub ipcalc_broadcast{
        shift;

        my ($ipaddr,$netmask)=@_;
        my $iadr=0;
        my $inetmask=0;
        my $ibroadcast=0;

        unless($netmask){
                $netmask=NetworkUtils->Pack($ipaddr);
        }

        $iaddr=NetworkUtils->Pack($ipaddr);
        $inetmask=NetworkUtils->Pack($netmask);

        my $ibroadcast= ($iaddr & $inetmask) | ~$inetmask;

        return NetworkUtils->Unpack($ibroadcast);
}

sub checkIface_SLES{
        shift;
        my ($idx,$dev)=@_;

        my $tmp="IFCONFIG".$idx;
        if(!exists($::RCCONFIG{$tmp})){
                return "/etc/rcconfig";
        }
        my $ifcfg=$::RCCONFIG{$tmp};
        $ifcfg=~s/\"//g;

        my $proto,$netmask,$broadcast,$ipaddr;
        if($ifcfg eq "dhcpclient" || $ifcfg eq "bootp"){
                $proto = "dhcp";
                NetworkUtils->check_dev($dev) && return "/sbin/ifconfig";
                NetworkUtils->check_up($dev)  && return "NOT UP";
        }else{
                $proto = "static";
                my $t1,$t2,$3;
                ($ipaddr,$t1,$broadcast,$t2,$netmask,$t3) = split(/ /,$ifcfg);
                if($t1 ne "broadcast"){
                        return "/etc/rc.config";
                }
                if($t2 ne "netmask"){
                        return "/etc/rc.config";
                }

                unless($broadcast){
                        $broadcast = NetworkUtils->ipcalc_broadcast("broadcast",$ipaddr);
                }
                unless($netmask){
                        $netmask = NetworkUtils->ipcalc_netmask("netmask",$ipaddr);
                }
                NetworkUtils->check_dev($dev) && return "/sbin/ifconfig";
                NetworkUtils->check_up($dev)  && return "NOT UP";

                # Now go through the attributes and see that they match up
                # with the attributes mentioned elsewhere on the system.
                my $strCmd = "/sbin/ifconfig $dev | ";
                $strCmd .= "awk '/inet addr:/{ match(\$0, /inet addr:";
                $strCmd .= "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+/); ";
                $strCmd .= "print substr(\$0, RSTART+10, RLENGTH-10); }'";
                #print "$strCmd\n";
                chomp(my $ipaddr_a = `$strCmd`);
                $ipaddr_a =~ s/\"//g;
                #m Check Broadcast address:
                $strCmd = "/sbin/ifconfig $dev | ";
                $strCmd .= "awk '/Bcast:/{ match(\$0, /Bcast:";
                $strCmd .= "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+/); ";
                $strCmd .= "print substr(\$0, RSTART+6, RLENGTH-6); \}'";
                #print "$strCmd\n";
                chomp(my $broadcast_a = `$strCmd`);
                # Check netmask
                $strCmd = "/sbin/ifconfig $dev | ";
                $strCmd .= "awk '/Mask:/{ match(\$0, /Mask:";
                $strCmd .= "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+/); ";
                $strCmd .= "print substr(\$0, RSTART+5, RLENGTH-5); }'";
                #print "$strCmd\n";
                chomp(my $netmask_a = `$strCmd`);
                $netmask_a =~ s/\"//g;
                if($ipaddr ne $ipaddr_a){ return "IPADDR does not match" }
                if(($broadcast ne $broadcast_a)&&($dev ne "lo")){
                        return "BROADCAST does not match"
                }
        }
        return 0;
}

sub checkIface_SuSE{
        shift;
        my ($adapter,$dev)=@_;

        my $proto=$::NETCFG{$adapter}{"BOOTPROTO"};
        if($proto eq "dhcp"){
                NetworkUtils->check_dev($dev) && return "/sbin/ifconfig";
                NetworkUtils->check_up($dev)  && return "NOT UP";
        }else{
                my $broadcast=$::NETCFG{$adapter}{"BROADCAST"};
                my $netmask=$::NETCFG{$adapter}{"NETMASK"};
                my $ipaddr=$::NETCFG{$adapter}{"IPADDR"};

                if(!$ipaddr){
                        return "/etc/sysconfig/network/$adapter";
                }
                unless($broadcast){
                        $broadcast = NetworkUtils->ipcalc_broadcast("broadcast",$ipaddr);
                }
                unless($netmask){
                        $netmask = NetworkUtils->ipcalc_netmask("netmask",$ipaddr);
                }
                NetworkUtils->check_dev($dev) && return "/sbin/ifconfig";
                NetworkUtils->check_up($dev)  && return "NOT UP";

                #print "DEBUG: hello , I m here...\n";
                # Now go through the attributes and see that they match up
                # with the attributes mentioned elsewhere on the system.
                my $strCmd = "/sbin/ifconfig $dev | ";
                $strCmd .= "awk '/inet addr:/{ match(\$0, /inet addr:";
                $strCmd .= "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+/); ";
                $strCmd .= "print substr(\$0, RSTART+10, RLENGTH-10); }'";
                #print "$strCmd\n";
                chomp(my $ipaddr_a = `$strCmd`);
                $ipaddr_a =~ s/\"//g;
                #m Check Broadcast address:
                $strCmd = "/sbin/ifconfig $dev | ";
                $strCmd .= "awk '/Bcast:/{ match(\$0, /Bcast:";
                $strCmd .= "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+/); ";
                $strCmd .= "print substr(\$0, RSTART+6, RLENGTH-6); \}'";
                #print "$strCmd\n";
                chomp(my $broadcast_a = `$strCmd`);
                # Check netmask
                $strCmd = "/sbin/ifconfig $dev | ";
                $strCmd .= "awk '/Mask:/{ match(\$0, /Mask:";
                $strCmd .= "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+/); ";
                $strCmd .= "print substr(\$0, RSTART+5, RLENGTH-5); }'";
                #print "$strCmd\n";
                chomp(my $netmask_a = `$strCmd`);
                $netmask_a =~ s/\"//g;
                if($ipaddr ne $ipaddr_a){ return "IPADDR does not match" }
                if(($broadcast ne $broadcast_a)&&($dev ne "lo")){
                        return "BROADCAST does not match"
                }
        }
        return 0;
}


sub checkIface{
    if(ProbeSupport->isAix()){
	return 0;       # Right now we do nothing.
    } 
    shift;
    my $config = shift;
    my (%devAttrs,$strCmd);
    unless($config){
        exit 127;    # You have called this function without arguments.
    }
    unless(open(FILE,"<$config")){ return 127}
    while(my $line = <FILE>){
	chomp $line;
	my ($var,$val) = split('=', $line);
	$devAttrs{$var} = $val;
    }
    my $broadcast = $devAttrs{"BROADCAST"};
    $broadcast =~ s/\"//g;
    $broadcast =~ s/\'//g;
    my $device = $devAttrs{"DEVICE"};
    $device =~ s/\'//g;
    $device =~ s/\"//g;
    my $macaddr = $devAttrs{"MACADDR"};
    $macaddr =~ s/\"//g;
    $macaddr =~ s/\'//g;
    my $ipaddr = $devAttrs{"IPADDR"};
    $ipaddr =~ s/\"//g;
    $ipaddr =~ s/\'//g;
    my $netmask = $devAttrs{"NETMASK"};
    $netmask =~ s/\"//g;
    $netmask =~ s/\'//g;
    my $bootproto = $devAttrs{"BOOTPROTO"};
    $bootproto =~ s/\"//g;
    $bootproto =~ s/\'//g;
    if($bootproto eq "bootp" or $bootproto eq "dhcp"){ 
	NetworkUtils->check_dev($device) && return "/sbin/ifconfig";
    	NetworkUtils->check_up($device)  && return "NOT UP";	
    }
    else{
	unless($netmask){
	    $strCmd = "/bin/ipcalc --netmask $ipaddr 2>&1";
	    chomp(my $line = `$strCmd 2>&1`);
	    $netmask = (split('=',$line))[1];
            $netmask =~ s/\"//g;
	}
	unless($broadcast){
	    $strCmd = "/bin/ipcalc --broadcast $ipaddr $netmask";
	    chomp(my $line  = `$strCmd 2>&1`);
	    $broadcast = (split('=',$line))[1];
        }    
	NetworkUtils->check_dev($device) && return "/sbin/ifconfig";
    	NetworkUtils->check_up($device)  && return "NOT UP";	

	if($macaddr){
	    $strCmd = "/sbin/ifconfig $device | grep \"HWaddr\"";
	    $strCmd .= " | awk '{print \$5}'";
	    print $strCmd,"\n";
	    my $macaddr_A = system("$strCmd 2>&1");
	    if($macaddr_A ne $macaddr){ return "MACADDR" }
	}
	# Now go through the attributes and see that they match up
	# with the attributes mentioned elsewhere on the system.
        $strCmd = "/sbin/ifconfig $device | ";
	$strCmd .= "awk '/inet addr:/{ match(\$0, /inet addr:";
	$strCmd .= "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+/); ";
	$strCmd .= "print substr(\$0, RSTART+10, RLENGTH-10); }'";
  	#print "$strCmd\n";
	chomp(my $ipaddr_a = `$strCmd`);
	$ipaddr_a =~ s/\"//g;
	#m Check Broadcast address:
        $strCmd = "/sbin/ifconfig $device | ";
	$strCmd .= "awk '/Bcast:/{ match(\$0, /Bcast:";
	$strCmd .= "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+/); ";
	$strCmd .= "print substr(\$0, RSTART+6, RLENGTH-6); \}'";
  	#print "$strCmd\n";
	chomp(my $broadcast_a = `$strCmd`);
	# Check netmask
        $strCmd = "/sbin/ifconfig $device | ";
	$strCmd .= "awk '/Mask:/{ match(\$0, /Mask:";
	$strCmd .= "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+/); ";
	$strCmd .= "print substr(\$0, RSTART+5, RLENGTH-5); }'";
  	#print "$strCmd\n";
	chomp(my $netmask_a = `$strCmd`);
        $netmask_a =~ s/\"//g;
	if($ipaddr ne $ipaddr_a){ return "IPADDR" }
	if(($broadcast ne $broadcast_a)&&($device ne "lo")){ 
	    return "BROADCAST" 
	}
	if($netmask ne $netmask_a){ return "NETMASK" }
    }
    return 0;
}







1;
