#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2001,2004 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
package ProbeSupport; 

=item ProbeSupport.pm

	The best way to read these comments is to go to the 
	directory where this file is and type:
	% pod2text ProbeSupport.pm

	
	ProbeSupport.pm has the following functions (utilities) that may
	be used by the probe developer:
	        unix_info	
		node_type
		isLinux
		isAix
		parseargs
		printtrace
		printfault
		printattn
		printexpl
		runutility
		probe_init
		probe_exit
		seterror	
	There is also other functions, that should
	not be called by the developer, unless absolutely 
	necessary.  (I cant think of any reasons why you would call them, 
        but maybe you can of reasons.) They are:

		chkExitandPrint
		ignore
		message


=cut



BEGIN { 

    $intDoWarn        = 1;
    # csmpm for development use
    $::csmpm         = $ENV{'CSM_PM'} ? $ENV{'CSM_PM'} : '/opt/csm/pm'; 
    $SIG{'__WARN__'} = sub {warn $_[0] if $intDoWarn };
    $::csmmsgmaps = $ENV{'CSM_MSGMAPS'} ? $ENV{'CSM_MSGMAPS'} : "/opt/diagnostics/msgmaps";

}  
use strict;
use lib $::csmpm;
require NodeUtils;
require CSMDefs;
use Getopt::Std;



#-----------------
# Global Variables
#-----------------


=item global variables

	
	The Probe developer should define the following variables:
           
	MSGCAT - the name of the message catalog containing the messages
	MSGMAPPATH - the path of the message map containing the messages
	MSGSET - the name of the msg set withing the catalog.  If not 
		set, it will default to the program name.

	The above variables are used for NLS support.  If they are
	not defined by the probe, then we assume that they do not
	want NLS support.  Print statements should be done by defining
	a global variable:
	   %::MESSAGES = (
		...
		EMsgCantRun => "Cant run the program %s",
		...
	   );

	 Notice, that these errors should not have \n, since the
	 messaging routine will add these in.


	 $::UTILITY should also be defined if you are writing
	 a utility.  This will only prevent -d from being interpreted
 	 in the command line.

	     

=cut

# The following will be defined in the user probe
$::UTILITY;   			 # 1 if the prog is a utility 
$::MSGCAT;			 # see above explaination.
$::MSGMAPPATH;
$::MSGSET;


$::EXIT_STATUS    = 0;
$::PROGNAME;                           
	
#private variables
my @utilArgs;	     		 # This is where we add args to utilities called
my $intDoWarn        = 1;
my $intPrintLevel    = 3;        # Default level of message output.    
my $strIgnore        = "";       # string where -e args go.
my %EXIT_LEVEL = (
	ERROR     => 20,
	FAULT     => 127,
	ATTN      => 10,
	EXPL      => 0,
	TRACE     => 0
);

my $types = NodeUtils->getMessageFromCat('stdProbes.cat',
					 $::csmmsgmaps,
					 'utils', 'IMsgType_utils');
my @types = split(' ', $types);
my %LEVEL2MSG;
$LEVEL2MSG{'ERROR'} = $types[0];
$LEVEL2MSG{'FAULT'} = $types[1];
$LEVEL2MSG{'ATTN'}  = $types[2];
$LEVEL2MSG{'EXPL'}  = $types[3];
$LEVEL2MSG{'TRACE'} = $types[4];



	
=item parseargs

    parseargs gets the arguments that were passed into it, of course!
    The following are valid options:
    -d This will print a dependency list 
    -e msgnum1,msgnum2,...
       Indicates that the messages identified by the passed message numbers
       must be ignored.  The probe must determine whether ignoring the 
       should affect the exit status or not.

	Question: Does this mean that we do not raise the exit status?
	I think it does.

    -l {|1|2|3|4}  Indicates the lowest level of output that should be
       reported by the probe.  The default is 1, which indicates
       attention and error messages should be reported.
       0 - Trace Messages
       1 - Attention
       2 - Error
    -s Return a one line summary of the purpose of the probe
    -m Return a detailed documentation for the probe.

=cut
sub parseargs {	
	my %opt;
	my $opts = 'l:e:smh';
	unless($::UTILITY){
	   $opts .= 'd';
	}
	my @notParsed = @ARGV;
	$intDoWarn = 0;
	# I dont check for bad options, because utilities may want there own options passed 
	# to them.  Who knows...

	getopts($opts,\%opt); #{ &printfault("FMsgBadArgs") }
	$intDoWarn = 1;
	
	if($opt{'d'}){
		exit 127 if not defined do main::printdep();
		exit 0;
	}
 	if($opt{'l'}){
		
		if($opt{'l'} > 0 && $opt{'l'} < 5){
			$intPrintLevel = $opt{'l'};	
			push @utilArgs, (" -l$opt{'l'}");			
		}
		else{
		    &printhelp;
    		}
	}
	if($opt{'e'}){
		$strIgnore = $opt{'e'};
		push @utilArgs, (" -e$opt{'e'}");
	}	
	if($opt{'s'}){
		$intPrintLevel = 0;
		ProbeSupport->printexpl('IMsgSummary');
		exit 0;
	}
	if($opt{'m'}){
		$intPrintLevel = 0;
		ProbeSupport->printexpl('IMsgDocumentation');
		exit 0;
	}
	if($opt{'h'}){
		&printhelp();
		exit 0;
	} 
	#push @utilArgs;# @_;
	#print @utilArgs;
	return @notParsed;
}

=item isLinux & isAix

	isLinux returns true if you are running on linux; false otherwise.
	isAix returns true if you are running on AIX; false otherwise.
	what a magical and symmetrical functions!

=cut

sub isLinux{
	return NodeUtils->isLinux();
}

sub isAix{
	return NodeUtils->isAIX();
}


=item printhelp

	printhelp just prints the usage message if you run the probe alone.

=cut

sub printhelp {
    my $msg;
    
    if($::UTILITY){
    	$msg = NodeUtils->getMessageFromCat('stdProbes.cat', $::csmmsgmaps, 'utils', 'IMsgUtilHelp_utils');
    }
    else{
    	$msg = NodeUtils->getMessageFromCat('stdProbes.cat', $::csmmsgmaps, 'utils', 'IMsgProbeHelp_utils');
    }
    $msg .= NodeUtils->getMessageFromCat('stdProbes.cat', $::csmmsgmaps, 'utils', 'IMsgCommonHelpMessage_utils');
    print $::PROGNAME . " " . $msg;
}


=item printattn, printtrace, printfault, printerror

 	The print* functions set the exit code and print the
	message.  They will use NLS if the $::MSG* variables
	are defined.  Otherwise they will look for the %::MESSAGES
	hash table and print the values found in there.  For each
	message you should give it a string name.  The name should start with
	either EMsg* or IMsg*.  For example, we have one called IMsgOS that
	has the value 'this machine is running %s'.  There is a global
	variable that contains the error value.  Each print* function will
	set the error number to the highest value:
	printfault -> 127
	printerror -> 20
	printattn -> 10
	printtrace -> 0
	printexpln -> 0

	The user can change the error value by calling ProbeSupport->seterror($val).

	The print functions should be called with the message name as the first
	value followed by the arguments.  Heres an example using printtrace:
	ProbeSupport->printtrace( 'IMsgString', $value1,$value2,...)

	Note: if the error value in this example was already at 20, then printtrace will not reset
	it to 0.


=cut 



sub printtrace {
	 shift;    # this is just probesupport
	 my $strMsg = shift;
	 my $msglevel = 'TRACE';
	 my $exitLevel = $EXIT_LEVEL{'TRACE'};
	 if($intPrintLevel < 2){
		  chkExitandPrint($exitLevel,$msglevel,$strMsg,@_);
	 }   
}

sub printfault {
	 shift;    # this is just probesupport
	 my $strMsg = shift;
	 my $msglevel = 'FAULT';
	 my $exitLevel = $EXIT_LEVEL{'FAULT'};
	 chkExitandPrint($exitLevel,$msglevel,$strMsg,@_);	
	 ProbeSupport->probe_exit();
}

sub printattn {
	 shift;    # this is just probesupport
	 my $strMsg = shift;
	 my $msglevel = 'ATTN';
	 my $exitLevel = $EXIT_LEVEL{'ATTN'};
	 if($intPrintLevel < 4){
		  chkExitandPrint($exitLevel,$msglevel,$strMsg,@_);   
	 }
}

sub printerror {
	shift;    # this is just probesupport
	my $strMsg = shift;
	my $msglevel = 'ERROR';
	my $exitLevel = $EXIT_LEVEL{'ERROR'};
	chkExitandPrint($exitLevel,$msglevel,$strMsg,@_);	
}

sub printexpl {
	 shift;    # this is just probesupport
	 my $strMsg = shift;
	 my $msglevel = 'EXPL';
	 my $exitLevel = $EXIT_LEVEL{'EXPL'};
	 if($intPrintLevel < 3){
		  chkExitandPrint($exitLevel,$msglevel,$strMsg,@_);
	 }   
}


=item chkExitandPrint

	chkExitandPrint will determine whether or not 
	the message will be printed.  If there is an entry
	for the message number in $::PROBE_IGNPRBMSGFILE, then
	it will not be printed.  This is the place where we determine
	if NLS is enabled or not.  Right now, we merely check if
	$::MSGMAPPATH is empty.  If it is, then we use the user
	defined %::MESSAGES hash.  Otherwise, we envoke 
	NodeUtils->message() which prints the routine.

=cut

sub chkExitandPrint {
	 
	 my $exitLevel = shift;
	 my $msgLevel = shift;
	 my $strMsg = shift;
	 my $msg;
	 if(defined($::MSGMAPPATH) && defined($::MSGCAT) && defined($::MSGSET) )
	 {
		  $strMsg .= "_$::MSGSET";
		  $msg = NodeUtils->getMessage($strMsg,@_);
		  $msg = "$::PROGNAME:$LEVEL2MSG{$msgLevel}:" . $msg;	
	 }
	 else
	 {
		  $msg = "$::PROGNAME:$LEVEL2MSG{$msgLevel}:";
		  $msg .= sprintf($::MESSAGES{$strMsg}, @_);
		  $msg .= "\n";
	 }

	 unless(&ignore($msg))
	 {
		  my $stdouterrf = \*STDERR;
		  my $stdocuterrd = '1>&2';
		  if( ($msgLevel eq $LEVEL2MSG{'EXPL'}) || ($msgLevel eq $LEVEL2MSG{'TRACE'}))
		  {
				$stdouterrf = \*STDOUT;
				$stdocuterrd = '';
		  }
		  print $stdouterrf $msg; # $stdocuterrd;
		  $::EXIT_STATUS = $exitLevel if $::EXIT_STATUS < $exitLevel;
	 }
}

=item ignore

	ignore message, checks to see if the given message should
	be ignored or not.  If it is to be ignored, returns true, if
	it is not to be ignored, return false.

=cut


sub ignore{
	my $msg = shift;
	#TODO do a reg exp check here to see if messae will be printed
	return 0;

}



=item runutility

	runtutility is envoked as follows:
	runtutility $utility, args
	Call this for utilities that are not perl modules.  The standard utilities
	are just modules and can be ran like functions.  See some of the standard
	probes for examples.

=cut

sub runutility{
	shift;  		# get rid of ProbeSupport.
   	my($pn, $retval, @arglist,$cmd,$returnCode);
	my (@output,@return);
	$pn=shift;
	# push the arguments on the command.
	push @arglist, @_;
	my $name = (split(' ', $pn))[0];
        # edit the cmd.
	if( -f $name ){ $cmd = "./".$pn;}
	else{ $cmd = $::STD_PROBE_UTILDIR."/".$pn }
	$cmd .= " @arglist";
	print "$cmd\n" if $intPrintLevel eq 1;
	if( !($cmd =~ /2>&1$/)){ $cmd .= ' 2>&1';}
        system("$cmd");
	my $rv = $? >> 8;
	return $rv;	
}


=item seterror

	Funtion sets the error return code

=cut

sub seterror{
	shift;
	my $exitCode = shift;
	$::EXIT_STATUS = $exitCode;

}


=item unix_info

	unix_info Tells you the unix brand the version

=cut 


sub unix_info{
    my @unix_name = `/bin/uname -a 2>&1`;
    if($? >> 8){
	return 127;
    }
    my @unix_info = split ' ', $unix_name[0];
    my ($Distrib, $Version);
    my $OS = $unix_info[0];
    
    if($OS =~ m/AIX/i){
	$Distrib = "IBM";
	$Version = join '.',$unix_info[3],$unix_info[2];
        #AIX c175n11 1 5 000029914C00
    }
    elsif($OS =~ m/Linux/i){
	my @file_info = `cat /proc/version 2>&1`;
	if($? >> 8){
	    return 127;
	}
	my @version_info = split '\(', $file_info[0];
	my @Dist_info = split ' Linux', $version_info[3];
	$Distrib = $Dist_info[0];
	my @level = split ' ', $Dist_info[1];
        $Version = $level[0];
    }
    my $rv = "$OS: $Distrib: $Version";
    return $rv;
}



=item node_type

	node_type is helpful to determine whether the node you are running on is a managment server
	for CSM or a CSM client.

=cut

sub node_type {
    my @subsys_info = `/usr/bin/lssrc -a 2>&1`;
    if($? >> 8){ return 127 }
    if(grep /IBM.DMSRM/, @subsys_info){ return "ManagementServer"}
    elsif(grep /IBM.CSMAgentRM/, @subsys_info){ return "ManagedNode"}
    else{ return "Unknown"}	
}




=item getLinuxDistributionName

        getDistributionName try do get Linux Distribution Name,
        On SuSE x.x, it will return "SuSEx"
        On SuSE SLES, it will return "SLESx"
	On RedHat8.0,it will return RedHat80
        On other linux distribution,it will return "Default"

=cut
sub getLinuxDistributionName {

    my $distru = "Default";
        my $version = "";

    if(-f "/etc/SuSE-release"){
            open(DIST,"</etc/SuSE-release") or return $distru.$version;
			my @contents=<DIST>;
            my $line= join('',@contents);
            close DIST;
            if( $line=~/SuSE Linux\s+(\d+\.\d+)\s+/ ){
                    $distru="SuSE";
                    $version=$1;
            } elsif ( $line=~/SuSE SLES-(\d+)/ ){
               if($line=~/VERSION\s+=\s+(\d+\.\d+)\s+/ )
               {
                      $distru="SLES";
                      $version=$1;
               }else{
                      $distru="SLES";
                      $version="7.2";
               }
			}else {
            # Set default handler
            $distru = "SuSE";
            $version = "8.0";
        }
    }elsif(-f "/etc/redhat-release"){
        open(DIST,"</etc/redhat-release") or return $distru.$version;
        my $line = <DIST>;
        close DIST;
        if( $line =~ /Red Hat Linux release\s+(\d+\.\d+)\s+/ ){
            if($1 eq "8.0"){
                $distru = "RedHat";
                $version=$1;
            }
        }elsif($line =~ /Red Hat Linux release\s+(\d+)\s+/){
                $distru="RedHat";
                $version=$1;
        }elsif($line =~ /Red Hat Linux Advanced Server release (\d+\.\d+)AS/){
	        $distru="RedHatEL-AS";
		$version=$1;
        }elsif($line =~ /Red Hat Linux Advanced Server release (\d+\.\d+)ES/){
	        $distru="RedHatEL-ES";
		$version=$1;
        }elsif($line =~ /Red Hat Linux Advanced Server release (\d+\.\d+)WS/){
	        $distru="RedHatEL-WS";
		$version=$1;
	}elsif($line=~ /Red Hat Enterprise Linux ES release (\d+\.\d+)/){
		$distru="RedHatEL-ES";
		$distru=$1;
	}elsif($line =~ /Red Hat Enterprise Linux AS release (\d+\.\d+)/ ){
		$distru="RedHatEL-AS";
		$version=$1;
	}elsif($line =~ /Red Hat Enterprise Linux WS release (\d+\.\d+)/ ){
		$distru="RedHatEL-WS";
		$version=$1;
	}elsif($line=~ /Red Hat Enterprise Linux ES release (\d+)/){
		$distru="RedHatEL-ES";
		$distru=$1;
	}elsif($line =~ /Red Hat Enterprise Linux AS release (\d+)/ ){
		$distru="RedHatEL-AS";
		$version=$1;
	}elsif($line =~ /Red Hat Enterprise Linux WS release (\d+)/ ){
		$distru="RedHatEL-WS";
		$version=$1;
	}
	#else Distribution name is "Default"

    }elsif(-f "/etc/UnitedLinux-release"){
        $distru = "United";
        open(DIST,"</etc/UnitedLinux-release") or return $distru.$version;
        my $line = <DIST>;
        close DIST;
        if( $line =~ /VERSION\s+=\s+(\d+\.\d+)\s+/ ){
            $version = $1;
        }else{
            $version ="1.0";
        }
    }

    return $distru.$version;
}

=item

get the list of nodes from the config file

=cut

sub fetch_input_nodes {
	 my ($class,%input) = @_;
	 
	 if ( ref($input{result}) ne 'ARRAY' ) { 
		  die "Expected an array reference, not a $input{result}\n"; 
	 }
	 
	 my $nodefile = $ENV{'PROBE_NODELISTFILE'};
	 if ( ! -f $nodefile ) {
		  my @Nodes = `$::LSNODE`;
		  foreach my $node ( sort @Nodes ) {
				chomp $node;
				push ( @{$input{result}} , $node );
		  }
		  return 0;
	 }
	 
	 my @CONFIG = `cat $nodefile`;
	 foreach my $node (sort (@CONFIG)) {
		  chomp $node;
		  $node =~s/\s+//g;
		  next if($node eq "");
		  push (@{$input{result}},$node);
	 }
	 
	 return 0;
}



=item probe_init and probe_exit

	probe_init sets $::PROGNAME to the name of the program.
	probe_exit simple exits with the correct return status.
	is there anything else these should do?

=cut




sub probe_init {
    $::PROGNAME=`basename $0`;
    chomp $::PROGNAME
}


sub probe_exit {
    print "$::EXIT_STATUS\n" unless $intPrintLevel; 	
    exit $::EXIT_STATUS
}

1;
