#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2001,2004 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
#
# @(#)91   1.352   src/csm/install/InstallUtils.pm.perl, setup, csm_rsir, rsir0339a 9/19/03 17:26:31
 
#####################################################################
#                                                                   #
# Module: InstallUtils.pm                                           #
#                                                                   #
#-------------------------------------------------------------------#
#                                                                   #
# Description: This module contains Perl subroutines used           #
#              commonly across the CSM installation commands.       #
#              Detailed descriptions appear in subroutine headers.  #
#                                                                   #
# Subroutines:                                                      #
#                                                                   #
#  make_node_list_file invokeLAP                                    #
#  get_target_nodes get_common_attrs findImagesInPath               #
#  make_config_file make_script_lists  name2list                    #
#  create_nodemap_file start_nfs export_csminstall                  #
#  unexport_csminstall                                              #
#  get_lock   release_lock  backup_logfile start_logging            #
#  stop_logging  append_logging  get_management_server  get_OSName  #
#  get_DistributionVersion                                          #
#  get_CSMVersion  get_PkgArchitecture  get_pkgVersion              #
#  get_DistributionName get_uniqu_arry_elemnts                      #
#  findRpmInPath findRpm installRequiredRpms getRequiredRpms        #
#  getReqsNotSatisfied parseRequire inc_ip                          #
#  SetAllowRequest SetMode genHex verifyTftp verify_dhcp            #
#  configureTftp testvalidconsole testvalidpowermethod chkethup     #
#  getRemoteShell checkHOMEforSSHperm installSSHFiles cpSSHFiles    #
#  setupAIXRSH setupLinuxRSH setupRemoteShellforFullInstall         #
#  getRecentVersionRpm subnet getInstallDir getUpdateDir getRPMSDir #
#  getBaseDir checkForNotFound isAutoupdateInstalled checkRPMInstall#
#  nfsprobe createRPMSdir copyDistroPKGPATH copyDistroCD            #
#  MountAixCdrom getBladeStatus checkDistroCopied                   #
#  getRPMVersionRelease write_status initialize_status              #
#  createRPMVersionListAlreadyInstalled get_definition_version      #
#  get_csminstall_pathVersion search_csm search_opensrc_prereqs     #
#  search_copy_distro_rpms create_rpm_list_to_copy copy_rpm         #
#  post_csm_copy search_rpm_csminstall_path                         #
#  create_rpm_list_to_install display_opensrcLicence syncServers    #
#  makeSyncHash findRpmInPath_java copyBinaries SetControlFlag      #
#                                                                   #
#####################################################################

package InstallUtils;

use strict;
use Fcntl qw(:flock);
use File::Basename;
use File::Find;
use File::Path;	# Provides mkpath()

#
# Subroutines
#

###########################################################
# getBladeStatus
#    Checks whether a list of nodes are blades or not.
#
#    Argument: a node list (getBladeStatus will not verify that the node is 
#              is defined.  It will only run the commmand on it.)
#    Return Values:
#		A hash ref of { node => yes or no }
#
###########################################################
sub getBladeStatus{
    # local variables.
    my ($node,$nodes,%nodes,$rc,$rv,$cmd,@stdout,$line);
    # first strip the class name:
    shift;
    # need to do 50 at a time so that we don't have scaling problems.
    $nodes = join(',',@_);
    # the command to find out whether we have a blade or not is this:
    $cmd = "/opt/csm/bin/lshwstat -n $nodes blade";
    # do runcmd, but do not exit if there is an error.  Just
    # set the return code.
    @stdout = NodeUtils->runcmd("$cmd");
    foreach $line (@stdout){
	($node,$rv) = split(': ',$line);
	$nodes{$node} = $rv;	
    }
    return \%nodes;
}
###########################################################
# invokeLAP
#  To invoke the LAP tool and display the license
#  agreements for CSM.
#
# input: a string - either: "tab" or "full"
# output: 3, 9, or 127
#  The LAP tool will be invoked and copy the LA and LI
#  files to /var/opt/csm/<VRnumber>/<type>/license/<lang>.txt
#  where <VRnumber> = CSM Version (1.3, etc)
#  <type> = tryandbuy or full, depending upon the license.
#  <lang> = one of the 17 supported LAP languages.
#  If everything ran successful, then LAP will return 9.
#  otherwise, if the user returns 3, the license was not 
#  accepted.  If something went wrong with the LAP tool, then
#  an error of 127 will be returned.
###########################################################
sub invokeLAP{
    my ($class, $licType) = @_; 
    my ($csmVersion,$laDir,$strCmd, $rc); 
    my $laptool = "/opt/csm/install/lapapp.jar";
    # make sure that $licType is either tab or full
    if($licType eq "full"){
	# do nothing.. everything's fine.
    }elsif($licType eq "tab"){
	$licType = "tryandbuy";
    }else{
	#print "don't know..\n";
	return 127;
    }  

    # get the CSM version:
    $csmVersion = InstallUtils->get_CSMVersion("csm.server"); 
    $csmVersion = sprintf "%.1f", $csmVersion;    # format to be
						  # x.x, e.g.: 1.3

    # check that the LA and LI files are in place:
    if(NodeUtils->isLinux()){
        $laDir = "/opt/csm/install/$csmVersion/Linux/license/$licType";
    }elsif(NodeUtils->isAIX()){
        $laDir = "/opt/csm/install/$csmVersion/AIX/license/$licType";
    }else{
	return 127;
    }

    unless( -f "$laDir/LA_en"){
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgCANT_READ_FILE', "$laDir/LA_en"); 
	return 127;   
    }

    # check that the LAP tool is in place:
    unless ( -f "$laptool"){
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
                'csminstall','E', 'EMsgCANT_READ_FILE', $laptool); 
	return 127;  
    }
	
    # Everything should be in place now, so let's call LAP:
    # construct the command:
    if(NodeUtils->isLinux()){
	# check that Java is installed... and use the right one.
	if(-d "/opt/IBMJava2-131"){
            $strCmd = "/opt/IBMJava2-131/jre/bin/java -cp ";
  	    $strCmd .= "/opt/csm/install/lapapp.jar:";
	    $strCmd .= "/opt/IBMJava2-131/jre/lib/rt.jar:";
	    $strCmd .= "/opt/IBMJava2-131/jre/lib/i18n.jar:";
	}elsif(-d "/opt/IBMJava2-13"){
            $strCmd = "/opt/IBMJava2-13/jre/bin/java -cp ";
  	    $strCmd .= "/opt/csm/install/lapapp.jar:";
	    $strCmd .= "/opt/IBMJava2-13/jre/lib/rt.jar:";
	    $strCmd .= "/opt/IBMJava2-13/jre/lib/i18n.jar:";
        }elsif(-d "/opt/IBMJava2-131"){
            $strCmd = "/opt/IBMJava2-131/jre/bin/java -cp ";
            $strCmd .= "/opt/csm/install/lapapp.jar:";
            $strCmd .= "/opt/IBMJava2-131/jre/lib/rt.jar:";
            $strCmd .= "/opt/IBMJava2-131/jre/lib/i18n.jar:";
	}elsif(-d "/opt/IBMJava2-ppc-131"){
            $strCmd = "/opt/IBMJava2-ppc-131/jre/bin/java -cp ";
            $strCmd .= "/opt/csm/install/lapapp.jar:";
            $strCmd .= "/opt/IBMJava2-ppc-131/jre/lib/rt.jar:";
            $strCmd .= "/opt/IBMJava2-ppc-131/jre/lib/i18n.jar:";
	}elsif(-d "/usr/lib/IBMJava2-1.3.1"){
            $strCmd = "/usr/lib/IBMJava2-1.3.1/jre/bin/java -cp ";
            $strCmd .= "/opt/csm/install/lapapp.jar:";
            $strCmd .= "/usr/lib/IBMJava2-1.3.1/jre/lib/rt.jar:";
            $strCmd .= "/usr/lib/IBMJava2-1.3.1/jre/lib/i18n.jar:";
        }else{
	    #print "IBMJava2-13[1] is not installed.\n";
	    exit 127;
	}
	
    }else{ # AIX
        $strCmd = "/usr/java131/jre/bin/java -cp ";
  	$strCmd .= "/opt/csm/install/lapapp.jar:";
	$strCmd .= "/usr/java131/jre/lib/rt.jar:";
	$strCmd .= "/usr/java131/jre/lib/i18n.jar:";
    }
     
    # this part of the command is common in both AIX and Linux:
    $strCmd .= " com.ibm.lex.lapapp.LAP ";
    $strCmd .= "-l $laDir ";
    $strCmd .= "-s /var/opt/csm/$csmVersion/$licType ";
    $strCmd .= "-text_only";
    $rc = system("$strCmd");
    $rc = $rc >> 8;

    # kill processes if they exist:
    if(NodeUtils->isLinux){
        foreach(`ps -efl --columns 300 | grep lapapp | awk '{print \$4}'`){
           chomp $_;
           `kill -9 $_ 2>/dev/null`;
        }
    }

    return $rc;
}


###########################################################
# make_node_list_file
#  this file is suitable for the ENV vars: 
#	DSH_LIST
#	RPOWER_LIST
#	RCONSOLE_LIST
#
#  IMPORTANT!!!: if you use this subroutine, make sure you use 
#  	InstallUtils->close_delete_file($file_handle, $file_name) 
#       to erase it.
#
# input: (\@list_of_nodes)
# output
#	$file_name
# sets global var: $::NODE_LIST_FILE
###########################################################
sub make_node_list_file{
	my ($class, $ref_node_list) = @_;
	my @node_list = @$ref_node_list;
	srand(time|$$); #random number generator start
	
	my $file = "/tmp/csm_$$";
        while(-e $file){
             $file = InstallUtils->CreateRandomName($file);
        }

	open($::NODE_LIST_FILE, ">$file") or NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', "$file");
	foreach my $node (@node_list){
		print $::NODE_LIST_FILE "$node\n";
	}
	return $file;
}
###########################################################
# close_delete_file
# This subroutine takes a file handle and file name
# First closes and then deletes the file
# input: $file_handle $file_name
# output null
###########################################################
sub close_delete_file{
	my ($class, $file_handle, $file_name) = @_;
	close $file_handle;
	
  	my $cmd = "$::RM -f $file_name";
        my $rc  = system ($cmd) >> 8;
        if ($rc) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        }
}
###########################################################
# CreateRandomName
# input : prefix of name
# output: prefix with 8 random letters appended
###########################################################
sub CreateRandomName {

    my ($class, $name) = @_;

    my $nI;
    for ($nI = 0; $nI < 8; $nI++) {
       my $char = ('a'..'z', 'A'..'Z')[int(rand( 52)) + 1];
       $name .= $char;
    }
    $name;
}
###########################################################
# get_target_nodes
#
# input:
#      (\@ARGV) either from @ARGV or $::opt_n
#      ($node group string) from $::opt_N
#	$file containing node list
#       1  -- skip the check for not found nodes
#
#Global variables:
#	$::ALL                     	#Distribute to all nodes
#	$::MANAGED     	#Distribute to all nodes where Mode="Managed"
#       $::PRE_MANAGED	#Distribute to all nodes where Mode="PreManaged"
#
# output
#	\@DestNode  = reference to list of all target nodes
#       \@lsnode_info = reference to a list of results from lsnode command (only for the nodes in @DestNode)
#	\%DestNodeHash = reference to a hash of hostnames and attributes (foreach node in @Destnode):
#			$DestNodeHash{$hostname}{$attribute_name} = "attribute value"
#			For the list of attributes see $lsnode_attr
###########################################################
sub get_target_nodes{
    my ($class, $ref_nodelist, $nodegrp_list, $file, $skipNFcheck) = @_;
    my @nodelist = @$ref_nodelist;
    my (@DestNode, @DestGroup, %DestKeyTemp, %all_groups, @lsnode_info, @empty_array);
    
    my @real_nodelist; #split the nodes if there are commas
        foreach my $n (@nodelist){
    	   if($n =~ m/,/){
    		   my @tmp = split ',', $n;
    		   foreach my $t (@tmp){
    			   push @real_nodelist, $t;
    		   }
    	   }
    	   else{
    		   push @real_nodelist, $n;
    	   }
       }
       @nodelist=@real_nodelist;
   undef @real_nodelist;
    
    my $lsnode_attr = "Hostname,ManagementServer,InstallOSName,InstallCSMVersion,InstallDistributionVersion,InstallDistributionName,InstallPkgArchitecture,Mode,InstallAdapterMacaddr,InstallAdapterType,InstallAdapterDuplex,InstallAdapterSpeed,InstallMethod,ConsoleSerialDevice,Status,InstallServer,InstallTemplate,PowerMethod,InstallDiskType,InstallDisk,PowerStatus";
    # These delimeters are used with the lsnode command.  A simple colon (:)
    # cannot be used since there are colons in InstallAdapterMacaddr.
    #my $delim = ':|:';		# Used on the lsnode command
    my $check_delim = ':\|:';	# Used when splitting the line apart
    
    if($::ALL || $::MANAGED || $::PRE_MANAGED ){ #distribute to all nodes of a certain set
        my $select_string;
        if($::MANAGED){
            $select_string = "Mode=\"Managed\"";
        } elsif($::PRE_MANAGED){
            $select_string = "Mode=\"PreManaged\"";
        } else{ #just managednodes
            $select_string = "";
        }
        my %opthash = (WhereStr => $select_string); 
	my $ref_lsnode_info = NodeUtils->listNodeAttrs( \@empty_array, $lsnode_attr, \%opthash);
	@lsnode_info = @$ref_lsnode_info;
        
        foreach my $line (@lsnode_info){
            chomp $line;
            my $hostname = (split /$check_delim/, $line)[0];
            push @DestNode, $hostname;
        }
    } 
    else{ #we are not distributing to all nodes of a certain mode 
        my %opthashref = ();
    	if($file){
		%opthashref = ( NodeFile => $file, NodeGrp => $nodegrp_list);
	}
	else{
		%opthashref = ( NodeFile => 1, NodeGrp => $nodegrp_list);
	}	
	my $refnodes = NodeUtils->gatherNodeList(\@nodelist, \%opthashref);
	if(scalar @$refnodes){
		my $ref_lsnode_info = NodeUtils->listNodeAttrs( $refnodes, $lsnode_attr);
		@lsnode_info = @$ref_lsnode_info;
		if(!$skipNFcheck){
			InstallUtils->checkForNotFound($refnodes);
		}
		
	}
	else{
		@lsnode_info = ();
	}
    }
    
    my  %DestNodeHash;
    foreach my $line (@lsnode_info){  #this only has node names in it from the cluster
	my @attributes = split /$check_delim/, $line;
	my $hostname = $attributes[0];
	chomp $line;
	my @attr_names = split ',', $lsnode_attr;
	my $index =0;
	foreach my $attr (@attr_names){
	    $DestNodeHash{$hostname}{$attr} = $attributes[$index];
	    $index++;
	}
    }
    
    if(@::VALID_INSTALLMETHODS){
	my %meths;
	foreach my $method (@::VALID_INSTALLMETHODS){
	    $meths{$method}=1;
	}
	my @methString = keys %meths;
	foreach my $target (keys %DestNodeHash){
	    if(!$meths{$DestNodeHash{$target}{'InstallMethod'}}){
		if(!($::ALL || $::MANAGED || $::PRE_MANAGED)){                  
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgINVALID_INSTALL_METH', $target, "@methString");
		}
		else{
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','W', 'EMsgINVALID_INSTALL_METH', $target, "@methString");
			delete $DestNodeHash{$target};
		    }
	    }
	}
    }

    my @tmpDestNode = keys %DestNodeHash;
    my @return_array = (\@tmpDestNode, \@lsnode_info, \%DestNodeHash);
    return @return_array;
}

# get_common_attrs
#
# Get the common attributes across all the nodes.
#  InstallDistributionName
#  InstallDistributionVersion
#  InstallPkgArchitecture
#  InstallCSMVersion
#  InstallMethod
#
# input:
#      none
# global varialbles:
#      %::NODEHASH - List of nodes to check
# output:
#      The following variables in an array:
#      ($distro_name, $distro_version, $arch, $csm_version);
#
# Exits with an error if one of the attributes is not the same on 
# all the nodes, 
#
sub get_common_attrs
{
#SSSBEGIN
    my ($hostname, $distro_name, $distro_version, $arch, $csm_version, $install_method);
#SSSEND
    my ($errors) = 0;
    my ($InstallDistributionName);
    my ($InstallDistributionVersion);
    my ($InstallPkgArchitecture);
    my ($InstallCSMVersion);
#SSSBEGIN
    my ($InstallMethod);
    my $InstallMethod_Err_Flg=0;
#SSSEND
    my $InstallCSMVersion_Err_Flg=0;
    my $InstallPkgArchitecture_Err_Flg=0;
    my $InstallDistributionVersion_Err_Flg=0;
    my $InstallDistributionName_Err_Flg=0;
    my $DistributionName_Err_Flg=0;
    my $DistributionVersion_Err_Flg=0;
    my $PkgArchitecture_Err_Flg=0;
    my $CSMVersion_Err_Flg=0;

    foreach $hostname (keys %::NODEHASH)
    {
	$InstallDistributionName = 
	    $::NODEHASH{$hostname}{'InstallDistributionName'};
	$InstallDistributionVersion = 
	    $::NODEHASH{$hostname}{'InstallDistributionVersion'};
	$InstallPkgArchitecture = 
	    $::NODEHASH{$hostname}{'InstallPkgArchitecture'};
	$InstallCSMVersion = 
	    $::NODEHASH{$hostname}{'InstallCSMVersion'};
#SSSBEGIN
	$InstallMethod = 
	    $::NODEHASH{$hostname}{'InstallMethod'};
#SSSEND

	if (! $distro_name)
	{
	    $distro_name = $InstallDistributionName;
	}
	if (! $distro_version)
	{
	    $distro_version = $InstallDistributionVersion;
	}
	if (! $arch)
	{
	    $arch = $InstallPkgArchitecture;
	}
	if (! $csm_version)
	{
	    $csm_version = $InstallCSMVersion;
	}
#SSSBEGIN
	if (! $install_method)
	{
	    $install_method = $InstallMethod
	}
#SSSEND

	if ("$distro_name" and 
	    ($InstallDistributionName ne $distro_name))
	{
	    $errors++;
            $InstallDistributionName_Err_Flg++;
	}

	if ("$distro_version" and 
	    ($InstallDistributionVersion ne $distro_version))
	{
	    $errors++;
            $InstallDistributionVersion_Err_Flg++;
	}

	if ("$arch" and 
	    ($InstallPkgArchitecture ne $arch))
	{
	    $errors++;
            $InstallPkgArchitecture_Err_Flg++;
	}

	if ("$csm_version" and 
	    ($InstallCSMVersion ne $csm_version))
	{
	    $errors++;
            $InstallCSMVersion_Err_Flg++;
	}

#SSSBEGIN
	if ("$install_method" and 
	    ($InstallMethod ne $install_method))
	{
	    $errors++;
            $InstallMethod_Err_Flg++;
	}
#SSSEND
    }

    if (! $distro_name) {
	$errors++;
        $DistributionName_Err_Flg++;
    }
    if (! $distro_version) {
	$errors++;
        $DistributionVersion_Err_Flg++;
    }
    if (! $arch) {
	$errors++;
        $PkgArchitecture_Err_Flg++;
    }
    if (! $csm_version) {
	$errors++;
        $CSMVersion_Err_Flg++;
    }
#SSSBEGIN
# Don't worry if the InstallMethod is not set.  This might have been called 
# from csmsetup[ks|yast] which sets it later.
#    if (! $install_method) {
#	$errors++;
#        $InstallMethod_Err_Flg++;
#    }
#SSSEND

    if ($InstallDistributionName_Err_Flg > 0)
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgDIFFERENT_ATTR', 'InstallDistributionName');
    }
    if ($InstallDistributionVersion_Err_Flg > 0)
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgDIFFERENT_ATTR', 'InstallDistributionVersion');
    }
    if ($InstallPkgArchitecture_Err_Flg > 0)
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgDIFFERENT_ATTR', 'InstallPkgArchitecture');
    }
    if ($InstallCSMVersion_Err_Flg > 0)
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgDIFFERENT_ATTR', 'InstallCSMVersion');
    }
    if ($DistributionName_Err_Flg > 0)
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgNO_ATTR', 'InstallDistributionName');
    }
    if ($DistributionVersion_Err_Flg > 0)
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgNO_ATTR', 'InstallDistributionVersion');
    }
    if ($PkgArchitecture_Err_Flg > 0)
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgNO_ATTR', 'InstallPkgArchitecture');
    }
    if ($CSMVersion_Err_Flg > 0)
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgNO_ATTR', 'InstallCSMVersion');
    }
#SSSBEGIN
    if ($InstallMethod_Err_Flg > 0)
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgNO_ATTR', 'InstallMethod');
    }
#SSSEND

    exit $errors if ($errors);

#SSSBEGIN
    my @return_array = ($distro_name, $distro_version, $arch, $csm_version, $install_method);
#SSSEND
    return (@return_array);
}

#-------------------------------------------------------------------------#
#                                                                         #
#  SetAllowRequest     Make sure the management server can update the     #
#                      node information. 				  #	
#		       	This should be set to 3 by installnode		  #
#		 	and by updatenode if the Mode is not managed.	  #
#			Otherwise it should be set to 2 by updatenode.    #
#			during updatenode exit, it should be set to 0.    #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub SetAllowRequest {

    my ($class, $ref_DestNodeHash, $value) = @_;
    my %DestNodeHash = %$ref_DestNodeHash;
    if(!$value){
	    $value = "0 ";
    }
    if(keys %DestNodeHash){
	    # DestNodeHash is a hash containing attribute values for 
	    # each node that was provided on the command line.
	    my ( $cmd, $rc);

	    # check each node on our list and set the AllowManageRequest 
	    # attribute if needed.
	    my @nodelist = (keys %DestNodeHash);
	    
	    my $nodelist_file = 
		InstallUtils->make_node_list_file(\@nodelist);
	    $cmd = "$::CHNODE -f $nodelist_file AllowManageRequest=$value";
	    $rc = system("$cmd");
	    if ($rc >> 8){
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', "$cmd", $rc);
	    }
	    InstallUtils->close_delete_file($::NODE_LIST_FILE, $nodelist_file);
	    return $::OK;
	    
    }
    else{
	    return 1; #a value and nodes must be specified.
    }
}
#-------------------------------------------------------------------------#
#                                                                         #
#  SetControlFlag      Set the ControlFlag attribute.  Currently, this    #
#                      is used to determine whether the node is allowed   #
#                      to write to the status log in                      #
#                      /csminstall/csm/status/<nodename>.                 #
#                                                                         #
#                      This should be set to 1 by installnode, and then   #
#                      csmfirstboot will run an RMC action to set it back #
#                      to 0 when the node install is complete and no more #
#                      status messages need to be written to the MS.      #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub SetControlFlag {

    my ($class, $ref_DestNodeHash, $value) = @_;
    my %DestNodeHash = %$ref_DestNodeHash;
    if(!$value){
	    $value = "0 ";
    }
    if(keys %DestNodeHash){
	    # DestNodeHash is a hash containing attribute values for 
	    # each node that was provided on the command line.
	    my ( $cmd, $rc);
	    my @nodelist = (keys %DestNodeHash);
	    
	    my $nodelist_file = 
		InstallUtils->make_node_list_file(\@nodelist);
	    $cmd = "$::CHNODE -f $nodelist_file ControlFlag=$value";
	    $rc = system("$cmd");
	    if ($rc >> 8){
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', "$cmd", $rc);
	    }
	    InstallUtils->close_delete_file($::NODE_LIST_FILE, $nodelist_file);
	    return $::OK;
	    
    }
    else{
	    return 1; #a value and nodes must be specified.
    }
}

# Set a node's Mode attribute to PreManaged, Installing or Managed.
sub SetMode
{
    my ($class, $node, $mode) = @_;
    my ($cmd, $rc, $fstatus);

    my $CSMLITE = $::CSMLITE;
    if (($mode ne "PreManaged") and 
	($mode ne "Installing") and 
	($mode ne "Managed")    and
	($mode ne $CSMLITE)   and
	($mode ne "${CSMLITE}-Installing"))
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgINVALID_MODE', $mode, "PreManaged, Installing, Managed, $CSMLITE, ${CSMLITE}-Installing");
    }

    $cmd = "$::CHNODE $node Mode=$mode";
    $rc = system($cmd) >> 8;
    if ($rc != 0)
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', $::CHNODE, $rc);
	$fstatus = $rc;
    }

    return $fstatus;
}

#-------------------------------------------------------------------------#
#                                                                         #
#   make_script_lists  - create lists of pre-install, post-install and    #
#                        update scrips for each node.  Put them in the    #
#                        following hashes:                                #
#                                                                         #
#                 %::PREINSTSCRIPTS - scripts get run from csmprereboot   #
#                                     during full install before the      #
#                                     first reboot.                       #
#                 %::POSTINSTSCRIPTS - scripts get run from csmfirstboot  #
#                                    after the first reboot of the system.#
#                 %::UPDATESCRIPTS - scripts get run from                 #
#                                   updatenode.client after a node has    #
#                                   been updated.                         #
#                                                                         #
#   Returns:      0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#

sub make_script_lists {

    my ($junk, @nodel)=@_; # the list of nodes passed in on the command line
    my $dirname;    #  directory to check for scripts
    my $script;   #  unique script name
    my $cmd;
    my $rc;

	@::scriptlist;  #  list of scripts found in a directory
    %::scriptnodes; #  lists of nodes to run each script on

    # get the list of all nodes so we can check 
    #       for valid node names later
    $cmd = "$::LSNODE";
    @::allnodes = NodeUtils->runcmd($cmd, -1);
    if ($::RUNCMD_RC) { 
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $::RUNCMD_RC);
        return $::NOK;
    }

    my @scriptdirlist;
    if ($::UPDATESCRIPTSONLY) {
        @scriptdirlist=($::UPDATEDIR);
    } else {
        @scriptdirlist=($::PREINSTALLDIR, $::POSTINSTALLDIR);
    }

    # check each subdirectory for user scripts
    foreach $dirname (@scriptdirlist) {

        $rc = InstallUtils->get_script_list($dirname,@nodel);

        if ($rc == $::NOK) {
            next;
        }

        # check each script name in this directory
        foreach $script ( sort @::scriptlist ) {
            chomp $script;
   
            # see if there are any nodes in the list
            if (@{$::scriptnodes{$script}}[0] eq $::NOK) {
                # msg - Skipping to the next script name.
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgSkipScript');
                next;
            }

            # see if the script is executable
            if (!(-x "$dirname/$script")) {
                # msg - Script $script is not executable.
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgExecScript', "$dirname/$script");
                # msg - Skipping to the next script name.
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgSkipScript');
                next;
            }

            #  add the file to the correct script list for the node
            foreach my $node (@{$::scriptnodes{$script}}) {

                chomp $node;

                if ($dirname eq $::PREINSTALLDIR) {
                    push @{$::PREINSTSCRIPTS{$node}}, $script;

                } elsif ($dirname eq $::POSTINSTALLDIR) {
                    push @{$::POSTINSTSCRIPTS{$node}}, $script;

                } elsif ($dirname eq $::UPDATEDIR) {
                    push @{$::UPDATESCRIPTS{$node}}, $script;
                }            
            }
        }
    }
    return $::OK;
}

#-------------------------------------------------------------------------#
#                                                                         #
#   name2list  - convert a script file name to a list of nodes            #
#                that the script should be run on                         #
#                                                                         #
#       The file name format is <basename>._<node or group>               #
#                                                                         #
#       If there is just a basename then it applies to all nodes          #
#       If there is a node name then it just applies to that node         #
#       If there is a group name then it applies to the members of        #
#             the group                                                   #
#                                                                         #
#   Returns:                                                              #
#                 list of nodes                                           # 
#                 1 - An error occured.                                   #
#                                                                         #
#   Note: requires @::allnodes be set by caller
#-------------------------------------------------------------------------#
sub name2list {
    my ($junk, $dirname, $filename, @nlist) = @_;
    my @nodelist=();
    my ($base, $name, @glist);
    my ($short, $target, $cmd);
    my $junk2;

    chomp $filename;

    # the second part of the file name tells us 
    #    what nodes to run it on
    # also check if additional target was used - error
    ($base, $target, $junk2)  = split (/\._/, $filename);

    if (defined($junk2)) {
        #  \'$filename\' is not a valid customization script name.
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgBadScriptName', "$dirname/$filename");
        return $::NOK;
    }

    if (!defined($target)) {

        # do on all nodes the calling
        #    command was asked to process
        return @nlist;

    }  else {

        # see if the target is a valid node name
        my ($hostname, $ipaddr) = NodeUtils->tryHost($target);
        if (grep(/^$hostname/, @::allnodes)) {
             # see if the target is one of the nodes
             #     specified on the command line
             if (grep(/^$hostname/, @nlist)) {
                 chomp(@nodelist =($hostname));
             }
             return @nodelist;
        }
               
        # see if target is a valid group name
        $cmd = "$::NODEGRP -p $target";
        @glist = NodeUtils->runcmd($cmd, -1);
        if ($::RUNCMD_RC) {
            # msg - An incorrect file name format was used for the script $filename. '$target' is neither a node name nor a group name.
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgBadScript', "$dirname/$filename", $target);
            return $::NOK;

        } else {

            foreach $name (@glist) {
                # we only need a node if it was specified 
                #    on the command line
                if (grep(/^$name/, @nlist)) {
                    push (@nodelist,$name);
                }
            }
            return @nodelist;
        }
    } 
    return $::NOK;
}

#-------------------------------------------------------------------------#
#                                                                         #
#   get_script_list  - for a given directory get a list of scripts and    #
#                        associated list of nodes to run it on.           #
#                                                                         #
#                                                                         #
#   Returns:                                                              #
#                 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_script_list {
    my ($junk, $dirname, @nlist) = @_;
    my %basename;
    my $target;
    my $filename;
    my @filelist;
    my $cmd;

    @::scriptlist=();

    if ( -d $dirname ) {
        $cmd = "$::LS $dirname";
        @filelist = NodeUtils->runcmd($cmd, -1);
        if ($::RUNCMD_RC) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_RUN',$cmd, $::RUNCMD_RC);
            return $::NOK;
        }
    } else {
        # msg - Warning: The \'$dirname\' does not exist.
        #        Skipping to the next directory.
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgSkipDir', $dirname);
        return $::NOK;
    }

    # check each script name in this directory
    # - add the filename to the script list
    # - get the initial list of node to run the script on
    # - save the basename for each script
    foreach $filename ( sort @filelist ) {
        chomp $filename;

        #  make sure this is not a subdir
        if ( -d "$dirname/$filename" ) {
            # error - Skipping subdirectory $dirname/$filename.
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"I", 'IMsgSkipSubDir', "$dirname/$filename");
            next;
        }

        # figure out what nodes to run the script on by
        #    deciphering the script name
        @{$::scriptnodes{$filename}}=InstallUtils->name2list($dirname,$filename,@nlist);
        if (${$::scriptnodes{$filename}}[0] eq $::NOK) {
            # msg - Skipping to the next script name.
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgSkipScript');
            next;
        }
        push @::scriptlist, $filename;
        (@basename{$filename}, $target)  = split (/\._/, $filename);
    }
    
    # now check each script that has no extension (target) and see
    #  if there are any scripts with the same basename that have targets
    # if there are - remove any nodes in the targeted script list from the
    #  list of nodes for the script with no extensions(target)
    # ex.  say script foo goes to all nodes and script foo._grpa goes
    # to node1 & node2.  In this case I need to remove node1 & node2
    # from the list of nodes for the script foo.

    foreach my $fname ( sort @::scriptlist ) {
        chomp $fname;

        # if the script name is the same as the basename then
        #   the list is all the nodes passed in on the command line.
        if ($fname eq @basename{$fname}) {
            # check the rest of the scripts to see if any
            # have the same basename - if we find one then redo
            # the list for the basename-only script
            foreach my $file ( @::scriptlist ) {
                chomp $file;
                #  see if the basenames are the same
                if (@basename{$fname} eq @basename{$file} ) {
                    # don't remove itself
                    if ( $file ne $fname ) {
                        my @tmplist=();
                        foreach my $node (@{$::scriptnodes{$fname}} ) {
                            # if the node is not in the list for $file
                            #  then add it to a temp list
                            if ( !grep /^$node/, @{$::scriptnodes{$file}} ){
                                push @tmplist, $node;
                            }
                        }
                        @{$::scriptnodes{$fname}}=@tmplist;
                    }
                }
            }
        }
    }
    return $::OK;
}

#-------------------------------------------------------------------------#
#                                                                         #
#  make_config_file  - Create configuration information files for each    #
#                      node provided in the list.                         #
#                                                                         #
#    requires $::REMOTE_SHELL and $::SETUP_REMOTE_SHELL to be set         #
#        optionally, $::NOSMS can be set to instruct makenode to not      #
#                    perform software maintenance on Linux nodes          #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub make_config_file {

    my ($class, $ref_nl, $ref_nodeinfo, $ref_DestNodeHash) = @_;
    my @nl = @$ref_nl;
    my @nodeinfo = @$ref_nodeinfo;

    #  DestNodeHash is a hash containing attribute values for each node
    #    that was provided on the command line.
    my %DestNodeHash = %$ref_DestNodeHash;

    my ($hname, $short, $rc);
    my $csmcfgdir = "/csminstall/csm/config";
    my $cmd;
    my $output;
    my $DirectorInstalled=0;

    #
    #  Check and/or create the /csminstall/csm/config directory
    #
    if (! -d $csmcfgdir) {
        $cmd = "$::MKDIR -m 755 -p $csmcfgdir";
        $rc = system("$cmd");
        if ($rc >> 8) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_CreateDir', $csmcfgdir);
            return($::NOK);
        }
    }

    # If this is a Linux management server, check if csm.director.agent 
    # is installed.
    if(NodeUtils->isLinux()==1) {
        $cmd = "$::RPMCMD -q csm.director.agent";
        $output = NodeUtils->runcmd($cmd, -1);
        if ($::RUNCMD_RC == 0) {
            $DirectorInstalled = 1;
        }
    }

    # get the lists of user-provided cstomization script
    #   that we need to run on the nodes
    if (InstallUtils->make_script_lists(@nl) ) {
        # msg  -Could not get lists of customization scripts
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNoCustScripts');
        # continue
    }

    #
    # create the config_info file
    #

    #  go through each node in the list
    #   (the keys are the hostnames as returned from tryHost() !!)
    foreach $hname (keys %DestNodeHash) { 
        
        # remove the old config file if any
        if ( -e "$csmcfgdir/$hname.config_info") {
            $cmd = "$::RM $csmcfgdir/$hname.config_info";
            $rc = system("$cmd");
            if ($rc >> 8){
                my $fn = "$csmcfgdir/$hname.config_info";
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_RM', "$fn");
                 # try to continue ??return $::NOK;
            }
        }

        # create the new file
        unless (open(CONFIG_OUT,">$csmcfgdir/$hname.config_info")) {
            my $fn = "$csmcfgdir/$hname.config_info";
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', "$fn");
            return $::NOK;
        }

        # Add cluster-wide configuration information to the file
        if ( defined($::REMOTE_SHELL) ) {
            print CONFIG_OUT "RemoteShell=$::REMOTE_SHELL\n";
        }
        if ( defined($::SETUP_REMOTE_SHELL) ) {
            print CONFIG_OUT "SetupRemoteShell=$::SETUP_REMOTE_SHELL\n";
        }

        # add the node configuration information to the file
        if ( defined($hname) ) {
            print CONFIG_OUT "Hostname=$hname\n";
        }
        if ( defined($DestNodeHash{$hname}{'ManagementServer'}) ) {
            print CONFIG_OUT "ManagementServer=$DestNodeHash{$hname}{'ManagementServer'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'InstallOSName'} ) ) {
            print CONFIG_OUT "InstallOSName=$DestNodeHash{$hname}{'InstallOSName'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'InstallCSMVersion'}) ) {
            print CONFIG_OUT "InstallCSMVersion=$DestNodeHash{$hname}{'InstallCSMVersion'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'InstallDistributionVersion'}) ) {
            print CONFIG_OUT "InstallDistributionVersion=$DestNodeHash{$hname}{'InstallDistributionVersion'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'InstallDistributionName'} ) ) {
            print CONFIG_OUT "InstallDistributionName=$DestNodeHash{$hname}{'InstallDistributionName'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'InstallPkgArchitecture'}) ) {
            print CONFIG_OUT "InstallPkgArchitecture=$DestNodeHash{$hname}{'InstallPkgArchitecture'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'Mode'}) ) {
            print CONFIG_OUT "Mode=$DestNodeHash{$hname}{'Mode'}\n";
        }
        if ( defined($DestNodeHash{$hname}{'ConsoleSerialDevice'}) ) {
            print CONFIG_OUT "ConsoleSerialDevice=$DestNodeHash{$hname}{'ConsoleSerialDevice'}\n";
        }
        if (defined($DestNodeHash{$hname}{'InstallServer'}) && ($DestNodeHash{$hname}{'InstallServer'} ne "") ) {
		my $progname = NodeUtils->programName();
		if($progname =~ m/installnode/){
			#make sure a machine is not an install server of itself
			my ($node_hostname, $node_ip) = NodeUtils->getHost($hname);
			my ($server, $rep_dir) = split ':', $DestNodeHash{$hname}{'InstallServer'};
			my ($isvr_hostname, $isvr_ip) = NodeUtils->getHost($server);
			if($node_hostname ne $isvr_hostname){
				print CONFIG_OUT "InstallServer=$DestNodeHash{$hname}{'InstallServer'}\n";
			}	    
		}
		elsif($DestNodeHash{$hname}{'InstallServer'} ne $DestNodeHash{$hname}{'ManagementServer'}){
		    #not using the ms as the install server
		    print CONFIG_OUT "InstallServer=$DestNodeHash{$hname}{'InstallServer'}\n";
		}
        }
		
		if ( defined($DestNodeHash{$hname}{'PowerMethod'}) ) {
	    print CONFIG_OUT "PowerMethod=$DestNodeHash{$hname}{'PowerMethod'}\n";
        }

        if ( defined($DestNodeHash{$hname}{'InstallAdapterMacaddr'}) ) {
            print CONFIG_OUT "InstallAdapterMacaddr=$DestNodeHash{$hname}{'InstallAdapterMacaddr'}\n";
        }

        # Handle Linux-specific attributes.
        if ( $DestNodeHash{$hname}{'InstallOSName'}eq "Linux") {

            # If csm.director.agent is installed on the management server,
            # then install it on the Linux nodes.
            print CONFIG_OUT "InstallDirector=$DirectorInstalled\n";
            # if $::NOSMS isn't set then perform software maintenance
            if(!$::NOSMS){
                print CONFIG_OUT "SMS=1\n";
            }
        }

        # add lists of scripts to run on the node
        if (defined(@{$::PREINSTSCRIPTS{$hname}} )) {
            print CONFIG_OUT "PREINSTSCRIPTS=@{$::PREINSTSCRIPTS{$hname}}\n";
        }
        if (defined ( @{$::POSTINSTSCRIPTS{$hname}}) ) {
            print CONFIG_OUT "POSTINSTSCRIPTS=@{$::POSTINSTSCRIPTS{$hname}}\n";
        }
        if (defined( @{$::UPDATESCRIPTS{$hname}})) {
            print CONFIG_OUT "UPDATESCRIPTS=@{$::UPDATESCRIPTS{$hname}}\n";
        }

        close(CONFIG_OUT);
        
        $cmd = "$::CHMOD 644 $csmcfgdir/$hname.config_info";
        $rc = system("$cmd");
        if ($rc >> 8){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', $::CHMOD, $rc);
            # try to continue
        }

	#Add symbol link short_hostname.config_info to long_hostname.config_info
	my ($short_hname) = split('\.', $hname);

        # this symbolic link will be done only when the short name is not the
        # same as the long name
       if ( $short_hname ne $hname ) {
        $cmd = "cd $csmcfgdir; $::LN -fs $hname.config_info $short_hname.config_info";
        $rc = system("$cmd");
        if ($rc >> 8){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', $::LN, $rc);
        }
       }
	
    }  # go do the next node

    return $::OK;
}

#-------------------------------------------------------------------------#
#    create_nodemap_file                                                  #
#                                                                         #
# Create /csminstall/csm/config/nodemap file that contains a mapping of   #
# ManagedNode hostnames as defined in the CSM database, and node          #
# hostnames as returned by the hostname command.                          #
#                                                                         #
# File format:                                                            #
#      csm_hostname<space>node_hostname                                   #
#                                                                         #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub create_nodemap_file {
    my ($class, $ref_nodelist, $ref_nodeinfo, $ref_DestNodeHash) = @_;
    my @nodelist = @$ref_nodelist;
    my @nodeinfo = @$ref_nodeinfo;
    #  DestNodeHash is a hash containing attribute values for each node
    #    that was provided on the command line.
    my %DestNodeHash = %$ref_DestNodeHash;
    %main::dsh_failed = ();
    my ($cmd, @dshout, $rc);
    my ($csm_nodename, $csmnode, $hostname);
    my $csmcfgdir = "/csminstall/csm/config";
    my $nodemap_file = "/csminstall/csm/config/nodemap";
    my $node;
    my $dshresult="";
    #
    #  Check and/or create the /csminstall/csm/config directory
    #
    if (! -d $csmcfgdir) {
        $cmd = "$::MKDIR -m 755 -p $csmcfgdir";
        $rc = system("$cmd");
        if ($rc >> 8) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_CreateDir', $csmcfgdir);
            return($::NOK);
        }
    }
    # Remove any old occurences of the nodemap file
    if (-f $nodemap_file) {
        $cmd = "$::RM $nodemap_file";
        $rc = system("$cmd") >> 8;
        if ($rc != 0) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_RM', "$nodemap_file");
            # try to continue
        }
    }
    # open the map file
    unless (open(NODEMAP,">$nodemap_file")) {
        # print error message
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', "$nodemap_file");
        return $::NOK;
    }
    my @AIX;
    my @Linux;
    my %found = ();
    my %dups = ();
    # for each node in the node list
    foreach $node (keys %DestNodeHash) {
	# set the correct command based on the InstallOSName
        if ( defined($DestNodeHash{$node}{'InstallOSName'}) ) {
            if ($DestNodeHash{$node}{'InstallOSName'} eq "AIX") {
                #$dshcmd = "$::DSH -n$node hostname";
                push @AIX, $node;
            } elsif ($DestNodeHash{$node}{'InstallOSName'} eq "Linux") {
                #$dshcmd = "$::DSH -n$node hostname -f";
                push @Linux, $node;
            }
	} 
    }
    if(@Linux){
	my $node_file = InstallUtils->make_node_list_file(\@Linux); #put all dest nodes in tmp file
	$ENV{'DSH_LIST'} = $node_file;
	my $dshcmd = "$::DSH hostname -f";
	my @dshresult = `$dshcmd`;
	$rc = $? >> 8;
	InstallUtils->close_delete_file($::NODE_LIST_FILE, $node_file); #delete node file
	if ($rc != 0) {
	    # print error message
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', $dshcmd, $rc);
	    #$main::dsh_failed{$node}=1; #take these out of target nodes
	    #next;
        }
	foreach my $line (@dshresult){
	    my ($csmnode, $hostname) = split(" ",$line);
	    $hostname     =~ s/://g;
	    $csmnode      =~ s/://g;
	    if ( defined($csmnode) && defined($hostname) ) {
		# make sure the format is consistent with the name that
		#     was used to create the configinfo file
		#$csm_nodename = NodeUtils->tryHost($csmnode);
		$found{$csmnode}=1;
		$dups{$hostname}{$csmnode}=1;
		# put the result in the nodemap file
		unless(print NODEMAP "$csmnode $hostname\n")  {
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_WRITEtoFILE', "$nodemap_file");
		}
	    }
	}
    }
    if(@AIX){
	my $node_file = InstallUtils->make_node_list_file(\@AIX); #put all dest nodes in tmp file
	$ENV{'DSH_LIST'} = $node_file;
	my $dshcmd = "$::DSH hostname";
	my @dshresult = `$dshcmd`;
	$rc = $? >> 8;
	InstallUtils->close_delete_file($::NODE_LIST_FILE, $node_file); #delete node file
	if ($rc != 0) {
	    # print error message
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCMD_FAILED_RC', $dshcmd, $rc);
	    #$main::dsh_failed{$node}=1; #take these out of target nodes
	    #next;
	}
	foreach my $line (@dshresult){
	    my ($csmnode, $hostname) = split(" ",$line);
	    $hostname     =~ s/://g;
	    $csmnode      =~ s/://g;
	    if ( defined($csmnode) && defined($hostname) ) {
		# make sure the format is consistent with the name that
		#     was used to create the configinfo file
		#$csm_nodename = NodeUtils->tryHost($csmnode);
		$found{$csmnode}=1;
		$dups{$hostname}{$csmnode}=1;
		# put the result in the nodemap file
		unless(print NODEMAP "$csmnode $hostname\n")  {
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_WRITEtoFILE', "$nodemap_file");
		}
	    }
	}
    }
    #foreach my $n (keys %DestNodeHash){
#	if(!$found{$n}){
#	    $main::dsh_failed{$n}=1;   
#	}
#    }
    foreach my $n (keys %dups){ #get rid of any nodes that map to the same hostname
	    if( scalar (keys %{$dups{$n}}) > 1 ){
		    my @dupNodes = keys %{$dups{$n}};
		    my $dNstring = join ',', @dupNodes;
		    my $hostname = $n;
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgHostnameMap', $hostname, $dNstring);
		    foreach my $d (@dupNodes){
			    $main::dsh_failed{$d}=1;
		    }
	    }
    }
    close (NODEMAP);
    if (!@AIX && !@Linux){
        return $::NOK;
    }
    return $::OK;
}

#-------------------------------------------------------------------------#
#                                                                         #
#   start_nfs                                                             #
#                                                                         #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub start_nfs{
    my ($cmd, $output);
    my $rc=0;
    my $exports = "/etc/exports";
    #my $PLTFRM = `uname`; chomp $PLTFRM;      #using $::PLTFRM instead
    if($::MS_INSTALL_SERVER){ #the MS is serving NFS files
	# Ensure nfs is running.
	if (! -s $exports){ 
	    # nfs will only start if /etc/exports has a size > 0
	    open (EXPORTS, ">$exports") || NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_READ_FILE', $exports); 
	    print EXPORTS "\n";
	    close (EXPORTS);
	}
	
	# Start nfs if needed.  
	if ($::PLTFRM eq "AIX") {
	    my $cmd = "$::MKNFS -B  >/dev/null 2>&1";
	    NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	    my $rc  = system ($cmd) >> 8;
	    if ($rc) {
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
	    }
	    
	} else {
	    # We need modify start_nfs in rc.config on SuSE
	    my $distro=NodeUtils->distribution();
	    if($distro =~ /SuSE/ || $distro =~ /SLES/ ){
		my $RCCONFIG="/etc/rc.config";
		my $s=NodeUtils->readFile($RCCONFIG);
		$s=~s/^\s*NFS_SERVER\s*=\s*"\s*[Nn][Oo]\s*"\s*$/NFS_SERVER="yes"/m;
		if(open(OUT, ">$RCCONFIG")){
		    print OUT ($s);
		    close(OUT);
		} else{
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgShow_Output','Can not open $RCCONFIG for write');
		}
	    }
	    
	    # The output of the 'service nfs probe' tells 
	    #    what needs to be done to start nfs.  It returns either 
	    #    'start', 'restart', 'reload' or nothing.
	    
	    # SuSE Change
	    #$output = `$::SERVICE nfs probe 2>&1`;
	    $output=InstallUtils->nfsprobe();
	    
	    if ($output){
		# SuSE Change
		#$cmd = "$::SERVICE nfs $output";
		$cmd=NodeUtils->service("nfs",$output);
		
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		$rc  = system ($cmd) >> 8;
		if ($rc) { 	
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCommandFailed', $cmd, $rc);
		} else{
		    #wait for nfsserver to start on SLES7
		    sleep(1);
		}
	    }
	}
    }
    if(defined %::INSTALL_SERVERS){ #There are install servers serving files
	#get info about install servers
	my @make_ex = ();
	my @AIX_servers = ();
	my @RedHat_servers = ();
	my @SUSE_servers = ();
	#my @Linux_servers = ();
	my %service;
	if(defined %{$::INSTALL_SERVERS{'isnode'}}){
	    #if some of the install servers are nodes then this is a lot easier
	    my @servers = keys %{$::INSTALL_SERVERS{'isnode'}};
	    foreach my $s (@servers){
		my $os = $::INSTALL_SERVERS{'isnode'}{$s}{'OS'};
		my $distro = $::INSTALL_SERVERS{'isnode'}{$s}{'DISTRO'};
		if($os eq "AIX"){
		    push @AIX_servers, $s;
		}
		elsif($os eq "Linux"){
		    if($distro =~ m/RedHat/){
			push @RedHat_servers, $s;
		    }
		    elsif($distro =~ m/SuSE/ || $distro =~ m/SLES/){ #suse?
			push @SUSE_servers, $s;
		    }
		    #    push @Linux_servers, $s;
		}
	    }
	    if(@RedHat_servers){
		my $list = join ',', @RedHat_servers;
		my $cmd = "LANG=C $::DSH -n $list \"$::LS -s $exports; /sbin/service nfs probe;\"";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		my @output = `$cmd`;
		foreach my $line (@output){
		    chomp $line;
		    my ($server, $data) = split ":", $line, 2;
		    if($line =~ m/$exports/){ #this is the ls line
			if($line !~ m/"does not exist"/ || $line !~ m/"not found"/){
			    #then /etc/exports exists
			    $data =~ s/^\s+//; #remove any leading whitespace
			    my ($size, $file) = split /\s+/, $data, 2;
			    if($size ==0){
				push @make_ex, $server;
			    }
			}
			else{ #/ext/exports does not exist
			    push @make_ex, $server;
			}
		    }
		    elsif(($data =~ m/start/) || ($data =~ m/restart/) || ($data =~ m/reload/)){
			$data =~ s/^\s+//; #remove any leading whitespace
			push @{$service{$data}}, $server;
		    }
		}
	    }
	    if(@AIX_servers || @SUSE_servers){
		my $list = join ',', @SUSE_servers, @AIX_servers;
		#make sure /exports is larger than 0
		my $cmd = "LANG=C $::DSH -n $list \"$::LS -s $exports;\" 2>&1";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		my @output = `$cmd`;
		foreach my $line (@output){
		    chomp $line;
		    my ($server, $data) = split ":", $line, 2;
		    if($line =~ m/$exports/){ #this is the ls line
			if($line !~ m/"does not exist"/ || $line !~ m/"not found"/){
			    #then /etc/exports exists
			    $data =~ s/^\s+//; #remove any leading whitespace
			    my ($size, $file) = split /\s+/, $data, 2;
			    if($size ==0){
				push @make_ex, $server;
			    }
			}
			else{ #no file
			    push @make_ex, $server;
			}
		    }
		}
	    }
	}
	if(defined %{$::INSTALL_SERVERS{'notnode'}}){
	    #size of /etc/exports
	    #OS
	    my $dsh_list = join ",", (keys %{$::INSTALL_SERVERS{'notnode'}});
	    my $cmd = "LANG=C $::DSH -n $dsh_list \"$::LS -s $exports; /bin/uname; /bin/cat /etc/*release; /sbin/service nfs probe;\" 2>&1";
	    NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	    my @output = `$cmd`;
	    foreach my $line (@output){
		chomp $line;
		my ($server, $data) = split ":", $line, 2;
		if($line =~ m/$exports/){ #this is the ls line
		    if($line !~ m/"does not exist"/ || $line !~ m/"not found"/){
			#then /etc/exports exists
			$data =~ s/^\s+//; #remove any leading whitespace
			my ($size, $file) = split /\s+/, $data, 2;
			if($size ==0){
			    push @make_ex, $server;
			}
		    }
		    else{
			push @make_ex, $server;
		    }
		}
		elsif($data =~ m/AIX/){
		    $::INSTALL_SERVERS{'notnode'}{$server}{'OS'}="AIX";
		    push @AIX_servers, $server;
		}
		elsif($data =~ m/Red Hat/){
		    $::INSTALL_SERVERS{'notnode'}{$server}{'OS'}="Linux";
		    $::INSTALL_SERVERS{'notnode'}{$server}{'DISTRO'}="RedHat";
		    push @RedHat_servers, $server;
		}
		elsif($data =~ m/SuSE/){
		    $::INSTALL_SERVERS{'notnode'}{$server}{'OS'}="Linux";
		    $::INSTALL_SERVERS{'notnode'}{$server}{'DISTRO'}="SUSE";
		    push @SUSE_servers, $server;
		}
		elsif(($data =~ m/start/) || ($data =~ m/restart/) || ($data =~ m/reload/)){
		    $data =~ s/^\s+//; #remove any leading whitespace
		    push @{$service{$data}}, $server;
		}
	    }
	}   
	#first make sure /etc/exports has a size > 0 on all of the install servers
	if(@make_ex){
	    my $ex_list = join ",", @make_ex;
	    $cmd = "$::DSH -n $ex_list \"/bin/echo ' ' >> $exports\" 2>&1";
	    NodeUtils->runcmd($cmd,0);
	}
	if(@RedHat_servers){
	    #/sbin/service
	    #output of service nfs probe
	    #then start NFS
	    #special steps required in SuSE / SLES case -- these will be implemented in rigel
	    foreach my $value (keys %service){
		my $list = join ",", @{$service{$value}};	
		$cmd = "$::DSH -n $list \"/sbin/service nfs $value &\" 2>&1";
		NodeUtils->runcmd($cmd,0);
	    }
	}
	if(@SUSE_servers){
	    my $list = join ',', @SUSE_servers;
	    #going to need to run sed command to edit /etc/rc.config on SUSE server
	    my $RCCONFIG="/etc/rc.config";
	    my $CHECKPROC = "/sbin/checkproc";
	    my $PIDOF = "/sbin/pidof";
	    my $NFSD="nfsd";
	    my $MOUNTD="/usr/sbin/rpc.mountd"; 
	    my $EXPORTS="/etc/exports";
	    my $ETAB="/var/lib/nfs/etab";
	    #$cmd = qq($::DSH -n $list '/bin/sed -e s/.*NFS_SERVER.*/NFS_SERVER=\\"yes\\"/ $RCCONFIG | cat '-' > /tmp/csm.rcconfig; /bin/mv /tmp/csm.rcconfig $RCCONFIG; $CHECKPROC -n $NFSD; echo \"nfsd proc rc=\$?\"; $CHECKPROC $MOUNTD; /bin/echo \"mountd proc rc=\$?\"; $PIDOF $NFSD; echo \"pidof nfsd rc=\$?\"; $PIDOF $MOUNTD; echo \"pidof mountd rc=\$?\"; /usr/bin/stat -c %n=%Y $EXPORTS; /usr/bin/stat -c %n=%Y $ETAB;');
	    $cmd = qq($::DSH -n $list '/bin/sed -e s/.*NFS_SERVER.*/NFS_SERVER=\\"yes\\"/ $RCCONFIG | cat '-' > /tmp/csm.rcconfig; /bin/mv /tmp/csm.rcconfig $RCCONFIG; $CHECKPROC -n $NFSD; echo \"nfsd proc rc=\$?\"; $CHECKPROC $MOUNTD; /bin/echo \"mountd proc rc=\$?\"; $PIDOF $NFSD; echo \"pidof nfsd rc=\$?\"; $PIDOF $MOUNTD; echo \"pidof mountd rc=\$?\";');
	    NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	    my @output = `$cmd`;
	    my %service_hash;
	    my %suse_service = ();
	    foreach my $line (@output){
		# print "LINE = $line";
		chomp $line;
		my $hostname;
		($hostname, $line) = split ':', $line, 2;
		# print "HOSTNAME = $hostname\n";
		my $rc = (split '=', $line)[1];
		if($line =~ m/nfsd proc/){
		    $service_hash{$hostname}{'NFSD'}{'PROC'}=$rc;
		#    print "NFSD PROC = $rc\n";
		}
		elsif($line =~ m/mountd proc/){
		    $service_hash{$hostname}{'MOUNTD'}{'PROC'}=$rc;
		# print "MOUNTD PROC = $rc\n";
		}
		elsif($line =~ m/pidof nfsd/){
		    $service_hash{$hostname}{'NFSD'}{'PIDOF'}=$rc;
		#    print "NFSD PIDOF = $rc\n";
		}
		elsif($line =~ m/pidof mountd/){
		    $service_hash{$hostname}{'MOUNTD'}{'PIDOF'}=$rc;
		#    print "MOUNTD PIDOF = $rc\n";
		}
		elsif($line =~ m/$EXPORTS/){
		    $service_hash{$hostname}{'EXPORTS'}=$rc;
		}
		elsif($line =~ m/$ETAB/){
		    $service_hash{$hostname}{'ETAB'}=$rc;
		}
	    }
	    foreach my $hostname (keys %service_hash){
		my $value;
		if($service_hash{$hostname}{'NFSD'}{'PROC'} || $service_hash{$hostname}{'MOUNTD'}{'PROC'}){
		    $value = "start";
		}
		elsif($service_hash{$hostname}{'NFSD'}{'PIDOF'} ==1 || $service_hash{$hostname}{'MOUNTD'}{'PIDOF'} == 1){
		    $value = "restart";
		}
		elsif($service_hash{$hostname}{'EXPORTS'} > $service_hash{$hostname}{'ETAB'}){
		    $value = "reload";
		}
		else{
		    $value = "status";
		}
		# print "VALUE=$value\n\n";
		push @{$suse_service{$value}}, $hostname;
	    }
	    foreach my $value (keys %suse_service){
		my $list = join ',', @{$suse_service{$value}};
		my $cmd = "$::DSH -n $list \"/etc/init.d/nfsserver $value;\"";
		NodeUtils->runcmd($cmd,0);
	    }
	}
	if(@AIX_servers){
	    my $list = join ',', @AIX_servers;
	    $cmd = "$::DSH -n $list \"$::MKNFS -B >/dev/null\" 2>&1";
	    NodeUtils->runcmd($cmd,0);
	}
    }
    return ($rc);
}

#-------------------------------------------------------------------------#
#                                                                         #
#   export_csminstall                                                     #
#                                                                         #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub export_csminstall{
    my ($class, $dir_to_be_exported)= @_;
    my $cmd;
    my $rc=0;
    #my $PLTFRM = `uname`; chomp $PLTFRM;    #using $::PLTFRM instead
    if($::MS_INSTALL_SERVER){ #the MS is serving NFS files
	if ( -d $dir_to_be_exported)
	{ 
	    if ($::PLTFRM eq "AIX") {
		#$cmd = "$::EXPORTFS -i -o ro /csminstall";
		$cmd = "$::EXPORTFS -i -o ro $dir_to_be_exported";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		$rc  = system ($cmd) >> 8;
		if ($rc) {
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
		}
		
	    } else {
		
		#$cmd = "$::EXPORTFS -o ro,no_root_squash \*:/csminstall";
		$cmd = "$::EXPORTFS -o ro,no_root_squash \*:$dir_to_be_exported";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		$rc  = system ($cmd) >> 8;
		if ($rc) { 
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCommandFailed', $cmd, $rc);
		}
	    }
	}
	else
	{
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','W', 'EMsgNo_Dir_Export', $dir_to_be_exported);
	    $rc=1;
	}
    }
    #determine AIX and Linux install servers
    my %AIX;
    my %Linux;
    foreach my $key (keys %::INSTALL_SERVERS){
	foreach my $server (keys %{$::INSTALL_SERVERS{$key}}){
	    my $os = $::INSTALL_SERVERS{$key}{$server}{'OS'};
	    my $dir = $::INSTALL_SERVERS{$key}{$server}{'dir'};
	    if($os eq "AIX"){
		$AIX{$dir}{$server}=1;
	    }
	    elsif($os eq "Linux"){
		$Linux{$dir}{$server}=1;
	    }
	    else{
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgINVALID_OSTYPE');
	    }
	}
    }
   foreach my $dir (keys %AIX){ #there are AIX install servers, this code will be done in rigel
   	my $list = join ",", keys %{$AIX{$dir}};
   	my $real_dir = $dir_to_be_exported;
   	#check to see if $dir is already exported
   	$cmd = "$::DSH -n $list \"$::EXPORTFS | $::GREP $dir\" 2>&1";
   	NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
   	my @output =`$cmd`;
   	my @unexport;
   	foreach my $line (@output){
   	    chomp $line;
   	    my ($hostname, $line) = split ':', $line;
   	    if($line =~ m/$dir/){
   		push @unexport, $hostname;		
   	    }
   	}
   	$real_dir =~ s/^\/csminstall/$dir/;
   	if(@unexport){
   	    $cmd = "$::DSH -n $list '$::EXPORTFS -i -u $real_dir'";
   	    NodeUtils->runcmd($cmd,0);
   	}
   	$cmd = "$::DSH -n $list '$::EXPORTFS -i -o ro $real_dir\'";
   	NodeUtils->runcmd($cmd,0);
    }
    foreach my $dir (keys %Linux){
	my $list = join ",", keys %{$Linux{$dir}};
	my $real_dir = $dir_to_be_exported;
	#check to see if $dir is already exported
	$cmd = "$::DSH -n $list \"$::EXPORTFS | $::GREP $dir\" 2>&1";
	NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	my @output =`$cmd`;
	my @unexport;
	foreach my $line (@output){
		chomp $line;
		my ($hostname, $line) = split ':', $line;
		if($line =~ m/$dir/){
			push @unexport, $hostname;		
		}
	}
	
	$real_dir =~ s/^\/csminstall/$dir/;
	
	if(@unexport){
		$cmd = "$::DSH -n $list '$::EXPORTFS -u \*:$real_dir'";
		NodeUtils->runcmd($cmd,0);
	}
	
	$cmd = "$::DSH -n $list \"$::EXPORTFS -o ro,no_root_squash \*:$real_dir\"";
	NodeUtils->runcmd($cmd,0);
    }
    return ($rc);
}

#-------------------------------------------------------------------------#
#                                                                         #
#   unexport_csminstall                                                   #
#                                                                         #
#                                                                         #
#   Return Codes: 0 - All was successful.                                 #
#                 1 - An error occured.                                   #
#-------------------------------------------------------------------------#
sub unexport_csminstall{
    my ($class, $dir_to_be_unexported)= @_;
    my $cmd;
    my $rc=0;
    #my $PLTFRM = `uname`; chomp $PLTFRM;      #using $::PLTFRM instead
    if($::MS_INSTALL_SERVER){ #the MS is serving NFS files
	if ($::PLTFRM eq "AIX") {
	    #$cmd = "$::EXPORTFS -i -u /csminstall";
	    $cmd = "$::EXPORTFS -i -u $dir_to_be_unexported";
	    NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	    $rc  = system ($cmd) >> 8;
	    if ($rc) { 
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
	    }
	} else {
	    #$cmd = "$::EXPORTFS -u \*:/csminstall";
	    $cmd = "$::EXPORTFS -u \*:$dir_to_be_unexported";
	    NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	    $rc  = system ($cmd) >> 8;
	    if ($rc) { 
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCommandFailed', $cmd, $rc);
	    }
	}    
    }
    #determine AIX and Linux install servers
    my %AIX;
    my %Linux;
    foreach my $key (keys %::INSTALL_SERVERS){
	foreach my $server (keys %{$::INSTALL_SERVERS{$key}}){
	    my $os = $::INSTALL_SERVERS{$key}{$server}{'OS'};
	    my $dir = $::INSTALL_SERVERS{$key}{$server}{'dir'};
	    if($os eq "AIX"){
		$AIX{$dir}{$server}=1;
	    }
	    elsif($os eq "Linux"){
		$Linux{$dir}{$server}=1;
	    }
	    else{
    		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgINVALID_OSTYPE');
	    }
	}
    }
   foreach my $dir (keys %AIX){ #there are AIX install servers, this code will be done in rigel
  	my $list = join ",", keys %{$AIX{$dir}};
  	my $real_dir = $dir_to_be_unexported;
  	$real_dir =~ s/^\/csminstall/$dir/;
  	$cmd = "$::DSH -n $list '$::EXPORTFS -i -u $real_dir'";
  	NodeUtils->runcmd($cmd,0);
    }
    foreach my $dir (keys %Linux){
	my $list = join ",", keys %{$Linux{$dir}};
	my $real_dir = $dir_to_be_unexported;
	$real_dir =~ s/^\/csminstall/$dir/;
	$cmd = "$::DSH -n $list '$::EXPORTFS -u \*:$real_dir'";
	NodeUtils->runcmd($cmd,0);
    }
    return ($rc);
}

###########################################################	
# Get the hostname of the management server.
#
# Since the nodes might need to contact the management server through a
# different address than its primary hostname, we cannot just use the output
# of the hostname command here.  Use the getSourceIP2Target command instead.
#
# If we cannot determine the route to the node via getSourceIP2Target
# just return nothing.  This indicates that there is no way to get to the node.
###########################################################

sub get_management_server
{
    my ($class, $node_hostname) = @_;
    my $ipaddr   = "";
    my $hostname = "";

    # Move this to InstallDefs.pm.perl
    $::GETSOURCEIP2TARGET = "/opt/csm/csmbin/getSourceIP2Target";

    my $cmd = "$::GETSOURCEIP2TARGET $node_hostname";
    my @output = NodeUtils->runcmd($cmd,0);
    my $rc = $::RUNCMD_RC;
    if (! $rc) # If no errors
    {
	$ipaddr = $output[0];

	if($ipaddr){
	    $hostname = NodeUtils->tryHost($ipaddr);
	}
    }

    if (wantarray) { return ($hostname, $ipaddr); }
    else { return $hostname; }
}

#################################################
#The following two subroutines are used for locking the tool
#by obtaining a lock at the beginning of execution and holding
#it until the tool completes. Therefore it is guaranteed that two or more
#instances of updatenode will NOT be able to be run.
#
#If process A invokes get_lock and process B invokes it after 
#process A has, process B will fail. 
#
#In the event that the program is aborted, the lock is automatically
#released.
#
#INPUT:
#	get_lock takes the filename you want to use as an argument
#OUTPUT:
#	returns a 2 if it couldn't open the file to lock it
#	returns a 3 if there was an error locking the file
###########################################################

sub get_lock 
{
	my ($class, $file) =@_;
	my $return_code = 0;
	my $program = NodeUtils->programName();
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"V", 'IMsgGet_Lock', $file, $program);
	if(!open(SEM,">$file")) { 
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E1", 'EMsgSEMAPHORE_ERROR', $file, "$? $!");
	}
	if(!flock(SEM,LOCK_EX|LOCK_NB)) 
	{ 
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E1", 'EMsgLOCK_ERROR', $program, $file, "$? $!");
	}
}

#-------------------------------------------------------------------------#
#                                                                         #
#   release lock							  #
#                                                                         #
#       Releases the lock of the file of get lock sub routine             #
#-------------------------------------------------------------------------#
sub release_lock
{
	my ($class, $file) =@_;
	
    close(SEM);
	if(-e "$file"){
		system("/bin/rm -f $file");
	}
}

#-------------------------------------------------------------------------#
#                                                                         #
#   backup_logfile                                                        #
#                                                                         #
#        Backup the current logfile. Move logfile to logfile.1.           #
#        Shift all other logfiles (logfile.[1-3]) up one number.          #
#        The original logfile.4 is removed.                               #
#                                                                         #
#-------------------------------------------------------------------------#
sub backup_logfile
{
    my ($logfile) = @_;
    my ($logfile1) = $logfile . ".1";
    my ($logfile2) = $logfile . ".2";
    my ($logfile3) = $logfile . ".3";
    my ($logfile4) = $logfile . ".4";

    if (-f $logfile) {

        rename ($logfile3, $logfile4) if (-f $logfile3);
        rename ($logfile2, $logfile3) if (-f $logfile2);
        rename ($logfile1, $logfile2) if (-f $logfile1);
        rename ($logfile,  $logfile1);
    }
    return $::OK;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    start_logging                                                        #
#                                                                         #
#           Start logging messages to a logfile. Return the log file      #
#           handle so it can be used to close the file when done logging. #
#                                                                         #
#-------------------------------------------------------------------------#
sub start_logging
{
    my ($class, $logfile, $quiet_flag) = @_;
    my ($cmd, $rc);
	if(!$quiet_flag){
		$quiet_flag =0;
	}
    &backup_logfile($logfile);

    # create the log directory if it's not already there
    if (! -d $::CSMLOG) {
        $cmd = "$::MKDIR -m 644 -p $::CSMLOG";
        $rc = system("$cmd");
        if ($rc >> 8) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_CreateDir', $::CSMLOG);
            return($::NOK);
        }
    }

    # open the log file
    unless (open(LOGFILE, ">>$logfile")) {
        # Cannot open file
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', $logfile);
        return $::NOK;
    }

    $::LOG_FILE_HANDLE = \*LOGFILE;

    # Print the date to the top of the logfile
    my $sdate = `/bin/date`;
    chomp $sdate;
    my $program = NodeUtils->programName();
    if ($quiet_flag !=1)
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgWRITING_LOGFILE', $program, $logfile);
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"LI", 'IMsgSTART_LOGGING', $sdate);
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"LI", 'IMsgShow_Output', $::command_line);
    }

    return ($::LOG_FILE_HANDLE);
}

#-------------------------------------------------------------------------#
#                                                                         #
#    append_logging                                                       #
#                                                                         #
#         Append logging messages to a logfile. Return the log file       #
#           handle so it can be used to close the file when done logging. #
#                                                                         #
#-------------------------------------------------------------------------#
sub append_logging
{
    my ($class, $logfile, $quiet_flag) = @_;
    my ($cmd, $rc);
if(!$quiet_flag){
		$quiet_flag =0;
	}
    # create the log directory if it's not already there
    if (! -d $::CSMLOG) {
        $cmd = "$::MKDIR -m 644 -p $::CSMLOG";
        $rc = system("$cmd");
        if ($rc >> 8) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_CreateDir', $::CSMLOG);
            return($::NOK);
        }
    }

    #  
    #  get log file ready
    #
    if (! -e $logfile) {  #  create the log file if not already there
        # open the log file
        unless (open(LOGFILE, ">$logfile")) {
            # Cannot open file
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', $logfile);
            return $::NOK;
        }
    } else {   # it's there so just append
        unless (open(LOGFILE, ">>$logfile")) {
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', $logfile);
            return $::NOK;
        }
    }

    $::LOG_FILE_HANDLE = \*LOGFILE;

    # Print the date to the top of the logfile
    my $sdate = `/bin/date`;
    chomp $sdate;
    my $program = NodeUtils->programName();
    if ($quiet_flag !=1)
    {
       NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgWRITING_LOGFILE', $program, $logfile);
       NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"LI", 'IMsgSTART_LOGGING', $sdate);
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"LI", 'IMsgShow_Output', $::command_line);
    }

    return ($::LOG_FILE_HANDLE);
}

#-------------------------------------------------------------------------#
#                                                                         #
#    stop_logging                                                         #
#                                                                         #
#                  Turn off message logging.  (Expects to have a file     #
#                   handle passed in. )                                   #
#                                                                         #
#-------------------------------------------------------------------------#
sub stop_logging
{
    my ($class, $quiet_flag) = @_;
    if(!$quiet_flag){
    		$quiet_flag =0;
	}
	if (defined($::LOG_FILE_HANDLE)) {
    		# Print the date at the bottom of the logfile
                if ($quiet_flag !=1)
                { 
			my $date = `/bin/date`;
			chomp $date;
       		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"LI", 'IMsgSTOP_LOGGING', $date);
                }

    		close($::LOG_FILE_HANDLE);
    		undef $::LOG_FILE_HANDLE;
	}
    return $::OK;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    get_OSName                                                           #
#                                                                         #
#                  Returns the Os name i.e. Linux , AIX else undefined    #
#                                                                         #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_OSName
{
    if(NodeUtils->isLinux()==1){
	    return "Linux";
    }
    elsif(NodeUtils->isAIX()==1){
	    return "AIX"
    }
    return undef;
}

#------------------------------------------------------------------------#
#                                                                        #
#    DistributionVersion                                                       #
#                                                                        #
#        Returns the Operating System version that is installed on the   #
#        management server.                                              #
#        For AIX, the version is in the form V.R.M.  For Linux, it is    #
#        in the form reported in the /etc/redhat-release file (V.R).     #
#------------------------------------------------------------------------#
sub get_DistributionVersion
{
    my ($cmd, $output, $rpm, $version, $release, $v1, $v2, $v3, $rest);

    if ($::PLTFRM eq "Linux")
    {
        
        $version = (split ' ', (NodeUtils->distribution()))[1];
        
        #if (-f '/etc/redhat-release')
        #{
        #    my $s = NodeUtils->readFile('/etc/redhat-release');
        #    ($version) = $s =~ /\s(\d+\.\d+)\s/;
        #}
    }
    elsif ($::PLTFRM eq "AIX")
    {
        #$cmd = "$::OSLEVEL";
        $cmd = "$::LSLPP -Lcq bos.rte | /usr/bin/cut -d: -f 3";
        $output = NodeUtils->runcmd($cmd);
        chomp ($output);

        ($v1, $v2, $v3, $rest) = split(/\./, $output);
        $version = join (".", $v1, $v2, $v3);
    }
    else
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
    }

    return (length($version) ? $version : undef);
}

#------------------------------------------------------------------#
#                                                                  #
#    get_CSMVersion                                                #
#                                                                  #
#       Get the version of csm component that is installed on the  #
#       management server.                                         #
#       The version should be in the form "V.R.M" (i.e 1.2.0).     #
#       If showfixlevel is true (1) the version will be in the     #
#       format "V.R.M.F" (i.e. 1.2.0.0)                            #
#                                                                  #
#------------------------------------------------------------------#
sub get_CSMVersion
{
    my ($class, $component, $showfixlevel) = @_;
    my ($cmd, $output, $rpm, $version, $release, $v1, $v2, $v3, $v4, $rest);

    if ($::PLTFRM eq "Linux")
    {
        $cmd = "$::RPMCMD -q $component";
        $output = NodeUtils->runcmd($cmd);
        chomp ($output);
        ($rpm, $version, $release) = split (/-/, $output);

        ($v1, $v2, $v3, $v4, $rest) = split(/\./, $version);
        if ($showfixlevel) { $version = join (".", $v1, $v2, $v3, $v4); }
        else { $version = join (".", $v1, $v2, $v3); }
    }
    elsif ($::PLTFRM eq "AIX")
    {
        $cmd = "$::LSLPP -cLq $component";
        $output = NodeUtils->runcmd($cmd);
        chomp ($output);

        # The third field in the lslpp output is the VRMF
        ($version) = (split(/:/, $output))[2];

        ($v1, $v2, $v3, $v4, $rest) = split(/\./, $version);
        if ($showfixlevel) { $version = join (".", $v1, $v2, $v3, $v4); }
        else { $version = join (".", $v1, $v2, $v3); }
    }
    else
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
    }

    return (length($version) ? $version : undef);
}

#-------------------------------------------------------------------------#
#                                                                         #
#    get_PkgArchitecture                                                  #
#                                                                         #
#        Returns the Os Architure of the OS i.e. i386 for linux           #
#        This only for Linux                                              #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_PkgArchitecture
{
  my ($cmd, $output, $osarch);
 if ($::PLTFRM eq "Linux")
 {
    $cmd = "$::UNAME -m";
    $output = NodeUtils->runcmd($cmd);
    chomp ($output);

    $osarch = $output;

   return (length($osarch) ? $osarch : undef);
 }
 elsif ($::PLTFRM eq "AIX")
 {
     $osarch="";
    return (length($osarch) ? $osarch : undef);
 }
 else
 {
   NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
 }
}

#-------------------------------------------------------------------------#
#                                                                         #
#    get_pkgVersion                                                       #
#                                                                         #
#       Returns the installp image version from Installp command which,   #
#        looks into the .toc file.                                        #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_pkgVersion
{
   my ($class, $pkg) = @_;
   my ($cmd, $output, $version);

   if ($::PLTFRM eq "Linux")
   {
        $version = "";
      return (length($version) ? $version : undef);
   }
   elsif ($::PLTFRM eq "AIX")
   {
      $cmd=qq($::INSTALLPCMD -L -d . | grep $pkg 2>&1);
      $output = NodeUtils->runcmd($cmd);
      chomp ($output);
      (undef,undef,$version,undef)=split(/:/,$output);
      return (length($version) ? $version : undef);
   }
   else
   {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
   }

}

#-------------------------------------------------------------------------#
#                                                                         #
#    get_DistributionName                                                 #
#                                                                         #
#       Get the distribution name for linux its RedHat and return         # 
#       nothing for AIX.						  #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_DistributionName
{
    my ($cmd, $output, $rpm, $distro);

    # IF LINUX
    if ($::PLTFRM eq "Linux")
    {
        $distro = (split ' ', (NodeUtils->distribution()))[0];
        
        #if (-f '/etc/redhat-release')
        #{
            # RedHat is the only Linux distro we support for now.
        #    $distro = "RedHat";
        #}
    }
    elsif ($::PLTFRM eq "AIX")
    {
        $distro = "";
    }
    else
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
    }

    return (length($distro) ? $distro : undef);
}

#-------------------------------------------------------------------------#
#                                                                         #
#    get_uniqu_arry_elemnts                                               #
#                                                                         #
#       Returns the unique elements in an array                           #
#                                                                         #
#-------------------------------------------------------------------------#
sub get_uniqu_arry_elemnts
{
   my ($class, @my_arry) =@_;
   my %my_hsh=();
   my @uniq=();
   my $itm;
   foreach $itm (@my_arry)
   {
     $my_hsh{$itm}++;
   }
   @uniq= keys %my_hsh;
   return @uniq;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    findImagesInPath                                                     #
#                                                                         #
#       Find the images  in the given path                                #
#                                                                         #
#-------------------------------------------------------------------------#
sub findImagesInPath{
    # the $image variable may have a *, so we may find more than one that 
    # matches.

    my ($class, $image, @path) = @_;
    my (@tmpImages,@returnImages,$tmpImage,$realImage);

    foreach my $p (@path){
     	my $filename = "$p/$image";

     	# use the ls command here instead of the -e operator 
     	# so we can handle *s in the names

     	if (`/bin/ls $filename 2> /dev/null`){
       	    # put all of the images we just got into @tmpImages array
       	    chomp(@tmpImages = `/bin/ls $filename`);  

       	    # Go through each tmpImage and see if the base name already
            # exists in the returnImages array.  If it does exist, don't
            # put it in, otherwise, put it in!


       	    foreach $tmpImage (@tmpImages){
	  
	  	# get the base name of the test image. 

	  	my $tstImageBaseName;
		my $defined = 0;
	  	($tstImageBaseName = $tmpImage) =~ s:^.*/::g;
		

	  	foreach $realImage (@returnImages){

	      	    # get the base name of the real image	

	      	    my $realImageBaseName;
	      	    ($realImageBaseName = $realImage) =~ s:^.*/::g;

	       	    if($realImageBaseName eq $tstImageBaseName){
		  	$defined = "true";
	          	last;
	      	    }
	   	}	

	   	unless($defined){
		    push @returnImages, $tmpImage;
	   	}else{

print "\n\nnot copying $tmpImage\n\n";
		}
     	    }
   	} 
    }
    return @returnImages;
}



#-------------------------------------------------------------------------#
#                                                                         #
#    findRpmInPath                                                        #
#                                                                         #
#       Find the rpm package in the given path                            #
#                                                                         #
#-------------------------------------------------------------------------#
sub findRpmInPath
{
   my ($class, $rpmfile, @path) = @_;
   foreach my $p (@path)
   {
     my $filename = "$p/$rpmfile";
     # use the ls command here instead of the -e operator so we can handle *s in the names
     #print "looking for $filename: ", `/bin/ls $filename`, "\n";
     if (`/bin/ls $filename 2> /dev/null`)
     {
       NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgFOUND_RPM', $rpmfile, $p);
       return $p;
     }
   } 
   return undef;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    findRpm                                                              #
#                                                                         #
#       Try to find and return the rpm file that is the required version  #
#                                                                         #
#-------------------------------------------------------------------------#
sub findRpm
{
        my ($class, $pkg, $operator, $version, @my_path) = @_;

        my $os_rpm_path=$my_path[0];
        my $pkg_rpm_path=$my_path[1];
     if ($::PREREQS_ATTR{'OSName'} eq "Linux")
     {
        # Get the current distribution if it was not already obtained.
        my $distro = $::DISTRO;
        if (! $distro)
        {
            $distro = &get_DistributionName." ".&get_DistributionVersion;
            if (!grep(/^$distro$/, @::VALID_DISTROS))
            {
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','W', 'EMsgUNSUPPORTED_DISTRO', $distro, join(', ',@::VALID_DISTROS));
            }
            else
            {
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDETECTED_DISTRO', $distro);
            }
        }
      }
       # note: The -vr options on the ls command gives us the entries in
        #       reverse version order to get the latest version first.
        my @rpms;
        my @rpms1;
        my @rpms2;

        # Look for the rpm in packages
        if (NodeUtils->isLinux)
        {
           @rpms1 = NodeUtils->runcmd("$::LS -vr $pkg_rpm_path/$pkg-*", -1);
        }
        else
        {
           @rpms1 = NodeUtils->runcmd("$::LS  $pkg_rpm_path/$pkg-*", -1);
        }
        my $rc1 = $::RUNCMD_RC;
        if (! $rc1) { @rpms = (@rpms, @rpms1); } # Only add if no errors

        # Look for the rpm in  RPMS
        #my $TFTPBOOT_DISTRORPM = "$::TFTPBOOT/$::DISTRO_SHORTNAME{$distro}/$::DISTRO_RPMDIR{$distro}";
        if (NodeUtils->isLinux)
        {
           @rpms2 = NodeUtils->runcmd("$::LS -vr $os_rpm_path/$pkg-*", -1);
        }
        else
        {
           @rpms2 = NodeUtils->runcmd("$::LS $os_rpm_path/$pkg-*", -1);
        }
        my $rc2 = $::RUNCMD_RC;
        if (! $rc2) { @rpms = (@rpms, @rpms2); } # Only add if no errors

        if (($rc1 && $rc2) || !scalar(@rpms)) { return undef; }

        foreach my $r (@rpms)
          {
                #my ($ipkg, $iversion, $irelease) = split('-', $r);
                my ($fn) = $r =~ m|^.*?([^/]+)$|;      # get rid of the directory, if any
                #print "fn=$fn\n";
                my ($ipkg, $iversion, $irelease) = $fn =~ /^(.+)-(\d.*)-(.*)$/;
                #print "ipkg=$ipkg, iversion=$iversion, irelease=$irelease\n";
                # Note: irelease has .i386.rpm on the end of it
                if ($ipkg ne $pkg || !length($iversion)) { next; }
                if (!length($operator) || !length($version) || InstallUtils->testVersion($iversion, $operator, $version))
                  {
                        $operator=" " if (!$operator);
                        $version=" " if (!$version);

                        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgFOUND', $r, $operator, $version);
                        return $r;
                  }
          }
        return undef;   # didn't find a suitable rpm

}

#-------------------------------------------------------------------------#
#                                                                         #
#   testVersion                                                           #
#                                                                         #
#       Compare version1 and version2 according to the operator and       #
#       return true or false. These expressions come from the Require     #
#       statement of our rpms.						  #
#                                                                         #
#-------------------------------------------------------------------------#
sub testVersion
  {
	my ($class, $version1, $operator, $version2, $release1, $release2) = @_;

	# Note: perl seems to be able to handle really big integers, but if we run into a limit, we
	#       can use the Math::BigInt pkg

	my @a1 = split(/\./, $version1);
	my @a2 = split(/\./, $version2);
	my $len = (scalar(@a1)>scalar(@a2) ? scalar(@a1) : scalar(@a2));
	$#a1 = $len -1;      # make the arrays the same length before appending release
	$#a2 = $len -1;
	push @a1, split(/\./, $release1);
	push @a2, split(/\./, $release2);
	$len = (scalar(@a1)>scalar(@a2) ? scalar(@a1) : scalar(@a2));
	my $num1 = '';
	my $num2 = '';

	for (my $i=0; $i<$len; $i++)
	  {
		my ($d1) = $a1[$i] =~ /^(\d*)/;     # remove any non-numbers on the end
		my ($d2) = $a2[$i] =~ /^(\d*)/;
		
		my $diff = length($d1) - length($d2);
		if ($diff > 0)     # pad d2
		  {
			$num1 .= $d1;
			$num2 .= ('0' x $diff) . $d2;
		  }
		elsif ($diff < 0)     # pad d1
		  {
			$num1 .= ('0' x abs($diff)) . $d1;
			$num2 .= $d2;
		  }
		else        # they are the same length
		  {
			$num1 .= $d1;
			$num2 .= $d2;
		  }
	  }

	# Remove the leading 0s or perl will interpret the numbers as octal
	$num1 =~ s/^0+//;
	$num2 =~ s/^0+//;

	#SuSE Changes
	# if $num1="", the "eval '$num1 $operator $num2'" will fail. So MUSTBE be sure that $num1 is not a "".
	if ( length($num1) == 0) { $num1 = 0; }
	if ( length($num2) == 0) { $num2 = 0; }
	#End of SuSE Changes

	if (length($release1)) { $release1 = "-$release1"; }   # this is just for error msgs
	if (length($release2)) { $release2 = "-$release2"; }   # this is just for error msgs
	
	if ($operator eq '=') { $operator = '=='; }
	my $bool = eval "$num1 $operator $num2";
	if (length($@))
	  {
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','W', 'EMsgVERSION_COMPARE_PROBLEM', "$version1$release1 $operator $version2$release2", $@);
		return undef;
	  }
	if($ENV{LANG} =~ /^(C|POSIX|en_US)/){  #this is a work around for NLS problems
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCOMPARING_VERSION', "$version1$release1 $operator $version2$release2", ($bool?'true':'false'));
	}
	return $bool;
  }

#-------------------------------------------------------------------------#
#                                                                         #
#    installRequiredRpms                                                  #
#                                                                         #
#     Install any rpms that this rpm is dependent on if they              #
#     are not already on the system at the required level.                #
#     (The rpm passed in is the whole rpm file name.)                     #
#                                                                         #
#-------------------------------------------------------------------------#
sub installRequiredRpms
{
        my ($class, $rpm, @my_path) = @_;
        my @reqs = InstallUtils->getRequiredRpms($rpm, @my_path);
        if (scalar(@reqs))        # need to install at least 1 rpm
          {
                my $reqlist = join(' ', @reqs);
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V','IMsgINSTALLING_RPM_REQS', $rpm, $reqlist);
		my $cmd = "$::RPMCMD -U ";
                if($::FORCE_INSTALL){ $cmd .= "--force "; }
                if($::TEST){ $cmd .= "--test "; }
                $cmd .= "$reqlist";
                NodeUtils->runcmd("$cmd");
          }

}

#-------------------------------------------------------------------------#
#                                                                         #
#    getRequiredRpms                                                      #
#                                                                         #
#       Return the names of any rpms that this rpm is dependent on if     #
#       they are not already on the system at the required level.         #
#       The rpm passed in as the 1st arg is the whole rpm file name.      # 
#       The rest of the args are the list of reqs we have already found   #
#       in previous, recursive invokations of the function.               #
#                                                                         #
#-------------------------------------------------------------------------#
sub getRequiredRpms
{
        my ($class, $rpm, @my_path) = @_;
        my @reqrpms;
        my @reqs = InstallUtils->getReqsNotSatisfied($rpm);
        #print "num=" . scalar(@reqs) . ' ,reqs=' . join(':', @reqs) . ".\n";
        foreach my $r (@reqs)
          {
                my ($pkg, $operator, $version) = split(' ', $r);

                my $newrpm = InstallUtils->findRpm($pkg, $operator, $version, @my_path);
                if (defined($newrpm))        # found an appropriate one to install
                  {
                        # make sure it is not already on the list
                        if (grep(/^$newrpm$/, @_)) { next; }   # already on the list

                        push @reqrpms, $newrpm;

                        push @reqrpms, InstallUtils->getRequiredRpms($newrpm, @reqrpms, @my_path);
                  }
                else { NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E11', 'EMsgCANT_FIND_RPM', $pkg, $operator, $version, $rpm); }
          }
        return @reqrpms;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    getReqsNotSatisfied                                                  #
#                                                                         #
#       Return the rpms (and versions) that the given rpm requires        #
#       that are not already on the system                                #
#                                                                         #
#-------------------------------------------------------------------------#
sub getReqsNotSatisfied
{
        my ($class, $rpm) = @_;
        my @requires;
        my @output = `LANG=C $::RPMCMD -i --test $rpm 2>&1`;
        my $RC = $? >> 8;
        if ($RC == 0) { return @requires; }    # all dependencies were satisfied
        if ($RC > 1) { return @requires; }
        my $firstline = shift @output;
        my $secondline = $output[0];
        if ($firstline=~/is already installed/) { return @requires; }
        if ($secondline=~/conflicts with/) { return @requires; }
        if (!($firstline=~/failed dependencies/)) { return @requires; }

        foreach my $o (@output)
          {
                #print "o=$o\n";
                my ($pkg, $operator, $version) = InstallUtils->parseRequire($o);
                #print "pkg=$pkg, operator=$operator, version=$version\n";
                if (!defined($pkg)) { next; }   # was not an rpm require

                # Hack for glibc version because I can not find a way to have the require
                # statement in the rpm be different for different distribution levels.
                if ($pkg eq 'glibc' && NodeUtils->distribution()=~/^RedHat 7/i)
                  { $o = "$pkg $operator 2.2"; }

                my $req = "$pkg $operator $version";
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgREQUIRE', $rpm, $req);
                push(@requires, $req);
          }
        return @requires;
}

#-------------------------------------------------------------------------#
#                                                                         #
#    parseRequire                                                         #
#                                                                         #
#       Parse a single require line, distinguishing between rpms and      #
#       individual libraries of executables.                              #
#                                                                         #
#-------------------------------------------------------------------------#
sub parseRequire
{
        my ($class, $line) = @_;
        $line =~ s/ is needed by .*$//;
        my ($pkg, $operator, $version, $therest) = split(' ', $line);
        if ($pkg=~/^rpmlib/ || $pkg=~m|/| || $pkg=~/\.so/) { return undef; }
        else { return ($pkg, $operator, $version); }
}


#-------------------------------------------------------------------------#
#									  #
#      inc_ip								  #
#           Increment an IP address by 1, carrying to the previous octet  #
#           as required. This code originally appeared in the LUI 	  #
#           product (luiapi.pm)						  #
#                                                                         #
#-------------------------------------------------------------------------#
sub inc_ip
{
    my $ip=$_[1];
    my ($a,$b,$c,$d);
    ($a,$b,$c,$d)=split(/\./,$ip);
    $d++;
    if ($d > 255) {
        $d = $d - 256;
        $c++;
        if ($c > 255) {
            $c = $c - 256;
            $b++;
            if ($b > 255) {
                $b = $b - 256;
                $a++;
                if ($a > 255) {
                    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgIPADDR_ERROR', "$a.$b.$c.$d");
                    #printf "Error - ip address out of valid range.\n";
                    return 1;
                }
            }
        }
    }
    return $a."\.".$b."\.".$c."\.".$d;

}



#-------------------------------------------------------------------------#
#									  #
#      genHex								  #
#      generates a hex representation of an ip address:                   #
#      for example, 9.114.133.193 --> 097285C1                            #
#      Takes in a list of hosts                                           #
#      and returns a hex where the key is the hostname and                #
#                             the value is the IP in hex form.            #
#-------------------------------------------------------------------------#

sub genHex{
    shift;
    my $host;
    my $uno;
    my $dos;
    my $tres;
    my $cuatro;
    my $hex;
    my %hex;
    foreach $host (@_){
        my $ip =  NodeUtils->getHost($host);            # Get ip address
        ($uno, $dos, $tres, $cuatro) = split(/\./, $ip);# Split by the .
        $hex = sprintf("%02X", $uno);                   # put formatted into hex
        $hex .= sprintf("%02X", $dos);
        $hex .= sprintf("%02X", $tres);
        $hex .= sprintf("%02X", $cuatro); 
        $hex{$host} = $hex;
    }
    return %hex;
}

#-------------------------------------------------------------------------#
#                                                                         #
#      configureTftp                                                      #
#                                                                         #
#      Modify /etc/inetd.conf for tftp                                    #
#                                                                         #
#                                                                         #
#-------------------------------------------------------------------------#
sub configureTftp_inetd{

    my $inetdconf         = "/etc/inetd.conf";
    my $inetdbackup   = "/tmp/tftp_inetd.bak";
    my $errors         = 0;
    my ($cmd, $output, $rc,$s);

    unless(-f "$inetdconf"){
         return($errors++);
    }
    # backup inetd.conf
    $cmd = "$::COPY -f $inetdconf $inetdbackup ";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    if ($rc){
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgShow_Output',$output);
        $errors++;
    }
    # now edit the inetd.conf file:
    $s=NodeUtils->readFile($inetdconf);
    #comment tftp entry
    $s=~s/^\s*(tftp\s+[\s\S]*$)//m;
    $s = $s . "\ntftp    dgram   udp     wait    root    /usr/sbin/in.tftpd in.tftpd -r blksize -vvv -p -u tftpd -s /tftpboot\n";
   if(open(FILE,">$inetdconf")){
        print FILE ($s);
        close(FILE);
    }else{ exit; }
    # Restart inetd to pick up the changes
    $errors = restartTftp($errors,"inetd"); # restart tftp server.
    return $errors;
}

#-------------------------------------------------------------------------#
#									  #
#      configureTftp  						          #
#									  #
#    configure the default tftp server 					  #
#									  #
#-------------------------------------------------------------------------#
sub configureTftp{

    # Configure inetd in SuSE
    my $distro=NodeUtils->distribution();
    if( $distro =~ /SuSE/ || $distro =~ /SLES/ ){
        my $inetderr=configureTftp_inetd();
        return $inetderr;
    }

    my $xinetd         = "/etc/xinetd.d";
    my $tftpConf     = $xinetd . "/tftp";
    my $xinetdbackup   = "/tmp/tftp.bak";
    my $errors         = 0;
    my ($cmd, $output, $rc);

    unless (-d $xinetd ){
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
		'csminstall',"E", 'EMsgXINETD_MISSING', "$xinetd");
        $errors++;
    }

    # save old file to tmp
    if(-f "$tftpConf"){
	$cmd = "$::COPY -f $tftpConf $xinetdbackup";
	$output = NodeUtils->runcmd($cmd,0);
	$rc = $::RUNCMD_RC;
   	if($rc){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
			'csminstall','I','IMsgShow_Output',$output);
	    $errors++;
	}
    }
    # Now, configure our file:
    open(FILE,">$tftpConf") or die NodeUtils->messageFromCat('csmInstall',
			$::MSGMAPPATH,'csminstall','E1','EMsgCANT_WRITE_FILE', $tftpConf);
    my @fC = <<EOF;
# description: The tftp server serves files using the trivial file transfer \
#       protocol.  The tftp protocol is often used to boot diskless \
#       workstations, download configuration files to network-aware printers, \
#       and to start the installation process for some operating systems.
service tftp
{
        socket_type             = dgram
        protocol                = udp
        wait                    = yes
        user                    = root
        server                  = /usr/sbin/in.tftpd
        server_args             = -vvv -r blksize -p -u tftpd -s /tftpboot
}
EOF
    foreach(@fC){ print FILE "$_";}
    close(FILE);
    $errors = restartTftp($errors,"xinetd"); # restart tftp server.
    return $errors;
}


sub restartTftp{
    my $errors = shift;
    my $service = shift;
    my $rc;
    # Restart xinetd to pick up the changes
    my $cmd=NodeUtils->service($service,"start");
    NodeUtils->runcmd($cmd,0);
    if($rc){$errors++}
    $cmd=NodeUtils->service($service,"reload");
    NodeUtils->runcmd($cmd,0);
    if($rc){$errors++}
    return $errors;
}

#-------------------------------------------------------------------------#
#									  #
#      verifyTftp  						          #
#									  #
#	   check that tftp server is running 				  #
#									  #
#									  #
#-------------------------------------------------------------------------#

sub verifyTftp{
    # if the value 1 is passed in then we should try to restart tftp if
    # it is not running.
    my $restart = shift;
    my $errors = 0;
    my $running = 0;
    my $cmd;

    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCheckingTftp');
    unless($::LSOF){ $::LSOF = '/usr/sbin/lsof' }
    $cmd = "$::LSOF -i UDP:tftp";
    NodeUtils->runcmd($cmd,-1);

    if($::RUNCMD_RC){
	# There is no tftp server running on this machine
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall'
			,'I', 'IMsgTftpNotRunning','tftp');
    }else{
	# A tftp server is up and running.
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall'
			,'V', 'IMsgRunning','tftp');
	$running = 1;
    }
	
    # try to restart it
    if($restart && $::RUNCMD_RC){ 
     	# trying to restart tftp server
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall'
			,'I', 'IMsgRestart', 'tftp');
        my $distro=NodeUtils->distribution();
	
    	if( $distro =~ /SuSE/ || $distro =~ /SLES/ ){
            $errors = restartTftp($errors,"inetd"); # restart tftp server.
    	}else{
            $errors = restartTftp($errors,"xinetd"); # restart tftp server.
	}
        $cmd = "$::LSOF -i UDP:tftp";
     	NodeUtils->runcmd($cmd,-1);
     	if($::RUNCMD_RC){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
			'csminstall','I', 'IMsgStillCantRestart', 'tftp');
	    
	}else{
	    $running = 1;
	}
    }
    return $running;
}

#-------------------------------------------------------------------------#
#									  #
#      verify_dhcp  						          #
#									  #
#          check that dhcp is up and running.  Right now we don't check   #
#          the /etc/dhcpd.conf file, since that should've been checked    #
#          when made.  This function is just for extreme paranoya.        #
#									  #
#-------------------------------------------------------------------------#

sub verify_dhcp
{
        my $errors = 0;
        my @output;
        my $cmd;
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDhcpCheck');
        unless(-f "/var/lib/dhcp/dhcpd.leases"){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I', 'IMsgDhcpMissing');
            unless(-d "var/lib/dhcp"){
                NodeUtils->runcmd("$::MKDIR -p /var/lib/dhcp");
            }
            NodeUtils->runcmd("$::TOUCH /var/lib/dhcp/dhcpd.leases");
        }
        # SuSE Change
        #$cmd = "$::SERVICE dhcpd status";
        $cmd=NodeUtils->service("dhcpd","status");

        chomp(@output = `$cmd`);
        my @line = split(" ",@output);
        if( grep /running/, @output){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDhcpOutput', @output);
        }
        else{
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDhcpOutput', @output);
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDhcpProblemRestart', @output);
            # SuSE Change
            #`$::SERVICE dhcpd start`;
            #$cmd = "$::SERVICE dhcpd status";
            $cmd=NodeUtils->service("dhcpd","start");
            `$cmd`;
            $cmd=NodeUtils->service("dhcpd","status");

            chomp(@output = `$cmd`);
            if( grep /running/, @output){
                #print "dhcp: @output\n" if $::VERBOSE;
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgDhcpOutput', @output);
            }
            else{
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I', 'IMsgDhcpNotWorking');
                $errors++;
            }
        }
        return $errors;
}

#-------------------------------------------------------------------------#
#									  #
#      testvalidconsole					                  #
#									  #
#            checks for the valid Console Method attributes. This         #
#            subroutine  compares the given Console method against the    #
#            list in the /opt/csm/bin/*_console Console methods.          #
#									  #
#-------------------------------------------------------------------------#
sub testvalidconsole
{
  my ($class, $my_console) = @_;
  my @temp_console=();
  @::valid_console=();
  my $console="";
  my $tmp_console="";
  @temp_console=NodeUtils->runcmd("$::LS \/opt\/csm\/bin\/\*_console", -1);
  if ($::RUNCMD_RC != 0)
  {
    return 2; 
  }
  foreach $tmp_console (@temp_console)
  {
     (undef, undef, undef,undef, $console)=split("/",$tmp_console);
     ($console, undef)=split("_",$console);
     push(@::valid_console, $console);
  }
  if (!grep(/^$my_console$/, @::valid_console))
  {
    return 1;
  }
  else
  {
    return 0;
  }
}


#-------------------------------------------------------------------------#
#									  #
#      testvalidpowermethod                                               #
#									  #
#            checks for the valid Power Method attributes. This           #
#            subroutine  compares the given Power method against the one  #
#            in the /opt/csm/lib/lib+<given Power Method>+_power.so.      #
#            If this Method is not found then it checks against the       #
#            /opt/csm/bin/*_power Power Methods                           #
#									  #
#-------------------------------------------------------------------------#
sub testvalidpowermethod
{
  my ($class, $my_powermethod) = @_;
  my @temp_powermethod=();
  @::valid_powermethod=();
  my $powermethod="";
  my $tmp_powermethod="";
  my $exit_code=0;
  @temp_powermethod=();
  @temp_powermethod=NodeUtils->runcmd("$::LS \/opt\/csm\/lib\/\*_power\.so", -1);
  if ($::RUNCMD_RC != 0)
  {
    $exit_code=2;
  }
  else
  {
    foreach $tmp_powermethod (@temp_powermethod)
    {
      if ( -z $tmp_powermethod)
      {
        next;
      }
      $powermethod = basename ($tmp_powermethod);
      $powermethod =~ s/^lib(.*)_power.*$/$1/;
      push(@::valid_powermethod, $powermethod);
    }
    if (!grep(/^$my_powermethod$/, @::valid_powermethod))
    {
      $exit_code=1; 
    }
    else
    {
      $exit_code=0; 
    }
  }
  if ( ($exit_code ==1) || ($exit_code ==2) )
  {
    @temp_powermethod=();
    @temp_powermethod=NodeUtils->runcmd("$::LS \/opt\/csm\/bin\/\*_power", -1);
    if ($::RUNCMD_RC != 0)
    {
      #$exit_code=2;
    }
    else
    {
      foreach $tmp_powermethod (@temp_powermethod)
      {
         if ( -z $tmp_powermethod)
         {
            next;
         }
	$powermethod = basename($tmp_powermethod);
	$powermethod =~ s/(.*)_power.*$/$1/;
        push(@::valid_powermethod, $powermethod);
      }

      if (!grep(/^$my_powermethod$/, @::valid_powermethod))
      {
        $exit_code=1; 
      }
      else
      {
        $exit_code=0; 
      }
    } 
   }
   return $exit_code;
# end new code
}

#-------------------------------------------------------------------------#
#									  #
#      chkethup                                                           #
#          Ensure that the eth0 is up and running                         #
#									  #
#-------------------------------------------------------------------------#
sub chkethup
{
    my $cmd = "$::NETSTAT -rn | $::GREP eth0";
    unless(`$cmd`)
    {
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1','EMsgEth0NotUp');
    }
  return;
}

#-------------------------------------------------------------------------#
#									  #
#      cpSSHFiles                                                         #
#
#	returns: 1 if it failed					
#		or locations of authorized keys in /csminstall
#-------------------------------------------------------------------------#
sub cpSSHFiles {
	my ($cmd, $rc);
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgCopySSHKeys');
	my $SSHdir = "/csminstall/csm/config/.ssh";
	if( !(-d "/csminstall/csm/config")){
		$cmd = "$::MKDIR /csminstall/csm/config";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		`$cmd 2>&1`;
		$rc  = $? >> 8;
		if ($rc) {
			NodeUtils->message('E', 'EMsgNO_CreateDir', "/csminstall/csm/config");
		}
	}
	if( !(-d "$SSHdir")){
        	$cmd = "$::MKDIR $SSHdir";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		`$cmd 2>&1`;
		$rc  = $? >> 8;
		if ($rc) {
                        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_CreateDir',$SSHdir);
        	}
	}
	my $home = InstallUtils->getRootHomeDir();
	my $authorized_keys = "$SSHdir/authorized_keys";
        my $authorized_keys2 = "$SSHdir/authorized_keys2";
        if(!(-e "$home/.ssh/identity.pub") || !(-e "$home/.ssh/id_rsa.pub") || !(-e "$home/.ssh/id_dsa.pub") ){
		return 1;
	}
	$cmd = "$::COPY $home/.ssh/identity.pub $authorized_keys";
	NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	`$cmd 2>&1`;
	$rc  = $? >> 8;
	if ($rc) {
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        }
        
        $cmd = "$::COPY $home/.ssh/id_rsa.pub $authorized_keys2";
	NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	`$cmd 2>&1`;
	$rc  = $? >> 8;
	if ($rc) {
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        }
        
	$cmd = "$::CAT $home/.ssh/id_dsa.pub >> $authorized_keys2";
	NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	`$cmd 2>&1`;
	$rc  = $? >> 8;
	if ($rc) {
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        }
        if(!(-e "$authorized_keys") || !(-e "$authorized_keys2")){
		return 1;
	}
        return ($authorized_keys, $authorized_keys2);
}

#-------------------------------------------------------------------------#
#									  #
#      installSSHFiles                                                    #
#
#	input: 
#		$filename_of_authorized_keys
#		$filename_of_authorized_keys2
#-------------------------------------------------------------------------#
sub installSSHFiles{
	my ($class, $authorized_keys, $authorized_keys2) = @_;
	my ($cmd, $rc);
	my $home = InstallUtils->getRootHomeDir();
	my $dest_dir = "$home/.ssh";
	my $dest_authorized_keys = "$dest_dir/authorized_keys";
	my $dest_authorized_keys2 = "$dest_dir/authorized_keys2";
	
	umask(0077);
	if (! -d "$dest_dir" ) {
	    # create a local directory
	    $cmd = "$::MKDIR -p $dest_dir";
	    $rc = system("$cmd");
	    if ($rc >> 8){
		# Could not create $::CSMINSTDIR directory.
		NodeUtils->message('E', 'EMsgNO_CreateDir', $dest_dir);
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgNO_CreateDir',$dest_dir);
		return $::NOK;
	    }
	    else{
		    chmod 0700, $dest_dir;
	    }
	}

	umask(0133);
	if( -e "$dest_authorized_keys"){
		$cmd = "$::CAT $authorized_keys >> $dest_authorized_keys";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		`$cmd 2>&1`;
		$rc  = $? >> 8;
		if ($rc) {
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        	}
	}
	else{
		#just copy the keys
		my $cmd = "$::COPY $authorized_keys $dest_authorized_keys";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		`$cmd 2>&1`;
		$rc  = $? >> 8;
		if ($rc) {
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        	}
		chmod 0644, $dest_authorized_keys;
	}
	if( -e "$dest_authorized_keys2"){
		$cmd = "$::CAT $authorized_keys2 >> $dest_authorized_keys2";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		`$cmd 2>&1`;
		$rc  = $? >> 8;
		if ($rc) {
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
		}
	}
	else{
		#just copy the keys
		my $cmd = "$::COPY $authorized_keys2 $dest_authorized_keys2";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		`$cmd 2>&1`;
		$rc  = $? >> 8;
		if ($rc) {
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
        	}
		chmod 0644, $dest_authorized_keys2;	
	}
}

#-------------------------------------------------------------------------#
#									  #
#      checkHOMEforSSHperm                                                #
#	
#	returns: 1 if $HOME has the wrong permissions -- SSH should not be setup
#		 0 if the permissions are correct
#-------------------------------------------------------------------------#
sub checkHOMEforSSHperm{
	my $home = InstallUtils->getRootHomeDir();
	my $mode = (stat($home))[2];
	my $real_mode = sprintf "%04o", $mode & 07777;
	my @perm = split '', $real_mode;
	if( ($perm[2] == 2) || ($perm[2] == 6) || ($perm[2] == 7) || ($perm[3] == 2) || ($perm[3] == 6) || ($perm[3] == 7) ){
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgWrongHomePermissions', $home);
		return 1;
	}
	else{
		return 0;
	}
}

#-------------------------------------------------------------------------#
#									  #
#      getRemoteShell                                                     #
#
#	returns: 1 if it failed
#		 "path_of_remote_shell" otherwise
#-------------------------------------------------------------------------#
sub getRemoteShell {
	# set local scope because we only want classes on the mgmt svr
	$ENV{'CT_MANAGEMENT_SCOPE'} = 1;
	# todo: remove when lsrsrc-api converts to above
	$ENV{'CT_SESSION_SCOPE'} = 1;
	if(NodeUtils->isMgmtSvr()){ #only defined on managment server
		if(!$ENV{'DSH_REMOTE_CMD'}){
			my $outref = NodeUtils->runrmccmd('lsrsrc-api', "-i", "-s IBM.DmsCtrl::::RemoteShell");
			return $$outref[0];
		}
		else{
			return $ENV{'DSH_REMOTE_CMD'};
		}
	}
	return 1;
}

#-------------------------------------------------------------------------#
#									  #
#      getSetupRemoteShell                                                     #
#
#	returns: value of SetupRemoteShell in CSM database, or null
#-------------------------------------------------------------------------#
sub getSetupRemoteShell {
	# set local scope because we only want classes on the mgmt svr
	$ENV{'CT_MANAGEMENT_SCOPE'} = 1;
	# todo: remove when lsrsrc-api converts to above
	$ENV{'CT_SESSION_SCOPE'} = 1;
	if(NodeUtils->isMgmtSvr()){ #only defined on managment server
			my $outref = NodeUtils->runrmccmd('lsrsrc-api', "-i", "-s IBM.DmsCtrl::::SetupRemoteShell");
			return $$outref[0];
	}
}

#-------------------------------------------------------------------------#
#									  #
#      setupLinuxRSH	                                                  #
#
#	input:
#		$full_hostname_of_management_server			  #
#-------------------------------------------------------------------------#
sub setupLinuxRSH{
	my ($class, $ms) = @_;
	umask(0177);
	my $home = InstallUtils->getRootHomeDir();
	my ($cmd, $rc);

	# SuSE Changes
	# Setup rsh in inetd on SuSE
	my $distroname = &get_DistributionName;
	if ( ($distroname =~ /SuSE/) || ($distroname =~ /SLES/) ) {
		`/bin/rpm -q inetd 2>&1`;
             	if  ($? >> 8) {#setup rsh by xinetd
	             	$cmd = "/bin/sed -e \"s/disable.*=.*yes/disable = no/\" /etc/xinetd.d/rsh";
			NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
			my @rsh = `$cmd 2>&1`;
			$rc  = $? >> 8;
			if ($rc) {
				NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
			}
			open (RSH, ">/etc/xinetd.d/rsh") or NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', "/etc/xinetd.d/rsh");
			print RSH @rsh;
			close RSH;
			chmod 0600, "/etc/xinetd.d/rsh";
			$cmd = "/etc/init.d/xinetd restart";
			NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
			`$cmd 2>&1`;
			$rc  = $? >> 8;
			if ($rc) {
				NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
			}
			if(-e "/etc/securetty"){
				$cmd = "/bin/grep rsh /etc/securetty";
				`$cmd 2>&1`;
				$rc  = $? >> 8;
				if($rc){
					`/bin/echo rsh >> /etc/securetty`;
				}
			}
             	}else{
			$cmd = "/bin/sed -e \"s/#.*shell/shell/\" /etc/inetd.conf";
			NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
			my @rsh = `$cmd 2>&1`;
			$rc  = $? >> 8;
			if ($rc) {
				NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
			}
			open (RSH, ">/etc/inetd.conf") or NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', "/etc/inetd.conf");
			print RSH @rsh;
			close RSH;
			chmod 0600, "/etc/inetd.conf";
			$cmd = "/etc/rc.d/inetd restart";
			NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
			`$cmd 2>&1`;
			$rc  = $? >> 8;
			if ($rc) {
				NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
			}
		}
	}
	else {
		$cmd = "/bin/sed -e \"s/disable.*=.*yes/disable = no/\" /etc/xinetd.d/rsh";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		my @rsh = `$cmd 2>&1`;
		$rc  = $? >> 8;
		if ($rc) {
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
		}
		open (RSH, ">/etc/xinetd.d/rsh") or NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', "/etc/xinetd.d/rsh");
		print RSH @rsh;
		close RSH;
		chmod 0600, "/etc/xinetd.d/rsh";
		$cmd = "/sbin/service xinetd restart";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
		`$cmd 2>&1`;
		$rc  = $? >> 8;
		if ($rc) {
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
		}
		if(-e "/etc/securetty"){
			$cmd = "/bin/grep rsh /etc/securetty";
			`$cmd 2>&1`;
			$rc  = $? >> 8;
			if($rc){
				`/bin/echo rsh >> /etc/securetty`;
			}
		}
	}
	# End of SuSE change

	$cmd = "/bin/echo $ms root >> $home/.rhosts";
	NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	`$cmd 2>&1`;
	$rc  = $? >> 8;
	if ($rc) {
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
	}
	chmod 0600, "$home/.rhosts";
}

#-------------------------------------------------------------------------#
#									  #
#      setupAIXRSH	                                                  #
#
#	input:
#		$full_hostname_of_management_server			  #
#									  #
#-------------------------------------------------------------------------#
sub setupAIXRSH{
	my ($class, $ms) = @_;
	umask(0177);
	my $home = InstallUtils->getRootHomeDir();
	my ($cmd, $rc);
	$cmd = "/bin/echo $ms root >> $home/.rhosts";
	NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	`$cmd 2>&1`;
	$rc  = $? >> 8;
	if ($rc) {
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cmd, $rc);
	}
	chmod 0600, "$home/.rhosts";
}

#-------------------------------------------------------------------------#
#									  #
#      supportedSSHversion                                                #
#                							  #
#	 requires $::REMOTE_SHELL to be set				  #
#									  #
#	 returns 1 if the version of SSH is supported by install	  #
#		 0 if not supported -- install should not setup ssh	  #
#									  #
#-------------------------------------------------------------------------#
sub supportedSSHversion{
	#check if its a supported version of OpenSSH
	my $ssh_check = "$::REMOTE_SHELL -V";
	my @ssh_res = `$ssh_check 2>&1`;
	my $ssh_rc = $? >>8;
	if(!$ssh_rc){ #successful
		if($ssh_res[0] =~ m/OpenSSH.(\d)\.\d/){ #we can set this up
			if($1 >= 2){
				return 1;
			}
		}
		else{
			return 0;
		}
	}
}

#-------------------------------------------------------------------------#
#									  #
#      setupRemoteShellforFullInstall                                     #
#           Determine which shell to set up (rsh or ssh) and do           #
#           everything that is required on the management server.  Also,  #
#           set the $::SETUP_REMOTE_SHELL global varible to 1 if the      #
#           remote shell should be set up on the nodes.  This used by     #
#           make_config_file to set the SetupRemoteShell attribute.       #
#									  #
#           This subroutine is called by installnode and csmsetupnim.	  #
#									  #
#        sets these global variables:                                     #
#           $::REMOTE_SHELL       : The remote shell to set up.           #
#           $::SETUP_REMOTE_SHELL : If 1, setup remote shell on nodes     #
#                                   If 1, do not set up remote shell      #
#									  #
#	 returns 1 if there was a problem                                 #
#		 0 no problems                                            #
#									  #
#-------------------------------------------------------------------------#
sub setupRemoteShellforFullInstall
{
    my $rc = 0;

    my ($class, $ref_nodes) = @_;
    my @node_hostnames = @$ref_nodes;

    # SETUP_REMOTE_SHELL controls whether or not to set up the remote shell 
    # on the nodes.  It is used when creating the config_file.
    #     0=do not set up remote shell
    #     1=set up whichever remote shell is listed in $::REMOTE_SHELL
    $::SETUP_REMOTE_SHELL = InstallUtils->getSetupRemoteShell();

    # If the SetupRemoteShell attribute is set to 0 (or null) in DMSCTRL,
    # don't do any remote shell setup.
    if (! $::SETUP_REMOTE_SHELL)
    {
	return $rc;
    }

    $::REMOTE_SHELL = InstallUtils->getRemoteShell();

    # If the REMOTE_SHELL is set to 1, there was an error getting it from
    # the dmsctrl.  Don't bother setting up the remote shell.
    if ($::REMOTE_SHELL == 1)
    {
	return $::NOK;
    }

    (my $cmd_basename = $::REMOTE_SHELL) =~ s:^.*/::;
    if ($cmd_basename eq "rsh")
    {
	$::SETUP_REMOTE_SHELL = 1;
    }
    else # Attempt to set up ssh
    {
	if (InstallUtils->supportedSSHversion())
	{
	    # Check permissions on home directory
	    $rc = InstallUtils->checkHOMEforSSHperm();
	    if ($rc)
	    {
		$::SETUP_REMOTE_SHELL = 0;
		return $rc;
	    }
  
	    # Setup ssh keys
	    NodeUtils->runcmd("/opt/csm/csmbin/remoteshell.expect -k", 0);

	    # Copy ssh keys to /csminstall
	    $rc = InstallUtils->cpSSHFiles();
	    if ($rc == 1)
	    {
		$::SETUP_REMOTE_SHELL = 0;
		return $rc;
	    }

	    # Clean up the Known Hosts file for each node so that when the 
	    # nodes are reinstalled, the hosts files will be regenerated.
	    #$rc = InstallUtils->rmNodesKnownHosts(\@node_hostnames);
	    #the rmNodesKnownHosts step was moved to GatherSSHHostKeys

	    # ssh was set up properly, so set it up on the nodes.
	    $::SETUP_REMOTE_SHELL = 1;

	    # Add RemoteShell to config_file
	}
    }

    return $rc;
}

#-------------------------------------------------------------------------#
#									  #
#      rmNodesKnownHosts                                                  #
#                							  #
#       input:								  #
#		\@node_hostnames					  #
#									  #
#	returns 	1 if failure					  #	
# 			0 if success					  #
#-------------------------------------------------------------------------#
sub rmNodesKnownHosts{
	my ($class, $ref_nodes) = @_;
	my @node_hostnames = @$ref_nodes;
	my $home = InstallUtils->getRootHomeDir();
	my @all_names;
	my $return =0;
	foreach my $n (@node_hostnames){
		my @parts = split /\./, $n;
		my $name = shift @parts;
		push @all_names, $name;
		foreach my $ext (@parts){
			$name = "$name.$ext";
			push @all_names, $name;
		}
	}
	#create the sed command
	my $sed = "/bin/sed -e ";
	$sed .= "\"";
	foreach my $n (@all_names){
		$sed .= "/^$n\[,| ]/d; "
	}
	chop $sed; #get rid of last space
	$sed .= "\"";
	my $sed2 = $sed;
	my $file = "$home/.ssh/$$";
	while(-e $file){
		$file = InstallUtils->CreateRandomName($file);
	}
	if(-e "$home/.ssh/known_hosts"){
		$sed .= " $home/.ssh/known_hosts";
		$sed .= " > $file";
		my $printsed = $sed;
		$printsed =~ s/"//g; #"
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $printsed);
		`$sed 2>&1`;
		my $rc  = $? >> 8;
		if ($rc) {
			$return =1;
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $printsed, $rc);
		}
		my $cp = "$::COPY $file $home/.ssh/known_hosts";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cp);
		`$cp 2>&1`;
		$rc  = $? >> 8;
		if ($rc) {
			$return =1;
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cp, $rc);
		}
		system("$::RM -f $file");
	}
	if(-e "$home/.ssh/known_hosts2"){
		$sed2 .= " $home/.ssh/known_hosts2";
		$sed2 .= " > $file";
		my $printsed2 = $sed2;
		$printsed2 =~ s/"//g; #"
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $printsed2);
		`$sed2 2>&1`;
		my $rc  = $? >> 8;
		if ($rc) {
			$return =1;
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $printsed2, $rc);
		}
		my $cp = "$::COPY $file $home/.ssh/known_hosts2";
		NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cp);
		`$cp 2>&1`;
		$rc  = $? >> 8;
		if ($rc) {
			$return =1;
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $cp, $rc);
		}
		system("$::RM -f $file");	
	}
	return $return;
}

#-------------------------------------------------------------------------#
#									  #
#      getRootHomeDir                                                     #
#                							  #
#	returns 	$path_of_root_home_dir				  #	
# 									  #
#-------------------------------------------------------------------------#
sub getRootHomeDir{
	my @root = split ':', (`/bin/grep ^root /etc/passwd 2>&1`);
	my $home = $root[5];
	return $home;
}
#-------------------------------------------------------------------------#
#                                                                         #
#     getRecentVersionRpm                                                 # 
#                                                                         #
#       returns         $recent rpm                                       #
#                                                                         #
#-------------------------------------------------------------------------#
sub getRecentVersionRpm
{
  my ($class, $path, $rpm_name) = @_;

  my %localversions_hash=();
  my %file_name_hash=();
  my %packver;
  my @output=NodeUtils->runcmd("$::LS  -1 $path/$rpm_name");   
  #my @output=`ls  -1 $path/$rpm_name`;
  chomp @output;
  # if there is only one rpm then return that rpm name instead comparing the version the same rpm
  if (scalar(@output) == 1)
  {
    #return ($packver{$versionout});
    return $output[0];
  }
  my $i=0;
  foreach my $rpm1 (@output)
  {
	  
	  ####jcran changes start 
	  #### --getting the version from the output of LS, instead of running rpm multiple times
	  
    # my $VERSION=NodeUtils->runcmd("$::RPMCMD -qp --qf '%{Version}' $rpm1");  
    #my $VERSION=`rpm -qp --qf '%{Version}' $rpm1`;
     my ($fn) = $rpm1 =~ m|^.*?([^/]+)$|;      # get rid of the directory, if any
    #print "fn=$fn\n";
    #my ($ipkg, $VERSION) = $fn =~ /^(.+)-(\d.*)-.*$/;
    my ($ipkg, $VERSION, $RELEASE) = $fn =~ /^(.+)-(\d.*)-(\d.*)\..+\..+$/;
    	
    	  ####jcran changes end
    	
    chomp $VERSION;chomp $RELEASE;
    $localversions_hash{$i}{'VERSION'}=$VERSION;
    $localversions_hash{$i}{'RELEASE'}=$RELEASE;
    $file_name_hash{$VERSION}{$RELEASE}=$rpm1;
    $i++;
  }
  my $versionout="";
  my $releaseout="";
  $i=0;
  foreach my $k (keys %localversions_hash)
  {
    if ($i==0) {$versionout=$localversions_hash{$k}{'VERSION'}; $releaseout=$localversions_hash{$k}{'RELEASE'};}
    if (InstallUtils->testVersion($localversions_hash{$k}{'VERSION'},">",$versionout,$localversions_hash{$k}{'RELEASE'},$releaseout))
    {
      $versionout=$localversions_hash{$k}{'VERSION'};
      $releaseout=$localversions_hash{$k}{'RELEASE'};
    }
    $i++;

  }

  return ($file_name_hash{$versionout}{$releaseout});
}

#-------------------------------------------------------------------------#
#									  #
#      subnet       						          #
#									  #
#          Calculate the subnet that a host is on.                        #
#      INPUT:                                                             #
#	   host    : hostname or ip address                               #
#	   netmask : subnet mask in dotted decimal format (a.b.c.d)       #
#      OUTPUT:                                                            #
#	   returns the subnet in dotted decimal format (a,b,c,d)          #
#									  #
#-------------------------------------------------------------------------#
sub subnet
{
    my ($class, $host, $netmask) = @_;

    #$host = `hostname` if (!$host);
    #$netmask = '255.255.255.0' if (!$netmask);

    chomp $host;
    chomp $netmask;

    #print "INPUT:  $host $netmask\n";

    my ($hostname, $ipaddr) = NodeUtils->getHost($host);

    my ($ia,$ib,$ic,$id) = split ('\.', $ipaddr);
    my ($na,$nb,$nc,$nd) = split ('\.', $netmask);

    # Convert to integers so the bitwise and (&) works correctly.
    int $ia; int $ib; int $ic; int $id;
    int $na; int $nb; int $nc; int $nd;

    my $sa = ($ia & $na);
    my $sb = ($ib & $nb);
    my $sc = ($ic & $nc);
    my $sd = ($id & $nd);
    my $subnet = "$sa.$sb.$sc.$sd";


    #print "IPADDR = $ia.$ib.$ic.$id\n";
    #print "NETMSK = $na.$nb.$nc.$nd\n";
    #print "SUBNET = $subnet\n";

    return $subnet;
}

#-------------------------------------------------------------------------#
#									  #
#      determineMS    						          #
#									  #
#	this returns the name of the localhost from a list of node names
#	input: @DestNode
#									  #
#-------------------------------------------------------------------------#
sub determineMS
{
	my ($class, $refnodes) = @_;
	my @nodes = @$refnodes;
	my ($localnode, $ifconfig, $label, $separator, $address, @ipaddrs, %localhostnames);
	
	if($::PLTFRM eq "AIX"){
		$ifconfig = "/usr/sbin/ifconfig -a";
		# format is: "inet xxx.xxx.xxx.xxx
		$label = "inet";
		$separator = " ";
	}
	else{ #Linux
		$ifconfig = "/sbin/ifconfig -a";
		# format is: "inet addr:xxx.xxx.xxx.xxx
		$label = "inet addr";
		$separator = ":";
	}
	
	NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $ifconfig);
	my @ips = `$ifconfig 2>&1`;
	my $rc  = $? >> 8;
	if ($rc) {
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_RUN', $ifconfig, $rc);
	}
	foreach(@ips){
		if ($_ =~ /$label$separator((\d{1,3}?\.){3}(\d){1,3})\s/o) {
			#print "found \"$1\" in \"$_\"\n";
			push @ipaddrs, $1;
		}
	}
	foreach my $addr (@ipaddrs){
		#see what it resolves to
		$localhostnames{NodeUtils->tryHost($addr)}=1;
	}
	FINDMS: foreach my $name (@nodes){
		my $rn = NodeUtils->tryHost($name);
		if($localhostnames{$rn}){
			#there should only be one ManagedNode that maps to the localhost
			$localnode = $name;
			last FINDMS;
		}
	}
	return $localnode;	
}

#-------------------------------------------------------------------------#
#									  #
#      returnSubdirectory    					          #
#									  #
#	this returns all the sub directories (and grandchildren directories, etc)
#       of a give directory
#	input: directory name
#									  #
#-------------------------------------------------------------------------#
sub returnSubdirectory{
	my ($class, $dir, $valid_dir) = @_;
	my $file;
	my @subdirs = ();
	my @returndirs = ();
	my @valid_subdirs = (); #these are ones whose children we can look at
	if($::PLTFRM eq "AIX"){
		if($dir =~ m/(installp.?\z)|(usr.?\z)|(RPMS.?\z)/){ #check to see if the parent is valid
			$valid_dir=1;
		}
	}
	opendir(DIRECTORY, $dir) or return();
	while ($file = readdir(DIRECTORY)){
		if ($file ne '.' and $file ne '..'){
			my $filename;
			if($dir =~ /\/\z/){ #if it ends with a slash
				$filename = "$dir$file";
			}
			else{ #no slash
				$filename = "$dir/$file";
			}
			if((-d "$filename") && ($::PLTFRM eq "AIX")){
				if(!$valid_dir){ #the parent wasn't an valid directory
					if(($file eq "installp") || ($file eq "usr") || ($file eq "RPMS")){
						push @valid_subdirs, $filename;
					}
				}
				else{ #a valid parent, so therefore its children are valid too
					push @valid_subdirs, $filename;		
				}
				
			}
		}
	}
	closedir(DIRECTORY);
	push @returndirs, @valid_subdirs;
	push @returndirs, @subdirs;
	foreach my $sub (@valid_subdirs){  #these are valid children
		my @tmpdirs = InstallUtils->returnSubdirectory($sub, 1);
		push @returndirs, @tmpdirs;
        }
        return @returndirs;
}

#-------------------------------------------------------------------------#
#									  #
#      getIntsallDir   						          #
#									  #
#	this returns the install directory corresponding to the 
#	Install attributes that it is passed
#
#	input: (InstallOSName, InstallDistributionName, 
#		InstallDistributionVersion, InstallPkgArchitecture);
#									  #
#-------------------------------------------------------------------------#

sub getInstallDir{
	my ($class, $os, $distributor, $version, $arch) = @_;
	if ($arch =~ /i.86/){ $arch="i386"; } #SSS
	return "/csminstall/$os/$distributor/$version/$arch/install";
}

#-------------------------------------------------------------------------#
#									  #
#      getUpdateDir    						          #
#									  #
#	this returns the updates directory corresponding to the 
#	Install attributes that it is passed
#
#	input: (InstallOSName, InstallDistributionName, 
#		InstallDistributionVersion, InstallPkgArchitecture);
#									  #
#-------------------------------------------------------------------------#

sub getUpdateDir{
	my ($class, $os, $distributor, $version, $arch) = @_;
	if ($arch =~ /i.86/){ $arch="i386"; } #SSS
	return "/csminstall/$os/$distributor/$version/$arch/updates";
}

#-------------------------------------------------------------------------#
#									  #
#      getRPMSDir    						          #
#									  #
#	this returns the RPMS directory (for SMS) corresponding to the 
#	Install attributes that it is passed
#
#	input: (InstallOSName, InstallDistributionName, 
#		InstallDistributionVersion, InstallPkgArchitecture);
#									  #
#-------------------------------------------------------------------------#

sub getRPMSDir{
	my ($class, $os, $distributor, $version, $arch) = @_;
	if ($arch =~ /i.86/){ $arch="i386"; } #SSS
	return "/csminstall/$os/$distributor/$version/$arch/RPMS";
}

#-------------------------------------------------------------------------#
#									  #
#      getRealRPMSDir    						          #
#									  #
#	this returns the RPMS directory corresponding to the 
#	Install attributes that it is passed
#
#	input: (InstallOSName, InstallDistributionName, 
#		InstallDistributionVersion, InstallPkgArchitecture);
#									  #
#-------------------------------------------------------------------------#

sub getRealRPMSDir{
	my ($class, $os, $distributor, $version, $arch) = @_;
	my $distro_dir = $distributor;
	if ($distributor =~ /RedHat/){ $distro_dir = "RedHat"; }
	if ($arch =~ /i.86/){ $arch="i386"; } #SSS
	return "/csminstall/$os/$distributor/$version/$arch/$distro_dir/RPMS";
}

#-------------------------------------------------------------------------#
#									  #
#      getBaseDir    						          #
#									  #
#	this returns the directory corresponding to the 
#	Install attributes that it is passed
#
#	input: (InstallOSName, InstallDistributionName, 
#		InstallDistributionVersion, InstallPkgArchitecture);
#									  #
#-------------------------------------------------------------------------#

sub getBaseDir{
	my ($class, $os, $distributor, $version, $arch) = @_;
	if ($arch =~ /i.86/){ $arch="i386"; } #SSS
	return "/csminstall/$os/$distributor/$version/$arch";
}


#-------------------------------------------------------------------------#
#									  #
#      checkForNotFound						          #
#									  #
#	checks to see if listnodeattrs couldn't find some nodes           #
#                                                                         #
#	input: original node list                                         # 
#									  #
#-------------------------------------------------------------------------#
sub checkForNotFound
  {
	my ($class, $nodelist) = @_;
	if ($::LISTNODEATTRS_NUMNOTFOUND)
	  {
		if (scalar(@$nodelist) == 1) { NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'lsnode', 'E12', 'EMsgNODE_NOT_FOUND', $$nodelist[0]); }
		elsif (scalar(@$nodelist) <= 10) { NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'lsnode', 'W12', 'EMsgSOME_NOT_FOUND_LIST', join(', ',@$nodelist)); }
		else { NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'lsnode', 'W12', 'EMsgSOME_NOT_FOUND'); }
	  }
  }

#-------------------------------------------------------------------------#
#									  #
#      isAutoupdateInstalled    				          #
#									  #
#	checks to see there is an Autoupdate RPM in the correct directory #
#       Only run this on the MS						  #
#                                                                         #
#	input: DestNodeHash                                               # 
#	output: returns 1 if installed, 0 otherwise                       #
#-------------------------------------------------------------------------#
sub isAutoupdateInstalled{
	my ($class, $destnodekey) = @_;
	my $exit =1; #assume it is installed
	#Autoupdate should be in /csminstall/Linux/<InstallDistributionName>/csm/<InstallCSMVersion>/packages/
	#EX: /csminstall/Linux/RedHat/csm/1.3.0/packages/autoupdate-4.0.1-1.noarch.rpm

	#determine possible directory structures
	my %directories = ();
	foreach my $node (keys %$destnodekey){
		if($$destnodekey{$node}{'InstallOSName'} eq "Linux"){
			my $distro = $$destnodekey{$node}{'InstallDistributionName'};
			my $version = $$destnodekey{$node}{'InstallCSMVersion'};
			if($version >= 1.3){
				my $dir = "/csminstall/Linux/$distro/csm/$version/packages/";
				$directories{$dir} =1;
			}
		}
	}
	
	#check if there is an autoupdate rpm in the directory
	foreach my $dir (keys %directories){
		`/bin/ls $dir/autoupdate*.rpm 2>/dev/null`;
		my $rc = $?>>8;
		if($rc !=0){ #autoupdate isn't there			
			NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgNoAutoupdate', $dir, "http://freshmeat.net/projects/autoupdate", $dir);
			$exit =0;
		}
	}
	return $exit;
}

#-------------------------------------------------------------------------#
#									  #
#      createRPMList             				          #
#									  #
#       creates a global hash of installed RPMS 			  #
#       must be called before checkRPMInstall                             #
#                                                                         #
#	input:                                                            # 
#	output:                                                           #
#-------------------------------------------------------------------------#
sub createRPMList{
	#first run rpm -qa to get a list of all rpms installed
	my @output = `$::RPMCMD -qa`;
	foreach my $line (@output){
		chomp $line;
		$::rpmShortnames{$line}=1;
		#now create the basename
		my @parts = split /-/, $line;
		my $basename = shift @parts;
		my $p;
		BNAME: while ($p = shift @parts){
			if($p !~ m/^\d/){ #if it doesn't start with a digit
				$basename .= "-$p";
			}
			else{
			unshift @parts, $p;
				last BNAME;
			}
		}
		$::rpmBasenames{$basename}=1;
	}
	
}

#-------------------------------------------------------------------------#
#									  #
#      checkRPMInstall             				          #
#									  #
#       checks to see if a given rpm is installed.                        #
#       The full pathname of the rpm must be given.                       #
#                                                                         #
#	checkRPMInstall must be called before this function is used       #
#                                                                         #
#	input:                                                            # 
#	output:                                                           #
#-------------------------------------------------------------------------#
sub checkRPMInstall{
	my ($class, $filename) = @_; #rpm name
	my $file;
	if($::PLTFRM eq "AIX"){
		my @output = `$::RPMCMD -q --qf %{NAME}-%{VERSION}-%{RELEASE} -p $filename`;
		chomp @output;
		$file = $output[0];
	}
	else{
		my @path = split '/', $filename;
		$file = pop @path;
	}
	#determine if the given rpm is installed on the system
	 #we need to check to see if this rpm is already installed
	 my ($shortname, $basename, $p);
	my @parts = split /-/, $file;
	$shortname = shift @parts;
	SNAME: while ($p = shift @parts){
		if($p !~ m/^\d/){ #if it doesn't start with a digit
			$shortname .= "-$p";
		}
		else{
			unshift @parts, $p;
			last SNAME;
		}
	}
	$basename = $shortname;
	$shortname .= "-" . $parts[0] . "-" . (split /\./, $parts[1])[0];
	if(!exists $::rpmShortnames{$shortname}){
		#so check the basename
		if(!exists $::rpmBasenames{$basename}){
			#no version of this rpm is installed
			#push @rpms, $wholename;
			return 0;
		}
		else{ # a different version of this rpm is installed
			#test to see if this rpm  is installed
			# Check if the RPM is already installed at the correct version
			my $cmd = "$::RPMCMD -U --test $filename";
			NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
			my @cmdout = `LANG=C $cmd 2>&1`;
			my $rc = $? >> 8;       
			my $cmdout = join('', @cmdout);        # Put output into a single string
			if ($cmdout !~ /package .* is already installed/){ 
				#not installed
				return 0;
			}
		}
	}
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgRPM_ALREADY_INSTALLED', $file);
	return 1; #it is installed
}

#-------------------------------------------------------------------------------#
#                                                                               #
#      nfsprobe                                                                 #
#          probe nfs status like "nfs probe" on redhat                          #
#                                                                               #
# return: a "nfs" service cmd like "start","restart","reload"                   #
#-------------------------------------------------------------------------------#
sub nfsprobe
{
        my $CHECKPROC = "/sbin/checkproc";
        my $PIDOF = "/sbin/pidof";
        my $NFSD="nfsd";
        my $MOUNTD="/usr/sbin/rpc.mountd";
        my $EXPORTS="/etc/exports";
        my $ETAB="/var/lib/nfs/etab";
        my ($nfsd_status,$mountd_status);
        my $cmd= NodeUtils->service("nfs","probe");
        #my $output=NodeUtils->runcmd($cmd,0);
        my $output=`$cmd 2>&1`;
        if( ! $? ){
                return $output;
        }

        NodeUtils->runcmd("$CHECKPROC -n $NFSD",-1);
        $nfsd_status=$::RUNCMD_RC;

        NodeUtils->runcmd("$CHECKPROC $MOUNTD",-1);
        $mountd_status=$::RUNCMD_RC;

        #either nfsd and mountd are not start
        if( $nfsd_status || $mountd_status ){
                return "start";
        }

        NodeUtils->runcmd("$PIDOF $NFSD",-1);
        $nfsd_status=$::RUNCMD_RC;

        NodeUtils->runcmd("$PIDOF $MOUNTD",-1);
        $mountd_status=$::RUNCMD_RC;

        if( $nfsd_status == 1 || $mountd_status == 1){
                return "restart";
        }

        my $exports_mtime= (stat($EXPORTS))[9];
        my $etab_mtime= (stat($ETAB))[9];

        if( $exports_mtime > $etab_mtime){
                return "reload";
        }

        return "status";
}

#-------------------------------------------------------------------------------#
#                                                                               #
#      getmacs_via_dsh                                                          #
#          gathers mac addrs via dsh                                            #
#                                                                               #
# input: a reference to an array of nodes                                       #
# returns: a list of unreachable nodes                                          #
#-------------------------------------------------------------------------------#
sub getmacs_via_dsh
{
    my ($class, $ref_nodelist) = @_;
    my @nodelist = @$ref_nodelist;
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgGETTING_MACS_VIA_DSH');
    my $dshcmd = '/sbin/ifconfig eth0 | /bin/grep HWaddr | /bin/awk "{print \$5}"';
    my $RemoteShell=InstallUtils->getRemoteShell();
    my @new_getmacslist;
    my @unreach_nodes = ();
    my @reach = ();
    
    my $node_file = InstallUtils->make_node_list_file(\@nodelist); #put all dest nodes in tmp file
    my $cmd = "$::REMOTESHELL_EXPECT -d $node_file";
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', "V", 'IMsgRUN_CMD', $cmd);
    my @output = `$cmd 2>&1`;
    InstallUtils->close_delete_file($::NODE_LIST_FILE, $node_file); #delete node file
    my %out;
    foreach my $line (@output){
    	chomp $line;
    	my @parts = split ":", $line;
    	if($parts[1] =~ m/test.success/){ #this node is reachable
    	    $out{$parts[0]}=1;
    	}
    }
    foreach my $n (@nodelist){
    	    if(!$out{$n}){
    		push @unreach_nodes, $n;
    	    }
    	    else{ #it is reachable
    	    	push @reach, $n;
    	    }
    }
    if(@unreach_nodes){
	my $node_str = join ', ', @unreach_nodes;
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'W', 'EMsgDSHCannotConnect', $RemoteShell, $node_str ); 
	push @new_getmacslist, @unreach_nodes;
    }
    if(@reach){
	#we can theoretically reach it with DSH
	$node_file = InstallUtils->make_node_list_file(\@reach); #put all dest nodes in tmp file
	$ENV{'DSH_LIST'} = $node_file; 
	my $output = NodeUtils->runcmd("$::DSH '$dshcmd'", -1, 1);
	InstallUtils->close_delete_file($::NODE_LIST_FILE, $node_file); #delete node file
	my %macs;
	foreach my $line (@$output){
	    chomp $line;
	    my ($nodename, $macaddr) = split(': ', $line);
	    if($macaddr =~ /(..:..:..:..:..:..)/){
		$macs{$nodename}=$macaddr;
	    }
	}
	foreach my $node (@reach){
	    if($macs{$node}){
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgFOUND_MACADDR', $node, $macs{$node});
		NodeUtils->runcmd("$::CHNODE $node InstallAdapterMacaddr=$macs{$node}");
	    }
	    else{
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgCANT_DSH_NODE', $node);
		push (@new_getmacslist, $node);
	    }
	}
    }
    return (@new_getmacslist);
}

#-------------------------------------------------------------------------------#
#                                                                               #
#      copyDistroCD                                                             #
# Copy the entire contents of the Distro CD-ROM(s) to                           #
#    SUSE:                                                                      #
# 	/csminstall/Linux/<distro>/<ver>/<arch>/RPMS                            #
#    RedHat:                                                                    #
#       /csminstall/Linux/<distro>/<ver>/<arch>/                                #
#										#
# input: ($distro_top, $distro_name, $distro_version)                           #
# global vars: $::MNTCDROM                        #
# returns:                                                                      #
#                                                                               #
# Before calling this function, you should ensure the correct directory         #
# structure has been created by calling InstallUtils->createRPMSdir();          #
#-------------------------------------------------------------------------------# 

sub copyDistroCD
{
    my ($cmd, $ismounted, $output,$rc, $fstatus, $disk);
    my ($class, $distro_top, $distro_name, $distro_version) = @_;
    my $orig_dir; #used on AIX
    my $destination	= $distro_top . "/RPMS";
    my $distro		= $distro_name . " " . $distro_version;
    #SSS
    # Determine the normalized architecture.  This is the same as 
    # InstallPkgArchitecture unless it is something like i*86, in which 
    # case, it is normalized to i386.
    my $arch_norm = $::ARCH;#SSS
    if ($arch_norm =~ /i.86/){ $arch_norm="i386"; }#SSS

    %::pkgdefs = InstallUtils->get_pkgdefs($::PLTFRM,$distro_name,$distro_version,$::ARCH);
#SSSBEGIN
#    if ( ($distro_name =~ /SuSE/) || ($distro_name =~ /SLES/) ) {
#	$destination	= $distro_top . "/RPMS";
#	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgCOPYING_RPMS_SuSE', $::MNTCDROM);
#    }
#    else {
#SSSEND
	$destination	= $distro_top;
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgCOPYING_RPMS_OS', $distro, $::MNTCDROM);
#SSSBEGIN
#    }
#SSSEND
    # Create the directory
    if (!(-e $destination))	
    {
	$cmd = "$::MKDIR -p $destination;";
	$output = NodeUtils->runcmd($cmd,0);
    }
    $cmd = "";
    # Unmount the CD-ROM if it is already mounted
    $ismounted = "$::MOUNT | $::GREP $::MNTCDROM > /dev/null 2>&1";
    $cmd = "$ismounted && $::UNMOUNT $::MNTCDROM ; ";
    $output = NodeUtils->runcmd($cmd,0);
    $fstatus = $::RUNCMD_RC;
    #NodeUtils->message('I','IMsgShow_Output',$output) if ($fstatus);
    # SuSE Changes
    # SIS need to copy the first disk only
#    my @disklist; 
    my @disks = (@{$::pkgdefs{'distro_disks'}});

#    #--Begin add for ppc64
#    if ($::ARCH eq "ppc64") {
#    	@disklist = ('1');
#    }
#    #--End  
#    else {
#	if ( ($distro_name =~ /SLES/) && ($distro_version =~ /8\.1/) ) {
#	    @disklist = ('1', '4');
#    	}
#    	elsif ( ($distro_name =~ /SuSE/) || ($distro_name  =~ /SLES/) ) {
#	    @disklist = ('1');
#        }
#    	else {
#	    @disklist = split(/\s/, $::pkgdefs{DISTRO_DISKLIST});
#    	}
#    }
    if(($::PLTFRM eq "AIX") && !( ($distro_name =~ /SuSE/) || ($distro_name =~ /SLES/) )){
	#change to the destination dir. if on AIX copying RedHat CDs
	use Cwd;
	$orig_dir = getcwd();
	chdir($destination);
    }
    $fstatus=0;
    #foreach $disk ( @disklist )
    foreach my $diskref (@disks)
    {
	
	my $diskname = $diskref->{name};
	my $disknum  = $diskref->{num};

	my $not_for_full_install = '';
	if (defined $diskref->{not_for_full_install})
	{
	    $not_for_full_install = $diskref->{not_for_full_install};
	}

	# SIS only needs the first SuSE disk for each level of SuSE.
	# Skip the other SuSE disks.  All other distros need all the disks.
        #if (($distro_name =~ /SuSE/) and ($disknum ne "1")) { next; }
	if ($not_for_full_install) { next; }
	

	# Prompt to insert a CD-ROM
	if($fstatus == 0){
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH, 'csminstall', 'I', 'IMsgINSERT_DISK', $diskname, $disknum);
	    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'P','IMsgPRESS_ENTER');
	    my $junk = getc;
	    my $wrong_disk = 1;
	    while ($wrong_disk)
	    {
		# Mount the CD-ROM
		if ($::PLTFRM eq "AIX")
		{
		    sleep(5); #this is needed for AIX and wont hurt Linux (too much)
		    `$::UNMOUNT $::MNTCDROM 2>/dev/null 1>/dev/null`;
		    InstallUtils->MountAixCdrom();
		}
		else{
		    $cmd = "$::MOUNT $::MNTCDROM";
		    # Ignore mount errors.  Usually they are because
		    # the CD was already mounted.
		    $output = NodeUtils->runcmd($cmd,-1);
		    #$fstatus = $::RUNCMD_RC;
		    #NodeUtils->message('I','IMsgShow_Output',$output) if ($fstatus);
		}

		# Check for valid disk inserted
		my $diskid_filename = "$::MNTCDROM/" . $diskref->{diskid_filename};
		$cmd = "$::LS $diskid_filename 2> /dev/null";
		chomp($diskid_filename = NodeUtils->runcmd($cmd, -1));
		my $diskid_searchstring = $diskref->{diskid_searchstring};
		# First make sure the file exists
		if (! $::RUNCMD_RC and -f $diskid_filename)
		{
		    if ($diskid_searchstring)
		    {
		    open (DISKID_FILE, $diskid_filename) || NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_READ_FILE', $diskid_filename); 
		    if (grep (/$diskid_searchstring/, <DISKID_FILE>))
		    {
			#print "CORRECT DISK!\n";
			$wrong_disk = 0;
		    }
		    else {
			#print "Search string $diskid_searchstring not found\n";
			$wrong_disk = 1; 
		    }
		    close DISKID_FILE;
		    }
		    else { $wrong_disk = 0; }
		}
		else {
		    #print "Can't find $::MNTCDROM/$diskref->{diskid_filename}.  \nERRORS:  $diskid_filename\n"; 
		    $wrong_disk = 1; 
		}

		if ($wrong_disk) {
		    if ($::PLTFRM eq "AIX")
		    {
		    `$::UNMOUNT $::MNTCDROM 2>/dev/null 1>/dev/null`;
		    }
		    else
		    {
		    $cmd = "$::UNMOUNT $::MNTCDROM > /dev/null 2>&1";
		    NodeUtils->runcmd("$cmd",-1);
		    }
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'W', 'EMsgWRONG_DISK', $diskname, $disknum);
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgINSERT_DISK', $diskname, $disknum);
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgPRESS_ENTER_OR_FORCE');


		    # Get user input
		    $junk = getc;
		    if ($junk eq "F" or $junk eq "f") { 
		    #Force it to be the right disk
		    $cmd = "mount $::MNTCDROM";
		    NodeUtils->runcmd("$cmd",-1);
		    if ($::PLTFRM eq "AIX")
		    {
			`$::UNMOUNT $::MNTCDROM 2>/dev/null 1>/dev/null`;
			InstallUtils->MountAixCdrom();
		    }
		    else{
			$cmd = "$::MOUNT $::MNTCDROM";
			# Ignore mount errors.  Usually they are because 
			# the CD was already mounted.
			$output = NodeUtils->runcmd($cmd,-1);
			#$fstatus = $::RUNCMD_RC;
			#NodeUtils->message('I','IMsgShow_Output',$output) if ($fstatus);
		    }
		    $wrong_disk = 0;
		    }
		    else {
		    # Try again
		    next;
		    }
		}

		# Copy the contents of the Distro disk to /csminstall
		# Handle SLES 8.1 on pSeries
   		if ($distro_name    =~ /SLES/ and 
		    $distro_version =~ /8\.1/ and 
		    $::ARCH eq "ppc64") {
   		 	if (-d "$distro_top/CD1") {
	    			$cmd = "$::COPY -a $::MNTCDROM/* $distro_top/CD1 1>/dev/null 2>&1;";
	    		} else {
	    			$cmd = "$::COPY -a $::MNTCDROM $distro_top/CD1 1>/dev/null 2>&1;";
	    		}
		}
		# Handle SLES 8.1 on i386 or x86_64
    		elsif ( ($distro_name =~ /SLES/) && ($distro_version =~ /8\.1/) ) {
		    # Check if this is the SLES disk or the UnitedLinux disk
		    if ($disknum eq '1' and $diskname =~ /SLES/){
			$cmd = "$::MKDIR -p $distro_top/sles8-$arch_norm";#SSS
			NodeUtils->runcmd($cmd,-1);
			$cmd = "$::TAR cf - -C $::MNTCDROM . | $::TAR xvf - -C $distro_top/sles8-$arch_norm ; ";#SSS
		    }
		    else{
			# Copy the UnitedLinux disk
			$cmd = "$::MKDIR -p $distro_top/unitedlinux-$arch_norm";#SSS
			NodeUtils->runcmd($cmd,-1);
			$cmd = "$::TAR cf - -C $::MNTCDROM . | $::TAR xvf - -C $distro_top/unitedlinux-$arch_norm ; "; #SSS
		    }
		}

		# Handle all other distros (and AIX)
		else 
		{
		    if($::PLTFRM eq "AIX"){ 
			$cmd = "$::TAR cf - -C $::MNTCDROM . | $::TAR xvf - ";
		    }
		    else{
			$cmd = "$::TAR cf - -C $::MNTCDROM . | $::TAR xvf - -C $destination ; ";
		    }
		}
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgCOPYING_DISK', $diskname, $disknum);
		$output = NodeUtils->runcmd($cmd,0);
		$fstatus = $::RUNCMD_RC;
		#NodeUtils->message('I','IMsgShow_Output',$output) if ($fstatus);
		# Unmount the CD-ROM
		$cmd = "$::UNMOUNT $::MNTCDROM";
		$output = NodeUtils->runcmd($cmd,0);
	    }
	}
    }
#SSSBEGIN
    #
    # For SuSE or SLES, make symbolic link from .../$arch/suse/*/*.rpm
    # to .../$arch/<distro>/$RPMS
    # 
    my $realRPMS = "$distro_top/$distro_name/RPMS";
    if (($distro_name =~ /SuSE/) || ($distro_name =~ /SLES/)) {
	if (($distro_name =~ /SLES/) && ($distro_version eq 8.1)) {
		$cmd = "cd $realRPMS; $::LN -s ../../sles8-$arch_norm/suse/*/*.rpm .";#SSS
		NodeUtils->runcmd($cmd,-1);
		$cmd = "cd $realRPMS; $::LN -s ../../unitedlinux-$arch_norm/UnitedLinux/*/*.rpm .";#SSS
		NodeUtils->runcmd($cmd,-1);

		# handle special case of autoyast on SLES8
		$cmd = "$::COPY -a $distro_top/sles8-$arch_norm/yast $distro_top/sles8-$arch_norm/boot $distro_top/sles8-$arch_norm/content $distro_top/sles8-$arch_norm/media.1 $distro_top";#SSS
		NodeUtils->runcmd($cmd,-1);
		$cmd = "$::MKDIR -p $distro_top/yast/sles8/suse/setup/descr";
		NodeUtils->runcmd($cmd,-1);
		$cmd = "$::COPY -a $distro_top/sles8-$arch_norm/suse/setup/descr $distro_top/yast/sles8/suse/setup";#SSS
		NodeUtils->runcmd($cmd,-1);
		$cmd = "$::COPY -a $distro_top/sles8-$arch_norm/content $distro_top/sles8-$arch_norm/media.1 $distro_top/yast/sles8";#SSS
		NodeUtils->runcmd($cmd,-1);
		my $order = "$distro_top/yast/order";
		if ( open(FILE, ">$order")) {
			print FILE "/yast/sles8\t/sles8-$arch_norm\n";
			print FILE "/yast/ul\t/unitedlinux-$arch_norm\n";
			close(FILE);
		}
	} else {
		$cmd = "cd $realRPMS; $::LN -s ../../suse/*/*.rpm .";
		NodeUtils->runcmd($cmd,-1);
	}
    }
    elsif (($distro_name =~ /RedHatEL/) || ($distro_version =~ /3/)) {
	$realRPMS = "$distro_top/RedHat/RPMS";
	$cmd = "cd $realRPMS; $::LN -s ../Updates/*.rpm .";
	NodeUtils->runcmd($cmd,-1);
    }

    # For AutoYaST, we need to remove the info.txt file so that
    # the install doesn't try to prompt to accept a license.
    if (($distro_name =~ /SuSE/) || ($distro_name =~ /SLES/)) {
	if (-f "$destination/media.1/info.txt") {
	    # Remove media.1/info.txt
	    $cmd = "$::RM $destination/media.1/info.txt";
	    $output = NodeUtils->runcmd($cmd,0);
	}
    }
#SSSEND

    if(($::PLTFRM eq "AIX") && !( ($distro_name =~ /SuSE/) || ($distro_name =~ /SLES/) )){
	#change to the destination dir. if on AIX copying RedHat CDs
	chdir($orig_dir);
    }
    `$::UNMOUNT $::MNTCDROM 2>/dev/null 1>/dev/null`; #just to make sure the cd is unmounted
    if($fstatus != 0){ #there was an error, exit
    	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'E2', 'EMsgERROR_COPYING_LINUX_RPMS');
    }
    
}

#-------------------------------------------------------------------------------#
#                                                                               #
#      createRPMSdir                                                            #
#                                                                               #
# Creates the proper /csminstall/.../RPMS directory structure                   #
# input: ($OS, $distro_name, $distro_version, $arch)                            #
# returns:                                                                      #
#-------------------------------------------------------------------------------# 

sub createRPMSdir
{
	my ($class, $OS, $dist, $distver, $arch) = @_;
	use File::Path; # Provides mkpath() 

	my $base = InstallUtils->getBaseDir($OS, $dist, $distver, $arch);
	my $SMS_RPMS = InstallUtils->getRPMSDir($OS, $dist, $distver, $arch);
	my $SMS_updates = InstallUtils->getUpdateDir($OS, $dist, $distver, $arch);
	my $SMS_install = InstallUtils->getInstallDir($OS, $dist, $distver, $arch);
	my $realRPMS = InstallUtils->getRealRPMSDir($OS, $dist, $distver, $arch);
	
	if(!-e "$base"){
       		mkpath ($base, $::VERBOSE, 0755);
	}
	if(!-e "$realRPMS"){
			mkpath ($realRPMS, $::VERBOSE, 0755);
	}
	if(!-e "$SMS_updates"){
		mkpath ($SMS_updates, $::VERBOSE, 0755);
	}
	if(!-e "$SMS_install"){
		mkpath ($SMS_install, $::VERBOSE, 0755);
	}
        # Now we make new symbolic links to make sure they are
	# up to date
	if($dist =~ /RedHat/){
		#this is a symbolic link to ->RedHat/RPMS
		# force the change to make the symbolic link
		# relative, instead of full path.
		if(-d $SMS_RPMS){
		    #print "removing $SMS_RPMS\n";
		    NodeUtils->runcmd("rm $SMS_RPMS");
		}
		#print "Now linking\n";
		#print "dir: $realRPMS\n";
		my $basedir = "$realRPMS/../../";
		my @pa = split("/", $realRPMS);
		my $srcd = pop @pa;
		$srcd =  pop( @pa) . "/" . $srcd;
		$basedir = join("/", @pa);
		#print "basedir: $basedir\nsrc: $srcd\n";
		my(undef,$target) = split("$basedir/", $SMS_RPMS);
		#print "target: $target\n";
	    	my $ln_cmd = "cd $basedir; $::LN -s $srcd $target";
		#print "LInk: $ln_cmd\n";
	       	NodeUtils->runcmd($ln_cmd,0);
	}
	#SuSE Changes
	elsif( $dist =~ /SuSE/ || $dist =~ /SLES/ ){
	    if(-d $SMS_RPMS){
	    	#print "removing $SMS_RPMS\n";
	    	NodeUtils->runcmd("rm $SMS_RPMS");
	    }
	    #print "Now linking\n";
	    #print "dir: $realRPMS\n";
	    my $basedir = "$realRPMS/../../";
	    my @pa = split("/", $realRPMS);
	    my $srcd = pop @pa;
	    $srcd =  pop( @pa) . "/" . $srcd;
	    $basedir = join("/", @pa);
	    #print "basedir: $basedir\nsrc: $srcd\n";
	    my(undef,$target) = split("$basedir/", $SMS_RPMS);
	    #print "target: $target\n";
	    my $ln_cmd = "cd $basedir; $::LN -s $srcd $target";
	    #print "LInk: $ln_cmd\n";
	    NodeUtils->runcmd($ln_cmd,0);
	#End of SuSE Changes
 	}
	else{
	    mkpath ($SMS_RPMS, $::VERBOSE, 0755);
	}
}

#-------------------------------------------------------------------------------#
#                                                                               #
#      copyDistroPKGPATH                                                        #
# Copy the entire contents of the path to                                       #
#    SUSE:                                                                      #
# 	/csminstall/Linux/<distro>/<ver>/<arch>/RPMS                            #
#    RedHat:                                                                    #
#       /csminstall/Linux/<distro>/<ver>/<arch>/                                #
#										#
# input: ($distro_top, $distro_name, $distro_version)                           #
# global vars: @::PKGPATH                                                       #
# returns:                                                                      #
#                                                                               #
# Before calling this function, you should ensure the correct directory         #
# structure has been created by calling InstallUtils->createRPMSdir();          #
#-------------------------------------------------------------------------------# 

sub copyDistroPKGPATH
{
    my ($class, $distro_top, $distro_name, $distro_version) = @_;
    my $distro		= $distro_name . " " . $distro_version;
    my $destination;
    my $fstatus		= 0;
    my $copied_something= 0;
    my ($cmd, $output, $rc, $orig_dir);
    %::pkgdefs = InstallUtils->get_pkgdefs($::PLTFRM,$distro_name,$distro_version,$::ARCH);
    if(($::PLTFRM eq "AIX") && !( ($distro_name =~ /SuSE/) || ($distro_name =~ /SLES/) )){
	#change to the destination dir. if on AIX copying RedHat CDs
	use Cwd;
	$orig_dir = getcwd();
	chdir($distro_top);
    }
    #SSS
    # Determine the normalized architecture.  This is the same as 
    # InstallPkgArchitecture unless it is something like i*86, in which 
    # case, it is normalized to i386.
    my $arch_norm = $::ARCH;#SSS
    if ($arch_norm =~ /i.86/){ $arch_norm="i386"; }#SSS

    my @disks = (@{$::pkgdefs{'distro_disks'}});
    foreach my $path (@::PKGPATH)
    {
	# Make sure the directory looks like a Distro disk
	#--Begin add for ppc64
	if ($::ARCH eq "ppc64") {
		$destination = $distro_top . "/CD1";
		if (-d $path) {
			if (-d "$destination") {
				$cmd = "$::COPY -a $path/* $destination 1>/dev/null 2>&1 ; ";
	    		} else {
	    			$cmd = "$::COPY -a $path $destination 1>/dev/null 2>&1 ; ";
	    		}
			$output = NodeUtils->runcmd($cmd,0);
			$rc = $::RUNCMD_RC;
			$fstatus = $rc if ($fstatus lt $rc);
			$copied_something++;
		}
	}
	#--End
#SSSBEGIN
#        elsif (( $distro_name =~ /SLES/ ) || ($distro_name =~ /SuSE/) )
#        {
#	    $destination = $distro_top . "/RPMS";
#	    if ((-d $path) && ((-d "$path/$::pkgdefs{DISTRO_RPMDIR}") || (-d "$path/UnitedLinux")) )
#	    {
#		# Create the directory if it doesn't exist
#		if (!(-d $destination))	
#		{
#		    $cmd = "$::MKDIR -p $destination;";
#		    $output = NodeUtils->runcmd($cmd,0);
#		}
#        	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgCOPYING_RPMS_SuSE', $path);
#		# Copy the contents of the directory
#		if (-d "$path/UnitedLinux"){
#            	    $cmd = "$::COPY -f ".$path."/UnitedLinux"."/*/* ".$destination." 1>/dev/null 2>&1 ; ";
#       	}
#        	else{
#            	    $cmd = "$::COPY -f ".$path."/".$::pkgdefs{DISTRO_RPMDIR}."/*/* ".$destination." 1>/dev/null 2>&1 ; ";
#            	}
#		$output = NodeUtils->runcmd($cmd,0);
#		$rc = $::RUNCMD_RC;
#		#NodeUtils->message('I','IMsgShow_Output',$output) if ($rc);
#		$fstatus = $rc if ($fstatus lt $rc);
#		$copied_something++;
#	    }
#	    else{
#        	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'W', 'EMsgNOT_COPYING_SuSE_RPMS', $path);
#	    }
#	}
#SSSEND
	else{
	    if (-d $path)
            {
		my $wrong_disk = 1;
		my ($disknum, $diskname);
    		foreach my $diskref (@disks) {
			# Check for valid distro contents
			my $diskid_filename = "$path/" . $diskref->{diskid_filename};
			$cmd = "$::LS $diskid_filename 2> /dev/null";
			chomp($diskid_filename = NodeUtils->runcmd($cmd, -1));
			my $diskid_searchstring = $diskref->{diskid_searchstring};
			# First make sure the file exists
			if ((!$::RUNCMD_RC) && (-f $diskid_filename))
			{
			    if ($diskid_searchstring)
			    {
			    open (DISKID_FILE, $diskid_filename) || NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_READ_FILE', $diskid_filename); 
			    if (grep (/$diskid_searchstring/, <DISKID_FILE>))
			    {
				$wrong_disk = 0;
				$disknum = $diskref->{num};
				$diskname = $diskref->{name};
				last;
			    }
			    else {
				$wrong_disk = 1; 
			    }
			    close DISKID_FILE;
			    }
			    else {
				$wrong_disk = 0;
				$disknum = $diskref->{num};
				$diskname = $diskref->{name};
				last;
			    }

			}
		}
		if ($wrong_disk) {
                	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'W', 'EMsgNOT_COPYING_LINUX_RPMS', $path, $distro);
		} else {
			$destination = $distro_top; #RedHat's stuff just goes in arch directory
	                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgCOPYING_RPMS_OS', $distro, $path);
			# Copy the contents of the directory
			if($::PLTFRM eq "AIX"){ 
			    $cmd = "$::TAR cf - -C $path . | $::TAR xvf - ";
			}
    			elsif ( ($distro_name =~ /SLES/) && ($distro_version =~ /8\.1/) ) {
			    # Check if this is the SLES disk or the UnitedLinux disk
			    if ($disknum eq '1' and $diskname =~ /SLES/){
				$cmd = "$::MKDIR -p $distro_top/sles8-$arch_norm";#SSS
				NodeUtils->runcmd($cmd,-1);
				$cmd = "$::TAR cf - -C $path . | $::TAR xvf - -C $distro_top/sles8-$arch_norm ; ";#SSS
			    }
			    else{
				# Copy the UnitedLinux disk
				$cmd = "$::MKDIR -p $distro_top/unitedlinux-$arch_norm";#SSS
				NodeUtils->runcmd($cmd,-1);
				$cmd = "$::TAR cf - -C $path . | $::TAR xvf - -C $distro_top/unitedlinux-$arch_norm ; ";#SSS
			    }
			} else {
			    $cmd = "$::TAR cf - -C $path . | $::TAR xvf - -C $destination ; ";
			}
	                $output = NodeUtils->runcmd($cmd,0);
	                $rc = $::RUNCMD_RC;
        	        #NodeUtils->message('I','IMsgShow_Output',$output) if ($rc);
	                $fstatus = $rc if ($fstatus lt $rc);
			$copied_something++;
		}
            }
            else{
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'W', 'EMsgNOT_COPYING_LINUX_RPMS', $path, $distro);
            }
	}
    }
    if (! $copied_something)
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'E', 'EMsgNOTHING_COPIED', join(':', @::PKGPATH));
	$fstatus = 2;
    }

#SSSBEGIN
    #
    # Make symbolic link from .../$arch/suse/*/*.rpm to .../$arch/<distro>/RPMS
    #
    my $realRPMS = "$distro_top/$distro_name/RPMS";
    if (($distro_name =~ /SuSE/) || ($distro_name =~ /SLES/)) {
	if (($distro_name =~ /SLES/) && ($distro_version =~ /8\.1/ )) {
		$cmd = "cd $realRPMS; $::LN -s ../../sles8-$arch_norm/suse/*/*.rpm .";#SSS
		NodeUtils->runcmd($cmd,-1);
		$cmd = "cd $realRPMS; $::LN -s ../../unitedlinux-$arch_norm/UnitedLinux/*/*.rpm .";#SSS
		NodeUtils->runcmd($cmd,-1);

		# handle special case of autoyast on SLES8
		$cmd = "$::COPY -a $distro_top/sles8-$arch_norm/yast $distro_top/sles8-$arch_norm/boot $distro_top/sles8-$arch_norm/content $distro_top/sles8-$arch_norm/media.1 $distro_top";#SSS
		NodeUtils->runcmd($cmd,-1);
		$cmd = "$::MKDIR -p $distro_top/yast/sles8/suse/setup/descr";
		NodeUtils->runcmd($cmd,-1);
		$cmd = "$::COPY -a $distro_top/sles8-$arch_norm/suse/setup/descr $distro_top/yast/sles8/suse/setup";#SSS
		NodeUtils->runcmd($cmd,-1);
		$cmd = "$::COPY -a $distro_top/sles8-$arch_norm/content $distro_top/sles8-$arch_norm/media.1 $distro_top/yast/sles8";#SSS
		NodeUtils->runcmd($cmd,-1);
		my $order = "$distro_top/yast/order";
		if ( open(FILE, ">$order")) {
			print FILE "/yast/sles8\t/sles8-$arch_norm\n";
			print FILE "/yast/ul\t/unitedlinux-$arch_norm\n";
			close(FILE);
		}
	} else {
		$cmd = "cd $realRPMS; $::LN -s ../../suse/*/*.rpm .";
		NodeUtils->runcmd($cmd,-1);
	}
    }
    elsif (($distro_name =~ /RedHatEL/) || ($distro_version =~ /3/)) {
	$realRPMS = "$distro_top/RedHat/RPMS";
	$cmd = "cd $realRPMS; $::LN -s ../Updates/*.rpm .";
	NodeUtils->runcmd($cmd,-1);
    }
	    
    # For AutoYaST, we need to remove the info.txt file so that
    # the install doesn't try to prompt to accept a license.
    if (($distro_name =~ /SuSE/) || ($distro_name =~ /SLES/)) {
	if (-f "$destination/media.1/info.txt") {
	    # Remove media.1/info.txt
	    $cmd = "$::RM $destination/media.1/info.txt";
	    $output = NodeUtils->runcmd($cmd,0);
	}
    }
#SSSEND

    if(($::PLTFRM eq "AIX") && !( ($distro_name =~ /SuSE/) || ($distro_name =~ /SLES/) )){
	#change to the destination dir. if on AIX copying RedHat CDs
	chdir($orig_dir);
    }
    return $fstatus;
}

#-------------------------------------------------------------------------------#
#                                                                               #
#      MountAixCdrom                                                            #
#                                                                               #
# Mounts Aix Cdrom               						#
#										#
#   Make sure to unmount the directory or cdrom before using this function      #
#   and after mounting and after finishing reading the cdrom make sure to 	#
#   unmount it 									#
#										#
# input: 									#
# returns:                                                                      #
#-------------------------------------------------------------------------------#

sub MountAixCdrom
{
   
   use File::Path; # Provides mkpath() 
   #$cmd = "$::MOUNT -v cdrfs -o ro /dev/cd0 $::MNTCDROM";
   #MOUNT_CMD="/usr/sbin/mount -rvcdrfs $DEVICE $2" -- mount command used

   if (!(-d $::MNTCDROM))
   {
     mkpath ($::MNTCDROM, $::VERBOSE, 0755);
   }
  my $cmd = "$::MOUNT -rvcdrfs $::AIX_MNT_DEV $::MNTCDROM";
  NodeUtils->runcmd($cmd, 0);
  if ($::RUNCMD_RC)
  {
     NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgNO_MOUNT', $::MNTCDROM);
  }
}

#-------------------------------------------------------------------------------#
#                                                                               #
#      checkDistroCopied                                                        #
#										#
#	returns 1 if more than 100 files in /csminstall/RPMS                    #
# 	returns 0 otherwise							#
#-------------------------------------------------------------------------------# 
sub checkDistroCopied
{
	my ($class, $rpmdir) = @_;

	if(!-e "$rpmdir"){
		return 0;
	}
	my @output = `$::LS -l $rpmdir/ | /usr/bin/wc -l`;
	chomp @output;
	my $lines = $output[0];
	$lines =~ s/\s+//;
	if($lines > 100){
		return 1;
	}
	else{
		return 0;
	}
}


#-------------------------------------------------------------------------------#
#                                                                               #
#      getRPMVersionRelease                                                     #
#       Input: RPM name and rpduct flag                                         #
#                       product flag value 0=already installed rpm or default   #
#                                          1=product rpm                        #
#                                                                               #
#       returns RPM version  and release                                        #
#                                                                               #

#										#
#	returns RPM version 							#
#                                                                               #
#-------------------------------------------------------------------------------# 
sub getRPMVersionRelease
{
   my ($cmd,$version, $release, $my_str, $my_cmd, $class, $RPM_NAME, $product_flag, @my_array);
   @my_array=();
   $product_flag=0;
   ($class, $RPM_NAME,$product_flag) = @_;
   if ($product_flag==0)
   {
     $my_cmd="$::RPMCMD -q --qf '%{Version}:|:%{Release}' $RPM_NAME";
     $my_str=NodeUtils->runcmd($my_cmd,-1);
   }
   else
   {
     $my_cmd="$::RPMCMD -qp --qf '%{Version}:|:%{Release}' $RPM_NAME";
     $my_str=NodeUtils->runcmd($my_cmd,-1);
   }
   if ($::RUNCMD_RC!=0)
   {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgCANT_RUN', $my_cmd, $::RUNCMD_RC);
   }
   ($version, $release)=split(/:\|:/,$my_str);
   push(@my_array,$version);
   push(@my_array,$release);
   return (@my_array);
}

#-------------------------------------------------------------------------------#
#                                                                               #
#      syncServers                                                              #
#										#
#	syncs the Install Servers						#
#                                                                               # 
#	input: %DestNodeHash (where InstallServer is one of the attributes)     #
#                                                                               #
#      use CFM -- all commands that call this function need to use CFM          #
#-------------------------------------------------------------------------------# 
sub syncServers
{
    #use CFM -- all commands that call this function need to use CFM
    
    my ($class, $DestNodeHash) = @_;	
    my $sHash = InstallUtils->makeSyncHash($DestNodeHash);
    if(!keys %::INSTALL_SERVERS){ #there are no install servers
	return 0;
    }
    my $distro = InstallUtils->get_DistributionName();
#rsync is no longer used for SuSE and SLES
#    if(($distro eq "SuSE") || ($distro eq "SLES")){
	   
#	    CFM->rsyncIsvrs($sHash);
	    #PROBLEM: the links are being made wrong:  RPMS -> /csminstall/Linux/SuSE/8.0/i386/SuSE/RPMS or wait, 
#    }
#    else{ #use rdist
    	my $distlocation = "/tmp/csm_servers";
    	CFM->createGenDistfile($sHash, $distlocation); 
    	CFM->distGenDistfile($distlocation);
#    }
}

#-------------------------------------------------------------------------------#
#                                                                               #
#      makeSyncHash                                                             #
#										#
#	makes a hash of which directories to distribute to which Install Servers#
#	also fills in %::INSTALL_SERVERS which is the machine names of all the  #
#		install servers							#
#                                                                               # 
#	input: %DestNodeHash (where InstallServer is one of the attributes)     #
#                                                                               #
#-------------------------------------------------------------------------------# 
sub makeSyncHash
{
    my ($class, $DestNodeHash) = @_;
    my $defDir = "/csmserver";
    my %sHash;
    my $ms_used = 0;
    %::INSTALL_SERVERS = ();
    my @install_servers =();
    my $progname = NodeUtils->programName();
    #hash of directory{replacement name}{installserver}
    NODE: foreach my $node (keys %$DestNodeHash){
	my $InstallOSName = $$DestNodeHash{$node}{'InstallOSName'};
	my $InstallDistributionName = $$DestNodeHash{$node}{'InstallDistributionName'};
	if($$DestNodeHash{$node}{'InstallServer'}){
		if($$DestNodeHash{$node}{'ManagementServer'} eq $$DestNodeHash{$node}{'Hostname'}){
		    #this machine is a MS and a node
		    $ms_used =1;
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'W', 'EMsgMSNoInstallServer',  $node);
		    next NODE;
	    	}
	    my ($server, $rep_dir) = split ':', $$DestNodeHash{$node}{'InstallServer'};
	    if($server){
		if($server eq $$DestNodeHash{$node}{'ManagementServer'}){
		    $ms_used=1;
		    #this node's install server is the same as the managment server
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'I', 'IMsgMSisInstallServer',  $node);
		    next NODE;
		}
		if($progname =~ m/installnode/){
			#make sure a machine is not an install server of itself
			my ($node_hostname, $node_ip) = NodeUtils->getHost($node);
			my ($isvr_hostname, $isvr_ip) = NodeUtils->getHost($server);
			if($node_hostname eq $isvr_hostname){
				$ms_used=1;
				NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH, 'csminstall','W', 'EMsgISVR_OF_ITSELF', $node); 
				next NODE;
			}	    
	        }
		my $dir = $defDir;
		if($rep_dir){
		    #todo: check to make sure this start with a /
			$dir = $rep_dir;
		    }
		#transfer /csminstall/csm/* to all nodes
		push @{$sHash{"/csminstall/csm/"}{$dir}}, $server;
		#%{$::INSTALL_SERVERS{$server}} = (); #hash of a hash
		push @install_servers, $$DestNodeHash{$node}{'InstallServer'};
		#find node attributes

		    my $InstallDistributionVersion = $$DestNodeHash{$node}{'InstallDistributionVersion'};
		#my $InstallPkgArchitecture = $DestNodeHash{$node}{'InstallPkgArchitecture'};
		if($InstallOSName eq "Linux"){
		    #All Linux nodes: /csminstall/Linux/<InstallDistributionName>/csm/*
		    # /csminstall/Linux/<InstallDistributionName>/<InstallDistributionVersion>/*
		    push @{$sHash{"/csminstall/Linux/$InstallDistributionName/csm/"}{$dir}}, $server;
		    push @{$sHash{"/csminstall/Linux/$InstallDistributionName/$InstallDistributionVersion"}{$dir}}, $server;
		}
		elsif($InstallOSName eq "AIX"){
		    #All AIX Nodes: /csminstall/AIX/csm/*
		    push @{$sHash{"/csminstall/AIX/csm/"}{$dir}}, $server;
		}
		else{
		    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'E1', 'EMsgUnsupportedOS',  $node, $InstallOSName);
		}
		
	    }
	    else{
		    #management server is a server
		    $ms_used =1;
	    }
	}
	else{
	    $ms_used =1;
	}
    }
    if(!$ms_used){
	 $::MS_INSTALL_SERVER =0;   
    }

    #figure out if any of the install servers are also nodes
    foreach my $entry (@install_servers){
	my ($server, $dir) = split ':', $entry;
	if(!$dir){ $dir = $::SERVER_DIRECTORY; }
	if(exists $$DestNodeHash{$server}){ #it is a node 
	    $::INSTALL_SERVERS{'isnode'}{$server}{'dir'} = $dir; 
	    $::INSTALL_SERVERS{'isnode'}{$server}{'OS'} = $$DestNodeHash{$server}{'InstallOSName'};
	    $::INSTALL_SERVERS{'isnode'}{$server}{'DISTRO'} = $$DestNodeHash{$server}{'InstallDistributionName'};
	}
	else{ #not a node
	    my ($real_hostname, $ip) = NodeUtils->getHost($server);
	    #print "REAL HOSTNAME = $real_hostname\n";
	    $::INSTALL_SERVERS{'notnode'}{$real_hostname}{'dir'} = $dir;
	}
    }
    if(scalar @install_servers){
	    for (@install_servers) { s/:.*// }  #remove directory
	    my $unreachable = InstallUtils->checkRSReachability(\@install_servers);
	    if($unreachable != 0){
		if(scalar @$unreachable >= 1){
			    #there are unreachable InstallServers
			    my $isvrs = join ",", @$unreachable;
			    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH, 'csminstall','E1', 'EMsgISDown', $isvrs); 
		}
	    }
    }

    return(\%sHash);
}

#--------------------------------------------------------------------------
# get_osdefs_files
#
# Determine which definition file(s) need to be loaded and load them
# Look for files in this order (Linux example):
#   Linux-RedHat7.3-i386.pm
#   Linux-RedHat7.3.pm
#   Linux-RedHat.pm
#   Linux.pm
#
# Look for files in this order (AIX example)  
#   AIX-5.2.pm
#   AIX.pm
#   
# Search for the files in either <pwd>/defs or $OSDEFS
#--------------------------------------------------------------------------
sub get_osdefs_files
{
    my ($class,
	$osname,
	$distro_name,
	$distro_version,
	$hw_arch,
	$csm_version,
	) = @_;

    my $isAIX = ($osname eq "AIX");

    # Set these global variables to load the correct values from osdefs.
    $::csm_version = $csm_version;

    my $distro = "${distro_name}${distro_version}";
    my $osdistro = "${osname}-${distro}";

    my @defsfiles;
    push (@defsfiles, "${osdistro}-${hw_arch}.pm") if (${hw_arch});
    push (@defsfiles, "${osdistro}.0-${hw_arch}.pm") if (${hw_arch} and $isAIX);
    push (@defsfiles, "${osdistro}.pm");
    push (@defsfiles, "${osdistro}.0.pm") if ($isAIX);
    push (@defsfiles, "${osname}-${distro_name}.pm") if (${distro_name});
    push (@defsfiles, "${osname}.pm");

    my $progdir = dirname($0);

    foreach my $file (@defsfiles)
    {
	foreach my $dir ("$progdir/defs", $::OSDEFS)
	{
	    my $defsfile = "$dir/$file";
	    ($::DEBUG) && NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgChecking_For', $defsfile);
	    if (-f $defsfile)
	    {
		# Using "do" rather than "require" because require only loads
		# a file once.  We might need to load it again with a different 
		# value for csm_version.
		#require "$defsfile";
		do "$defsfile";
		return $defsfile;
	    }
	}
    }

    # If we get here, then there was nothing loaded.
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgNoDefsFiles', $::OSDEFS);
    #die "Error:  No OS Definition files to load in $::OSDEFS\n";
}

#--------------------------------------------------------------------------
# get_pkgdefs_files
#
# Determine which definition file(s) need to be loaded and load them
# Look for files in this order (Linux example):
#   Linux-RedHat7.3-i386.pm
#   Linux-RedHat7.3.pm
#   Linux-RedHat.pm
#   Linux.pm
#
# Look for files in this order (AIX example)  
#   AIX-5.2.pm
#   AIX.pm
#   
# Search for the files in either <pwd>/pkgdefs or $PKGDEFS
#--------------------------------------------------------------------------
sub get_pkgdefs_files
{
    my ($class,
	$osname,
	$distro_name,
	$distro_version,
	$hw_arch,
	$mgmt_type,
	$csm_version,
	) = @_;

    my $isAIX = ($osname eq "AIX");

    # Set these global variables to load the correct values from pkgdefs.
    $::mgmt_type = $mgmt_type;
    $::csm_version = $csm_version;

    my $distro = "${distro_name}${distro_version}";
    my $osdistro = "${osname}-${distro}";

    my @defsfiles;
    push (@defsfiles, "${osdistro}-${hw_arch}.pm") if (${hw_arch});
    push (@defsfiles, "${osdistro}.0-${hw_arch}.pm") if (${hw_arch} and $isAIX);
    push (@defsfiles, "${osdistro}.pm");
    push (@defsfiles, "${osdistro}.0.pm") if ($isAIX);
    push (@defsfiles, "${osname}-${distro_name}.pm") if (${distro_name});
    push (@defsfiles, "${osname}.pm");

    my $progdir = dirname($0);

    foreach my $file (@defsfiles)
    {
	foreach my $dir ("$progdir/pkgdefs", $::PKGDEFS)
	{
	    my $defsfile = "$dir/$file";
	    ($::DEBUG) && NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgChecking_For', $defsfile);
	    if (-f $defsfile)
	    {
		# Using "do" rather than "require" because require only loads
		# a file once.  We might need to load it again with different 
		# values for mgmt_type or csm_version.
		#require "$defsfile";
		do "$defsfile";
		return $defsfile;
	    }
	}
    }

    # If we get here, then there was nothing loaded.
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgNoDefsFiles', $::PKGDEFS);
}

#--------------------------------------------------------------------------
# get_OSDefs                                                          
#
# Read the set of OSDefs variables into a hash and assigns them each to
# global variables.  Calls get_osdefs_files to read the files in 
# /opt/csm/install/defs.  The correct file(s) are chosen based on the 
# parameters passed in to get_OSDefs.
#
# If no parameters are provided, default values based on the current
# machine's configuration are filled in.
# 
# Here is an example of how to use the hash that is returned by get_OSDefs:
# 
#   my %osdefs = InstallUtils->get_OSDefs();  # Get for current machine
#   print "CHMOD = $osdefs{'CHMOD'}\n";
#
# You can also get the same information via the global variable:
# 
#   my %osdefs = InstallUtils->get_OSDefs();  # Get for current machine
#   print "CHMOD = $::CHMOD\n";
#
#--------------------------------------------------------------------------
sub get_OSDefs
{
    my $class		= shift;
    my $args_provided = (@_);

    my ($osname,		# eg Linux or AIX (case sensitive)
	$distro_name,		# eg RedHat or SuSE (case sensitive)
	$distro_version,	# eg 7.3 or 5.2
	$hw_arch,		# eg i386, ppc
	$csm_version,		# eg 1.3.0
	$no_globals,		# Don't set global variables if set to 1
	) = @_;

    # Default to setting global variables
    $no_globals = 0 if (! $no_globals);

    # Set defaults if no arguments were provided
    # Note below that some of these require command pathnames to be defined.
    # Therefore, the pathnames must be defined in CSMDefs.pm.
    if (! $args_provided)
    {
	$osname = $::PLTFRM;
	$distro_name = InstallUtils->get_DistributionName;#requires no cmds
	$distro_version = InstallUtils->get_DistributionVersion;#requires LSLPP
	$hw_arch = InstallUtils->get_PkgArchitecture;#requires UNAME for Linux
	$csm_version = InstallUtils->get_CSMVersion("csm.core");#requires RPMCMD and LSLPP
    }
    my $munge = join ("|::|", ($osname, $distro_name, $distro_version, $hw_arch, $csm_version));
    if ($::all_OSDefs{$munge})
    {
	($::DEBUG) && print "Found $munge in OSDefs hash\n";
	return (%{$::all_OSDefs{$munge}});
    }

    InstallUtils->get_osdefs_files($osname, $distro_name, $distro_version, $hw_arch, $csm_version);

    # Assign each hash element to a global variable unless $no_globals is set
    unless ($no_globals)
    {
	foreach my $key  (keys %::OSDefs)
	{
	    my $value = $::OSDefs{$key};

	    # Display the key and the value
	    #print "$key : $value\n";

	    # Assign each hash element to a global variable.
	    # For example, the following has the effect of setting
	    #    $::GREP = $value
	    # In order for this to work, we need to temporarily turn off
	    # strict type checking, until the end of this block.
	    no strict 'refs';
	    ${"main::$key"} = $value;
	}
    }

    # Store the attributes that were used to obtain this OSdefs 
    $::OSDefs{OSDefs_osname} = $osname;
    $::OSDefs{OSDefs_distro_name} = $distro_name;
    $::OSDefs{OSDefs_distro_version} = $distro_version;
    $::OSDefs{OSDefs_hw_arch} = $hw_arch;
    $::OSDefs{OSDefs_csm_version} = $csm_version;


    # Make a local copy of OSDefs
    my %OSDefs = %::OSDefs;

    # Add it to all_pkgdefs
    $::all_OSDefs{$munge} = \%OSDefs;

    return %OSDefs;

}

#--------------------------------------------------------------------------
# get_pkgdefs                                                          
#
# Read the set of pkgdefs variables into a hash.  Calls get_pkgdefs_files
# to read the files in /opt/csm/install/pkgdefs.  The correct file(s) are
# chosen based on the parameters passed in to get_pkgdefs.
#
# If no parameters are provided, default values based on the current
# machine's configuration are filled in.
# 
# Here is an example of how to use the hash that is returned by get_pkgdefs:
# 
#   my %pkgdefs = InstallUtils->get_pkgdefs();  # Get for current machine
#   print "DISTRO_NAME = $pkgdefs{'DISTRO_NAME'}\n";	# string example
#   print "opensrc_prereqs = " .                         # array example
#              join(" ", @{$pkgdefs{'opensrc_prereqs'}}) ."\n";
#
#--------------------------------------------------------------------------
sub get_pkgdefs
{
    my $class		= shift;
    my $args_provided = (@_);

    my ($osname,		# eg Linux or AIX (case sensitive)
	$distro_name,		# eg RedHat or SuSE (case sensitive)
	$distro_version,	# eg 7.3 or 5.2
	$hw_arch,		# eg i386, ppc
	$mgmt_type,		# Either MgmtServer or MgdNode
	$csm_version,		# eg 1.3.0
	) = @_;

    # Set defaults if no arguments were provided
    # Note below that some of these require command pathnames to be defined.
    # Therefore, the pathnames must be defined in CSMDefs.pm.
    if (! $args_provided)
    {
	$osname = $::PLTFRM;
	$distro_name = InstallUtils->get_DistributionName;#requires no cmds
	$distro_version = InstallUtils->get_DistributionVersion;#requires LSLPP
	$hw_arch = InstallUtils->get_PkgArchitecture;#requires UNAME for Linux
	$mgmt_type   = "MgmtServer";
	$csm_version = InstallUtils->get_CSMVersion("csm.core");#requires RPMCMD and LSLPP
    }

    my $munge = join ("|::|", ($osname, $distro_name, $distro_version, $hw_arch, $mgmt_type, $csm_version));
    if ($::all_pkgdefs{$munge})
    {
	($::DEBUG) && print "Found $munge in pkgdefs hash\n";
	return (%{$::all_pkgdefs{$munge}});
    }

    # Set these global variables to load the correct values from pkgdefs.
    $::mgmt_type = $mgmt_type;
    $::csm_version = $csm_version;

    InstallUtils->get_pkgdefs_files($osname, $distro_name, $distro_version, $hw_arch, $mgmt_type, $csm_version);

    # Store the attributes that were used to obtain this pkgdefs 
    $::pkgdefs{pkgdefs_osname} = $osname;
    $::pkgdefs{pkgdefs_distro_name} = $distro_name;
    $::pkgdefs{pkgdefs_distro_version} = $distro_version;
    $::pkgdefs{pkgdefs_hw_arch} = $hw_arch;
    $::pkgdefs{pkgdefs_mgmt_type} = $mgmt_type;
    $::pkgdefs{pkgdefs_csm_version} = $csm_version;

    # Make a local copy of pkgdefs
    my %pkgdefs = %::pkgdefs;

    # Add it to all_pkgdefs
    $::all_pkgdefs{$munge} = \%pkgdefs;


    return %pkgdefs;
}

#-------------------------------------------------------------------------------#
#                                                                               #
#      groupNFSnodes                                                            #
#										#
#                                                                               # 
#	input: \%DestNodeHash (where InstallServer is one of the attributes)    #
#              Fanout value                                                     #
#              ref to an array of nodes, if you are only dsh to a select group  #
#                                                                               #
#-------------------------------------------------------------------------------# 
sub groupNFSnodes{
    my ($class, $DestNodeHash, $fanout, $nodes) = @_;
    #put nodes into hashes based on install server
    my %is;
    my @norm;
    my @slist;
    if($nodes){
        foreach my $node (@$nodes){
            my $server = $$DestNodeHash{$node}{'InstallServer'};
            if($server){ #there is an install server for this node
                #$is{$server}{$node}=1;
                if(!exists $is{$server}){ #haven't scene this one before
                    push @slist, $server;
                }
                push @{$is{$server}}, $node;
            }
            else{ #just uses the ms as an install server
                #$norm{$node}=1;
                push @norm, $node;
                #print "pushed node $node onto norm\n";
            }
        }
    }
    else{ #using the entire hash of nodes
        foreach my $node (keys %$DestNodeHash){
            my $server = $$DestNodeHash{$node}{'InstallServer'};
            if($server){ #there is an install server for this node
                #$is{$server}{$node}=1;
                if(!exists $is{$server}){ #haven't scene this one before
                    push @slist, $server;
                }
                push @{$is{$server}}, $node;
            }
            else{ #just uses the ms as an install server
                #$norm{$node}=1;
                push @norm, $node;
                #print "pushed node $node onto norm\n";
            }
        }
    }
    my %dsh;
    my $num_servers = (scalar @slist) + 1;
    my $num_nodes;
    if($nodes){ $num_nodes = scalar @$nodes; }
    else{ $num_nodes = scalar keys %$DestNodeHash; }

    #print "the number of install servers is $num_servers -1\nthe number of nodes is $num_nodes\nthe fanout is $fanout\n";
    my $count =0;
    my $group =0;
    my $fancount =0;
  GROUPNODES: while($num_nodes){
      #print "in GROUPNODES list: count=$count\tgroup=$group\tnum_nodes=$num_nodes\n";
      if($fancount == $fanout){ #we are done with one list
          $fancount =0;
          $group++;
      }
      if($count > $num_servers){
          $count =0;
      }
      if($count == $num_servers){
          $count =0;
      }
      if($count == 0){ #grab one from the normal list if any are left
          if(@norm){
              #print "pushing onto group $group\n";
              push @{$dsh{$group}}, shift @norm;
              $num_nodes--;
              $fancount++;
              $count++;
              next GROUPNODES
            }
      }
      while(!@{$is{$slist[$count-1]}}){ #none in this array
          #print "server $slist[$count] is not defined\n";
          #print "incrementing count $count\n";
          $count++;
          if($count == $num_servers || !($slist[$count-1])){
              next GROUPNODES;
          }
      }
      push @{$dsh{$group}}, shift @{$is{$slist[$count-1]}};
      $fancount++;
      $num_nodes--;
      $count++;
  }
    my %fan = ();
    foreach my $g (keys %dsh){
        @{$fan{$g}{'ALL'}} = @{$dsh{$g}};
        foreach my $node (@{$dsh{$g}}){
            my $server = $$DestNodeHash{$node}{'InstallServer'};
            my ($s_hostname, $dir) = split(/:/, $server);
            if(!$server){
                $server = $$DestNodeHash{$node}{'ManagementServer'};
            }
            else{ #server exists
                if($server !~ m/:/){ #doesn't have a directory
                    $server .= ":$::SERVER_DIRECTORY";
                }
                if($$DestNodeHash{$node}{'ManagementServer'} eq $$DestNodeHash{$node}{'Hostname'}){
                        #this node is also the MS
                        $server = $$DestNodeHash{$node}{'ManagementServer'};
                }
                elsif($s_hostname eq $$DestNodeHash{$node}{'ManagementServer'}){
                    $server = $$DestNodeHash{$node}{'ManagementServer'};
                }
            }
            push @{$fan{$g}{'Server'}{$server}}, $node;
        }
        my @s = keys %{$fan{$g}{'Server'}};
        #print "group $g has nodes @{$dsh{$g}}\tand servers @s \n";
    }
    return(\%fan);
}


#-------------------------------------------------------------------------------#
#                                                                               #
#      write_status                                                             #
# Write an entry in the node install status file (/csminstall/csm/status/<node>)#
#                                                                               #
# Arguments:									#
#      node: The hostname of the node						#
#       msg: The message to add to the status file				#
#    status: (optional) The status of the operation				#
#-------------------------------------------------------------------------------# 
sub write_status
{
    my ($class, $node, $msg, $status) = @_;
    my $statusfile = "$::CSMSTATUSDIR/$node";
    open (STATUSFILE, ">>$statusfile")
	|| NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall', 'E2', 'EMsgCANT_WRITE_FILE', $statusfile);

    my $date = `date`;
    chomp $date;

    chomp $msg;

    if ($status) {print STATUSFILE "$date: $msg: status=$status\n";}
    else         {print STATUSFILE "$date: $msg\n";}

    close(STATUSFILE);
}


#-------------------------------------------------------------------------------#
#                                                                               #
#      initialize_status                                                        #
# The status directory is used to collect the installation status of 		#
# nodes that are getting full installs.  Each node contains a separate file	#
# in the status directory with the name of the node's hostname.			#
#										#
# Create the /csminstall/csm/status directory and make it world writable 	#
# so that the node can write to it with the atftp command.  Also, create 	#
# initial files for each node to be installed.					#
#-------------------------------------------------------------------------------#
sub initialize_status
{
    my ($class, $nodes) = @_;
    my ($mode, $rc);
    my ($errors) = 0;
    unlink ($::CSMSTATUSDIR);
    mkpath ($::CSMSTATUSDIR, $::VERBOSE, 0110);
    $mode = "0110";
    NodeUtils->runcmd("$::CHMOD $mode $::CSMSTATUSDIR");
    if ($::RUNCMD_RC) {
	NodeUtils->message('W', 'EMsgCANT_CHMOD', $::CSMSTATUSDIR, $mode);
	$errors++;
    }
    #symlink ($::CSMSTATUSDIR, $::CSMSTATUSLINK);

    foreach my $node (@$nodes)
    {
	my $statusfile = "$::CSMSTATUSDIR/$node";
	unlink ($statusfile);
	NodeUtils->runcmd("$::TOUCH $statusfile");
	if ($::RUNCMD_RC) {
	    NodeUtils->message('W', 'EMsgCANT_CREATE', $statusfile);
	    $errors++;
	}
	$mode = "0440";
	NodeUtils->runcmd("$::CHMOD $mode $statusfile");
	if ($::RUNCMD_RC) {
	    NodeUtils->message('W', 'EMsgCANT_CHMOD', $statusfile, $mode);
	    $errors++;
	}
    }

    return ($errors);
}

#-------------------------------------------------------------------------------#
# createRPMVersionListAlreadyInstalled						#
#    Create a list(hash) of rpm's version, release and long name        	#
#    which are already been installed on the machine.				#
#    This routine can replace the routine createRPMList.			#
#    This routine also sets the variable which is been already used.		#
#                                                                               #
#-------------------------------------------------------------------------------#
sub createRPMVersionListAlreadyInstalled
{
   #first run rpm -qa to get a list of all rpms installed
   my @output = `$::RPMCMD -qa --qf '%{Name}::%{Version}::%{Release}::%{Arch}\n'`;
   chomp @output;
   my ($name,$version, $release, $arch);
   foreach my $line (@output)
   {
     chomp $line;
     ($name, $version, $release, $arch)=split(/::/,$line);
      my $longname=$name."-".$version."-".$release;
      $::rpmBasenames{$name}{'Version'}=$version;
      $::rpmBasenames{$name}{'Release'}=$release;
      $::rpmBasenames{$name}{'Longname'}=$longname;
      $::rpmBasenames{$name}{'Arch'}=$arch;
      $::rpmShortnames{$longname}=1;
   }
}

#-------------------------------------------------------------------------------#
#                                                                               #
#  get_definition_version							#
#	return the version and release of the definition rpm/fileset which      #
#	is defined in the defs file.						#
#   I/P: rpm name from the defs file 						#
#   O/P: version and release delimited by "::"					#
#                                                                               #
#-------------------------------------------------------------------------------#
sub get_definition_version
{
   my ($class, $my_rpm_def)=@_;
   my ($req_ver, $req_rel);
   my ($tmp_var1, $tmp_var2, $tmp_var3);

   #get the base name of the rpm
   my $my_rindex=rindex($my_rpm_def,"*");
   if ($my_rindex!=-1) {  $tmp_var1=substr($my_rpm_def,0,rindex($my_rpm_def,"*")); }
   else { $tmp_var1=$my_rpm_def; }
   #get the version and release by processing the filename
   $my_rindex=rindex($tmp_var1,"-");
   if ( ($my_rindex+1) == 0 ) { $req_ver=0; $req_rel=0;}
   else {
       $tmp_var2=substr($tmp_var1,rindex($tmp_var1,"-")+1);
       $tmp_var1=substr($tmp_var1,0,rindex($tmp_var1,"-"));
       $my_rindex=rindex($tmp_var1,"-");
       if ($my_rindex != -1)
       {
         $tmp_var3=substr($tmp_var1,rindex($tmp_var1,"-")+1);
         if (length($tmp_var2)>0)
         {
           if ($tmp_var2 =~ /^[\d\.]*$/)
           {
            if ($tmp_var3 =~ /^[\d\.]*$/) { $req_ver=$tmp_var3;$req_rel=$tmp_var2; }
            else { $req_ver=$tmp_var2;$req_rel=0; }
           }
           else { $req_ver=0;$req_rel=0; }
         }
         else { $req_ver=0;$req_rel=0; }
       }
       else
       {
         if (length($tmp_var2)>0)
         {
           if ($tmp_var2 =~ /^[\d\.]*$/) { $req_ver=$tmp_var2;$req_rel=0; }
           else { $req_ver=0;$req_rel=0; }
         }
         else { $req_ver=0;$req_rel=0; }
       }
   }
   my $return_str=$req_ver."::".$req_rel;
   return ($return_str);
}

#-------------------------------------------------------------------------------#
#                                                                               #
# get_csminstall_pathVersion							#
#	gets the version release of the rpms in the /csminstall/(Linux/AIX)     #
#	path. This subroutine will populate the PathVersionName hash		#
#                                                                               #
#                                                                               #
#-------------------------------------------------------------------------------#
sub get_csminstall_pathVersion
{
  $::CSMINSTALLPATH_FLAG=0;
  my ($my_path1, $my_path2);
  if ( ($::PREREQS_ATTR{'OSName'} eq "Linux") || ($::ATTRS{'InstallOSName'} eq "Linux") )
  {
    $my_path1=$::INSTALLDIR_CSMPKG;
    $my_path2=$::INSTALLDIR_OS_NAME_RPMS;
    my (@my_rpms, @my_result, @my_temp_array)=();
    my @my_path=($my_path1,$my_path2);
    foreach my $mydir(@my_path)
    {
      opendir RPMDIR, $mydir;
      # use two temp files to handle each rpm file
      # only grep *.rpm files ( not . and .. files )
      my $myfile1="/tmp/csm_$$.1";
      my $myfile2="/tmp/csm_$$.2";
      open (TMPFILE1, ">$myfile1") or NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_WRITE_FILE', "$myfile1");
      while ( $_ = readdir(RPMDIR) ) {
            my $file = "$mydir/$_";
            if($file !~ /\.rpm$/) {next;}
            print TMPFILE1 "$file\n";
      }
      close TMPFILE1;
      if ( -z $myfile1 ) { next; }
      my $cmd = "cat $myfile1 | xargs /bin/rpm -qp --qf '%{Name}::%{Version}::%{Release}::%{Arch}\n' > $myfile2";
      system($cmd);
      open (TMPFILE2, "<$myfile2") or NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E', 'EMsgCANT_READ_FILE', $myfile2);
      while ( <TMPFILE2> )
          {
            my $k = $_;
            if (!($k=~/.*::.*::.*::.*/)) { next;} 
            my ($name, $version, $release, $arch)=split(/::/,$k);
            # check if the arch exits
            if (exists $::PathVersionName{$arch})
            {
              # check of the rpm already exits, if yes, than get the latest
              if (exists $::PathVersionName{$arch}{$name})
              {
                my $ext_ver=$::PathVersionName{$arch}{$name}{'Version'};
                my $ext_rel=$::PathVersionName{$arch}{$name}{'Release'};
                if (InstallUtils->testVersion($version,">=",$ext_ver,$release,$ext_rel))
                {
                  $::PathVersionName{$arch}{$name}{'Version'}=$version;
                  $::PathVersionName{$arch}{$name}{'Release'}=$release;
                  $::PathVersionName{$arch}{$name}{'Location'}=$mydir;
                  $::PathVersionName{$arch}{$name}{'Longname'}=$name."-".$version."-".$release;
                }
              }
              else
              {
                $::PathVersionName{$arch}{$name}{'Version'}=$version;
                $::PathVersionName{$arch}{$name}{'Release'}=$release;
                $::PathVersionName{$arch}{$name}{'Location'}=$mydir;
                $::PathVersionName{$arch}{$name}{'Longname'}=$name."-".$version."-".$release;
              }
            }
            else
            {
                  $::PathVersionName{$arch}{$name}{'Version'}=$version;
                  $::PathVersionName{$arch}{$name}{'Release'}=$release;
                  $::PathVersionName{$arch}{$name}{'Location'}=$mydir;
                  $::PathVersionName{$arch}{$name}{'Longname'}=$name."-".$version."-".$release;
            }
          
          }
      close TMPFILE2;
      system("$::RM -f $myfile1");
      system("$::RM -f $myfile2");
      closedir RPMDIR;
    }
  }
  elsif ($::PREREQS_ATTR{'OSName'} eq "AIX")
  {
  }
  else { NodeUtils->message('E2', 'EMsgINVALID_OSTYPE'); }
}

#-------------------------------------------------------------------------------#
#                                                                               #
#  search_csm                                                                   #
#	This sub routine will search for the rpms in the installms path and     #
#	populate the Rpm_name_path and temp_CmdLinePathVersionName hash         #
#	if the rpm is not found than it will populate the 			#
#	RpmsNotFoundInCmdPath hash.						#
#                                                                               #
#-------------------------------------------------------------------------------#
sub search_csm
{
 if ( ($::PREREQS_ATTR{'OSName'} eq "Linux") || ($::ATTRS{'InstallOSName'} eq "Linux") )
 {
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgSearchingfor',"CSM, RSCT and  Director");
    my @my_res_path=();
    my ($rpmfile, $rpm, $path);

    #SSS_BEGIN
    # Determine the RPM architectures to search for on each platform.
    my @archs;
    my $arch = $::PREREQS_ATTR{'PkgArchitecture'};
    if (! $arch) {$arch = $::ATTRS{'InstallPkgArchitecture'}; }
  
    if ($arch =~ /i.86/)
    {
      @archs = ('i?86','noarch');
    }
    elsif ($arch =~ /ppc64/)
    {
      @archs = ($arch,'ppc','noarch');
    }
    elsif ($arch =~ /x86_64/)
    {
      @archs = ($arch,'i?86','noarch');
    }
    elsif ($arch =~ /ia64/)
    {
      @archs = ($arch,'i?86','noarch');
    }
    else 	# Any other unknown architectures.
    {
      @archs = ($arch,'noarch');
    }
    #SSS_END

    # Search RPMs from CSM distribution CD-ROM or directory to dirctory structure
    foreach $rpm (@::csm_packages, @::rsct_packages, @::director_packages)
    {
	  #SSS_BEGIN
	  $rpmfile = '';
          $path = '';
	  foreach my $a (@archs)
	  {
	      if (! ($path)) {
		  $rpmfile = "${rpm}-*.$a.rpm";
		  $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
	      }
	      else { last; }
	  }
	  if (! ($path)) {
              # we take out the dash because the images, may not have them.
	      foreach my $a (@archs)
	      {
		  if (! ($path)) {
		      $rpmfile = "${rpm}*.$a.rpm";
		      $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
		  }
		  if ($path) { last; }
	      }
	  }
	  #SSS_END

#          if ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/)  || ($::ATTRS{'InstallPkgArchitecture'} =~ /i?86/) )
#          {
#	    $rpmfile = "${rpm}-*.i?86.rpm";
#	    $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#          }
# 		elsif ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/)  || ($::ATTRS{'InstallPkgArchitecture'} =~ /ppc64/) )
#          {
#        	$rpmfile = "${rpm}-*.ppc64.rpm";
#        	$path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#			if( !$path ){
#				 $rpmfile = "${rpm}-*.ppc.rpm";
#            	 $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#			}
#          }
#          else
#          {
#	    $rpmfile = "${rpm}-*.ia64.rpm";
#	    $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#            if (!($path))
#            {
#              $rpmfile = "${rpm}-*.i?86.rpm";
#	      $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#            }
#          }
#          if (!($path))
#          {
#	     $rpmfile = "${rpm}-*.noarch.rpm";
#	     $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#          }
#          if (!($path))
#          {
#             if ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/)
#             {
#                 # PTF: (09/09/2002)
#                 # we take out the dash because the images, may not have them.
#
#	     $rpmfile = "${rpm}*.i?86.rpm";
#                 $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#             }
#			 if ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/) )
#          	{
#            	$rpmfile = "${rpm}-*.ppc64.rpm";
#            	$path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#            	if( !$path ){
#                 	$rpmfile = "${rpm}-*.ppc.rpm";
#                 	$path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#            	}
#          	}
#             else
#             {
#                $rpmfile = "${rpm}*.ia64.rpm";
#                $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#                if (!($path))
#                {
#                  $rpmfile = "${rpm}*.i?86.rpm";
#                  $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#                }
#             }
#             if (!($path))
#             {
#                $rpmfile = "${rpm}*.noarch.rpm";
#                $path = InstallUtils->findRpmInPath($rpmfile, @::PKGPATH);
#             }
#           }

	  if (defined($path)) 
	  { 
            $rpmfile = InstallUtils->getRecentVersionRpm($path, $rpmfile);
            chomp $rpmfile;
            my $rpmfile_basename;
            ($rpmfile_basename = $rpmfile) =~ s:^.*/::g;

             #find the shortname
             my @parts=split /-/, $rpmfile_basename;
             my $shortname = shift @parts;
             my $p;
             SNAME: while ($p = shift @parts){
                      if($p !~ m/^\d/){ #if it doesn't start with a digit
                          $shortname .= "-$p"; }
                      else{ unshift @parts, $p; last SNAME; }
                    }
          
             push @{$::temp_CmdLinePathVersionName{$path}},$rpmfile_basename;
             $::Rpm_name_path{$shortname}{'BaseName'}=$rpmfile_basename;
             $::Rpm_name_path{$shortname}{'Location'}=$path;
             $::Rpm_name_path{$shortname}{'Destination'}=$::INSTALLDIR_CSMPKG;
	  }
          else 
          {
            #get the shortname of the rpm
            my @parts=split /-/, $rpm; 
            my $shortname = shift @parts;
            my $p;
            SNAME: while ($p = shift @parts){
                      if($p !~ m/^\d/){ #if it doesn't start with a digit
                          $shortname .= "-$p"; }
                      else{ unshift @parts, $p; last SNAME; }
                    }
            my $tmp_var1;
            my $my_rindex=rindex($shortname,"*");
            if ($my_rindex!=-1) {  $tmp_var1=substr($shortname,0,rindex($shortname,"*")); }
            else { $tmp_var1=$shortname; }
            $my_rindex=rindex($tmp_var1,"-");
            if ( ($my_rindex+1) == 0 ) { $shortname=$tmp_var1; }
            elsif ( ($my_rindex+1) == length($tmp_var1) ) { $shortname=substr($tmp_var1,0,rindex($tmp_var1,"*")); }
            else { $shortname=$tmp_var1; }
            push(@::RpmsNotFoundInCmdPath, $shortname); 
          }
        
    }
    if ($::COPY_OPT_FLAG)
    {
      if (scalar(@::RpmsNotFoundInCmdPath)>0)
      {
         NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgNoRpmFound', join(", ",@::RpmsNotFoundInCmdPath));
      }
    }
 } 
 elsif ($::PREREQS_ATTR{'OSName'} eq "AIX")
 {
 }
 else
 {
     NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgINVALID_OSTYPE');
 }
}

#-------------------------------------------------------------------------------#
#                                                                               #
#  search_opensrc_prereqs                                                       #
#	Search for the open source rpms required for csm. If found than populate#
#	temp_CmdLinePathVersionName and Rpm_name_path hash else populate        #
#	RpmsNotFoundInCmdPath hash.						#
#                                                                               #
#-------------------------------------------------------------------------------#
sub search_opensrc_prereqs
{
  # Append /reqs and /reqs/<distro> to each PKGPATH element to search for pkgs
  my (@reqs_PKGPATH) = @::PKGPATH;  # Used for AIX
  map {s/$/\/reqs/} @reqs_PKGPATH;  # Used for AIX
  my $distroname = ($::PREREQS_ATTR{'DistributionName'} ?
		    $::PREREQS_ATTR{'DistributionName'} :
		    $::ATTRS{'InstallDistributionName'});
  my $distroversion = ($::PREREQS_ATTR{'DistributionVersion'} ?
			$::PREREQS_ATTR{'DistributionVersion'} :
			$::ATTRS{'InstallDistributionVersion'});
  my $distro = $distroname . $distroversion;

  # Assemble the full_PKGPATH as follows:
  #  path1/reqs/distro:path1/reqs:path1:path2/reqs/distro/path2/reqs:path2
  my (@full_PKGPATH);
  foreach my $pkgpath (@::PKGPATH)
  {
    push (@full_PKGPATH, "$pkgpath/reqs/$distro");
    push (@full_PKGPATH, "$pkgpath/reqs");
    push (@full_PKGPATH, "$pkgpath");
  }

  if ( ($::PREREQS_ATTR{'OSName'} eq "Linux") || ($::ATTRS{'InstallOSName'} eq "Linux") )
  {
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgSearchingfor',"Open Source");
    my ($rpm_pre_file, $rpm_pre, $path);

    #SSS_BEGIN
    # Determine the RPM architectures to search for on each platform.
    my @archs;
    my $arch = $::PREREQS_ATTR{'PkgArchitecture'};
    if (! $arch) {$arch = $::ATTRS{'InstallPkgArchitecture'}; }
  
    if ($arch =~ /i.86/)
    {
      @archs = ('i?86','noarch');
    }
    elsif ($arch =~ /ppc64/)
    {
      @archs = ($arch,'ppc','noarch');
    }
    elsif ($arch =~ /x86_64/)
    {
      @archs = ($arch,'i?86','noarch');
    }
    elsif ($arch =~ /ia64/)
    {
      @archs = ($arch,'i?86','noarch');
    }
    else 	# Any other unknown architectures.
    {
      @archs = ($arch,'noarch');
    }
    #SSS_END

    # Search open source prereqs from CSM distribution CD-ROM or directory structure
    foreach $rpm_pre (@::opensrc_rpm_prereqs)
    {
     $rpm_pre_file = ''; #SSS
     $path = ''; #SSS

     # Process the IBMJava rpm begin.....
     if ($rpm_pre =~/^IBMJava2-JRE/)
     {
      #SSS_BEGIN
      foreach my $a (@archs)
      {
	  if (! ($path)) {
	      $rpm_pre_file = "${rpm_pre}-*.$a.rpm";
	      $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
	  }
	  else { last; }
      }
      if (! ($path)) {
	  foreach my $a (@archs)
	  {
	      if (! ($path)) {
		  $rpm_pre_file = "${rpm_pre}*.$a.rpm";
		  $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
	      }
	      if ($path) { last; }
	  }
      }
      #SSS_END

#      # check for the i*86 rpm in path and path/reqs....
#      if ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/) || ($::ATTRS{'InstallPkgArchitecture'} =~ /i?86/) )
#      {
#         $rpm_pre_file = "${rpm_pre}-*.i?86.rpm";
#         $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#      }
#      elsif ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/)  || ($::ATTRS{'InstallPkgArchitecture'} =~ /ppc64/) )
#      {
#         $rpm_pre_file= "${rpm_pre}-*.ppc64.rpm";
#         $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#         if( !$path ){
#            $rpm_pre_file= "${rpm_pre}-*.ppc.rpm";
#            $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#         }
#      }
#      else
#      {
#         $rpm_pre_file = "${rpm_pre}-*.ia64.rpm";
#         $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#         if (!($path))
#         {
#            $rpm_pre_file = "${rpm_pre}-*.i?86.rpm";
#            $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#         }
#      }
#      #...If i*86 not found in path then check for the noarch rpm ....
#      if (!($path))
#      {
#          $rpm_pre_file = "${rpm_pre}-*.noarch.rpm";
#          $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#      }
#      if (!($path))
#      {
#         if ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/)
#         {
#            $rpm_pre_file = "${rpm_pre}*.i?86.rpm";
#            $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#         }
#	 elsif ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/) )
#          {
#            $rpm_pre_file= "${rpm_pre}-*.ppc64.rpm";
#            $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#            if( !$path ){
#                 $rpm_pre_file= "${rpm_pre}-*.ppc.rpm";
#                 $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#            }
#          }
#         else
#         {
#             $rpm_pre_file = "${rpm_pre}*.ia64.rpm";
#             $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#             if (!($path))
#             {
#                $rpm_pre_file = "${rpm_pre}*.i?86.rpm";
#                $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#             }
#          }
#          #...If i*86 not found in path then check for the noarch rpm ....
#          if (!($path))
#          {
#              $rpm_pre_file = "${rpm_pre}*.noarch.rpm";
#              $path = InstallUtils->findRpmInPath_java($rpm_pre_file, @full_PKGPATH);
#          }
#       }# Process the IBMJava rpm end.....
     }
     else # Not IBMJava
     {
      #SSS_BEGIN
      foreach my $a (@archs)
      {
	  if (! ($path)) {
	      $rpm_pre_file = "${rpm_pre}-*.$a.rpm";
              $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
	  }
	  else { last; }
      }
      if (! ($path)) {
	  foreach my $a (@archs)
	  {
	      if (! ($path)) {
		  $rpm_pre_file = "${rpm_pre}*.$a.rpm";
		  $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
	      }
	      if ($path) { last; }
	  }
      }
      #SSS_END

#      # check for the i*86 rpm in path and path/reqs....
#      if ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/) || ($::ATTRS{'InstallPkgArchitecture'} =~ /i?86/) )
#      {
#         $rpm_pre_file = "${rpm_pre}-*.i?86.rpm";
#         $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#      }
#      elsif ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/)  || ($::ATTRS{'InstallPkgArchitecture'} =~ /ppc64/) )
#      {
#         $rpm_pre_file= "${rpm_pre}-*.ppc64.rpm";
#         $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#         if( !$path ){
#            $rpm_pre_file= "${rpm_pre}-*.ppc.rpm";
#            $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#         }
#      }
#      else
#      {
#         $rpm_pre_file = "${rpm_pre}-*.ia64.rpm";
#         $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#         if (!($path))
#         {
#            $rpm_pre_file = "${rpm_pre}-*.i?86.rpm";
#            $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#         }
#      }
#      #...If i*86 not found in path then check for the noarch rpm ....
#      if (!($path))
#      {
#          $rpm_pre_file = "${rpm_pre}-*.noarch.rpm";
#          $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#      }
#      if (!($path))
#      {
#         if ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/)
#         {
#            $rpm_pre_file = "${rpm_pre}*.i?86.rpm";
#            $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#         }
#		 elsif ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/) )
#          {
#            $rpm_pre_file= "${rpm_pre}-*.ppc64.rpm";
#            $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#            if( !$path ){
#                 $rpm_pre_file= "${rpm_pre}-*.ppc.rpm";
#                 $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#            }
#          }
#         else
#         {
#             $rpm_pre_file = "${rpm_pre}*.ia64.rpm";
#             $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#             if (!($path))
#             {
#                $rpm_pre_file = "${rpm_pre}*.i?86.rpm";
#                $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#             }
#          }
#          #...If i*86 not found in path then check for the noarch rpm ....
#          if (!($path))
#          {
#              $rpm_pre_file = "${rpm_pre}*.noarch.rpm";
#              $path = InstallUtils->findRpmInPath($rpm_pre_file, @full_PKGPATH);
#          }
#       }
      }
      if (defined($path))
      {
          # Get the latest version of the file in the first location found
          #my $rpm_pre_file = NodeUtils->runcmd("$::LS -vr $path/$rpm_pre_file | $::HEAD -1", -1);
          $rpm_pre_file = InstallUtils->getRecentVersionRpm($path, $rpm_pre_file);
          chomp $rpm_pre_file;
          my $rpmfile_basename;
          ($rpmfile_basename = $rpm_pre_file) =~ s:^.*/::g;
          #find the shortname
          my @parts=split /-/, $rpmfile_basename;
          my $shortname = shift @parts;
          my $p;
          SNAME: while ($p = shift @parts){
                   if($p !~ m/^\d/){ #if it doesn't start with a digit
                       $shortname .= "-$p"; }
                   else{ unshift @parts, $p; last SNAME; }
                 }
          push @{$::temp_CmdLinePathVersionName{$path}},$rpmfile_basename;
          $::Rpm_name_path{$shortname}{'BaseName'}=$rpmfile_basename;
          $::Rpm_name_path{$shortname}{'Location'}=$path;
          $::Rpm_name_path{$shortname}{'Destination'}=$::INSTALLDIR_CSMPKG;
      }
      else 
      { 
            #get the shortname of the rpm
            my @parts=split /-/, $rpm_pre; 
            my $shortname = shift @parts;
            my $p;
            SNAME: while ($p = shift @parts){
                      if($p !~ m/^\d/){ #if it doesn't start with a digit
                          $shortname .= "-$p"; }
                      else{ unshift @parts, $p; last SNAME; }
                    }
            my $tmp_var1;
            my $my_rindex=rindex($shortname,"*");
            if ($my_rindex!=-1) {  $tmp_var1=substr($shortname,0,rindex($shortname,"*")); }
            else { $tmp_var1=$shortname; }
            $my_rindex=rindex($tmp_var1,"-");
            if ( ($my_rindex+1) == 0 ) { $shortname=$tmp_var1; }
            elsif ( ($my_rindex+1) == length($tmp_var1) ) { $shortname=substr($tmp_var1,0,rindex($tmp_var1,"*")); }
            else { $shortname=$tmp_var1; }
            push(@::RpmsNotFoundInCmdPath, $shortname); 
      }
    }
  }
  elsif ($::PREREQS_ATTR{'OSName'} eq "AIX")
  {
    # Search open source prereqs from CSM distribution CD-ROM or directory structure
    foreach my $rpm_pre (@::opensrc_rpm_prereqs)
    {
      # check for the ppc rpm in path and path/reqs....
      my $rpm_pre_file = "${rpm_pre}-*.ppc.rpm";
      my $path = InstallUtils->findRpmInPath($rpm_pre_file, @::PKGPATH);
      if (!($path))
      {
        $path = InstallUtils->findRpmInPath($rpm_pre_file, @reqs_PKGPATH);
      }
      #...If ppc not found in path then check for the noarch rpm ....
      if (!($path))
      {
          $rpm_pre_file = "${rpm_pre}-*.noarch.rpm";
          $path = InstallUtils->findRpmInPath($rpm_pre_file, @::PKGPATH);
      }
      if (!($path))
      {
        $path = InstallUtils->findRpmInPath($rpm_pre_file, @reqs_PKGPATH);
      }
      if (!($path))
      {
         $rpm_pre_file = "${rpm_pre}*.ppc.rpm";
         $path = InstallUtils->findRpmInPath($rpm_pre_file, @::PKGPATH);
         if (!($path))
         {
           $path = InstallUtils->findRpmInPath($rpm_pre_file, @reqs_PKGPATH);
         }
         #...If ppc not found in path then check for the noarch rpm ....
         if (!($path))
         {
            $rpm_pre_file = "${rpm_pre}*.noarch.rpm";
            $path = InstallUtils->findRpmInPath($rpm_pre_file, @::PKGPATH);
         }
         if (!($path))
         {
            $path = InstallUtils->findRpmInPath($rpm_pre_file, @reqs_PKGPATH);
         }
       }
      # And finally exit if the rpm is not found else copy
      if (defined($path))
      {
          # Get the latest version of the file in the first location found
          #my $rpmfile = NodeUtils->runcmd("$::LS $path/$rpm_pre_file | $::HEAD -1", -1);
          my  $rpmfile = InstallUtils->getRecentVersionRpm($path, $rpm_pre_file);
          chomp $rpmfile;
          my $rpmfile_basename;
          ($rpmfile_basename = $rpmfile) =~ s:^.*/::g;

          &copyRpm($rpmfile_basename, $path, $::INSTALLDIR_CSMRPM);
       }
       else
       {
            NodeUtils->message('E1', 'EMsgCANT_FIND', $rpm_pre, join(':',@::PKGPATH));
       }

    }
  }
  else
  {
     NodeUtils->message('E2', 'EMsgINVALID_OSTYPE');
  }
  if ($::COPY_OPT_FLAG)
  {
    if (scalar(@::RpmsNotFoundInCmdPath)>0)
    {
         my @rpm_ignored=();
         my @rpm_not_ignored=();
         foreach my $rpm_name (@::RpmsNotFoundInCmdPath)
         {
           if ( ($rpm_name =~ m/autoupdate/) || ($rpm_name =~ m/perl-libnet/) || ($rpm_name =~ m/perl-DBI/) ||
                ($rpm_name =~ m/perl-XML-Parser/) || ($rpm_name =~ m/perl-XML-Simple/) || ($rpm_name =~ m/fping/) )
           { push (@rpm_ignored,$rpm_name); }
           else { push (@rpm_not_ignored,$rpm_name); }
         }
         @::RpmsNotFoundInCmdPath=@rpm_not_ignored;
         if (scalar(@rpm_ignored)>0)
         {
           NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I', 'IMsgMissingRPM', join(", ",@rpm_ignored));
         }
         if (scalar(@::RpmsNotFoundInCmdPath)>0)
         {
           NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgNoRpmFound', join(", ",@::RpmsNotFoundInCmdPath));
         }
     }
  }
}


#-------------------------------------------------------------------------------#
#                                                                               #
#  search_copy_distro_rpms                                                      #
#	Check the operating system(os) required rpms in tha path, if rpms not   #
#	found than check the /csminstall.. directory, if still the rpm not found#
#       the prompt for the  os cds						#
#	If the rpms are found than the temp_CmdLinePathVersionName and 		#
#	Rpm_name_path hash are populate else RpmsNotFoundInCmdPath hash is 	#
#	populated.								#
#                                                                               #
#-------------------------------------------------------------------------------#
sub search_copy_distro_rpms
{
 if ( ($::PREREQS_ATTR{'OSName'} eq "Linux") || ($::ATTRS{'InstallOSName'} eq "Linux") )
 {
    #if( $::DISTRO =~ /SLES/ || $::DISTRO =~/SuSE/ )
    #{
    #  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgSearchAndCopy',"Distribution and SIS");
    #}
    #else
    #{
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I','IMsgSearchAndCopy',"Distribution");
    #}
    my $distro		= $::DISTRO;
    my $destination	= $::INSTALLDIR_OS_NAME_RPMS;
    my ($disk, $cmd, $rc);
    my (@notfound);
    my ($my_prereq, $prereq, $path, $dir);
	
    # First try to get the distro rpms out of the pkg path (which may 
    # include the CSM CD).  This allows us to put newer versions of some 
    # distro rpms on our media if we need to.
    # Note:  DISTRO_PREREQS is a hash of array references, thus the 
    #        funny syntax below.
    my (@DISTRO_PKGPATH) = @::PKGPATH;

    #SuSE Changes
    if( $distro =~ /SLES/ || $distro =~ /SuSE/ )
    {
       if ($::command_line=~/copycsmpkgs/)
       {  
	  my @Temp = @DISTRO_PKGPATH;
	  map {s|$|/$::pkgdefs{DISTRO_RPMDIR}/*|} @DISTRO_PKGPATH;
	  if(($::pkgdefs{DISTRO_NAME} =~ /SLES/) && ($::pkgdefs{DISTRO_NAME} =~ /8\.1/))		
	  {
	      map {s|$|/UnitedLinux/*|} @Temp;
	      @DISTRO_PKGPATH=(@DISTRO_PKGPATH, @Temp);
	  }
       }
       else
       { 
          my @Temp = @DISTRO_PKGPATH;
	  map {s|$|/$::Svrpkgdefs{DISTRO_RPMDIR}/*|} @DISTRO_PKGPATH;
	  if(($distro =~ /SLES/) && ($distro =~ /8\.1/))
	  {
              map {s|$|/UnitedLinux/*|} @Temp;
	      @DISTRO_PKGPATH=(@DISTRO_PKGPATH, @Temp);
	  }	
       }
    }
    else
    {
       if ($::command_line=~/copycsmpkgs/)
       { map {s|$|/$::pkgdefs{DISTRO_RPMDIR}|} @DISTRO_PKGPATH;}
       else { map {s|$|/$::Svrpkgdefs{DISTRO_RPMDIR}|} @DISTRO_PKGPATH;} 
    }
    @DISTRO_PKGPATH=(@DISTRO_PKGPATH,@::PKGPATH);
    
    #Empty the the hashes already used
    %::temp_CmdLinePathVersionName=();
    %::Rpm_name_path=();
    %::CmdLinePathVersionName=();
    @::Rpms_to_be_copied=();
    @::Rpms_not_found=();
    @::RpmsNotFoundInCmdPath=();

    #SSS_BEGIN
    # Determine the RPM architectures to search for on each platform.
    my @archs;
    my $arch = $::PREREQS_ATTR{'PkgArchitecture'};
    if (! $arch) {$arch = $::ATTRS{'InstallPkgArchitecture'}; }
  
    if ($arch =~ /i.86/)
    {
      @archs = ('i?86','noarch');
    }
    elsif ($arch =~ /ppc64/)
    {
      @archs = ($arch,'ppc','noarch');
    }
    elsif ($arch =~ /x86_64/)
    {
      @archs = ($arch,'i?86','noarch');
    }
    elsif ($arch =~ /ia64/)
    {
      @archs = ($arch,'i?86','noarch');
    }
    else 	# Any other unknown architectures.
    {
      @archs = ($arch,'noarch');
    }
    #SSS_END

    #foreach $prereq (@::os_prereq, @::sis_packages)
    foreach $prereq (@::os_prereq)
      {
	    #SSS_BEGIN
	    NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ', $prereq);
	    $my_prereq = '';
		$path = ''; 
	    foreach my $a (@archs)
	    {
		  if (! ($path)) {
		      $my_prereq = "${prereq}.$a.rpm";
		      $path = InstallUtils->findRpmInPath($my_prereq, @DISTRO_PKGPATH);
		  }
		  else { last; }
	    }
	    #SSS_END

#            if ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/) || ($::ATTRS{'InstallPkgArchitecture'} =~ /i?86/) )
#            {
#               $my_prereq="$prereq.i?86.rpm";
#	       NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ', $prereq);
#	       $path = InstallUtils->findRpmInPath($my_prereq, @DISTRO_PKGPATH);
#            }
#	    elsif ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/) || ($::ATTRS{'InstallPkgArchitecture'} =~ /ppc64/) )
#            {
#               $my_prereq="$prereq.ppc64.rpm";
#           		NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ', $prereq);
#           		$path = InstallUtils->findRpmInPath($my_prereq, @DISTRO_PKGPATH);
#
#				if( !($path) ){
# 					$my_prereq="$prereq.ppc.rpm";
#                	NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ', $prereq);
#                	$path = InstallUtils->findRpmInPath($my_prereq, @DISTRO_PKGPATH);
#				}
#            } 
#            else
#            {
#               $my_prereq="$prereq.ia64.rpm";
#	       NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ', $prereq);
#	       $path = InstallUtils->findRpmInPath($my_prereq, @DISTRO_PKGPATH);
#               if (!($path))
#               {
#                 $my_prereq="$prereq.i?86.rpm";
#	         NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ', $prereq);
#	         $path = InstallUtils->findRpmInPath($my_prereq, @DISTRO_PKGPATH);
#               }
#            }
#            if (!($path))
#            {
#              $my_prereq="$prereq.noarch.rpm";
#	      $path = InstallUtils->findRpmInPath($my_prereq, @DISTRO_PKGPATH);
#            } 

	    if (defined($path)) 
            {
                # Get the latest version of the file in the first location found
                #my $rpm_pre_file = NodeUtils->runcmd("$::LS -vr $path/$my_prereq | $::HEAD -1", -1);
                my $rpm_pre_file = InstallUtils->getRecentVersionRpm($path, $my_prereq);
                chomp $rpm_pre_file;
                my $rpmfile_basename;
                ($rpmfile_basename = $rpm_pre_file) =~ s:^.*/::g;
                #find the shortname
                my @parts=split /-/, $rpmfile_basename;
                my $shortname = shift @parts;
                my $p;
                SNAME: while ($p = shift @parts){
                         if($p !~ m/^\d/){ #if it doesn't start with a digit
                             $shortname .= "-$p"; }
                         else{ unshift @parts, $p; last SNAME; }
                       }
                push @{$::temp_CmdLinePathVersionName{$path}},$rpmfile_basename;
                $::Rpm_name_path{$shortname}{'BaseName'}=$rpmfile_basename;
                $::Rpm_name_path{$shortname}{'Location'}=$path;
                $::Rpm_name_path{$shortname}{'Destination'}=$destination;
            }
            else 
            { 
               #get the shortname of the rpm
               my @parts=split /-/, $prereq; 
               my $shortname = shift @parts;
               my $p;
               SNAME: while ($p = shift @parts){
                         if($p !~ m/^\d/){ #if it doesn't start with a digit
                             $shortname .= "-$p"; }
                         else{ unshift @parts, $p; last SNAME; }
                       }
               my $tmp_var1;
               my $my_rindex=rindex($shortname,"*");
               if ($my_rindex!=-1) {  $tmp_var1=substr($shortname,0,rindex($shortname,"*")); }
               else { $tmp_var1=$shortname; }
               $my_rindex=rindex($tmp_var1,"-");
               if ( ($my_rindex+1) == 0 ) { $shortname=$tmp_var1; }
               elsif ( ($my_rindex+1) == length($tmp_var1) ) { $shortname=substr($tmp_var1,0,rindex($tmp_var1,"*")); }
               else { $shortname=$tmp_var1; }
               push(@::RpmsNotFoundInCmdPath, $shortname); 
               #push(@::RpmsNotFoundInCmdPath,$prereq);
            }
      }
    my @temp_array2=();
    #copy the pacakges found in the path
    InstallUtils->create_rpm_list_to_copy();

    foreach my $my_rpm_name (@::Rpms_not_found)
    { push(@temp_array2,$::PackageVersionName{$my_rpm_name}{'FullName'}); }

    @notfound=@temp_array2;
    InstallUtils->copy_rpm();

    # return if all the packages are found
    if (!scalar(@notfound)) { return; }
    # if the packages are not found in the path than check in the 
    # /csminstall.. directory if the copy flag is not set
         

    # Copy RPMs from distribution CD-ROM(s) 
    $cmd = "$::UNMOUNT $::MNTCDROM > /dev/null 2>&1";
    NodeUtils->runcmd("$cmd",-1);
    #&run_cmd($cmd, "W", "ignore_errors");
    #my $my_DISTRO_DISKLIST;
    my $my_DISTRO_NAME;
    my $my_DISTRO_RPMDIR;
    my @disks;
    if ($::command_line=~/copycsmpkgs/)
    { 
       #$my_DISTRO_DISKLIST=$::pkgdefs{DISTRO_DISKLIST}; 
       @disks = (@{$::pkgdefs{'distro_disks'}});
       $my_DISTRO_NAME=$::pkgdefs{DISTRO_NAME};
       $my_DISTRO_RPMDIR=$::pkgdefs{DISTRO_RPMDIR};
    }
    else
    { 
       #$my_DISTRO_DISKLIST=$::Svrpkgdefs{DISTRO_DISKLIST}; 
       @disks = (@{$::Svrpkgdefs{'distro_disks'}});
       $my_DISTRO_NAME=$::Svrpkgdefs{DISTRO_NAME};
       $my_DISTRO_RPMDIR=$::Svrpkgdefs{DISTRO_RPMDIR};
    }
    #foreach $disk (split (/\s/, $my_DISTRO_DISKLIST))
    foreach my $diskref (@disks)
    {
	my $diskname = $diskref->{name};
	my $disknum  = $diskref->{num};
	NodeUtils->message('I', 'IMsgINSERT_DISK', $diskname, $disknum);

	NodeUtils->message('P', 'IMsgPRESS_ENTER');
        #Empty the the hashes already used
        %::temp_CmdLinePathVersionName=();
        %::Rpm_name_path=();
        %::CmdLinePathVersionName=();
        @::Rpms_to_be_copied=();
        @::Rpms_not_found=();
        @::RpmsNotFoundInCmdPath=();

	my $junk = getc;

	my $wrong_disk = 1;
	while ($wrong_disk)
	{
         # Mount the CD-ROM
	    if ($::PLTFRM eq "AIX")
            {
		 sleep(5); #this is needed for AIX
		 `$::UNMOUNT $::MNTCDROM 2>/dev/null 1>/dev/null`;
		 InstallUtils->MountAixCdrom();
            }
            else {
	         $cmd = "$::MOUNT $::MNTCDROM";
		 # Ignore mount errors.  Usually they are because
		 # the CD was already mounted.
	         NodeUtils->runcmd("$cmd",-1);
            }
	    # Check for valid disk inserted
	    my $diskid_searchstring = $diskref->{diskid_searchstring};
	    my $diskid_filename = "$::MNTCDROM/" . $diskref->{diskid_filename};
	    $cmd = "$::LS $diskid_filename 2> /dev/null";
	    chomp($diskid_filename = NodeUtils->runcmd($cmd, -1));
	    # First make sure the file exists
	    if (! $::RUNCMD_RC and -f $diskid_filename)
	    {
		if ($diskid_searchstring)
		{
		    open (DISKID_FILE, $diskid_filename) || 
			NodeUtils->message('E', 'EMsgCANT_READ_FILE', $diskid_filename); 
		    if (grep (/$diskid_searchstring/, <DISKID_FILE>))
		    {
			#print "CORRECT DISK!\n";
			$wrong_disk = 0;
		    }
		    else {
			#print "Search string $diskid_searchstring not found\n";
			$wrong_disk = 1; 
		    }
		    close DISKID_FILE;
		}
		else { $wrong_disk = 0; }
	    }
	    else {
		#print "Can't find $::MNTCDROM/$diskref->{diskid_filename}.  \nERRORS:  $diskid_filename\n"; 
		$wrong_disk = 1; 
	    }

	    if ($wrong_disk) {
                if ($::PLTFRM eq "AIX")
                {
                 `$::UNMOUNT $::MNTCDROM 2>/dev/null 1>/dev/null`;
                }
                else
                {
		 $cmd = "$::UNMOUNT $::MNTCDROM > /dev/null 2>&1";
		 NodeUtils->runcmd("$cmd",-1);
                }
		NodeUtils->message('W', 'EMsgWRONG_DISK', $diskname, $disknum);
		NodeUtils->message('I', 'IMsgINSERT_DISK', $diskname, $disknum);

		NodeUtils->message('P', 'IMsgPRESS_ENTER_OR_FORCE');

		# Get user input
		$junk = getc;
		if ($junk eq "F" or $junk eq "f") { 
		    #Force it to be the right disk
		    # Mount the CD-ROM
                    if ($::PLTFRM eq "AIX")
                    {
		       `$::UNMOUNT $::MNTCDROM 2>/dev/null 1>/dev/null`;
		       InstallUtils->MountAixCdrom();
                    }
                    else{
		       $cmd = "$::MOUNT $::MNTCDROM";
		       NodeUtils->runcmd($cmd,-1);
                    }
		    $wrong_disk = 0;
		}
		else {
		    # Try again
		    next;
		}
	    }
	}

	#SSS_BEGIN
	# Determine the RPM architectures to search for on each platform
	# and determine the directory to search for RPMs on each CD.
	my @archs;
	my $arch = $::PREREQS_ATTR{'PkgArchitecture'};
	if (! $arch) {$arch = $::ATTRS{'InstallPkgArchitecture'}; }
      
	if ($arch =~ /i.86/)
	{
	  @archs = ('i?86','noarch');

	  if ( ($distro =~ /SLES/) && ($distro =~ /8\.1/) ) {
	    # See if this is the SLES disk (not the UnitedLinux disk)
		if ($disknum eq '1' and $diskname =~ /$my_DISTRO_NAME/){
		$dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
	      }
	      else{
		$dir = "$::MNTCDROM/UnitedLinux/*";
	      }
	  }
	  elsif (($distro =~ /SuSE/) || ($distro=~ /SLES/) )
	  {
	      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
	  }
	  else
	  {
	      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR";
	  }
	}
	elsif ($arch =~ /ppc64/)
	{
	  @archs = ($arch,'ppc','noarch');

	  if (($distro =~ /SuSE/) || ($distro=~ /SLES/) )
	  {
	      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
	  }
	  else
	  {
	      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR";
	  }
	}
	elsif ($arch =~ /x86_64/)
	{
	  @archs = ($arch,'i?86','noarch');

	  if ( ($distro =~ /SLES/) && ($distro =~ /8\.1/) ) {
	      # See if this is the SLES disk (not the UnitedLinux disk)
	      if ($disknum eq '1' and $diskname =~ /$my_DISTRO_NAME/){
		$dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
	      }
	      else{
		$dir = "$::MNTCDROM/UnitedLinux/*";
	      }
	  }
	  elsif (($distro =~ /SuSE/) || ($distro=~ /SLES/) )
	  {
	      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
	  }
	  else
	  {
	      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR";
	  }
	}
	elsif ($arch =~ /ia64/)
	{
	  @archs = ($arch,'i?86','noarch');

	  if ( ($distro=~ /SLES/) && ($distro=~ /8\.1/) ) {
	      # See if this is the SLES disk (not the UnitedLinux disk)
	      if ($disknum eq '1' and $diskname =~ /$my_DISTRO_NAME/){
		  $dir = "$::MNTCDROM/my_DISTRO_RPMDIR/*";
	      }
	      else{
		  $dir = "$::MNTCDROM/UnitedLinux/*";
	      }
	  }
	  elsif (($distro=~ /SuSE/) || ($distro=~ /SLES/) )
	  {
	      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
	  }
	  else
	  {
	      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR";
	  }
	}
	else 	# Any other unknown architectures.
	{
	  @archs = ($arch,'noarch');
	  $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR";
	}
	#SSS_END

	foreach $prereq (@notfound)
	  {
		#SSS_BEGIN
		$my_prereq = '';
		$path = '';
		if (!length($prereq)) { next; }    # already found that one
                NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ_ON_CD', $prereq, $disknum);
		foreach my $a (@archs)
		{
		      if (! ($path)) {
			  $my_prereq = "${prereq}.$a.rpm";
			  $path = InstallUtils->findRpmInPath($my_prereq, $dir);
		      }
		      else { last; }
		}
		#SSS_END

#                if ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/) || ($::ATTRS{'InstallPkgArchitecture'} =~ /i?86/) )
#                { 
#                  $my_prereq="$prereq.i?86.rpm";
#		  if (!length($prereq)) { next; }    # already found that one
#		  NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ_ON_CD', $prereq, $disknum);
#
#                  # SuSE Changes
#     		  if ( ($distro =~ /SLES/) && ($distro =~ /8\.1/) ) {
#		      # See if this is the SLES disk (not the UnitedLinux disk)
# 		      if ($disknum eq '1' and $diskname =~ /$my_DISTRO_NAME/){
#                       	$dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
# 		      }
# 		      else{
#                       	$dir = "$::MNTCDROM/UnitedLinux/*";
# 		      }
# 		  }
#     		  elsif (($distro =~ /SuSE/) || ($distro=~ /SLES/) )
#                  {
#                      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
#                  }
#                  else
#                  {
#		      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR";
#                  }
#
#		  $path = InstallUtils->findRpmInPath($my_prereq, $dir);
#                }
#                elsif ( ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/) || ($::ATTRS{'InstallPkgArchitecture'} =~ /ppc64/) )
#                {
#                  $my_prereq="$prereq.ppc64.rpm";
#                  if (!length($prereq)) { next; }    # already found that one
#                  NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ_ON_CD', $prereq, $disknum);
# 
#                  # SuSE Changes
#                  if (($distro =~ /SuSE/) || ($distro=~ /SLES/) )
#                  {
#                      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
#                  }
#                  else
#                  {
#                      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR";
#                  }
# 
#                  $path = InstallUtils->findRpmInPath($my_prereq, $dir);
#
#                  if( !($path)){
#                        $my_prereq="$prereq.ppc.rpm";
#                        if (!length($prereq)) { next; }    # already found that one
#                        NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ_ON_CD', $prereq, $disknum);
# 
#                        # SuSE Changes
#                        if (($distro =~ /SuSE/) || ($distro=~ /SLES/) )
#                        {
#                            $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
#                        }
#                        else
#                        {
#                            $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR";
#                        }
# 
#                        $path = InstallUtils->findRpmInPath($my_prereq, $dir);
#                  }
#                }
#                else	# IA64
#                {
#                  $my_prereq="$prereq.ia64.rpm";
#                  if (!length($prereq)) { next; }    # already found that one
#                  NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ_ON_CD', $prereq, $disk);
#
#                  #SuSE Changes
#                  if ( ($distro=~ /SLES/) && ($distro=~ /8\.1/) ) {
#                      # See if this is the SLES disk (not the UnitedLinux disk)
#                      if ($disknum eq '1' and $diskname =~ /$my_DISTRO_NAME/){
#                          $dir = "$::MNTCDROM/my_DISTRO_RPMDIR/*";
#                      }
#                      else{
#                          $dir = "$::MNTCDROM/UnitedLinux/*";
#                      }
#                  }
#                  elsif (($distro=~ /SuSE/) || ($distro=~ /SLES/) )
#                  {
#                      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR/*";
#                  }
#                  else
#                  {
#                      $dir = "$::MNTCDROM/$my_DISTRO_RPMDIR";
#                  }
#
#                  $path = InstallUtils->findRpmInPath($my_prereq, $dir);
#                  if (!($path))
#                  {
#                    $my_prereq="$prereq.i?86.rpm";
#                    if (!length($prereq)) { next; }    # already found that one
#                    NodeUtils->message('V', 'IMsgSEARCHING_FOR_PREREQ_ON_CD', $prereq, $disknum);
#                    $path = InstallUtils->findRpmInPath($my_prereq, $dir);
#                  }
#                }
#                if (!($path))
#                {
#                    $my_prereq="$prereq.noarch.rpm";
#                    $path = InstallUtils->findRpmInPath($my_prereq, $dir);
#                }

                if (defined($path))
                {
                        # Get the latest version of the file in the first location found
                        #my $rpm_pre_file = NodeUtils->runcmd("$::LS -vr $path/$my_prereq | $::HEAD -1", -1);
                        my $rpm_pre_file = InstallUtils->getRecentVersionRpm($path, $my_prereq);
                        chomp $rpm_pre_file;
                        my $rpmfile_basename;
                        ($rpmfile_basename = $rpm_pre_file) =~ s:^.*/::g;
                        #find the shortname
                        my @parts=split /-/, $rpmfile_basename;
                        my $shortname = shift @parts;
                        my $p;
                        SNAME: while ($p = shift @parts){
                                 if($p !~ m/^\d/){ #if it doesn't start with a digit
                                     $shortname .= "-$p"; }
                                 else{ unshift @parts, $p; last SNAME; }
                               }
                        push @{$::temp_CmdLinePathVersionName{$path}},$rpmfile_basename;
                        $::Rpm_name_path{$shortname}{'BaseName'}=$rpmfile_basename;
                        $::Rpm_name_path{$shortname}{'Location'}=$path;
                        $::Rpm_name_path{$shortname}{'Destination'}=$destination;
                        $prereq = '';         # so we do not search for it on the next cd
                } 
          }
          # copy the rpm collected from the cd
          InstallUtils->create_rpm_list_to_copy();
          my $nf = join(' ', @notfound);      # this eliminates the ones we blanked out
          @notfound = split(' ', $nf);

          if (scalar(@::Rpms_not_found) >=1)
          { @notfound=(@notfound,@::Rpms_not_found); }
          @::Rpms_not_found=();
          InstallUtils->copy_rpm();
          # return if all the packages are found
          if (!scalar(@notfound)) { last; } # do not need to go to the next cd
  
 	  $cmd = "$::UNMOUNT $::MNTCDROM > /dev/null 2>&1";
          NodeUtils->runcmd("$cmd",-1);
    }

        # make sure to unmount cdrom after copying all the rpms
	$cmd = "$::UNMOUNT $::MNTCDROM > /dev/null 2>&1";
        NodeUtils->runcmd("$cmd",-1);
	#&run_cmd($cmd, "W");

	# Ignore all SIS packages
	my $rpm_temp;
	my (@ignore_temp);
	my (@notfound_temp);
	foreach $rpm_temp (@notfound)
	{
		my $short_rpm;
	        my $my_rindex=rindex($rpm_temp,"*");
	        if ($my_rindex!=-1) {  
			$short_rpm=substr($rpm_temp,0,$my_rindex); 
		}
		#my @foo = grep(/$short_rpm/, @::sis_packages);
		#if (scalar(@foo)) {
		#	push (@ignore_temp,$rpm_temp);
		#}
		#else {
			push (@notfound_temp,$rpm_temp); 
		#}
	}

	if (scalar(@ignore_temp)) { NodeUtils->message('I', 'IMsgMissingRPM', join(", ",@ignore_temp)); }
	if (scalar(@notfound_temp)) { NodeUtils->message('E1', 'EMsgPREREQ_NOT_FOUND', join(", ",@notfound_temp)); }
 }
 elsif ($::PREREQS_ATTR{'OSName'} eq "AIX")
 {
 }
}

#-------------------------------------------------------------------------------#
#                                                                               #
#  create_rpm_list_to_copy                                                      #
#	This subroutine will create a real hash of the rpms to be copied by     #
#	comparing the versions, the final hash populated is Rpms_to_be_copied   #
#                                                                               #
#-------------------------------------------------------------------------------#
sub create_rpm_list_to_copy
{
  my (@my_rpm_array, @my_result);
  my $my_rpm_string;
  my @temp_RpmsNotFoundInCmdPath_array=();
  foreach my $path (keys %::temp_CmdLinePathVersionName)
  {
   @my_rpm_array=();
   foreach my $my_rpm (@{$::temp_CmdLinePathVersionName{$path}})
   {
     push(@my_rpm_array,($path."/".$my_rpm));
   }
   $my_rpm_string=join(" ",@my_rpm_array);
   my $cmd="$::RPMCMD -qp --qf '%{Name}::%{Version}::%{Release}::%{Arch}\n' $my_rpm_string";
   @my_result=NodeUtils->runcmd("$cmd",-1);
   foreach my $i (@my_result)
   {
     if (!($i=~/.*::.*::.*::.*/)) { next;} 
     my ($name, $version, $release, $arch)=split(/::/,$i);
     if (exists $::CmdLinePathVersionName{$arch})
     {
       if (exists $::CmdLinePathVersionName{$arch}{$name})
       {
         my $ext_ver=$::CmdLinePathVersionName{$arch}{$name}{'Version'};
         my $ext_rel=$::CmdLinePathVersionName{$arch}{$name}{'Release'};
         if (InstallUtils->testVersion($version,">=",$ext_ver,$release,$ext_rel))
         {
           $::CmdLinePathVersionName{$arch}{$name}{'Version'}=$version;
           $::CmdLinePathVersionName{$arch}{$name}{'Release'}=$release;
           $::CmdLinePathVersionName{$arch}{$name}{'Location'}=$path;
           $::CmdLinePathVersionName{$arch}{$name}{'Longname'}=$name."-".$version."-".$release;
          }
       }
       else
       {
         $::CmdLinePathVersionName{$arch}{$name}{'Version'}=$version;
         $::CmdLinePathVersionName{$arch}{$name}{'Release'}=$release;
         $::CmdLinePathVersionName{$arch}{$name}{'Location'}=$path;
         $::CmdLinePathVersionName{$arch}{$name}{'Longname'}=$name."-".$version."-".$release;
       }
     }
     else
     {
       $::CmdLinePathVersionName{$arch}{$name}{'Version'}=$version;
       $::CmdLinePathVersionName{$arch}{$name}{'Release'}=$release;
       $::CmdLinePathVersionName{$arch}{$name}{'Location'}=$path;
       $::CmdLinePathVersionName{$arch}{$name}{'Longname'}=$name."-".$version."-".$release;
     }
   } 
 }
 # check for the result with the temp_CmdLinePathVersionName
 #foreach my $arch (%::CmdLinePathVersionName)
 #{
 #}


 #check the version and populate the to be copied hash and version not staisfied hash
 foreach my $arch (keys %::CmdLinePathVersionName)
 {
  if ($::COPY_OPT_FLAG)
  {
     #compare cm line vs defs
     foreach my $rpm_name (keys %{$::CmdLinePathVersionName{$arch}})
     {
       my $def_ver=$::PackageVersionName{$rpm_name}{'Version'};
       my $def_rel=$::PackageVersionName{$rpm_name}{'Release'};
       my $cmd_line_ver=$::CmdLinePathVersionName{$arch}{$rpm_name}{'Version'};
       my $cmd_line_rel=$::CmdLinePathVersionName{$arch}{$rpm_name}{'Release'};
       if (InstallUtils->testVersion($cmd_line_ver,">",$def_ver,$cmd_line_rel,$def_rel))
       {
         #populate the copy hash 
         push (@::Rpms_to_be_copied, $rpm_name);
       }
       elsif (InstallUtils->testVersion($cmd_line_ver,"=",$def_ver,$cmd_line_rel,$def_rel))
       {
         #populate the copy hash 
         push (@::Rpms_to_be_copied, $rpm_name);
       }
       else { 
           # error :rpm of the required version not found
           push(@::Rpms_not_found, $rpm_name);
       }
     }
  }
  else 
  { #compare and populate the copy hash only necessary
    # First check against the command line and if the rpms are
    # found than  check the /csminstall.. directory
    foreach my $rpm_name (keys %{$::CmdLinePathVersionName{$arch}})
    {
      if (exists $::PathVersionName{$arch}{$rpm_name})
      {
        my $def_ver=$::PackageVersionName{$rpm_name}{'Version'};
        my $def_rel=$::PackageVersionName{$rpm_name}{'Release'};
        my $cmd_line_ver=$::CmdLinePathVersionName{$arch}{$rpm_name}{'Version'};
        my $cmd_line_rel=$::CmdLinePathVersionName{$arch}{$rpm_name}{'Release'};
        my $path_ver=$::PathVersionName{$arch}{$rpm_name}{'Version'};
        my $path_rel=$::PathVersionName{$arch}{$rpm_name}{'Release'};
        if ($::CmdLinePathVersionName{$arch}{$rpm_name}{'Longname'} eq $::PathVersionName{$arch}{$rpm_name}{'Longname'})
        {
          #compare:Cmd vs defs and populate the copy hash the rpm
          if (InstallUtils->testVersion($cmd_line_ver,">=",$def_ver,$cmd_line_rel,$def_rel))
          {
            #populate the copy hash 
            push (@::Rpms_to_be_copied, $rpm_name); next;
          }
          else { 
             push(@::Rpms_not_found, $rpm_name); next;
          }
        }
        else
        { #compare: command line vs path 
         if (InstallUtils->testVersion($cmd_line_ver,">=",$path_ver,$cmd_line_rel,$path_rel)) 
         {
           #compare :cmd vs defs
           if (InstallUtils->testVersion($cmd_line_ver,">=",$def_ver,$cmd_line_rel,$def_rel))
           {
             #populate the copy csm rpm 
             push (@::Rpms_to_be_copied, $rpm_name); next;
           }
           else { 
              push(@::Rpms_not_found, $rpm_name); next;
           }
         }
         else
         {
           if (InstallUtils->testVersion($path_ver,">=",$def_ver,$path_rel,$def_rel))
           {   
             next;
           } 
           else {
               push(@::Rpms_not_found, $rpm_name); next
           }
         }
        }
      }
      else
      { # path doesn't exist
        my $def_ver=$::PackageVersionName{$rpm_name}{'Version'};
        my $def_rel=$::PackageVersionName{$rpm_name}{'Release'};
        my $cmd_line_ver=$::CmdLinePathVersionName{$arch}{$rpm_name}{'Version'};
        my $cmd_line_rel=$::CmdLinePathVersionName{$arch}{$rpm_name}{'Release'};
        if (InstallUtils->testVersion($cmd_line_ver,">=",$def_ver,$cmd_line_rel,$def_rel))
        { 
          #populate the copy hash
          push (@::Rpms_to_be_copied, $rpm_name); next;
        }
        else
        {
           push(@::Rpms_not_found, $rpm_name); next;
        }
      }
    }
  }
 }
    if ( (scalar(@::Rpms_not_found) >=1) ||  (scalar(@my_result) ==0)  || (scalar(@::RpmsNotFoundInCmdPath) >=1) )
    {
      # check the /csminstall.. directory
      # gather the arch name and populate the valid arch array of 
      # the PathVersionName hash depending on the os
      my (@my_temp_array, @valid_arch)=();
      foreach my $arch (keys %::PathVersionName)
      { push(@my_temp_array,$arch); }
      if ($::PREREQS_ATTR{'OSName'} eq "AIX")
      {
        foreach my $arch (@my_temp_array)
        {
          if (($arch =~/ppc/) || ($arch =~/noarch/))
          { push(@valid_arch, $arch); }
        }
      }
      elsif (($::PREREQS_ATTR{'OSName'} eq "Linux") || ($::ATTRS{'InstallOSName'} eq "Linux"))
      {
        foreach my $arch (@my_temp_array)
        {
          if (($arch =~/86/) || ($arch =~/ia64/) || ( $arch =~ /ppc64/ ) || ($arch =~/noarch/))
          { push(@valid_arch, $arch); }
        }
      }
      my (%temp_hash)=();
      # remove the number for the rpm name in the RpmsNotFoundInCmdPath variable
      foreach my $rpm_name_notfound (@::RpmsNotFoundInCmdPath)
      {
        my @parts=split /-/, $rpm_name_notfound;
        my $shortname = shift @parts;
        my $p;
        SNAME: while ($p = shift @parts)
        {
           if($p !~ m/^\d/)
           { #if it doesn't start with a digit
             $shortname .= "-$p";
           }
           else{ unshift @parts, $p; last SNAME; }
        }
        my $tmp_var1;
        my $my_rindex=rindex($shortname,"*");
        if ($my_rindex!=-1) {  $tmp_var1=substr($shortname,0,rindex($shortname,"*")); }
        else { $tmp_var1=$shortname; }
        $my_rindex=rindex($tmp_var1,"-");
        if ( ($my_rindex+1) == 0 ) { $shortname=$tmp_var1; }
        elsif ( ($my_rindex+1) == length($tmp_var1) ) { $shortname=substr($tmp_var1,0,rindex($tmp_var1,"*")); }
        push(@temp_RpmsNotFoundInCmdPath_array,$shortname);
      }
      @my_temp_array=(@::Rpms_not_found,@temp_RpmsNotFoundInCmdPath_array);
      @::Rpms_not_found=InstallUtils->get_uniqu_arry_elemnts(@my_temp_array);
      my @temp_not_found_rpm=();
      @temp_not_found_rpm=@::Rpms_not_found; 
      @::Rpms_not_found=();
      @::RpmsNotFoundInCmdPath=();
      my (@temp_array,@Rpmfound)=(); 
      if (scalar(@valid_arch) <= 0)
      {
         @::Rpms_not_found=@temp_not_found_rpm;
      }
      foreach my $my_rpm (@temp_not_found_rpm)
      {
         PATHVER: foreach my $arch (@valid_arch)
         {
            if (exists $::PathVersionName{$arch}{$my_rpm})
            {
               my $def_ver=$::PackageVersionName{$my_rpm}{'Version'};
               my $def_rel=$::PackageVersionName{$my_rpm}{'Release'};
               my $path_ver=$::PathVersionName{$arch}{$my_rpm}{'Version'};
               my $path_rel=$::PathVersionName{$arch}{$my_rpm}{'Release'};
               if (InstallUtils->testVersion($path_ver,">=",$def_ver,$path_rel,$def_rel))
               {
                   my %temp_Rpmsfound_hash=();
                   foreach my $my_Rpmfound (@::Rpmfound) { $temp_Rpmsfound_hash{$my_Rpmfound}++; }
                   if (!exists($temp_Rpmsfound_hash{$my_rpm}))
                   {
                     push (@::Rpmfound,$my_rpm); last PATHVER;
                   }
                   else { push(@::Rpms_not_found, $my_rpm); next PATHVER; }
 
               }
               else { push(@::Rpms_not_found, $my_rpm); next PATHVER; }
            }
            else { push(@::Rpms_not_found, $my_rpm); next PATHVER; }
         }
      }
      @::Rpms_not_found=InstallUtils->get_uniqu_arry_elemnts(@::Rpms_not_found);
      my %temp_Rpms_hash=();
      foreach my $my_Rpmfound (@::Rpmfound) { $temp_Rpms_hash{$my_Rpmfound}++; }
      foreach my $my_rpm_notfound (@::Rpms_not_found)
      {
        if (!exists($temp_Rpms_hash{$my_rpm_notfound})) { push (@temp_array,$my_rpm_notfound); }
      }
      @::Rpms_not_found=@temp_array;
      @temp_array=();
      foreach my $my_rpm_notfound (@temp_RpmsNotFoundInCmdPath_array)
      {
         if (!exists($temp_Rpms_hash{$my_rpm_notfound})) { push (@temp_array,$my_rpm_notfound); }
      }
      @temp_RpmsNotFoundInCmdPath_array=@temp_array;

    }

  my %temp_hash=();
  if (scalar(@::Rpms_to_be_copied) >=1)
  {
     foreach my $rpm (@::Rpms_to_be_copied) { $temp_hash{$rpm}++; }
     foreach my $rpm (@temp_RpmsNotFoundInCmdPath_array) 
     {
        if (!exists($temp_hash{$rpm})) { push (@::Rpms_not_found,$rpm); }
     }
     @::Rpms_not_found=InstallUtils->get_uniqu_arry_elemnts(@::Rpms_not_found);
  }
}

#-------------------------------------------------------------------------------#
#                                                                               #
#   copy_rpm                                                                    #
#	Copies the rpms using the Rpms_to_be_copied and Rpm_name_path hash      #
#                                                                               #
#-------------------------------------------------------------------------------#
sub copy_rpm
{
 foreach my $my_rpm (@::Rpms_to_be_copied)
 {
   my ($location, $destination, $full_name);
   $full_name=$::Rpm_name_path{$my_rpm}{'BaseName'};
   $location=$::Rpm_name_path{$my_rpm}{'Location'};
   $destination=$::Rpm_name_path{$my_rpm}{'Destination'};
   if ( (defined($full_name)) && (defined($location)) && (defined($destination)) )
   {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','I', 'IMsgCOPYING', $full_name);
      my $cmd="$::COPY $location/$full_name $destination";
      NodeUtils->runcmd("$cmd");
      $cmd="$::CHMOD +r $destination/$full_name";
      NodeUtils->runcmd("$cmd");
   }
   else
   { NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E1', 'EMsgNoRpmFound', $my_rpm); }
 }    
}

#-------------------------------------------------------------------------------#
# post_csm_copy                                                                 #
#  post copy functions                                                          #
#                                                                               #
#-------------------------------------------------------------------------------#
sub post_csm_copy
{
 if ( ($::PREREQS_ATTR{'OSName'} eq "Linux") || ($::ATTRS{'InstallOSName'} eq "Linux") )
 {
    # Link the packages directory to a location that addlcsext can use with its default -p
    if ($::LCSEXT eq "true")
    {
        my $cmd;
        if ( ! -d "/opt/csm/lcsext")
        {
            $cmd = "$::MKDIR /opt/csm/lcsext";
            NodeUtils->runcmd($cmd, 0);
        }
        else
        {
            $cmd = "$::RM /opt/csm/lcsext/packages";
            NodeUtils->runcmd($cmd, 0);
        }
        $cmd = "$::LN -sf $::INSTALLDIR_CSMPKG /opt/csm/lcsext/packages";
        NodeUtils->runcmd($cmd, 0);
    }
 }
 elsif ($::PREREQS_ATTR{'OSName'} eq "AIX") { }
 else { NodeUtils->message('E2', 'EMsgINVALID_OSTYPE'); }
}

#-------------------------------------------------------------------------------#
# search_rpm_csminstall_path                                                    #
#	Search the /csminstall.. path for the rpms not found in the installms 	#
#	path. Valid ony if -c arg is not used while using installms 		#
#	and copycsmpkgs.                                                        #
#	Populate the temp_CmdLinePathVersionName and Rpm_name_path hash and if  #
#	not found than populate the RpmsNotFoundInCmdPath hash   		#
#                                                                               #
#-------------------------------------------------------------------------------#
sub search_rpm_csminstall_path
{
  #Empty the the hashes already used
  %::temp_CmdLinePathVersionName=();
  %::Rpm_name_path=();
  %::CmdLinePathVersionName=();
  @::Rpms_to_be_installed=();
  @::Rpms_already_installed=();
  @::Rpms_not_found=();
  @::RpmsNotFoundInCmdPath=();

 my ($class, @rpm_list)=@_;
 my (@my_path);
 if ($::PREREQS_ATTR{'OSName'} eq "Linux")
 {
    @my_path=($::INSTALLDIR_CSMPKG, $::INSTALLDIR_OS_NAME_RPMS);
    my ($rpmfile, $rpm, $path);

    #SSS_BEGIN
    # Determine the RPM architectures to search for on each platform.
    my @archs;
    my $arch = $::PREREQS_ATTR{'PkgArchitecture'};
    if (! $arch) {$arch = $::ATTRS{'InstallPkgArchitecture'}; }
  
    if ($arch =~ /i.86/)
    {
      @archs = ('i?86','noarch');
    }
    elsif ($arch =~ /ppc64/)
    {
      @archs = ($arch,'ppc','noarch');
    }
    elsif ($arch =~ /x86_64/)
    {
      @archs = ($arch,'i?86','noarch');
    }
    elsif ($arch =~ /ia64/)
    {
      @archs = ($arch,'i?86','noarch');
    }
    else 	# Any other unknown architectures.
    {
      @archs = ($arch,'noarch');
    }
    #SSS_END

    # Search the RPMs from path 
    foreach $rpm (@rpm_list)
    {
       $rpmfile = ''; #SSS
       $path = ''; #SSS

       # Process the IBMJava rpm begin.....
       if ($rpm =~/^IBMJava2-JRE/)
       {
	  #SSS_BEGIN
	  foreach my $a (@archs)
	  {
	      if (! ($path)) {
		  $rpmfile = "${rpm}-*.$a.rpm";
		  $path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
	      }
	      else { last; }
	  }
	  if (! ($path)) {
              # we take out the dash because the images, may not have them.
	      foreach my $a (@archs)
	      {
		  if (! ($path)) {
		      $rpmfile = "${rpm}*.$a.rpm";
		      $path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
		  }
		  if ($path) { last; }
	      }
	  }
	  #SSS_END

#          if ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/)
#          {
#	    $rpmfile = "${rpm}-*.i?86.rpm";
#	    $path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
#          }
#		  elsif ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/)
#          {
#   		     $rpmfile = "${rpm}-*.ppc64.rpm";
#        	$path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
#			if( !($path) )
#			{
#				$rpmfile = "${rpm}-*.ppc.rpm";
#            	$path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
#			}
#          }
#          else
#          {
#	    $rpmfile = "${rpm}-*.ia64.rpm";
#	    $path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
#            if (!($path))
#            {
#              $rpmfile = "${rpm}-*.i?86.rpm";
#	      $path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
#            }
#          }
#          if (!($path))
#          {
#	     $rpmfile = "${rpm}-*.noarch.rpm";
#	     $path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
#          }
#          if (!($path))
#          {
#             if ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/)
#             {
#                 # PTF: (09/09/2002)
#                 # we take out the dash because the images, may not have them.
#
#                 $rpmfile = "${rpm}*.i?86.rpm";
#                 $path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
#             }
#             else
#             {
#                $rpmfile = "${rpm}*.ia64.rpm";
#                $path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
#                if (!($path))
#                {
#                  $rpmfile = "${rpm}*.i?86.rpm";
#                  $path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
#                }
#             }
#             if (!($path))
#             {
#                $rpmfile = "${rpm}*.noarch.rpm";
#                $path = InstallUtils->findRpmInPath_java($rpmfile, @my_path);
#             }
#           }
       } #Process the IBMJava rpm end.....
       else # Not IBMJava
       {
	  #SSS_BEGIN
	  foreach my $a (@archs)
	  {
	      if (! ($path)) {
		  $rpmfile = "${rpm}-*.$a.rpm";
		  $path = InstallUtils->findRpmInPath($rpmfile, @my_path);
	      }
	      else { last; }
	  }
	  if (! ($path)) {
              # we take out the dash because the images, may not have them.
	      foreach my $a (@archs)
	      {
		  if (! ($path)) {
		      $rpmfile = "${rpm}*.$a.rpm";
		      $path = InstallUtils->findRpmInPath($rpmfile, @my_path);
		  }
		  if ($path) { last; }
	      }
	  }
	  #SSS_END

#          if ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/)
#          {
#	    $rpmfile = "${rpm}-*.i?86.rpm";
#	    $path = InstallUtils->findRpmInPath($rpmfile, @my_path);
#          }
#		  elsif ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/)
#          {
#   		     $rpmfile = "${rpm}-*.ppc64.rpm";
#        	$path = InstallUtils->findRpmInPath($rpmfile, @my_path);
#			if( !($path) )
#			{
#				$rpmfile = "${rpm}-*.ppc.rpm";
#            	$path = InstallUtils->findRpmInPath($rpmfile, @my_path);
#			}
#          }
#          else
#          {
#	    $rpmfile = "${rpm}-*.ia64.rpm";
#	    $path = InstallUtils->findRpmInPath($rpmfile, @my_path);
#            if (!($path))
#            {
#              $rpmfile = "${rpm}-*.i?86.rpm";
#	      $path = InstallUtils->findRpmInPath($rpmfile, @my_path);
#            }
#          }
#          if (!($path))
#          {
#	     $rpmfile = "${rpm}-*.noarch.rpm";
#	     $path = InstallUtils->findRpmInPath($rpmfile, @my_path);
#          }
#          if (!($path))
#          {
#             if ($::PREREQS_ATTR{'PkgArchitecture'} =~ /i?86/)
#             {
#                 # PTF: (09/09/2002)
#                 # we take out the dash because the images, may not have them.
#
#                 $rpmfile = "${rpm}*.i?86.rpm";
#                 $path = InstallUtils->findRpmInPath($rpmfile, @my_path);
#             }
#             else
#             {
#                $rpmfile = "${rpm}*.ia64.rpm";
#                $path = InstallUtils->findRpmInPath($rpmfile, @my_path);
#                if (!($path))
#                {
#                  $rpmfile = "${rpm}*.i?86.rpm";
#                  $path = InstallUtils->findRpmInPath($rpmfile, @my_path);
#                }
#             }
#             if (!($path))
#             {
#                $rpmfile = "${rpm}*.noarch.rpm";
#                $path = InstallUtils->findRpmInPath($rpmfile, @my_path);
#             }
#           }
          }
	  if (defined($path)) 
	  { 
            $rpmfile = InstallUtils->getRecentVersionRpm($path, $rpmfile);
            chomp $rpmfile;
            my $rpmfile_basename;
            ($rpmfile_basename = $rpmfile) =~ s:^.*/::g;

             #find the shortname
             my @parts=split /-/, $rpmfile_basename;
             my $shortname = shift @parts;
             my $p;
             SNAME: while ($p = shift @parts){
                      if($p !~ m/^\d/){ #if it doesn't start with a digit
                          $shortname .= "-$p"; }
                      else{ unshift @parts, $p; last SNAME; }
                    }
          
             push @{$::temp_CmdLinePathVersionName{$path}},$rpmfile_basename;
             $::Rpm_name_path{$shortname}{'BaseName'}=$rpmfile_basename;
             $::Rpm_name_path{$shortname}{'Location'}=$path;
	  }
          else { push(@::RpmsNotFoundInCmdPath, $rpm); }
    }
 } 
 elsif ($::PREREQS_ATTR{'OSName'} eq "AIX")
 {
    NodeUtils->message("I", 'IMsgCOPYING_INSTALLP');
    # Copy RPMs from CSM distribution CD-ROM or directory to dirctory structure
    foreach my $image (@::csm_packages){
        my $file = "${image}";
        my @images = InstallUtils->findImagesInPath($file, @::PKGPATH);
        foreach my $image (@images){
            my $basename;              
            ($basename = $image) =~ s:^.*/::g;
            my $path;
            ($path = $image) =~ s/$basename//g;
            &copyRpm($basename, $path, $::INSTALLDIR_CSMINSTLP);
        }
    }

    `inutoc $::INSTALLDIR_CSMINSTLP`;
 }
 else
 {
     NodeUtils->message('E2', 'EMsgINVALID_OSTYPE');
 }
}

#-------------------------------------------------------------------------------#
#  create_rpm_list_to_install                                                   #
#	Create the rpm list to be installed. The Rpms_to_be_installed hash the  #
#	List of the rpms to be installed.Rpms_already_installed hash has the    #
#	list of rpms that are already been installed and RpmsNotFoundInCmdPath  #
# 	hash has the rpms list which are not found.				#
#                                                                               #
#-------------------------------------------------------------------------------#
sub create_rpm_list_to_install
{
  my (@my_rpm_array, @my_result);
  my $my_rpm_string;
  foreach my $path (keys %::temp_CmdLinePathVersionName)
  {
   @my_rpm_array=();
   foreach my $my_rpm (@{$::temp_CmdLinePathVersionName{$path}})
   {
     push(@my_rpm_array,($path."/".$my_rpm));
   }
   $my_rpm_string=join(" ",@my_rpm_array);
   my $cmd="$::RPMCMD -qp --qf '%{Name}::%{Version}::%{Release}::%{Arch}\n' $my_rpm_string";
   @my_result=NodeUtils->runcmd("$cmd",-1);
   foreach my $i (@my_result)
   {
     if (!($i=~/.*::.*::.*::.*/)) { next;} 
     my ($name, $version, $release, $arch)=split(/::/,$i);
     if (exists $::CmdLinePathVersionName{$arch})
     {
       if (exists $::CmdLinePathVersionName{$arch}{$name})
       {
         my $ext_ver=$::CmdLinePathVersionName{$arch}{$name}{'Version'};
         my $ext_rel=$::CmdLinePathVersionName{$arch}{$name}{'Release'};
         if (InstallUtils->testVersion($version,">=",$ext_ver,$release,$ext_rel))
         {
           $::CmdLinePathVersionName{$arch}{$name}{'Version'}=$version;
           $::CmdLinePathVersionName{$arch}{$name}{'Release'}=$release;
           $::CmdLinePathVersionName{$arch}{$name}{'Location'}=$path;
           $::CmdLinePathVersionName{$arch}{$name}{'Longname'}=$name."-".$version."-".$release;
          }
       }
       else
       {
         $::CmdLinePathVersionName{$arch}{$name}{'Version'}=$version;
         $::CmdLinePathVersionName{$arch}{$name}{'Release'}=$release;
         $::CmdLinePathVersionName{$arch}{$name}{'Location'}=$path;
         $::CmdLinePathVersionName{$arch}{$name}{'Longname'}=$name."-".$version."-".$release;
       }##
     }
     else
     {
       $::CmdLinePathVersionName{$arch}{$name}{'Version'}=$version;
       $::CmdLinePathVersionName{$arch}{$name}{'Release'}=$release;
       $::CmdLinePathVersionName{$arch}{$name}{'Location'}=$path;
       $::CmdLinePathVersionName{$arch}{$name}{'Longname'}=$name."-".$version."-".$release;
     }
   } 
 }
 #check the version and populate the to be copied hash and version not staisfied hash
 foreach my $arch (keys %::CmdLinePathVersionName)
 {
     foreach my $rpm_name (keys %{$::CmdLinePathVersionName{$arch}})
     {
       if (exists $::rpmBasenames{$rpm_name})
       {
         my $def_ver=$::PackageVersionName_Server{$rpm_name}{'Version'};
         my $def_rel=$::PackageVersionName_Server{$rpm_name}{'Release'};
         my $path_ver=$::CmdLinePathVersionName{$arch}{$rpm_name}{'Version'};
         my $path_rel=$::CmdLinePathVersionName{$arch}{$rpm_name}{'Release'};
         my $exist_ver=$::rpmBasenames{$rpm_name}{'Version'};
         my $exist_rel=$::rpmBasenames{$rpm_name}{'Release'};
         if (InstallUtils->testVersion($path_ver,">=",$def_ver,$path_rel,$def_rel))
         {
            if ($::CmdLinePathVersionName{$arch}{$rpm_name}{'Longname'} eq $::rpmBasenames{$rpm_name}{'Longname'})
            {
              push (@::Rpms_already_installed, $rpm_name);
              next;
            }
            elsif (InstallUtils->testVersion($path_ver,">",$exist_ver,$path_rel,$exist_rel))
            {
               push (@::Rpms_to_be_installed, $rpm_name);
               next;
            } 
            elsif (InstallUtils->testVersion($path_ver,"<=",$exist_ver,$path_rel,$exist_rel))
            {
              push (@::Rpms_already_installed, $rpm_name);
              next;
            }
         }
         else
         {
            if (InstallUtils->testVersion($exist_ver,">=",$def_ver,$exist_rel,$def_rel))
            {
               push (@::Rpms_not_found_installed, $rpm_name);
               next;
            }
            else
            {
              push(@::Rpms_not_found, $rpm_name);
              next;
            }
         }
       }
       else 
       {
         # rpm not in the already installed list
         my $def_ver=$::PackageVersionName_Server{$rpm_name}{'Version'};
         my $def_rel=$::PackageVersionName_Server{$rpm_name}{'Release'};
         my $path_ver=$::CmdLinePathVersionName{$arch}{$rpm_name}{'Version'};
         my $path_rel=$::CmdLinePathVersionName{$arch}{$rpm_name}{'Release'};
         if (InstallUtils->testVersion($path_ver,">=",$def_ver,$path_rel,$def_rel))
         {
            push (@::Rpms_to_be_installed, $rpm_name);
            next;
         } else { push(@::Rpms_not_found, $rpm_name); next; }
       }
     }
 }
 my %temp_hash=();
 if ($::FORCE_INSTALL)
 {
   @::Rpms_to_be_installed=(@::Rpms_to_be_installed,@::Rpms_already_installed);
   @::Rpms_already_installed=();
 }
 foreach my $rpm (@::Rpms_to_be_installed) { $temp_hash{$rpm}++; }
 foreach my $rpm (@::RpmsNotFoundInCmdPath) 
 {
    if (!exists($temp_hash{$rpm})) { push (@::Rpms_not_found,$rpm); }
 }
}

#-------------------------------------------------------------------------------#
# display_opensrcLicence                                                        #
#	Display the open source product licence					#
#                                                                               #
#-------------------------------------------------------------------------------#
sub display_opensrcLicence
{
  my ($class, $rpm)=@_;
  # If this is one of the GPL open source packages, display a license
  # agreement.  Don't display the license again if it has already been
  # displayed.
  if (grep(/^$rpm$/, @::GPL_PACKAGES) and ! $::DISPLAYED_GPL_LICENSE)
  {
     if ($::PREREQS_ATTR{'PkgArchitecture'} =~ /ppc64/)
     {
       # NodeUtils->message('I', 'IMsgGPL_LICENSE_PPC64');
       # NodeUtils->message('P', 'IMsgPRESS_ENTER_TO_INSTALL_PPC64');
     }
     else
     {
        NodeUtils->message('I', 'IMsgGPL_LICENSE');
        NodeUtils->message('P', 'IMsgPRESS_ENTER_TO_INSTALL');
     	my $t = <STDIN>; chop($t);
     	if (length($t) > 0){
       	    NodeUtils->message('I', 'IMsgCSM_NOT_INSTALLING');
       	    exit 1;
        }
     }
     # Set to 1 so the license is not displayed again.
     $::DISPLAYED_GPL_LICENSE = 1;
   }
}

#-------------------------------------------------------------------------#
#                                                                         #
#    findRpmInPath_java                                                   #
#                                                                         #
#       Find the IBMJava rpm package in the given path                    #
#                                                                         #
#-------------------------------------------------------------------------#
sub findRpmInPath_java
{
   my ($class, $rpmfile, @path) = @_;
   foreach my $p (@path)
   {
     my $filename = "$p/$rpmfile";
     # use the ls command here instead of the -e operator so we can handle *s in the names
     if (`/bin/ls $filename 2> /dev/null`)
     {
        # process the rpm  name for the packager...
        my @outout=NodeUtils->runcmd("$::RPMCMD -qp --qf '%{Name}::%{Version}::%{Release}::%{Arch}::%{Vendor}\n' $filename", -1);
        foreach my $i (@outout)
        {
           if (!($i=~/.*::.*::.*::.*::.*/)) { next;}
           my ($name, $version, $release, $arch, $packager_name)=split(/::/,$i);
           if ( ($packager_name =~ /International Business Machines/) || ($packager_name =~ /UnitedLinux/) )
           {
             NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','V', 'IMsgFOUND_RPM', $rpmfile, $p);
             return $p;
           }
           else
           { next;}
        }
     }
   }
   return undef;
}


#-------------------------------------------------------------------------------#
#  copyBinaries                                                                 #
#    This function takes functionality from installms so that the commands      #
#    updatenode, rmnode, and csmsetupnim can ensure that the latest binaries    #
#    are located in the /csminstall/csm directory.  This function does the      #
#    following:                                                                 #
#      1. create the /csminstall/csm directory if it doesn't exist              #
#      2. create the /csminstall/csm/script directories                         #
#      3. copy the binaries into /csminstall/csm, including the defs and pkdefs #
#         directory, if the timestamp of the files in /csminstall/csm are older #
#         than those installed on the system.                                   #
#      4. copies updateglibc._RedHat80Nodes for RedHat                          #
#      5. copies /tftpboot files for Linux                                      #
#      6. Copies hardware control ucode files on AIX to /csminstall/AIX/ucode   #
#                                                                               #
#-------------------------------------------------------------------------------#
sub copyBinaries
{

    my $INSTALLDIR_CSM = "/csminstall/csm";

    #
    # Get information about the OS, distribution, and package
    #
    my %pkginfo;
    $pkginfo{'OSName'} = InstallUtils->get_OSName;   #AIX/Linux
    $pkginfo{'DistributionVersion'} = InstallUtils->get_DistributionVersion;
    $pkginfo{'PkgArchitecture'} = InstallUtils->get_PkgArchitecture;
    $pkginfo{'DistributionName'} = InstallUtils->get_DistributionName; #RedHat
    if ( $pkginfo{'PkgArchitecture'} =~ /i.86/) #SSS
    {
	$pkginfo{'PkgArchitecture'}="i386";
    }
    $pkginfo{'CsmCoreVersion'} = InstallUtils->get_CSMVersion("csm\.core");
    if (length($pkginfo{'CsmCoreVersion'})==0)
    {
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall','E2', 'EMsgNO_CORE');
    }
 
    #
    # Create the /csminstall/csm directory if it doesn't exist
    # NOTE: This function will not create any operating system specific
    #       directories, such as /csminstall/csm/AIX
    if ( ! -d "$INSTALLDIR_CSM" ) 
    {
	mkpath ($INSTALLDIR_CSM, $::VERBOSE, 0755);
    }

    # make the script sub-directories if they don't exist
    if ( ! -d "$::PREINSTALLDIR" ) 
    {
	mkpath ($::PREINSTALLDIR, $::VERBOSE, 0755);
    }
    if ( ! -d "$::POSTINSTALLDIR" ) 
    {
	mkpath ($::POSTINSTALLDIR, $::VERBOSE, 0755);
    }
    if ( ! -d "$::UPDATEDIR" ) 
    {
	mkpath ($::UPDATEDIR, $::VERBOSE, 0755);
    }
    if ( ! -d "$::SCRIPTDATADIR" ) 
    {
	mkpath ($::SCRIPTDATADIR, $::VERBOSE, 0755);
    }

    #
    # Store the pkgdefs into a hash
    #
    my %Svrpkgdefs = InstallUtils->get_pkgdefs($pkginfo{'OSName'}, $pkginfo{'DistributionName'}, $pkginfo{'DistributionVersion'}, $pkginfo{'PkgArchitecture'}, "MgmtServer", $pkginfo{'CsmCoreVersion'});

    #
    # Get the list of binaries to copy
    #
    my @csm_bin_file_csm =(@{$Svrpkgdefs{csm_bin_copy_csm}});
    my @csm_bin_file_tftpboot=(@{$Svrpkgdefs{csm_bin_copy_tftpboot}});

    # Files to copy to /csminstall/csm
    foreach my $bin_file (@csm_bin_file_csm)
    {
          my @temper_val=();
          my $temper_file=();
          @temper_val=split(/\//,$bin_file);
          $temper_file= $temper_val[$#temper_val];
          $temper_file = $INSTALLDIR_CSM."\/".$temper_file;
          if (-e $temper_file)
          {
             NodeUtils->runcmd("$::RM -rf $temper_file", -2);
             NodeUtils->runcmd("$::COPY -pr $bin_file $INSTALLDIR_CSM", -2);
          }
          else
          {
             NodeUtils->runcmd("$::COPY -pr $bin_file $INSTALLDIR_CSM", -2);
          }

    }

    #
    # copy the files in /opt/csm/install/sample_scripts to /csminstall/csm/scripts
    #
    NodeUtils->runcmd("$::COPY -p /opt/csm/install/sample_scripts/* $::SCRIPTDIR", 0);

    # Copy updateglibc scripts to /csminstall/csm/scripts if DistributionName is
    # RedHat
    #todo: only do copies of scripts with ._ if the nodegrp exists.
#    if ( $pkginfo{'DistributionName'} eq "RedHat" )
#    {
#	NodeUtils->runcmd("$::COPY -p /opt/csm/install/scripts/installprereboot/updateglibc._RedHat80Nodes $::SCRIPTDIR/installprereboot/.", -2 );
#    }

    if ($pkginfo{'OSName'} eq "Linux")
    {
       # Files to copy to /tftpboot
       foreach my $bin_file (@csm_bin_file_tftpboot)
       {
             NodeUtils->runcmd("$::COPY $bin_file /tftpboot", -2);
       }
       # after files are copied, change the permission:
       InstallUtils->secureFilePermissions('/tftpboot','0110','0440');
       #files needed for RMC thin client which is used by the nodes to communicate
       # to the MS during install 
       my @files = ( "/usr/sbin/rsct/lib/libct_mc.so",
                     "/usr/sbin/rsct/lib/libct_cu.so",
                     "/usr/sbin/rsct/lib/libct_tr.so",
                     "/usr/sbin/rsct/lib/libct_pmsg.so",
                     "/usr/sbin/rsct/lib/libct_sec.so",
                     "/usr/sbin/rsct/lib/libct_idm.so",
                     "/usr/sbin/rsct/lib/libct_ffdc.so",
                     "/usr/sbin/rsct/cfg/ctsec.cfg",
                     "/usr/sbin/rsct/bin/runact-api",
                     "/opt/csm/csmbin/nodestatus.client" );

       foreach my $file (@files)
       {
         if(! -f $file)
         {
	   NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E","EMsgNO_CopyFile",$file, "/csminstall/csm");
         }
         else
         {
           NodeUtils->runcmd("$::COPY -p $file /csminstall/csm", -2);
         }
       }
    }
    
    # Copies hardware control ucode files on AIX to /csminstall/AIX/ucode   
    if ($pkginfo{'OSName'} eq "AIX")
    {
	    if(! -e "/csminstall/AIX/ucode"){
		    mkdir "/csminstall/AIX/ucode",0077;
	    }
	    NodeUtils->runcmd("$::COPY -pr /opt/csm/install/ucode/ /csminstall/AIX/", -2);
    }

}

#-----------------------------------------------------------------------------------------------#
#  copy_csm_scripts                                                                             #
#	This function copies customization scripts from /opt/csm/install/scripts/ to                #
#	/csminstall/csm/scripts.								                                    #
#	If the extension of the script matches a defined node group or there is no extension,       #
#	then copy the script.																		#
#																								#
#	If the $verify parameter is 1,this function checks the target dir if the scripts exist,     #
#	if no, it will copy the scripts, otherwise, it will do nothing.                             #
#	If the $verify parameter is 0,this function will always copy the scripts.                   #
#-----------------------------------------------------------------------------------------------#
sub copy_csm_scripts
{
	my ($class,$verify) = @_;
	my $INSTALLPREREBOOT = "/opt/csm/install/scripts/installprereboot";
	my $INSTALLPOSTREBOOT = "/opt/csm/install/scripts/installpostreboot";
	my $UPDATE = "/opt/csm/install/scripts/update";

	my @groupname;
	my $targetdir;
	my $filename;
	my $basename;
	my $extension;
	my $junk;
	my $cmd;

	$cmd = "$::NODEGRP";
	@groupname = NodeUtils->runcmd($cmd,-1);
	if ($::RUNCMD_RC)
	{
		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E","EMsgBadScript",$cmd,$::RUNCMD_RC);
		return $::NOK;
	}

	foreach my $dirname ($INSTALLPREREBOOT,$INSTALLPOSTREBOOT,$UPDATE)
	{
		if ($dirname eq $INSTALLPREREBOOT)
		{
			$targetdir = $::PREINSTALLDIR;
		}elsif ($dirname eq $INSTALLPOSTREBOOT)
		{
			$targetdir = $::POSTINSTALLDIR;
		}elsif ($dirname eq $UPDATE)
		{
			$targetdir = $::UPDATEDIR;
		}
	
		if (-d $dirname)
		{
			if (! -d $targetdir)
			{
				mkpath ($targetdir, $::VERBOSE, 0755);
			}
			$cmd = "$::LS $dirname";
			my @filelist = NodeUtils->runcmd($cmd,-1);
        	if ($::RUNCMD_RC) 
			{
            	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgCANT_RUN',$cmd, $::RUNCMD_RC);
	            return $::NOK;
			}
			foreach $filename (sort @filelist)
			{
				chomp $filename;
				if (-d "$dirname/$filename") 
				{
            		NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"I", 'IMsgSkipSubDir', "$dirname/$filename");
					next;
				}

				if (! -x "$dirname/$filename")
				{
                	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"E", 'EMsgSkipScript', "$dirname/$filename");
					next;
				}
				($basename,$extension,$junk) = split(/\._/,$filename);
				if (defined($junk)) { next; }
				if (!defined($extension) || grep(/^$extension$/,@groupname)) 
				{
					if ( (! $verify) || (($verify) && (! -e "$targetdir/$filename" )))
					{
						$cmd = "$::COPY -f $dirname/$filename $targetdir";
						NodeUtils->runcmd($cmd,-2);
					}
				}
			}
		
		}
	}
	return $::OK;
}

# here we secure file permissions for a directory
# first argument: the root directory
# second argument: permissions for directories and sub directories
# third argument: permissions for files in these directories


sub secureFilePermissions{
    my $class = shift;
    my $dir = shift;
    #my $msg = shift;
    # set these variables global, since chPerm can not take
    # in any parameters:
    $::dirPermissions = shift;
    $::filePermissions = shift;
    #print "Securing $dir and all subdirectories and files\n";
    find(\&chPerm,$dir);
}

$::TFTPUSER = "tftpd";
$::TFTPGROUP = $::TFTPUSER;
sub chPerm{
    #my $v = $::VERBOSE;
    #$::VERBOSE = 1;
    if(-f "$File::Find::name"){
        NodeUtils->runcmd("chmod $::filePermissions $File::Find::name");
    }elsif(-d "$File::Find::name"){
        NodeUtils->runcmd("chmod $::dirPermissions $File::Find::name");
    }else{
        print "---> $File::Find::name is not a file nor a directory.\n";
	return;
    }
    # change owner and group:
    NodeUtils->runcmd("chown $::TFTPUSER $File::Find::name");
    NodeUtils->runcmd("chgrp $::TFTPGROUP $File::Find::name");
    #$::VERBOSE = $v;
}

sub createTftpUser{
    unless($::GREP){ $::GREP = '/bin/grep'}
    my $tftpuser = $::TFTPUSER;
    my $tftpgrp = $::TFTPGROUP;
    my $passwdFile = "/etc/passwd";
    my $grpFile	= "/etc/group";
    my $bak = "/tmp/csmjunk";
    
    # determine whether or not a tftpd user already exists:
    NodeUtils->runcmd("$::GREP -e '^$tftpuser' $passwdFile", -1);
    if($::RUNCMD_RC eq 0){
    	#print "tftpd user has already been created.\n" if $::VERBOSE;
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"V", 'IMsgUserAlreadyCreated', $tftpuser);
    }else{
        #print "Creating default tftpd user for the tftp server.\n";
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"I", 'IMsgCreatingTFTPUser', $tftpuser);
        # create tftpd user
        #
        NodeUtils->runcmd("cp $passwdFile $bak");
	NodeUtils->runcmd("echo '$tftpuser:x:6969:6969::/dev/null:/dev/null' | \
		cat $bak - > $passwdFile");
	# remove backup file
	NodeUtils->runcmd("rm $bak",-1);
    }
    NodeUtils->runcmd("$::GREP -e '^$tftpgrp' $grpFile", -1);
    if($::RUNCMD_RC eq 0){
    	#print "tftpd group has already been created.\n" if $::VERBOSE;
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"V", 'IMsgGroupAlreadyCreated', $tftpgrp);
    }else{
	#print "Creating default tftpd group for the tftp server.\n";
        NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,'csminstall',"I", 'IMsgCreatingTFTPGroup', $tftpgrp);
        NodeUtils->runcmd("cp $grpFile $bak");
	NodeUtils->runcmd("echo '$tftpgrp:x:6969:' | \
		cat $bak - > $grpFile");
	# remove backup file
	NodeUtils->runcmd("rm $bak",-1);
    }
}

#-----------------------------------------------------------------------------------------------#
#  checkRSReachability                                                                          #
#  this checks a list of machines (they do not need to be CSM nodes) to make sure they can be   #
#  reached via dsh                      							#
#												#
#  input: a reference to an array of hostnames							#
#  returns: a reference to an array of unreachable nodes                                        #
#-----------------------------------------------------------------------------------------------#
sub checkRSReachability{
	
	my ($class, $machines) = @_;
	$::REMOTE_SHELL = InstallUtils->getRemoteShell();
	my $file = InstallUtils->make_node_list_file($machines); #put all dest nodes in tmp file
	my $cmd;
	(my $cmd_basename = $::REMOTE_SHELL) =~ s:^.*/::;
	if ($cmd_basename ne "rsh" && (InstallUtils->supportedSSHversion())){ #can do SSH with options
		$cmd = "$::REMOTESHELL_EXPECT -ds $file";
	}
	else{
		$cmd = "$::REMOTESHELL_EXPECT -d $file";   
	}
	my $value = InstallUtils->supportedSSHversion();
	NodeUtils->messageFromCat('nodecmds.cat',$::MSGMAPPATH,'NodeUtils','V', 'IMsgCMD', $cmd);
	my @output = `$cmd 2>&1`;
	InstallUtils->close_delete_file($::NODE_LIST_FILE, $file); #delete node file
	my %reachable;
	my @unreachable;
	foreach my $line (@output){
		chomp $line;
		my @parts = split ":", $line;
		if($parts[1] =~ m/test.success/){ #this node is reachable
			$reachable{$parts[0]}=1;
		}
	}

	foreach my $n (@$machines){
		if(!$reachable{$n}){
			push @unreachable, $n;
		}
	}
	if(@unreachable){
		return(\@unreachable);
	}
}

#-----------------------------------------------------------------------------------------------#
# createSMSConfig										#
# put SMS node group info in file /csminstall/csm/var/smsupdate.info				#
#	input:											#
#												#
#	output:	a ref to a hash of all node groups						#
#-----------------------------------------------------------------------------------------------#
sub createSMSConfig{
	my %all_groups = ();
	my %node_index;
	my @empty_array = ();
	if(-e "/csminstall"){
		if(! -e "/csminstall/csm"){
			mkdir "/csminstall/csm",0077;
			mkdir "/csminstall/csm/config",0077;
		}
	}
	my $ref_nodegrp = NodeUtils->listNodeGroupExpMem(@empty_array);
	foreach my $row (@$ref_nodegrp){
		my ($groupname, $nodelist) = $row =~ /^(.+):\|:\{(.*)\}$/;
		my @nodes = split(/,/, $nodelist);
		foreach my $node (@nodes){
			push @{$node_index{$node}}, $groupname;
			push @{$all_groups{$groupname}}, $node;

		}
	}
	system("$::MKDIR -p /csminstall/csm/config/");
	open (GROUPFILE, ">/csminstall/csm/config/sms.info") or NodeUtils->message( 'E', 'EMsgCANT_OpenFile', "/csminstall/csm/config/sms.info", "$? $!"); 
	foreach my $node (keys %node_index){
		print GROUPFILE "$node: @{$node_index{$node}} \n";
	}
	close GROUPFILE;
	return \%all_groups;
}

#---------------------------------------------------------------------------#
# check_valid_install_distros					     						#
# Check whether current node's InstallDistributionName and 					#
# InstallDistributionVersion is valid for full installation					#
#	input:																	#
#	output:																	#
#-----------------------------------------------------------------------------------------------#
sub check_valid_install_distros{
	my ($class, $installmethod, $distro) = @_;
	my @VALID_AUTOYAST_DISTROS = ('SuSE8.1', 'SLES8.1');

	if (!grep(/^$distro$/,@VALID_AUTOYAST_DISTROS)) {
		return -1;
	}
	return 0;
}

1;
